package com.hywa.map.common.exception;

import static com.hywa.map.common.ConstantPool.SPLIT;
import static com.hywa.map.common.enums.CommonEnum.FAILURE;

import com.hywa.map.common.enums.CommonEnum;
import com.hywa.map.common.utils.StringUtils;
import com.hywa.map.common.utils.UUIDUtils;
import com.hywa.map.config.interceptor.TokenManager;
import com.hywa.map.config.interceptor.TokenModel;
import com.hywa.map.dao.entity.Log;
import com.hywa.map.dao.mapper.LogMapper;
import com.hywa.map.representation.ResponseBase;

import java.io.FileNotFoundException;
import java.util.Date;
import java.util.List;
import javax.servlet.http.HttpServletRequest;

import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.dao.DataIntegrityViolationException;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.util.CollectionUtils;
import org.springframework.validation.ObjectError;
import org.springframework.web.bind.MethodArgumentNotValidException;
import org.springframework.web.bind.MissingServletRequestParameterException;
import org.springframework.web.bind.annotation.CrossOrigin;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.RestControllerAdvice;
import org.springframework.web.multipart.MaxUploadSizeExceededException;
import org.springframework.web.multipart.support.MissingServletRequestPartException;


@Slf4j
@CrossOrigin
@RestControllerAdvice
public class GlobalDefaultExceptionHandler {
    @Autowired
    TokenManager tokenManager;
    @Autowired
    private LogMapper logMapper;

    @ExceptionHandler(value = Exception.class)
    public ResponseEntity<ResponseBase> defaultErrorHandler(
            HttpServletRequest req, Exception e) {
        String userId = "临时用户";
        try {
            TokenModel accessToken = tokenManager.getToken(req.getHeader("accessToken"));
            userId = accessToken.getUserId().split(SPLIT)[1];
        } catch (Exception internalException) {

        }

        Log operationLog = new Log();
        operationLog.setId(UUIDUtils.randomUUID());
        operationLog.setCreateTime(new Date());
        operationLog.setReqIp(req.getRemoteAddr());
        operationLog.setReqPort(req.getRemotePort());
        operationLog.setReqUri(req.getRequestURI());
        operationLog.setErrorMessage(e.getMessage());
        operationLog.setParameter(req.getQueryString());
        log.error(req.getRequestURL() + "<:=====:>" +
                req.getMethod() + "<:=====:>" +
                req.getQueryString() + "<:=====:>" +
                e.getMessage());
        e.printStackTrace();

        ResponseBase responseBase = new ResponseBase<>();

        if (e instanceof GlobalException) {
            log.error(e.getClass().toString());
            GlobalException globalException = (GlobalException) e;
            Short errorCode = globalException.getErrorCode();
            String message = globalException.getMessage();
            operationLog.setErrorCode(errorCode);
            operationLog.setErrorMessage(message);
            operationLog.setUserId(globalException.getUserName() == null ? userId : globalException.getUserName());
            if("/user/pcLogin".equals(req.getRequestURI())){
                operationLog.setMenuName("登录");
                operationLog.setContent("用户登录系统");
            } else {
                operationLog.setMenuName(globalException.getMenuName().split("\\-")[0]);
                operationLog.setContent(globalException.getMenuName().split("\\-")[1]);
            }

            responseBase.setRetHead(errorCode, message);
        } else if (e instanceof MethodArgumentNotValidException) {
            log.error(e.getClass().toString());
            List<ObjectError> allErrors = ((MethodArgumentNotValidException) e).getBindingResult()
                    .getAllErrors();
            if (!CollectionUtils.isEmpty(allErrors)) {
                String errorMessage = allErrors.get(0).getDefaultMessage();
                log.error(errorMessage);
                responseBase.setRetHead(FAILURE.getIndex(), errorMessage);
            }
//            //获得所有错误提示码,将错误提示码拼接成字符串
//            List<String> errorMessages = new ArrayList<>();
//            allErrors.forEach(objectError -> {
//                if (!errorMessages.contains(objectError.getDefaultMessage())) {
//                    errorMessages.add(objectError.getDefaultMessage());
//                }
//                log.error(objectError.getDefaultMessage());
//            });
//            responseBase.setRetHead(FAILURE.getIndex(), Joiner.on(",").join(errorMessages));
        } else if (e instanceof MissingServletRequestParameterException) {
            log.error(e.getClass().toString());
            responseBase.setRetHead(
                    CommonEnum.PARAMETER_NOT_NULL.getIndex(),
                    CommonEnum.PARAMETER_NOT_NULL.getValue());
        } else if (e instanceof FileNotFoundException) {
            log.error(e.getClass().toString());
            responseBase.setRetHead(
                    CommonEnum.FILE_NOT_FOUND.getIndex(),
                    CommonEnum.FILE_NOT_FOUND.getValue());
        } else if (e instanceof MaxUploadSizeExceededException) {
            log.error(e.getClass().toString());
            responseBase.setRetHead(
                    CommonEnum.MAX_UPLOAD_SIZE_EXCEEDED.getIndex(),
                    CommonEnum.MAX_UPLOAD_SIZE_EXCEEDED.getValue());
        } else if (e instanceof MissingServletRequestPartException) {
            log.error(e.getClass().toString());
            responseBase.setRetHead(
                    CommonEnum.FILE_NOT_NULL.getIndex(),
                    CommonEnum.FILE_NOT_NULL.getValue());
        } else if (e instanceof NumberFormatException) {
            log.error(e.getClass().toString());
            responseBase.setRetHead(
                    CommonEnum.NUMBER_FORMAT.getIndex(),
                    CommonEnum.NUMBER_FORMAT.getValue());
        } else if (e instanceof MaxUploadSizeExceededException) {
            log.error(e.getClass().toString());
            responseBase.setRetHead(
                    CommonEnum.TOO_BIG_A_DOCUMENT.getIndex(),
                    CommonEnum.TOO_BIG_A_DOCUMENT.getValue());
        } else if (e instanceof DataIntegrityViolationException) {
            responseBase.setRetHead(
                    CommonEnum.THE_CONTENT_IS_TOO_LARGE_TO_SAVE.getIndex(),
                    CommonEnum.THE_CONTENT_IS_TOO_LARGE_TO_SAVE.getValue());
        } else {
            log.error(e.getClass().toString() + ":" + e.getMessage());
            responseBase.setRetHead(
                    CommonEnum.INTERNAL_ERROR.getIndex(),
                    CommonEnum.INTERNAL_ERROR.getValue());
        }

        try {
            if (!StringUtils.isEmpty(operationLog.getUserId())) {
                logMapper.insert(operationLog);
            }
        } catch (Exception internalException) {
            log.error(internalException.getMessage());
        }

        return new ResponseEntity<>(responseBase, HttpStatus.OK);
    }
}
