package com.hywa.map.common.log;

import javassist.ClassClassPath;
import javassist.ClassPool;
import javassist.CtClass;
import javassist.CtMethod;
import javassist.NotFoundException;
import javassist.bytecode.AnnotationsAttribute;
import javassist.bytecode.ConstPool;
import javassist.bytecode.MethodInfo;
import javassist.bytecode.annotation.Annotation;
import javassist.bytecode.annotation.StringMemberValue;

public class AnnotationUtils {

    private static AnnotationUtils mInstance;

    public AnnotationUtils() {
    }

    public static AnnotationUtils getInstance() {
        if (mInstance == null) {
            synchronized (AnnotationUtils.class) {
                if (mInstance == null) {
                    mInstance = new AnnotationUtils();
                }
            }
        }
        return mInstance;
    }

    public void setAnnotationFieldValue(String className, String methodName, String annoName,
            String fieldName, String fieldValue) throws NotFoundException {
        ClassPool classPool = ClassPool.getDefault();
        ClassClassPath classClassPath = new ClassClassPath(this.getClass());
        classPool.insertClassPath(classClassPath);
        CtClass ctClass = classPool.get(className);
        CtMethod ctMethod = ctClass.getDeclaredMethod(methodName);
        MethodInfo methodInfo = ctMethod.getMethodInfo();
        ConstPool constPool = methodInfo.getConstPool();
        AnnotationsAttribute aa = (AnnotationsAttribute) methodInfo
                .getAttribute(AnnotationsAttribute.visibleTag);
        Annotation annotation = aa.getAnnotation(annoName);
        if (annotation != null) {
            annotation.addMemberValue(fieldName, new StringMemberValue(fieldValue, constPool));
            aa.setAnnotation(annotation);
            methodInfo.addAttribute(aa);
        }

    }

    public String getAnnotationFieldValue(String className, String methodName, String annoName,
            String fieldName)
            throws NotFoundException {
        ClassPool classPool = ClassPool.getDefault();
        ClassClassPath classClassPath = new ClassClassPath(this.getClass());
        classPool.insertClassPath(classClassPath);
        CtClass ctClass = classPool.get(className);
        CtMethod ctMethod = ctClass.getDeclaredMethod(methodName);
        MethodInfo methodInfo = ctMethod.getMethodInfo();
        AnnotationsAttribute aa = (AnnotationsAttribute) methodInfo
                .getAttribute(AnnotationsAttribute.visibleTag);
        String value = "";
        if (aa != null) {
            Annotation annotation = aa.getAnnotation(annoName);
            if (annotation != null) {
                value = ((StringMemberValue) annotation.getMemberValue(fieldName)).getValue();
            }
        }
        return value;
    }
}
