package com.hywa.map.common.log;

import static com.hywa.map.common.ConstantPool.SPLIT;
import static com.hywa.map.common.enums.CommonEnum.SUCCESS;

import com.hywa.map.common.utils.StringUtils;
import com.hywa.map.common.utils.UUIDUtils;
import com.hywa.map.config.interceptor.TokenManager;
import com.hywa.map.config.interceptor.TokenModel;
import com.hywa.map.dao.mapper.LogMapper;

import java.util.Date;

import javassist.NotFoundException;

import javax.servlet.http.HttpServletRequest;

import lombok.extern.slf4j.Slf4j;
import org.aspectj.lang.JoinPoint;
import org.aspectj.lang.annotation.AfterReturning;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.annotation.Pointcut;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

@Slf4j
@Aspect
@Component
public class LogAspect {

    @Autowired
    TokenManager tokenManager;
    @Autowired
    private LogMapper logMapper;

    // 设置切点表达式
    @Pointcut("execution(* com.hywa.map.controller..*(..))")
    private void pointcut() {
    }

    //方法后置切面
    //@After(value = "pointcut()")
    @AfterReturning(value = "pointcut()")
    public void after(JoinPoint joinPoint) throws NotFoundException {
        String className = joinPoint.getTarget().getClass().getName();
        String methodName = joinPoint.getSignature().getName();

        String logStr = AnnotationUtils.getInstance()
                .getAnnotationFieldValue(className, methodName,
                        com.hywa.map.common.log.Log.class.getName(), "log");
        if (StringUtils.isNotEmpty(logStr)) {
            Date date = new Date();
            HttpServletRequest request = ((ServletRequestAttributes) RequestContextHolder
                    .getRequestAttributes()).getRequest();

            String userId = null;
            try {
                TokenModel accessToken = tokenManager.getToken(request.getHeader("accessToken"));
                userId = accessToken.getUserId().split(SPLIT)[1];
            } catch (Exception internalException) {
                try{
                    userId = request.getSession().getAttribute("userId").toString();
                }catch (Exception e){
                    userId ="anonymous";
                }
            }
            com.hywa.map.dao.entity.Log log = new com.hywa.map.dao.entity.Log();
            log.setUserId(userId);
            log.setId(UUIDUtils.randomUUID());
            log.setReqUri(request.getRequestURI());
            log.setCreateTime(date);
            log.setMenuName(logStr.split("\\-")[0]);
            log.setContent(logStr.split("\\-")[1]);
            log.setReqPort(request.getRemotePort());
            log.setErrorCode(SUCCESS.getIndex());
            log.setReqIp(request.getRemoteAddr());
            logMapper.insert(log);
        }
    }
}
