package com.hywa.map.common.utils;

import org.apache.log4j.Logger;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.springframework.web.multipart.MultipartFile;

import java.io.*;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * excel读写工具类
 *
 * @author hywa
 * 2016年8月21日
 */
public class ReadXlsUtils {
    private static Logger logger = Logger.getLogger(ReadXlsUtils.class);
    private final static String xls = "xls";
    private final static String xlsx = "xlsx";

    /**
     * 读入excel文件，解析后返回
     *
     * @param file
     * @throws IOException
     */
    public static List<String[]> readExcel(String file, int numberOfRow, int numberOfCells) throws IOException {


        //获得Workbook工作薄对象
        Workbook workbook = getWorkBook(file);
        //创建返回对象，把每行中的值作为一个数组，所有行作为一个集合返回
        List<String[]> list = new ArrayList<String[]>();
        if (workbook != null) {
            //for(int sheetNum = 0;sheetNum < workbook.getNumberOfSheets();sheetNum++){
            //获得当前sheet工作表
            Sheet sheet = workbook.getSheetAt(0);
                /*if(sheet == null){
                    continue;
                }*/
            //获得当前sheet的开始行
            int firstRowNum = sheet.getFirstRowNum();
            //获得当前sheet的结束行
            int lastRowNum = sheet.getLastRowNum();
            //循环除了第一行的所有行
            for (int rowNum = firstRowNum + numberOfRow; rowNum <= lastRowNum; rowNum++) {
                //获得当前行
                Row row = sheet.getRow(rowNum);
                if (row == null) {
                    continue;
                }
                //获得当前行的开始列
                //int firstCellNum = row.getFirstCellNum();
                int firstCellNum = 0;
                //获得当前行的列数
                //int lastCellNum = row.getPhysicalNumberOfCells()+1;
                int lastCellNum = numberOfCells;
                String[] cells = new String[lastCellNum];
                //循环当前行
                for (int cellNum = firstCellNum; cellNum < lastCellNum; cellNum++) {
                    Cell cell = row.getCell(cellNum);
                    cells[cellNum] = getCellValue(cell).replaceAll("\n", "");
                }
                String cellStr = String.join("", cells);
                if (StringUtils.isNotEmpty(cellStr)) {
                    list.add(cells);
                }
            }
            //}
            workbook.close();
        }
        return list;
    }

    public static void checkFile(MultipartFile file) throws IOException {
        //判断文件是否存在
        if (null == file) {
            logger.error("文件不存在！");
            throw new FileNotFoundException("文件不存在！");
        }
        //获得文件名
        String fileName = file.getOriginalFilename();
        //判断文件是否是excel文件
        if (!fileName.endsWith(xls) && !fileName.endsWith(xlsx)) {
            logger.error(fileName + "不是excel文件");
            throw new IOException(fileName + "不是excel文件");
        }
    }

    public static Workbook getWorkBook(String fileName) {
        File file = new File(fileName);
        //获得文件名
        //创建Workbook工作薄对象，表示整个excel
        Workbook workbook = null;
        try {
            //获取excel文件的io流
            InputStream is = new FileInputStream(file);
            //根据文件后缀名不同(xls和xlsx)获得不同的Workbook实现类对象
            if (fileName.endsWith(xls)) {
                //2003
                workbook = new HSSFWorkbook(is);
            } else if (fileName.endsWith(xlsx)) {
                //2007
                workbook = new XSSFWorkbook(is);
            }
        } catch (IOException e) {
            logger.info(e.getMessage());
        }
        return workbook;
    }

    public static String getCellValue(Cell cell) {
        String cellValue = "";
        if (cell == null) {
            return cellValue;
        }
        //把数字当成String来读，避免出现1读成1.0的情况
        if (cell.getCellType() == Cell.CELL_TYPE_NUMERIC) {
            cell.setCellType(Cell.CELL_TYPE_STRING);
        }
        //判断数据的类型
        switch (cell.getCellType()) {
            case Cell.CELL_TYPE_NUMERIC: //数字
                cellValue = String.valueOf(cell.getNumericCellValue());
                break;
            case Cell.CELL_TYPE_STRING: //字符串
                cellValue = String.valueOf(cell.getStringCellValue());
                break;
            case Cell.CELL_TYPE_BOOLEAN: //Boolean
                cellValue = String.valueOf(cell.getBooleanCellValue());
                break;
            case Cell.CELL_TYPE_FORMULA: //公式
                cellValue = String.valueOf(cell.getCellFormula());
                break;
            case Cell.CELL_TYPE_BLANK: //空值
                cellValue = "";
                break;
            case Cell.CELL_TYPE_ERROR: //故障
                cellValue = "非法字符";
                break;
            default:
                cellValue = "未知类型";
                break;
        }
        return cellValue;
    }
    /**
     * 将list数据写入文件
     *
     * @param list     数据集合
     * @param fileName 文件名称
     * @return String 0-写文件成功 1-写文件失败
     */
    public static boolean writeFile(ArrayList<String> list, String fileName, boolean isAppend) {
        File file = new File(fileName);
        try {
            org.apache.commons.io.FileUtils.writeLines(file, "utf-8", list, isAppend);
        } catch (IOException e) {
            return false;
        }
        return true;
    }
    public static void main(String[] args) throws IOException {
        List<String[]> list = readExcel("C:\\Users\\Administrator\\Desktop\\标准数据导入(农产品库).xlsx", 2, 3);
        String[] nameArray = new String[list.size()];
        String[] dataArray = new String[list.size()];
        Map<String, String> idMap = new HashMap<>();
        ArrayList<String> sqlList = new ArrayList();
        for (int i = 0; i < list.size(); i++) {
            dataArray[i] = list.get(i)[2];
        }
        for (int i = 0; i < list.size(); i++) {
            String uuid = UUIDUtils.randomUUID();
            String allParentId = "";
            String parentId = "";
            boolean flag = false;
            String[] label = list.get(i);
            String name = label[1];
            String code = label[2];
            String codeBak = label[2];
            String codeTemp = "";
            idMap.put(codeBak, uuid);
            if (code.length() == 2) {
                code = "000" + code;
                allParentId = "#";
                parentId = "#";
            } else if (code.length() == 3) {
                code = "000" + code.substring(0, 2) + "00" + code;
                allParentId = idMap.get(codeBak.substring(0, 2)) + ",#";
                parentId = idMap.get(codeBak.substring(0, 2));
            } else if (code.length() == 4) {
                code = "000" + code.substring(0, 2) + "00" + code.substring(0, 3) + "0" + code;
                allParentId = idMap.get(codeBak.substring(0, 2)) + "," + idMap.get(codeBak.substring(0, 3)) + ",#";
                parentId = idMap.get(codeBak.substring(0, 3));
            } else if (code.length() == 5) {
                code = "000" + code.substring(0, 2) + "00" + code.substring(0, 3) + "0" + code.substring(0, 4) + code;
                allParentId = idMap.get(codeBak.substring(0, 2)) + "," + idMap.get(codeBak.substring(0, 3)) + "," + idMap.get(codeBak.substring(0, 4)) + ",#";
                parentId = idMap.get(codeBak.substring(0, 4));
            } else {
                code = code.replaceAll(" ", "");
                String[] codeArray = code.split("\\•");
                codeTemp = codeArray[1];
                if (codeTemp.endsWith("0")) {
                    codeTemp = "00" + codeTemp;
                    allParentId = idMap.get(codeBak.substring(0, 2)) + "," + idMap.get(codeBak.substring(0, 3)) + "," + idMap.get(codeBak.substring(0, 4)) + "," + idMap.get(codeBak.substring(0, 5)) + ",#";
                    parentId = idMap.get(codeBak.substring(0, 5));
                } else {
                    for (int j = 0; j < dataArray.length; j++) {
                        if (dataArray[j].equals(codeBak.substring(0, codeBak.length() - 1) + "0")) {
                            flag = true;
                            break;
                        }
                    }
                    if (flag) {
                        codeTemp = "00" + codeArray[1].substring(0, 2) + "0" + "00" + codeArray[1];
                        allParentId = idMap.get(codeBak.substring(0, 2)) + "," + idMap.get(codeBak.substring(0, 3)) + "," + idMap.get(codeBak.substring(0, 4)) + "," + idMap.get(codeBak.substring(0, 5)) + "," + idMap.get(codeBak.substring(0, codeBak.length() - 1) + "0") + ",#";
                        parentId = idMap.get(codeBak.substring(0, codeBak.length() - 1) + "0");
                    } else {
                        codeTemp = "00" + codeTemp;
                        allParentId = idMap.get(codeBak.substring(0, 2)) + "," + idMap.get(codeBak.substring(0, 3)) + "," + idMap.get(codeBak.substring(0, 4)) + "," + idMap.get(codeBak.substring(0, 5)) + ",#";
                        parentId = idMap.get(codeBak.substring(0, 5));
                    }
                }
                String parentCode = "000" + codeArray[0].substring(0, 2) + "00" + codeArray[0].substring(0, 3) + "0" + codeArray[0].substring(0, 4) + codeArray[0];
                code = parentCode + codeTemp;
            }
            String sqlTitle = "INSERT INTO sys_dict (id, lib_id, dict_name, dict_code, parent_id, remark, state, is_del, create_user, create_date, update_user,update_date, all_parent_id, system_id, dict_type) ";
            String sqlValue = " VALUES(" + "'" + uuid + "'," + "'2178002a6ce345edb39dbcbea018ea3f','" + name + "','" + code + "','" + parentId + "',null" + ",0" + ",0" + ",'b8d11ee289394be688ef3a4f6968efed'" + ",now()" + ",null" + ",null,'" + allParentId + "','N0001'" + ",0" + ");";
            //System.out.println("第" + i + "行：" + sqlTitle + sqlValue);
            sqlList.add(sqlTitle + sqlValue);
        }
        writeFile(sqlList, "C:\\Users\\Administrator\\Desktop\\标准数据导入(农产品库).sql", false);
    }
}
