package com.hywa.map.config.interceptor;

import static com.hywa.map.common.ConstantPool.SPLIT;
import static com.hywa.map.common.enums.CommonEnum.NO_AUTHORITY;
import static com.hywa.map.common.enums.CommonEnum.RE_LOGIN;

import com.alibaba.fastjson.JSON;
import com.hywa.map.common.utils.StringUtils;
import com.hywa.map.representation.ResponseBase;

import java.io.*;
import java.util.Arrays;
import java.util.List;
import javax.servlet.Filter;
import javax.servlet.FilterChain;
import javax.servlet.FilterConfig;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.annotation.WebFilter;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;


@Slf4j
@Component
@WebFilter(urlPatterns = {"/*"}, filterName = "tokenAuthorFilter") //过滤规则
public class TokenAuthorFilter implements Filter {

    @Autowired
    private TokenManager tokenManager;
    @Autowired
    private UserAuthContext userAuthContext;

    private static final List<String> UN_FILTER_URL_LIST = Arrays
            .asList("/user/login", "/user/pcLogin", "user/checkToken", "user/pcCheckToken","/statis","/gis","/public","/map",
                    "/region", "/file", "/druid", "/downLoadZipFile", "/uploadImage", "/showImage", "/uploadFile","/web",
                    "/getExcel", "/downloadExcel", "downloadTemplate", "downloadXls", "/tracing", "/downloadAttach", "/download");

    /* 判断请求路径是否为不拦截的请求路径 */
    private boolean isNotFilter(String url) {
        for (String s : UN_FILTER_URL_LIST) {
            if (url.contains(s)) {
                return true;
            }
        }
        return false;
    }

    @Override
    public void init(FilterConfig filterConfig) {
    }


    @Override
    public void doFilter(ServletRequest request, ServletResponse response, FilterChain chain)
            throws IOException, ServletException {
        HttpServletRequest req = (HttpServletRequest) request;
        HttpServletResponse rep = (HttpServletResponse) response;

        //设置允许跨域的配置
        // 这里填写你允许进行跨域的主机ip（正式上线时可以动态配置具体允许的域名和IP）
        rep.setHeader("Access-Control-Allow-Origin", "*");
        // 允许的访问方法
        rep.setHeader("Access-Control-Allow-Methods", "POST, GET, PUT, OPTIONS, DELETE, PATCH");
        // Access-Control-Max-Age 用于 CORS 相关配置的缓存
        rep.setHeader("Access-Control-Max-Age", "3600");
        rep.setHeader("Access-Control-Allow-Headers",
                "token,Origin, X-Requested-With, Content-Type, Accept, accessToken");

        response.setCharacterEncoding("UTF-8");
        response.setContentType("application/json; charset=utf-8");

        String accessToken = req.getHeader("accessToken");
        ResponseBase resultInfo = new ResponseBase();

        boolean isFilter = false;

        String method = ((HttpServletRequest) request).getMethod();
        String servletPath = req.getServletPath();
        servletPath = servletPath.endsWith("/") ? servletPath.substring(0, servletPath.length() - 1)
                : servletPath;
        if ("OPTIONS".equals(method)) {
            rep.setStatus(HttpServletResponse.SC_OK);
        } else {
            if (isNotFilter(req.getRequestURL().toString())) {
                isFilter = true;
            } else if (StringUtils.isEmpty(accessToken)) {
                resultInfo.setRetHead(RE_LOGIN.getIndex(), RE_LOGIN.getValue());
            } else {
                log.info(accessToken);
                //验证token
                TokenModel model = tokenManager.getToken(accessToken);
                if (tokenManager.checkToken(model)) {
                    //验证权限
                    if (userAuthContext.executeAuth(model.getUserId().split(SPLIT)[1],
                            servletPath + "|" + method)) {
                        isFilter = true;
                    } else {
                        isFilter = false;
                        resultInfo.setRetHead(NO_AUTHORITY.getIndex(), NO_AUTHORITY.getValue());
                    }
                } else {
                    resultInfo.setRetHead(RE_LOGIN.getIndex(), RE_LOGIN.getValue());
                }
            }

            if (isFilter) {
                chain.doFilter(request, response);
            } else {// 验证失败
                PrintWriter writer = null;
                OutputStreamWriter osw = null;
                try {
                    osw = new OutputStreamWriter(response.getOutputStream(),
                            "UTF-8");
                    writer = new PrintWriter(osw, true);
                    String jsonStr = JSON.toJSONString(resultInfo);
                    writer.write(jsonStr);
                    writer.flush();
                    writer.close();
                    osw.close();
                } catch (IOException e) {
                    log.error("过滤器返回信息失败:" + e.getMessage(), e);
                } finally {
                    if (null != writer) {
                        writer.close();
                    }
                    if (null != osw) {
                        osw.close();
                    }
                }
            }
        }
    }

    @Override
    public void destroy() {
    }
}
