package com.hywa.map.controller.branch;

import com.hywa.map.common.log.Log;
import com.hywa.map.config.EnvProperties;
import com.hywa.map.config.interceptor.TokenManager;
import com.hywa.map.config.interceptor.TokenModel;
import com.hywa.map.representation.*;
import com.hywa.map.service.branch.BranchService;
import io.swagger.annotations.Api;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;

import static com.hywa.map.common.ConstantPool.SPLIT;

/**
 * t_sys_branch  对应Controller 实现类
 *
 * @author hywa
 * @create by 2018-12-12 10:01:56
 */

@Slf4j
@Api(tags = "组织机构")
@RestController
@RequestMapping("/branch")
public class BranchController {

    @Autowired
    private BranchService branchService;
    @Autowired
    EnvProperties envProperties;

    @Autowired
    TokenManager tokenManager;

    @PostMapping
    @Transactional
    @Log(log = "组织机构-新增组织机构信息")
    public ResponseEntity<ResponseBase> save(@RequestBody BranchReq reqInfo, HttpServletRequest req) {
        TokenModel accessToken = tokenManager.getToken(req.getHeader("accessToken"));
        String userId = accessToken.getUserId().split(SPLIT)[1];
        try {
            branchService.save(reqInfo, userId);
        } catch (Exception e) {
            log.error("新增组织机构信息异常，请求数据|" + reqInfo + "，异常信息|" + e);
            throw e;
        }
        return new ResponseEntity<>(new ResponseBase().success(), HttpStatus.OK);
    }

    @DeleteMapping
    @Log(log = "组织机构-删除组织机构信息")
    public ResponseEntity<ResponseBase> delete(@RequestBody BranchReq reqInfo, HttpServletRequest req) {
        TokenModel accessToken = tokenManager.getToken(req.getHeader("accessToken"));
        String userId = accessToken.getUserId().split(SPLIT)[1];
        try {
            branchService.delete(reqInfo, userId);
        } catch (Exception e) {
            log.error("删除组织机构信息异常，请求数据|" + reqInfo + "，异常信息|" + e);
            throw e;
        }
        return new ResponseEntity<>(new ResponseBase().success(), HttpStatus.OK);
    }

    @RequestMapping(value = "/findAll", method = RequestMethod.GET)
    public ResponseEntity<ResponseBase> select() {
        ResponseBase<BranchReps> responseBase = new ResponseBase<>();
        responseBase.success().setRetBody(branchService.selectAll());
        return new ResponseEntity<>(responseBase, HttpStatus.OK);
    }

    @RequestMapping(value = "/findById", method = RequestMethod.GET)
    public ResponseEntity<ResponseBase<BranchRep>> findById(@RequestParam(required = false) String id) {
        ResponseBase<BranchRep> responseBase = new ResponseBase<>();
        responseBase.success().setRetBody(branchService.selectById(id));
        responseBase.success();
        return new ResponseEntity<>(responseBase, HttpStatus.OK);
    }


    @PutMapping
    @Log(log = "组织机构-编辑组织机构信息")
    public ResponseEntity<ResponseBase> update(@RequestBody BranchReq reqInfo, HttpServletRequest req) {
        TokenModel accessToken = tokenManager.getToken(req.getHeader("accessToken"));
        String userId = accessToken.getUserId().split(SPLIT)[1];
        try {
            branchService.update(reqInfo, userId);
        } catch (Exception e) {
            log.error("编辑组织机构信息异常，请求数据|" + reqInfo + "，异常信息|" + e);
            throw e;
        }
        return new ResponseEntity<>(new ResponseBase().success(), HttpStatus.OK);
    }

    @RequestMapping(value = "/findByParam", method = RequestMethod.GET)
    public ResponseEntity<ResponseBase> findByParam(@RequestParam(required = false) String queryParam,
                                                    @RequestParam(defaultValue = "1") int pageNo,
                                                    @RequestParam(defaultValue = "100") int pageSize) {
        ResponseBase<BranchReps> responseBase = new ResponseBase<>();
        responseBase.success().setRetBody(branchService.selectByParam(pageNo, pageSize, queryParam));
        return new ResponseEntity<>(responseBase, HttpStatus.OK);
    }

}

