package com.hywa.map.controller.common;

import com.hywa.map.common.exception.GlobalException;
import com.hywa.map.common.utils.FileUtils;
import com.hywa.map.common.utils.StringUtils;
import com.hywa.map.config.EnvProperties;
import com.hywa.map.representation.ResponseBase;
import com.hywa.map.representation.UploadFileRep;
import io.swagger.annotations.Api;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.UnsupportedEncodingException;

import static com.hywa.map.common.enums.CommonEnum.ILLEGAL_PARAMETER_ERROR;

/**
 * dap_weather  对应Controller 实现类
 *
 * @author hywa
 * @create by 2018-08-06 15:02:49
 */
@Slf4j
@Api(tags = "文件管理")
@RestController
@RequestMapping("/file")
public class FileController {
    @Autowired
    EnvProperties envProperties;

    @RequestMapping(value = "/uploadFile", method = RequestMethod.POST)
    @Transactional
    public ResponseEntity<ResponseBase> uploadFile(@RequestParam("file") MultipartFile file, @RequestParam("fileType") String fileType,HttpServletRequest req) {
        if (file.isEmpty()) {
            throw new GlobalException(ILLEGAL_PARAMETER_ERROR.getIndex(), ILLEGAL_PARAMETER_ERROR.getValue() + "，文件为空", null, "上传文件");
        }
        if (StringUtils.isEmpty(fileType)) {
            throw new GlobalException(ILLEGAL_PARAMETER_ERROR.getIndex(), ILLEGAL_PARAMETER_ERROR.getValue() + "，文件类型不能为空", null, "文件管理-上传文件");
        }
        String fileName = file.getOriginalFilename();
        try {
            String filePath=null;
            if ("map".equals(fileType)) {
                filePath = envProperties.getMapPicturePath();
            }
            FileUtils.upload(file, filePath);
        } catch (Exception e) {
            log.error("上传文件异常，请求数据|" + fileName + "，异常信息|" + e);
            throw e;
        }
        ResponseBase<UploadFileRep> responseBase = new ResponseBase<>();
        UploadFileRep uploadFileRep = new UploadFileRep();
        uploadFileRep.setFileName(fileName);
        String fileUrl=envProperties.getPubFileDownloadUrl() + fileName+"&fileType="+fileType;
        uploadFileRep.setFileUrl(fileUrl);
        responseBase.success().setRetBody( uploadFileRep);
        log.info("上传文件成功，请求数据：fileName|" + fileName + "，fileType|" + fileType);
        return new ResponseEntity<>(responseBase, HttpStatus.OK);
    }

    @RequestMapping(value = "/uploadDocument", method = RequestMethod.POST)
    @Transactional
    public ResponseEntity<ResponseBase> uploadDocument(@RequestParam("file") MultipartFile file,HttpServletRequest req) {
        if (file.isEmpty()) {
            throw new GlobalException(ILLEGAL_PARAMETER_ERROR.getIndex(), ILLEGAL_PARAMETER_ERROR.getValue() + "，文件为空", null, "上传文件");
        }
        String fileName = file.getOriginalFilename();
        try {
            String filePath = envProperties.getComFilePath();
            fileName = FileUtils.uploadByNewName(file, filePath);
        } catch (Exception e) {
            log.error("上传文件异常，请求数据|" + fileName + "，异常信息|" + e);
            throw e;
        }
        ResponseBase<UploadFileRep> responseBase = new ResponseBase<>();
        UploadFileRep uploadFileRep = new UploadFileRep();
        uploadFileRep.setFileName(fileName);
        String fileUrl=envProperties.getComFileDownloadUrl() + fileName;
        uploadFileRep.setFileUrl(fileUrl);
        responseBase.success().setRetBody( uploadFileRep);
        log.info("上传文件成功，请求数据：fileName|" + fileName);
        return new ResponseEntity<>(responseBase, HttpStatus.OK);
    }

    /**
     * 下载附件
     */
    @GetMapping(value = "/downloadDocument")
    public void downloadDocument(
            HttpServletResponse response,
            HttpServletRequest request,
            @RequestParam(required = false) String fileName) throws UnsupportedEncodingException {
        String menuName = "文件管理-查看文件信息";
        String filePath = envProperties.getComFilePath();
        log.info("下载文件成功，请求数据：fileName|" + fileName);
        FileUtils.downloadFile(filePath + fileName, fileName, response, request, menuName);
    }

    /**
     * 下载附件
     */
    @GetMapping(value = "/downloadAttach")
    public void downloadAttach(
            HttpServletResponse response,
            HttpServletRequest request,
            @RequestParam(required = false) String fileName,
            @RequestParam(required = false) String fileType) throws UnsupportedEncodingException {
        String menuName = "文件管理-查看文件信息";
        String filePath = "";
        if ("map".equals(fileType)) {
            filePath = envProperties.getMapPicturePath();
        }
        log.info("下载文件成功，请求数据：fileName|" + fileName + "，fileType|" + fileType);
        FileUtils.downloadFile(filePath + fileName, fileName, response, request, menuName);
    }

}

