package com.hywa.map.controller.common;

import com.hywa.map.representation.ResponseBase;
import com.hywa.map.representation.TempHour;
import com.hywa.map.representation.WeatherForecast;
import com.hywa.map.service.common.WeatherService;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.ResponseBody;

/**
 * dap_weather  对应Controller 实现类
 *
 * @author hywa
 * @create by 2018-08-06 15:02:49
 */
@Controller
@RequestMapping("/public")
public class WeatherController{
    @Autowired
    private WeatherService weatherService;

    private static Log log = LogFactory.getLog(WeatherController.class);

    /**
     * 获取天气预报信息
     */
    @ResponseBody
    @RequestMapping(value = "/weather", method = RequestMethod.GET)
    public ResponseEntity<ResponseBase> forecast(String cityCode) {
        try {
            ResponseBase<WeatherForecast> responseBase = new ResponseBase<>();
            responseBase.success().setRetBody(weatherService.getWeather(cityCode));
            return new ResponseEntity<>(responseBase, HttpStatus.OK);
        } catch (Exception e) {
            log.error("查询天气信息，城市：" + cityCode + "|查询异常：" + e);
            throw e;
        }
    }

    /**
     * 获取24小时天气预报信息
     */
    @ResponseBody
    @RequestMapping(value = "/hourWeather", method = RequestMethod.GET)
    public ResponseEntity<ResponseBase> getHourWeather(String cityCode) {
        try {
            ResponseBase<TempHour> responseBase = new ResponseBase<>();
            responseBase.success().setRetBody(weatherService.getHourWeather(cityCode));
            return new ResponseEntity<>(responseBase, HttpStatus.OK);
        } catch (Exception e) {
            log.error("查询24小时天气信息，城市：" + cityCode + "|查询异常：" + e);
            throw e;
        }
    }
}

