package com.hywa.map.controller.csv;

import com.hywa.map.common.exception.GlobalException;
import com.hywa.map.common.utils.FileUtils;
import com.hywa.map.config.EnvProperties;
import com.hywa.map.config.interceptor.TokenManager;
import com.hywa.map.config.interceptor.TokenModel;
import com.hywa.map.representation.*;
import com.hywa.map.service.csv.CsvService;
import io.swagger.annotations.Api;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;

import static com.hywa.map.common.ConstantPool.SPLIT;
import static com.hywa.map.common.enums.CommonEnum.ILLEGAL_PARAMETER_ERROR;

/**
 * t_mon_dealer_invent  对应Controller 实现类
 *
 * @author hywa
 * @create by 2019-03-04 13:49:14
 */
@Slf4j
@Api(tags = "csv文件操作")
@RestController
@RequestMapping("/csv")
public class CsvController {

    @Autowired
    private CsvService csvService;
    @Autowired
    EnvProperties envProperties;
    @Autowired
    TokenManager tokenManager;

    /*
     *查询csv数据
     */
    @RequestMapping(value = "/findAll", method = RequestMethod.GET)
    public ResponseEntity<ResponseBase> findByParam(@RequestParam(defaultValue = "1") int pageNo,
                                                    @RequestParam(defaultValue = "10") int pageSize,
                                                    @RequestParam(required = false) String tableName) {
        ResponseBase<CsvReps> responseBase = new ResponseBase<>();
        responseBase.success().setRetBody(csvService.findAll(pageNo, pageSize, tableName));
        return new ResponseEntity<>(responseBase, HttpStatus.OK);
    }

    /*
     *上传csv文件
     */
    @RequestMapping(value = "/uploadFile", method = RequestMethod.POST)
    @Transactional
    public ResponseEntity<ResponseBase> uploadFile(@RequestParam("file") MultipartFile file, HttpServletRequest req) {
        TokenModel accessToken = tokenManager.getToken(req.getHeader("accessToken"));
        String userId = accessToken.getUserId().split(SPLIT)[1];
        if (file.isEmpty()) {
            throw new GlobalException(ILLEGAL_PARAMETER_ERROR.getIndex(), ILLEGAL_PARAMETER_ERROR.getValue() + "，文件为空", null, "上传文件");
        }
        String fileName = file.getOriginalFilename();
        try {
            FileUtils.upload(file, envProperties.getCsvFilePath());
        } catch (Exception e) {
            log.error("上传文件异常，请求数据|" + fileName + "，异常信息|" + e);
            throw e;
        }
        String csvId = csvService.doCsvFile(envProperties.getCsvFilePath(), fileName, userId);
        ResponseBase<UploadCsvRep> responseBase = new ResponseBase<>();
        UploadCsvRep uploadCsvRep = new UploadCsvRep();
        uploadCsvRep.setCsvId(csvId);
        responseBase.success().setRetBody(uploadCsvRep);
        return new ResponseEntity<>(responseBase, HttpStatus.OK);
    }
}