package com.hywa.map.controller.csv;

import com.hywa.map.common.log.Log;
import com.hywa.map.common.utils.FileUtils;
import com.hywa.map.config.EnvProperties;
import com.hywa.map.config.interceptor.TokenManager;
import com.hywa.map.config.interceptor.TokenModel;
import com.hywa.map.representation.DbRep;
import com.hywa.map.representation.DbReps;
import com.hywa.map.representation.DbReq;
import com.hywa.map.representation.ResponseBase;
import com.hywa.map.service.csv.DbService;
import io.swagger.annotations.Api;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import java.io.UnsupportedEncodingException;

import static com.hywa.map.common.ConstantPool.SPLIT;

/**
 * t_mon_dealer_invent  对应Controller 实现类
 *
 * @author hywa
 * @create by 2019-03-04 13:49:14
 */
@Slf4j
@Api(tags = "我的数据源")
@RestController
@RequestMapping("/db")
public class DbController {

    @Autowired
    private DbService dbService;
    @Autowired
    EnvProperties envProperties;
    @Autowired
    TokenManager tokenManager;

    @PostMapping
    @Transactional
    @Log(log = "我的数据源-新增数据源信息")
    public ResponseEntity<ResponseBase> save(@RequestBody DbReq reqInfo, HttpServletRequest req) {
        TokenModel accessToken = tokenManager.getToken(req.getHeader("accessToken"));
        String userId = accessToken.getUserId().split(SPLIT)[1];
        try {
            dbService.save(reqInfo, userId);
        } catch (Exception e) {
            log.error("新增数据源信息异常，请求数据|" + reqInfo + "，异常信息|" + e);
            throw e;
        }
        return new ResponseEntity<>(new ResponseBase().success(), HttpStatus.OK);
    }

    @PutMapping
    @Log(log = "我的数据源-编辑数据源信息")
    public ResponseEntity<ResponseBase> update(@RequestBody DbReq reqInfo, HttpServletRequest req) {
        TokenModel accessToken = tokenManager.getToken(req.getHeader("accessToken"));
        String userId = accessToken.getUserId().split(SPLIT)[1];
        try {
            dbService.update(reqInfo, userId);
        } catch (Exception e) {
            log.error("编辑数据源信息异常，请求数据|" + reqInfo + "，异常信息|" + e);
            throw e;
        }
        return new ResponseEntity<>(new ResponseBase().success(), HttpStatus.OK);
    }

    @DeleteMapping
    @Log(log = "我的数据源-删除数据源信息")
    public ResponseEntity<ResponseBase> delete(@RequestBody DbReq reqInfo, HttpServletRequest req) {
        TokenModel accessToken = tokenManager.getToken(req.getHeader("accessToken"));
        String userId = accessToken.getUserId().split(SPLIT)[1];
        try {
            dbService.delete(reqInfo, userId);
        } catch (Exception e) {
            log.error("删除数据源信息异常，请求数据|" + reqInfo + "，异常信息|" + e);
            throw e;
        }
        return new ResponseEntity<>(new ResponseBase().success(), HttpStatus.OK);
    }

    @RequestMapping(value = "/findAll", method = RequestMethod.GET)
    public ResponseEntity<ResponseBase> findAll(@RequestParam(defaultValue = "1") int pageNo,
                                                @RequestParam(defaultValue = "100") int pageSize,
                                                @RequestParam(required = false) String sourceType,
                                                @RequestParam(required = false) String sourceName) {
        ResponseBase<DbReps> responseBase = new ResponseBase<>();
        responseBase.success().setRetBody(dbService.selectAll(pageNo, pageSize, sourceType, sourceName));
        return new ResponseEntity<>(responseBase, HttpStatus.OK);
    }

    @RequestMapping(value = "/findById", method = RequestMethod.GET)
    public ResponseEntity<ResponseBase<DbRep>> findById(@RequestParam(required = false) String id) {
        ResponseBase<DbRep> responseBase = new ResponseBase<>();
        responseBase.success().setRetBody(dbService.selectById(id));
        responseBase.success();
        return new ResponseEntity<>(responseBase, HttpStatus.OK);
    }

    /**
     * 下载附件
     */
    @GetMapping(value = "/downloadAttach")
    public void downloadAttach(
            HttpServletResponse response,
            HttpServletRequest request, @RequestParam(required = false) String fileName) throws UnsupportedEncodingException {
        String menuName = "我的数据源-下载csv文件";
        String filePath = envProperties.getCsvFilePath();
        FileUtils.downloadFile(filePath + fileName, fileName, response, request, menuName);
    }
}