package com.hywa.map.controller.depart;

import com.hywa.map.common.log.Log;
import com.hywa.map.config.EnvProperties;
import com.hywa.map.config.interceptor.TokenManager;
import com.hywa.map.config.interceptor.TokenModel;
import com.hywa.map.representation.*;
import com.hywa.map.service.depart.DepartService;
import io.swagger.annotations.Api;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;

import static com.hywa.map.common.ConstantPool.SPLIT;

/**
 * t_sys_depart  对应Controller 实现类
 *
 * @author hywa
 * @create by 2019-02-14 15:50:31
 */
@Slf4j
@Api(tags = "单位基本信息")
@RestController
@RequestMapping("/depart")
public class DepartController {
    @Autowired
    private DepartService departService;
    @Autowired
    EnvProperties envProperties;

    @Autowired
    TokenManager tokenManager;

    @PostMapping
    @Transactional
    @Log(log = "单位基本信息-保存单位基本信息")
    public ResponseEntity<ResponseBase> save(@RequestBody DepartReq reqInfo, HttpServletRequest req) {
        TokenModel accessToken = tokenManager.getToken(req.getHeader("accessToken"));
        String userId = accessToken.getUserId().split(SPLIT)[1];
        try {
            departService.save(reqInfo, userId);
        } catch (Exception e) {
            log.error("保存单位基本信息异常，请求数据|" + reqInfo + "，异常信息|" + e);
            throw e;
        }
        return new ResponseEntity<>(new ResponseBase().success(), HttpStatus.OK);
    }

    @RequestMapping(value = "/findAll", method = RequestMethod.GET)
    public ResponseEntity<ResponseBase> select() {
        ResponseBase<DepartRep> responseBase = new ResponseBase<>();
        responseBase.success().setRetBody(departService.select());
        return new ResponseEntity<>(responseBase, HttpStatus.OK);
    }

    @RequestMapping(value = "/findProvince", method = RequestMethod.GET)
    public ResponseEntity<ResponseBase> findProvince() {
        ResponseBase<ProvinceReps> responseBase = new ResponseBase<>();
        responseBase.success().setRetBody(departService.selectProvince());
        return new ResponseEntity<>(responseBase, HttpStatus.OK);
    }

    @RequestMapping(value = "/findCity", method = RequestMethod.GET)
    public ResponseEntity<ResponseBase> findCity(@RequestParam(required = false) String id) {
        ResponseBase<CityReps> responseBase = new ResponseBase<>();
        responseBase.success().setRetBody(departService.selectCity(id));
        return new ResponseEntity<>(responseBase, HttpStatus.OK);
    }

    @RequestMapping(value = "/findCounty", method = RequestMethod.GET)
    public ResponseEntity<ResponseBase> findCounty(@RequestParam(required = false) String id) {
        ResponseBase<CountyReps> responseBase = new ResponseBase<>();
        responseBase.success().setRetBody(departService.selectCounty(id));
        return new ResponseEntity<>(responseBase, HttpStatus.OK);
    }

    @RequestMapping(value = "/findTown", method = RequestMethod.GET)
    public ResponseEntity<ResponseBase> findTown(@RequestParam(required = false) String id) {
        ResponseBase<TownReps> responseBase = new ResponseBase<>();
        responseBase.success().setRetBody(departService.selectTown(id));
        return new ResponseEntity<>(responseBase, HttpStatus.OK);
    }

    @RequestMapping(value = "/findVillage", method = RequestMethod.GET)
    public ResponseEntity<ResponseBase> findVillage(@RequestParam(required = false) String id) {
        ResponseBase<VillageReps> responseBase = new ResponseBase<>();
        responseBase.success().setRetBody(departService.selectVillage(id));
        return new ResponseEntity<>(responseBase, HttpStatus.OK);
    }
}

