package com.hywa.map.controller.dict;

import com.hywa.map.common.log.Log;
import com.hywa.map.representation.*;
import com.hywa.map.service.dict.DictService;
import io.swagger.annotations.Api;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;

/**
 * t_sys_dict  对应Controller 实现类
 *
 * @author hywa
 * @create by 2019-02-15 09:56:54
 */
@Slf4j
@Api(tags = "数据字典")
@RestController
@RequestMapping("/dict")
public class DictController {

    @Autowired
    private DictService dictService;

    @PostMapping
    @Transactional
    @Log(log = "数据字典-新增数据字典信息")
    public ResponseEntity<ResponseBase> save(@RequestBody DictReq reqInfo) {
        try {
            dictService.save(reqInfo, null);
        } catch (Exception e) {
            log.error("新增数据字典信息异常，请求数据|" + reqInfo + "，异常信息|" + e);
            throw e;
        }
        return new ResponseEntity<>(new ResponseBase().success(), HttpStatus.OK);
    }

    @PutMapping
    @Log(log = "数据字典-编辑数据字典信息")
    public ResponseEntity<ResponseBase> update(@RequestBody DictReq reqInfo) {
        try {
            dictService.update(reqInfo, null);
        } catch (Exception e) {
            log.error("编辑数据字典信息异常，请求数据|" + reqInfo + "，异常信息|" + e);
            throw e;
        }
        return new ResponseEntity<>(new ResponseBase().success(), HttpStatus.OK);
    }

    @DeleteMapping
    @Log(log = "数据字典-删除数据字典信息")
    public ResponseEntity<ResponseBase> delete(@RequestBody DictReq reqInfo) {
        try {
            dictService.delete(reqInfo, null);
        } catch (Exception e) {
            log.error("删除数据字典信息异常，请求数据|" + reqInfo + "，异常信息|" + e);
            throw e;
        }
        return new ResponseEntity<>(new ResponseBase().success(), HttpStatus.OK);
    }

    @RequestMapping(value = "/findAll", method = RequestMethod.GET)
    public ResponseEntity<ResponseBase> findByParam(@RequestParam(required = false) String queryParam,
                                                    @RequestParam(defaultValue = "1") int pageNo,
                                                    @RequestParam(defaultValue = "10") int pageSize) {
        ResponseBase<DictReps> responseBase = new ResponseBase<>();
        responseBase.success().setRetBody(dictService.select(pageNo, pageSize, queryParam));
        return new ResponseEntity<>(responseBase, HttpStatus.OK);
    }

    @RequestMapping(value = "/findById", method = RequestMethod.GET)
    public ResponseEntity<ResponseBase<DictRep>> findById(@RequestParam(required = false) String id) {
        ResponseBase<DictRep> responseBase = new ResponseBase<>();
        responseBase.success().setRetBody(dictService.selectById(id));
        responseBase.success();
        return new ResponseEntity<>(responseBase, HttpStatus.OK);
    }

    @RequestMapping(value = "/findValue", method = RequestMethod.GET)
    public ResponseEntity<ResponseBase> findValue(@RequestParam(required = false) String queryParam,
                                                  @RequestParam(required = true) String dictId,
                                                  @RequestParam(defaultValue = "1") int pageNo,
                                                  @RequestParam(defaultValue = "10") int pageSize) {
        ResponseBase<DictReps> responseBase = new ResponseBase<>();
        responseBase.success().setRetBody(dictService.selectVal(pageNo, pageSize, dictId, queryParam));
        return new ResponseEntity<>(responseBase, HttpStatus.OK);
    }

    @RequestMapping(value = "/saveVale", method = RequestMethod.POST)
    @Log(log = "数据字典-新增字典值信息")
    public ResponseEntity<ResponseBase> saveVale(@RequestBody DictReq reqInfo) {
        try {
            dictService.saveVal(reqInfo, null);
        } catch (Exception e) {
            log.error("新增字典值信息异常，请求数据|" + reqInfo + "，异常信息|" + e);
            throw e;
        }
        return new ResponseEntity<>(new ResponseBase().success(), HttpStatus.OK);
    }

    @RequestMapping(value = "/updateVale", method = RequestMethod.POST)
    @Log(log = "数据字典-编辑字典值信息")
    public ResponseEntity<ResponseBase> updateVale(@RequestBody DictReq reqInfo) {
        try {
            dictService.updateVal(reqInfo, null);
        } catch (Exception e) {
            log.error("编辑字典值信息异常，请求数据|" + reqInfo + "，异常信息|" + e);
            throw e;
        }
        return new ResponseEntity<>(new ResponseBase().success(), HttpStatus.OK);
    }

    @RequestMapping(value = "/deleteVale", method = RequestMethod.POST)
    @Log(log = "数据字典-删除字典值信息")
    public ResponseEntity<ResponseBase> deleteVale(@RequestBody DictReq reqInfo) {
        try {
            dictService.deleteVal(reqInfo, null);
        } catch (Exception e) {
            log.error("删除字典值信息异常，请求数据|" + reqInfo + "，异常信息|" + e);
            throw e;
        }
        return new ResponseEntity<>(new ResponseBase().success(), HttpStatus.OK);
    }

    @RequestMapping(value = "/findValueById", method = RequestMethod.GET)
    public ResponseEntity<ResponseBase<DictRep>> findValueById(@RequestParam(required = false) String id) {
        ResponseBase<DictRep> responseBase = new ResponseBase<>();
        responseBase.success().setRetBody(dictService.selectValById(id));
        responseBase.success();
        return new ResponseEntity<>(responseBase, HttpStatus.OK);
    }

    @RequestMapping(value = "/findValueByCode", method = RequestMethod.GET)
    public ResponseEntity<ResponseBase> findValueByCode(@RequestParam(required = true) String dictCode) {
        ResponseBase<DictReps> responseBase = new ResponseBase<>();
        responseBase.success().setRetBody(dictService.selectVal(dictCode));
        return new ResponseEntity<>(responseBase, HttpStatus.OK);
    }
}

