package com.hywa.map.controller.dict;

import com.hywa.map.common.log.Log;
import com.hywa.map.representation.DictRep;
import com.hywa.map.representation.DictReq;
import com.hywa.map.representation.ResponseBase;
import com.hywa.map.service.dict.DictService;
import io.swagger.annotations.Api;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

/**
 * t_sys_dict  对应Controller 实现类
 *
 * @author hywa
 * @create by 2019-02-15 09:56:54
 */
@Slf4j
@Api(tags = "数据字典")
@RestController
@RequestMapping("/dictVal")
public class DictValController {

    @Autowired
    private DictService dictService;

    @PostMapping
    @Log(log = "数据字典-新增字典值信息")
    public ResponseEntity<ResponseBase> saveVale(@RequestBody DictReq reqInfo) {
        try {
            dictService.saveVal(reqInfo, null);
        } catch (Exception e) {
            log.error("新增字典值信息异常，请求数据|" + reqInfo + "，异常信息|" + e);
            throw e;
        }
        return new ResponseEntity<>(new ResponseBase().success(), HttpStatus.OK);
    }

    @PutMapping
    @Log(log = "数据字典-编辑字典值信息")
    public ResponseEntity<ResponseBase> updateVale(@RequestBody DictReq reqInfo) {
        try {
            dictService.updateVal(reqInfo, null);
        } catch (Exception e) {
            log.error("编辑字典值信息异常，请求数据|" + reqInfo + "，异常信息|" + e);
            throw e;
        }
        return new ResponseEntity<>(new ResponseBase().success(), HttpStatus.OK);
    }

    @DeleteMapping
    @Log(log = "数据字典-删除字典值信息")
    public ResponseEntity<ResponseBase> deleteVale(@RequestBody DictReq reqInfo) {
        try {
            dictService.deleteVal(reqInfo, null);
        } catch (Exception e) {
            log.error("删除字典值信息异常，请求数据|" + reqInfo + "，异常信息|" + e);
            throw e;
        }
        return new ResponseEntity<>(new ResponseBase().success(), HttpStatus.OK);
    }

    @RequestMapping(value = "/findValueById", method = RequestMethod.GET)
    public ResponseEntity<ResponseBase<DictRep>> findValueById(@RequestParam(required = false) String id) {
        ResponseBase<DictRep> responseBase = new ResponseBase<>();
        responseBase.success().setRetBody(dictService.selectValById(id));
        responseBase.success();
        return new ResponseEntity<>(responseBase, HttpStatus.OK);
    }
}

