package com.hywa.map.controller.expert;

import com.hywa.map.common.ConstantPool;
import com.hywa.map.common.log.Log;
import com.hywa.map.common.utils.FileUtils;
import com.hywa.map.config.EnvProperties;
import com.hywa.map.config.interceptor.TokenManager;
import com.hywa.map.config.interceptor.TokenModel;
import com.hywa.map.representation.*;
import com.hywa.map.service.expert.AreaDataService;
import com.hywa.map.service.login.UserRoleService;
import io.swagger.annotations.Api;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.io.IOUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.io.ClassPathResource;
import org.springframework.core.io.Resource;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import java.io.InputStream;
import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.nio.charset.StandardCharsets;

import static com.hywa.map.common.ConstantPool.SPLIT;


@Slf4j
@Api(tags = "区域数据管理")
@RestController
@RequestMapping("areaData")
public class AreaDataController {

    @Autowired
    EnvProperties envProperties;

    @Autowired
    TokenManager tokenManager;

    @Autowired
    private AreaDataService areaDataService;

    @Autowired
    UserRoleService userRoleService;

    /*
     * 获取所有区域数据
     */
    @RequestMapping(value = "/findAll", method = RequestMethod.GET)
    public ResponseEntity<ResponseBase> findAll(@RequestParam(defaultValue = "1") int pageNo,
                                                @RequestParam(defaultValue = "500") int pageSize,
                                                @RequestParam(required = false) String areaName,
                                                @RequestParam(required = false) String dataType,
                                                @RequestParam(required = false) String dataName,
                                                @RequestParam(required = false) String dataTime) {
        ResponseBase<AreaDataReps> responseBase = new ResponseBase<>();
        responseBase.success().setRetBody(areaDataService.queryAll(pageNo, pageSize, areaName, dataType, dataName, dataTime));
        return new ResponseEntity<>(responseBase, HttpStatus.OK);
    }

    /*
     * 根据id查询区域数据信息
     */
    @RequestMapping(value = "/findById", method = RequestMethod.GET)
    public ResponseEntity<ResponseBase> findById(@RequestParam(required = false) String id) {
        ResponseBase<AreaDataDetailRep> responseBase = new ResponseBase<>();
        responseBase.success().setRetBody(areaDataService.queryById(id));
        return new ResponseEntity<>(responseBase, HttpStatus.OK);
    }

    /*
     * 删除区域数据信息
     */
    @DeleteMapping
    @Log(log = "区域数据管理-删除区域数据信息")
    public ResponseEntity<ResponseBase> delete(@RequestBody DamInfoReq reqInfo, HttpServletRequest req) {
        TokenModel accessToken = tokenManager.getToken(req.getHeader("accessToken"));
        String userId = accessToken.getUserId().split(SPLIT)[1];
        try {
            areaDataService.delete(reqInfo, userId);
        } catch (Exception e) {
            log.error("坝区管理-删除坝区信息异常，请求数据：reqInfo|" + reqInfo + "，异常信息|" + e);
            throw e;
        }
        return new ResponseEntity<>(new ResponseBase().success(), HttpStatus.OK);
    }

    /*
     * 是否应用于大屏
     */
    @RequestMapping(value = "/updateState", method = RequestMethod.POST)
    @Log(log = "区域数据管理-是否应用于大屏")
    public ResponseEntity<ResponseBase> updateState(@RequestBody DamInfoReq reqInfo, HttpServletRequest req) {
        TokenModel accessToken = tokenManager.getToken(req.getHeader("accessToken"));
        String userId = accessToken.getUserId().split(SPLIT)[1];
        try {
            areaDataService.updateIsShow(reqInfo, userId);
        } catch (Exception e) {
            log.error("区域数据管理-是否应用于大屏异常，请求数据：reqInfo|" + reqInfo + "，异常信息|" + e);
            throw e;
        }
        return new ResponseEntity<>(new ResponseBase().success(), HttpStatus.OK);
    }

    /*
     * 导入excel数据
     */
    @RequestMapping(value = "/importXls", method = RequestMethod.POST)
    @Log(log = "区域数据管理-导入excel数据")
    public ResponseEntity<ResponseBase> importXls(@RequestParam("file") MultipartFile file, HttpServletRequest req) {
        TokenModel accessToken = tokenManager.getToken(req.getHeader("accessToken"));
        String userId = accessToken.getUserId().split(SPLIT)[1];
        try {
            areaDataService.importXls(file, userId);
        } catch (Exception e) {
            log.error("区域数据管理-导入excel数据异常，请求数据|" + file.getOriginalFilename() + "，异常信息|" + e);
            throw e;
        }
        return new ResponseEntity<>(new ResponseBase().success(), HttpStatus.OK);
    }

    /**
     * 下载结果文件
     */
    @GetMapping(value = "/downloadXls")
    public void downloadExcel(
            HttpServletResponse response,
            HttpServletRequest request,
            @RequestParam(required = false) String id) throws UnsupportedEncodingException {
        String fileName = areaDataService.exportXls(id);
        FileUtils.downloadFile(envProperties.getComFilePath() + fileName, fileName, response, request, "根据id导出区域数据信息");
    }

    /**
     * 下载模版
     */
    /*@GetMapping(value = "/downloadTemplate")
    public void downloadTemplate(
            HttpServletResponse response,
            HttpServletRequest request) throws UnsupportedEncodingException {
        Resource resource = new ClassPathResource(ConstantPool.IMPORT_TEMPLATE + "/表单导入模版.rar");
        File file = null;
        try {
            file = resource.getFile();
        } catch (IOException e) {
            e.printStackTrace();
        }
        FileUtils.downloadFile(file.getPath(), file.getName(), response, request, "区域数据管理-下载模版文件");
    }*/

    /**
     * 下载模板
     *
     * @param response
     * @param request
     */
    @RequestMapping(value = "/downloadTemplate")
    public void downloadTemp(HttpServletResponse response, HttpServletRequest request) {
        InputStream inputStream = null;
        ServletOutputStream servletOutputStream = null;
        try {
            String fileName = "表单导入模版.rar";
            Resource resource = new ClassPathResource(ConstantPool.IMPORT_TEMPLATE + "/表单导入模版.rar");
            response.setContentType("application/force-download");// 设置强制下载不打开
            //不能直接在header中设置中文，普通浏览器要使用URLEncoder.encode方法encode一下，其中火狐浏览器要区别处理，要转成ISO-8859-1编码
            if (request.getHeader("user-agent").toLowerCase().indexOf("firefox") > -1) {
                response.addHeader("Content-Disposition",
                        "attachment;filename=" + new String(fileName.getBytes("utf-8"), "ISO-8859-1"));
            } else {
                response.addHeader("Content-Disposition",
                        "attachment;filename=" + URLEncoder.encode(fileName, "utf-8"));
            }
            String encodeName = URLEncoder.encode(fileName, StandardCharsets.UTF_8.toString());
            response.setHeader("Content-Disposition", "attachment; filename=\"" + encodeName + "\"; filename*=utf-8''" + encodeName);

            inputStream = resource.getInputStream();
            servletOutputStream = response.getOutputStream();
            IOUtils.copy(inputStream, servletOutputStream);
            response.flushBuffer();
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            try {
                if (servletOutputStream != null) {
                    servletOutputStream.close();
                }
                if (inputStream != null) {
                    inputStream.close();
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
    }

    /*
     * 获取所属区域
     */
    @RequestMapping(value = "/findArea", method = RequestMethod.GET)
    public ResponseEntity<ResponseBase> findArea(@RequestParam String sn) {
        ResponseBase<SysAreaReps> responseBase = new ResponseBase<>();
        responseBase.success().setRetBody(areaDataService.queryAreaBySn(sn));
        return new ResponseEntity<>(responseBase, HttpStatus.OK);
    }
}


