package com.hywa.map.controller.expert;

import com.hywa.map.common.log.Log;
import com.hywa.map.config.EnvProperties;
import com.hywa.map.config.interceptor.TokenManager;
import com.hywa.map.config.interceptor.TokenModel;
import com.hywa.map.representation.*;
import com.hywa.map.service.expert.DamService;
import com.hywa.map.service.login.UserRoleService;
import io.swagger.annotations.Api;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;

import static com.hywa.map.common.ConstantPool.SPLIT;


@Slf4j
@Api(tags = "坝区管理")
@RestController
@RequestMapping("dam")
public class DamController {

    @Autowired
    EnvProperties envProperties;

    @Autowired
    TokenManager tokenManager;

    @Autowired
    private DamService damService;

    @Autowired
    UserRoleService userRoleService;

    /*
     * 获取所有坝区
     */
    @RequestMapping(value = "/findAll", method = RequestMethod.GET)
    public ResponseEntity<ResponseBase> findAll(@RequestParam(defaultValue = "1") int pageNo,
                                                @RequestParam(defaultValue = "500") int pageSize,
                                                @RequestParam(required = false) String areaCode,
                                                @RequestParam(required = false) String damName) {
        ResponseBase<DamInfoReps> responseBase = new ResponseBase<>();
        String state = (500 == pageSize) ? "0" : "";
        responseBase.success().setRetBody(damService.queryAll(pageNo, pageSize, areaCode, damName, state,null));
        return new ResponseEntity<>(responseBase, HttpStatus.OK);
    }

    /*
     * 根据id查询坝区信息
     */
    @RequestMapping(value = "/findById", method = RequestMethod.GET)
    public ResponseEntity<ResponseBase> findById(@RequestParam(required = false) String id) {
        ResponseBase<DamDetailInfoRep> responseBase = new ResponseBase<>();
        responseBase.success().setRetBody(damService.queryById(id));
        return new ResponseEntity<>(responseBase, HttpStatus.OK);
    }

    /*
     * 新增坝区信息
     */
    @PostMapping
    @Transactional
    @Log(log = "坝区管理-新增坝区信息")
    public ResponseEntity<ResponseBase> save(@RequestBody DamInfoReq reqInfo, HttpServletRequest req) {
        TokenModel accessToken = tokenManager.getToken(req.getHeader("accessToken"));
        String userId = accessToken.getUserId().split(SPLIT)[1];
        ResponseBase<DamDetailInfoRep> responseBase = new ResponseBase<>();
        try {
            DamDetailInfoRep rep = damService.save(reqInfo, userId);
            responseBase.success().setRetBody(rep);
        } catch (Exception e) {
            log.error("坝区管理-新增坝区信息异常，请求数据：reqInfo|" + reqInfo + "，异常信息|" + e);
            throw e;
        }
        return new ResponseEntity<>(responseBase, HttpStatus.OK);
    }

    /*
     * 修改坝区信息
     */
    @PutMapping
    @Log(log = "坝区管理-修改坝区信息")
    public ResponseEntity<ResponseBase> update(@RequestBody DamInfoReq reqInfo, HttpServletRequest req) {
        TokenModel accessToken = tokenManager.getToken(req.getHeader("accessToken"));
        String userId = accessToken.getUserId().split(SPLIT)[1];
        try {
            damService.update(reqInfo, userId);
        } catch (Exception e) {
            log.error("坝区管理-修改坝区信息异常，请求数据：reqInfo|" + reqInfo + "，异常信息|" + e);
            throw e;
        }
        return new ResponseEntity<>(new ResponseBase().success(), HttpStatus.OK);
    }

    /*
     * 删除坝区信息
     */
    @DeleteMapping
    @Log(log = "坝区管理-删除坝区信息")
    public ResponseEntity<ResponseBase> delete(@RequestBody DamInfoReq reqInfo, HttpServletRequest req) {
        TokenModel accessToken = tokenManager.getToken(req.getHeader("accessToken"));
        String userId = accessToken.getUserId().split(SPLIT)[1];
        try {
            damService.delete(reqInfo, userId);
        } catch (Exception e) {
            log.error("坝区管理-删除坝区信息异常，请求数据：reqInfo|" + reqInfo + "，异常信息|" + e);
            throw e;
        }
        return new ResponseEntity<>(new ResponseBase().success(), HttpStatus.OK);
    }

    /*
     * 是否启用
     */
    @RequestMapping(value = "/updateState", method = RequestMethod.POST)
    @Log(log = "坝区管理-是否启用")
    public ResponseEntity<ResponseBase> updateState(@RequestBody DamInfoReq reqInfo, HttpServletRequest req) {
        TokenModel accessToken = tokenManager.getToken(req.getHeader("accessToken"));
        String userId = accessToken.getUserId().split(SPLIT)[1];
        try {
            damService.updateState(reqInfo, userId);
        } catch (Exception e) {
            log.error("坝区管理-是否启用异常，请求数据：reqInfo|" + reqInfo + "，异常信息|" + e);
            throw e;
        }
        return new ResponseEntity<>(new ResponseBase().success(), HttpStatus.OK);
    }
}


