package com.hywa.map.controller.expert;

import com.hywa.map.common.log.Log;
import com.hywa.map.config.EnvProperties;
import com.hywa.map.config.interceptor.TokenManager;
import com.hywa.map.config.interceptor.TokenModel;
import com.hywa.map.representation.*;
import com.hywa.map.service.expert.ExpertService;
import com.hywa.map.service.login.UserRoleService;
import io.swagger.annotations.Api;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;

import static com.hywa.map.common.ConstantPool.SPLIT;


@Slf4j
@Api(tags = "专家团队管理")
@RestController
@RequestMapping("expert")
public class ExpertController {

    @Autowired
    EnvProperties envProperties;

    @Autowired
    TokenManager tokenManager;

    @Autowired
    private ExpertService expertService;

    @Autowired
    UserRoleService userRoleService;

    /*
     * 获取所属区域
     */
    @RequestMapping(value = "/findArea", method = RequestMethod.GET)
    public ResponseEntity<ResponseBase> findArea(@RequestParam String sn) {
        ResponseBase<SysAreaReps> responseBase = new ResponseBase<>();
        responseBase.success().setRetBody(expertService.queryAreaBySn(sn));
        return new ResponseEntity<>(responseBase, HttpStatus.OK);
    }

    /*
     * 获取所有专家
     */
    @RequestMapping(value = "/findAll", method = RequestMethod.GET)
    public ResponseEntity<ResponseBase> findAll(@RequestParam(defaultValue = "1") int pageNo,
                                                @RequestParam(defaultValue = "10") int pageSize,
                                                @RequestParam(required = false) String areaCode,
                                                @RequestParam(required = false) String expertName,
                                                @RequestParam(required = false) String typeId) {
        ResponseBase<ExpertInfoReps> responseBase = new ResponseBase<>();
        responseBase.success().setRetBody(expertService.queryAll(pageNo, pageSize, areaCode, expertName,typeId));
        return new ResponseEntity<>(responseBase, HttpStatus.OK);
    }

    /*
     * 根据id查询专家信息
     */
    @RequestMapping(value = "/findById", method = RequestMethod.GET)
    public ResponseEntity<ResponseBase> findById(@RequestParam(required = false) String id) {
        ResponseBase<ExpertInfoRep> responseBase = new ResponseBase<>();
        responseBase.success().setRetBody(expertService.queryById(id));
        return new ResponseEntity<>(responseBase, HttpStatus.OK);
    }

    /*
     * 新增专家信息
     */
    @PostMapping
    @Transactional
    @Log(log = "专家团队管理-新增专家信息")
    public ResponseEntity<ResponseBase> save(@RequestBody ExpertInfoReq reqInfo, HttpServletRequest req) {
        TokenModel accessToken = tokenManager.getToken(req.getHeader("accessToken"));
        String userId = accessToken.getUserId().split(SPLIT)[1];
        try {
            expertService.save(reqInfo, userId);
        } catch (Exception e) {
            log.error("专家团队管理-新增专家信息异常，请求数据：reqInfo|" + reqInfo + "，异常信息|" + e);
            throw e;
        }
        return new ResponseEntity<>(new ResponseBase().success(), HttpStatus.OK);
    }

    /*
     * 修改专家信息
     */
    @PutMapping
    @Log(log = "专家团队管理-修改专家信息")
    public ResponseEntity<ResponseBase> update(@RequestBody ExpertInfoReq reqInfo, HttpServletRequest req) {
        TokenModel accessToken = tokenManager.getToken(req.getHeader("accessToken"));
        String userId = accessToken.getUserId().split(SPLIT)[1];
        try {
            expertService.update(reqInfo, userId);
        } catch (Exception e) {
            log.error("专家团队管理-修改专家信息异常，请求数据：reqInfo|" + reqInfo + "，异常信息|" + e);
            throw e;
        }
        return new ResponseEntity<>(new ResponseBase().success(), HttpStatus.OK);
    }

    /*
     * 删除专家信息
     */
    @DeleteMapping
    @Log(log = "专家团队管理-删除专家信息")
    public ResponseEntity<ResponseBase> delete(@RequestBody ExpertInfoReq reqInfo, HttpServletRequest req) {
        TokenModel accessToken = tokenManager.getToken(req.getHeader("accessToken"));
        String userId = accessToken.getUserId().split(SPLIT)[1];
        try {
            expertService.delete(reqInfo, userId);
        } catch (Exception e) {
            log.error("专家团队管理-删除专家信息异常，请求数据：reqInfo|" + reqInfo + "，异常信息|" + e);
            throw e;
        }
        return new ResponseEntity<>(new ResponseBase().success(), HttpStatus.OK);
    }

    /*
     * 是否启用
     */
    @RequestMapping(value = "/updateState", method = RequestMethod.POST)
    @Log(log = "专家团队管理-是否启用")
    public ResponseEntity<ResponseBase> updateState(@RequestBody ExpertInfoReq reqInfo, HttpServletRequest req) {
        TokenModel accessToken = tokenManager.getToken(req.getHeader("accessToken"));
        String userId = accessToken.getUserId().split(SPLIT)[1];
        try {
            expertService.updateState(reqInfo, userId);
        } catch (Exception e) {
            log.error("专家团队管理-是否启用异常，请求数据：reqInfo|" + reqInfo + "，异常信息|" + e);
            throw e;
        }
        return new ResponseEntity<>(new ResponseBase().success(), HttpStatus.OK);
    }
}


