package com.hywa.map.controller.gis;

import com.hywa.map.common.utils.FileUtils;
import com.hywa.map.config.EnvProperties;
import com.hywa.map.config.interceptor.TokenManager;
import com.hywa.map.representation.*;
import com.hywa.map.service.orchard.OrchardService;
import com.hywa.map.service.topic.TopicInfoService;
import io.swagger.annotations.Api;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.UnsupportedEncodingException;

/**
 * t_sys_branch  对应Controller 实现类
 *
 * @author hywa
 * @create by 2018-12-12 10:01:56
 */

@Slf4j
@Api(tags = "GIS地图")
@RestController
@RequestMapping("/gis")
public class GisController {

    @Autowired
    private OrchardService orchardService;
    @Autowired
    private TopicInfoService topicInfoService;
    @Autowired
    EnvProperties envProperties;
    @Autowired
    TokenManager tokenManager;

    @RequestMapping(value = "/findTopicInfo", method = RequestMethod.GET)
    public ResponseEntity<ResponseBase> queryTopicInfo(@RequestParam(required = false) String topicName,
                                                       @RequestParam(defaultValue = "1") int pageNo,
                                                       @RequestParam(defaultValue = "10") int pageSize) {
        ResponseBase<TopicInfoReps> responseBase = new ResponseBase<>();
        responseBase.success().setRetBody(topicInfoService.selectTopicInfo(pageNo, pageSize, topicName));
        return new ResponseEntity<>(responseBase, HttpStatus.OK);
    }

    @RequestMapping(value = "/findTopicDetail", method = RequestMethod.GET)
    public ResponseEntity<ResponseBase> queryTopicDetail(@RequestParam(required = false) String id,
                                                         @RequestParam(required = false) String areaName) {
        ResponseBase<ViewTemplateReps> responseBase = new ResponseBase<>();
        responseBase.success().setRetBody(topicInfoService.selectTopicDetail(id,areaName));
        return new ResponseEntity<>(responseBase, HttpStatus.OK);
    }

    @RequestMapping(value = "/findGisPark", method = RequestMethod.GET)
    public ResponseEntity<ResponseBase> queryGisPark(@RequestParam(required = false) String type,
                                                     @RequestParam(required = false) String id) {
        ResponseBase<GisParkReps> responseBase = new ResponseBase<>();
        responseBase.success().setRetBody(orchardService.queryGisPark(type,id));
        return new ResponseEntity<>(responseBase, HttpStatus.OK);
    }
}

