package com.hywa.map.controller.login;

import static com.hywa.map.common.ConstantPool.SPLIT;
import static com.hywa.map.common.enums.CommonEnum.AREA_NO_FOUND;
import static com.hywa.map.common.enums.CommonEnum.ILLEGAL_PARAMETER_ERROR;

import com.hywa.map.common.exception.GlobalException;
import com.hywa.map.common.utils.StringUtils;
import com.hywa.map.config.EnvProperties;
import com.hywa.map.config.interceptor.TokenManager;
import com.hywa.map.config.interceptor.TokenModel;
import com.hywa.map.representation.AreaMenuReq;
import com.hywa.map.representation.MenuRep;
import com.hywa.map.representation.MenuReps;
import com.hywa.map.representation.ResponseBase;
import com.hywa.map.service.login.AreaMenuService;
import com.hywa.map.service.login.MenuService;
import com.hywa.map.service.login.UserService;
import io.swagger.annotations.Api;
import java.lang.reflect.InvocationTargetException;
import java.util.List;
import java.util.Map;
import javax.servlet.http.HttpServletRequest;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

@Slf4j
@Api(tags = "区域菜单管理")
@RestController
@RequestMapping("area/menu")
public class AreaMenuController {

    @Autowired
    private AreaMenuService areaMenuService;
    @Autowired
    EnvProperties envProperties;
    @Autowired
    TokenManager tokenManager;
    @Autowired
    private MenuService menuService;
    @Autowired
    private UserService userService;

    @PostMapping
    public ResponseEntity<ResponseBase> save(@Validated @RequestBody AreaMenuReq areaMenuReq,
            HttpServletRequest req) {
        TokenModel accessToken = tokenManager.getToken(req.getHeader("accessToken"));
        String userId = accessToken.getUserId().split(SPLIT)[1];
        if (!userService.checkArea(userId, areaMenuReq.getRegionId())) {
            throw new GlobalException(AREA_NO_FOUND.getIndex(), AREA_NO_FOUND.getValue());
        }
        areaMenuService.save(areaMenuReq);
        return new ResponseEntity<>(new ResponseBase<>().success(), HttpStatus.OK);
    }

    @PutMapping
    public ResponseEntity<ResponseBase> update(@Validated @RequestBody AreaMenuReq areaMenuReq,
            HttpServletRequest req) {
        TokenModel accessToken = tokenManager.getToken(req.getHeader("accessToken"));
        String userId = accessToken.getUserId().split(SPLIT)[1];
        if (!userService.checkArea(userId, areaMenuReq.getRegionId())) {
            throw new GlobalException(AREA_NO_FOUND.getIndex(), AREA_NO_FOUND.getValue());
        }
        areaMenuService.update(areaMenuReq);
        return new ResponseEntity<>(new ResponseBase<>().success(), HttpStatus.OK);
    }

    @DeleteMapping
    public ResponseEntity<ResponseBase> delete(@RequestParam String regionId) {
        if (!StringUtils.isEmpty(regionId)) {
            try {
                areaMenuService.delete(regionId);
            } catch (Exception e) {
                log.error(e.getMessage());
                return new ResponseEntity<>(new ResponseBase<>().success(), HttpStatus.OK);
            }
        } else {
            throw new GlobalException(ILLEGAL_PARAMETER_ERROR.getIndex(),
                    ILLEGAL_PARAMETER_ERROR.getValue() + ",regionId为空");
        }
        return new ResponseEntity<>(new ResponseBase<>().success(), HttpStatus.OK);
    }

    @RequestMapping(value = "/getAll", method = RequestMethod.GET)
    public ResponseEntity<ResponseBase> get(
            @RequestParam(required = false) String regionId,
            @RequestParam(defaultValue = "1") Integer pageNo,
            @RequestParam(defaultValue = "10") Integer pageSize, HttpServletRequest req)
            throws IllegalAccessException, NoSuchMethodException, InvocationTargetException {
        TokenModel accessToken = tokenManager.getToken(req.getHeader("accessToken"));
        String userId = accessToken.getUserId().split(SPLIT)[1];
        Map<String, Object> userMap = userService.findAllById(userId);
        String userName = (String) userMap.get("name");
        List<MenuRep> menuRepList = null;
        menuRepList = areaMenuService.selectAll(userId, userName, regionId);
        ResponseBase<MenuReps> menuRepResponseBase = new ResponseBase<>();
        MenuReps menuReps = new MenuReps(menuRepList);
        menuRepResponseBase.setRetBody(menuReps);
        menuRepResponseBase.success();
        return new ResponseEntity<>(menuRepResponseBase, HttpStatus.OK);
    }

    //根据区域代码，查询菜单
    @RequestMapping(value = "/getByRegionId", method = RequestMethod.GET)
    public ResponseEntity<ResponseBase> getAreaMenuTreeByRegionId(
            @RequestParam(defaultValue = "") String type,
            @RequestParam(defaultValue = "") String regionId, HttpServletRequest req)
            throws IllegalAccessException, NoSuchMethodException, InvocationTargetException {
        TokenModel accessToken = tokenManager.getToken(req.getHeader("accessToken"));
        String userId = accessToken.getUserId().split(SPLIT)[1];
        Map<String, Object> userMap = userService.findAllById(userId);
        String userName = (String) userMap.get("name");
        List<MenuRep> menuRepList = null;
        if ("admin".equals(userName)) {
            menuRepList = menuService.getMenuAllTreeByRegionId("", type, regionId);
        } else {
            menuRepList = menuService.getAreaMenuTreeByRegionId(userId, type, regionId);
        }
        ResponseBase<MenuReps> menuRepResponseBase = new ResponseBase<>();
        MenuReps menuReps = new MenuReps(menuRepList);
        menuRepResponseBase.setRetBody(menuReps);
        menuRepResponseBase.success();
        return new ResponseEntity<>(menuRepResponseBase, HttpStatus.OK);
    }
}
