package com.hywa.map.controller.login;

import static com.hywa.map.common.ConstantPool.SPLIT;
import static com.hywa.map.common.enums.CommonEnum.FAILURE;
import static com.hywa.map.common.enums.CommonEnum.ILLEGAL_PARAMETER_ERROR;

import com.hywa.map.common.exception.GlobalException;
import com.hywa.map.common.log.Log;
import com.hywa.map.common.utils.StringUtils;
import com.hywa.map.config.interceptor.TokenManager;
import com.hywa.map.config.interceptor.TokenModel;
import com.hywa.map.dao.entity.Menu;
import com.hywa.map.representation.*;
import com.hywa.map.service.login.MenuService;
import com.hywa.map.service.login.UserService;
import io.swagger.annotations.Api;

import java.lang.reflect.InvocationTargetException;
import java.util.List;
import java.util.Map;
import javax.servlet.http.HttpServletRequest;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

@Api(tags = "菜单管理")
@RestController
@RequestMapping("menu")
public class MenuController {

    @Autowired
    private MenuService menuService;
    @Autowired
    private UserService userService;
    @Autowired
    TokenManager tokenManager;

    @RequestMapping(value = "/getChildren", method = RequestMethod.GET)
    public ResponseEntity<ResponseBase> getChildren(
            @RequestParam String menuId,
            @RequestParam String userId) {
        List<MenuRep> menuRepList = menuService.getChildren(userId, menuId);
        ResponseBase<MenuReps> menuRepResponseBase = new ResponseBase<>();
        menuRepResponseBase.setRetBody(new MenuReps(menuRepList));
        menuRepResponseBase.success();
        return new ResponseEntity<>(menuRepResponseBase, HttpStatus.OK);
    }

    /*@RequestMapping(value = "/userMenuTree", method = RequestMethod.GET)
    public ResponseEntity<ResponseBase> getMenuTreeById(
            @RequestParam String userId,
            @RequestParam String requestMode) {
        List<MenuRep> menuRepList = menuService.getMenuTreeByUserId(userId, requestMode);

        ResponseBase<MenuReps> menuRepResponseBase = new ResponseBase<>();
        MenuReps menuReps = new MenuReps(menuRepList);
        menuRepResponseBase.setRetBody(menuReps);
        menuRepResponseBase.success();
        return new ResponseEntity<>(menuRepResponseBase, HttpStatus.OK);
    }*/

    @RequestMapping(value = "/userMenuTree", method = RequestMethod.GET)
    public ResponseEntity<ResponseBase> getMenuTreeById(@RequestParam(defaultValue = "900004") String sysId, HttpServletRequest req) {
        TokenModel accessToken = tokenManager.getToken(req.getHeader("accessToken"));
        String userId = accessToken.getUserId().split(SPLIT)[1];
        List<MenuItemRep> menuRepList = menuService.getUserMenuTreeByUserId(userId, sysId);

        ResponseBase<MenuItemReps> menuRepResponseBase = new ResponseBase<>();
        MenuItemReps menuReps = new MenuItemReps(menuRepList);
        menuRepResponseBase.setRetBody(menuReps);
        menuRepResponseBase.success();
        return new ResponseEntity<>(menuRepResponseBase, HttpStatus.OK);
    }

    @RequestMapping(value = "/menuAllTree", method = RequestMethod.GET)
    public ResponseEntity<ResponseBase> getMenuAllTree(
            @RequestParam(defaultValue = "") String roleId) {
        List<MenuRep> menuRepList = menuService.getMenuAllTree(roleId, null);

        ResponseBase<MenuReps> menuRepResponseBase = new ResponseBase<>();
        MenuReps menuReps = new MenuReps(menuRepList);
        menuRepResponseBase.setRetBody(menuReps);
        menuRepResponseBase.success();
        return new ResponseEntity<>(menuRepResponseBase, HttpStatus.OK);
    }

    @RequestMapping(value = "/getMenus", method = RequestMethod.GET)
    public ResponseEntity<ResponseBase> getMenusByRoleId(@RequestParam String roleId,
                                                         HttpServletRequest req)
            throws IllegalAccessException, NoSuchMethodException, InvocationTargetException {
        TokenModel accessToken = tokenManager.getToken(req.getHeader("accessToken"));
        String userId = accessToken.getUserId().split(SPLIT)[1];
        Map<String, Object> userMap = userService.findAllById(userId);
        String userName = (String) userMap.get("name");

        List<MenuRep> menuRepList = menuService.selectMenusByRoleId(roleId, userId, userName);
        ResponseBase<MenuReps> responseBase = new ResponseBase<>();
        MenuReps menuReps = new MenuReps(menuRepList);
        responseBase.success();
        responseBase.setRetBody(menuReps);
        return new ResponseEntity<>(responseBase, HttpStatus.OK);
    }

    @PostMapping
    public ResponseEntity<ResponseBase> save(@RequestBody MenuRep menuRep, HttpServletRequest req) {
        TokenModel accessToken = tokenManager.getToken(req.getHeader("accessToken"));
        menuRep.setCreateUser(accessToken.getUserId().split(SPLIT)[1]);
        if (1 != menuService.save(menuRep)) {
            throw new GlobalException(FAILURE.getIndex(), "保存失败");
        }
        ResponseBase<Menu> menuRepResponseBase = new ResponseBase<>();
        menuRepResponseBase.setRetBody(menuRep);
        menuRepResponseBase.success();
        return new ResponseEntity<>(menuRepResponseBase, HttpStatus.OK);
    }

    @PutMapping
    public ResponseEntity<ResponseBase> update(@RequestBody MenuRep menuRep,
                                               HttpServletRequest req) {
        TokenModel accessToken = tokenManager.getToken(req.getHeader("accessToken"));
        menuRep.setUpdateUser(accessToken.getUserId().split(SPLIT)[1]);
        if (1 != menuService.update(menuRep)) {
            throw new GlobalException(FAILURE.getIndex(), "更新失败");
        }
        return new ResponseEntity<>(new ResponseBase<>().success(), HttpStatus.OK);
    }

    @DeleteMapping
    public ResponseEntity<ResponseBase> delete(@RequestParam String menuId) {
        menuService.delete(menuId);
        return new ResponseEntity<>(new ResponseBase<>().success(), HttpStatus.OK);
    }

    //查询菜单基本信息
    @RequestMapping(value = "/search", method = RequestMethod.GET)
    public ResponseEntity<ResponseBase> search(
            @RequestParam(defaultValue = "1") int pageNo,
            @RequestParam(defaultValue = "10") int pageSize,
            @RequestParam(required = false) String parentId,
            @RequestParam(required = false) String id,
            @RequestParam(required = false) String menuName
    ) {

        ResponseBase menuRepResponseBase = new ResponseBase<>();
        menuRepResponseBase.success();
        menuRepResponseBase
                .setRetBody(menuService.search(pageNo, pageSize, parentId, id, menuName));
        return new ResponseEntity<>(menuRepResponseBase, HttpStatus.OK);
    }

    //根据用户角色，地区查询菜单
    @RequestMapping(value = "/areaMenuTree", method = RequestMethod.GET)
    public ResponseEntity<ResponseBase> getAreaMenuTree(
            @RequestParam(defaultValue = "") String type,
            @RequestParam(defaultValue = "") String regionId, HttpServletRequest req)
            throws IllegalAccessException, NoSuchMethodException, InvocationTargetException {
        TokenModel accessToken = tokenManager.getToken(req.getHeader("accessToken"));
        String userId = accessToken.getUserId().split(SPLIT)[1];
        Map<String, Object> userMap = userService.findAllById(userId);
        String userName = (String) userMap.get("name");
        List<MenuRep> menuRepList = null;
        if ("admin".equals(userName)) {
            menuRepList = StringUtils.isEmpty(regionId) ? menuService.getMenuAllTree("", type)
                    : menuService.getMenuAllTreeByRegionId("", type, regionId);
        } else {
            menuRepList = StringUtils.isEmpty(regionId) ? menuService.getAreaMenuTree(userId, type)
                    : menuService.getAreaMenuTreeByRegionId(userId, type, regionId);
        }
        ResponseBase<MenuReps> menuRepResponseBase = new ResponseBase<>();
        MenuReps menuReps = new MenuReps(menuRepList);
        menuRepResponseBase.setRetBody(menuReps);
        menuRepResponseBase.success();
        return new ResponseEntity<>(menuRepResponseBase, HttpStatus.OK);
    }

    @RequestMapping(value = "/getMenuByRole", method = RequestMethod.GET)
    public ResponseEntity<ResponseBase> getMenuByRole(@RequestParam String roleId) {
        if (StringUtils.isEmpty(roleId)) {
            throw new GlobalException(ILLEGAL_PARAMETER_ERROR.getIndex(),
                    ILLEGAL_PARAMETER_ERROR.getValue() + ",角色ID为空");
        }
        MenuSysReps menuSysReps = menuService.getMenuByRole(roleId);
        ResponseBase<MenuSysReps> responseBase = new ResponseBase<>();
        responseBase.success();
        responseBase.setRetBody(menuSysReps);
        return new ResponseEntity<>(responseBase, HttpStatus.OK);
    }

    @Log(log = "角色管理-保存权限")
    @RequestMapping(value = "/saveRoleMenu", method = RequestMethod.POST)
    public ResponseEntity<ResponseBase> saveRoleMenu(@RequestBody MenuReq menuReq) {
        menuService.saveRoleMenu(menuReq);
        return new ResponseEntity<>(new ResponseBase<>().success(), HttpStatus.OK);
    }

    @RequestMapping(value = "/sysMenuTree", method = RequestMethod.GET)
    public ResponseEntity<ResponseBase> getMenuTreeBySysId(@RequestParam String sysId, HttpServletRequest req) {
        if (StringUtils.isEmpty(sysId)) {
            throw new GlobalException(ILLEGAL_PARAMETER_ERROR.getIndex(),
                    ILLEGAL_PARAMETER_ERROR.getValue() + ",系统ID为空");
        }
        TokenModel accessToken = tokenManager.getToken(req.getHeader("accessToken"));
        String userId = accessToken.getUserId().split(SPLIT)[1];
        List<MenuRep> menuRepList = menuService.getMenuTreeBySysId(userId, sysId);
        ResponseBase<MenuReps> menuRepResponseBase = new ResponseBase<>();
        MenuReps menuReps = new MenuReps(menuRepList);
        menuRepResponseBase.setRetBody(menuReps);
        menuRepResponseBase.success();
        return new ResponseEntity<>(menuRepResponseBase, HttpStatus.OK);
    }

    @RequestMapping(value = "/getButtonByMenuId", method = RequestMethod.GET)
    public ResponseEntity<ResponseBase> getButtonByMenuId(@RequestParam String menuId, HttpServletRequest req) {
        if (StringUtils.isEmpty(menuId)) {
            throw new GlobalException(ILLEGAL_PARAMETER_ERROR.getIndex(),
                    ILLEGAL_PARAMETER_ERROR.getValue() + ",菜单ID为空");
        }
        TokenModel accessToken = tokenManager.getToken(req.getHeader("accessToken"));
        String userId = accessToken.getUserId().split(SPLIT)[1];
        ButtonReps buttonRep = menuService.selectButtonByMenuId(menuId, userId);
        ResponseBase<ButtonReps> responseBase = new ResponseBase<>();
        responseBase.success();
        responseBase.setRetBody(buttonRep);
        return new ResponseEntity<>(responseBase, HttpStatus.OK);
    }
}