package com.hywa.map.controller.login;

import static com.hywa.map.common.ConstantPool.SPLIT;
import static com.hywa.map.common.enums.CommonEnum.*;

import com.hywa.map.common.exception.GlobalException;
import com.hywa.map.common.log.Log;
import com.hywa.map.common.utils.StringUtils;
import com.hywa.map.config.EnvProperties;
import com.hywa.map.config.interceptor.TokenManager;
import com.hywa.map.config.interceptor.TokenModel;
import com.hywa.map.representation.ResponseBase;
import com.hywa.map.representation.RoleRep;
import com.hywa.map.representation.RoleReps;
import com.hywa.map.service.login.RoleService;
import com.hywa.map.service.login.UserService;
import io.swagger.annotations.Api;
import java.lang.reflect.InvocationTargetException;
import java.util.Map;
import javax.servlet.http.HttpServletRequest;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

@Slf4j
@Api(tags = "角色管理")
@RestController
@RequestMapping("role")
public class RoleController {

    @Autowired
    private RoleService roleService;

    @Autowired
    EnvProperties envProperties;
    @Autowired
    TokenManager tokenManager;
    @Autowired
    private UserService userService;

    @PostMapping
    @Log(log = "角色管理-新增")
    public ResponseEntity<ResponseBase> save(@Validated @RequestBody RoleRep roleRep,
            HttpServletRequest req) {
        TokenModel accessToken = tokenManager.getToken(req.getHeader("accessToken"));
        String userId = accessToken.getUserId().split(SPLIT)[1];
        roleRep.setCreateUser(userId);
        roleService.save(roleRep);
        return new ResponseEntity<>(new ResponseBase<>().success(), HttpStatus.OK);
    }

    @PutMapping
    @Log(log = "角色管理-修改")
    public ResponseEntity<ResponseBase> update(@RequestBody RoleRep roleRep,
            HttpServletRequest req) {
        if (StringUtils.isEmpty(roleRep.getId())) {
            throw new GlobalException(ILLEGAL_PARAMETER_ERROR.getIndex(),
                    ILLEGAL_PARAMETER_ERROR.getValue() + ",角色ID为空",null,"角色管理-修改");
        }
        if (envProperties.getAdminRoleId().equals(roleRep.getId())) {
            throw new GlobalException(DEL_ADMIN.getIndex(),
                    DEL_ADMIN.getValue(),null,"角色管理-修改");
        }
        TokenModel accessToken = tokenManager.getToken(req.getHeader("accessToken"));
        String userId = accessToken.getUserId().split(SPLIT)[1];
        roleRep.setUpdateUser(userId);
        roleService.update(roleRep);
        return new ResponseEntity<>(new ResponseBase<>().success(), HttpStatus.OK);
    }

    @DeleteMapping
    @Log(log = "角色管理-删除")
    public ResponseEntity<ResponseBase> delete(@RequestParam String roleId) {
        if (StringUtils.isEmpty(roleId)) {
            throw new GlobalException(ILLEGAL_PARAMETER_ERROR.getIndex(),
                    ILLEGAL_PARAMETER_ERROR.getValue() + ",角色ID为空",null,"角色管理-删除");
        }
        if (envProperties.getAdminRoleId().equals(roleId)) {
            throw new GlobalException(DEL_ADMIN.getIndex(),
                    DEL_ADMIN.getValue(),null,"角色管理-删除");
        }
        roleService.delete(roleId);
        return new ResponseEntity<>(new ResponseBase<>().success(), HttpStatus.OK);
    }

    /*@RequestMapping(value = "/getAll", method = RequestMethod.GET)
    public ResponseEntity<ResponseBase> get(
            @RequestParam(required = false) String name,
            @RequestParam(defaultValue = "1") Integer pageNo,
            @RequestParam(defaultValue = "10") Integer pageSize, HttpServletRequest req)
            throws IllegalAccessException, NoSuchMethodException, InvocationTargetException {
        TokenModel accessToken = tokenManager.getToken(req.getHeader("accessToken"));
        String userId = accessToken.getUserId().split(SPLIT)[1];
        Map<String, Object> userMap = userService.findAllById(userId);
        String userName = (String) userMap.get("name");
        RoleReps roleReps = roleService.selectAll(pageNo, pageSize, name, userName, userId);
        ResponseBase<RoleReps> responseBase = new ResponseBase<>();
        responseBase.success();
        responseBase.setRetBody(roleReps);
        return new ResponseEntity<>(responseBase, HttpStatus.OK);
    }*/

    @RequestMapping(value = "/getAll", method = RequestMethod.GET)
    public ResponseEntity<ResponseBase> get(
            @RequestParam(required = false) String name,@RequestParam(required = false) String flag) {
        RoleReps roleReps = roleService.selectAll(name,flag);
        ResponseBase<RoleReps> responseBase = new ResponseBase<>();
        responseBase.success();
        responseBase.setRetBody(roleReps);
        return new ResponseEntity<>(responseBase, HttpStatus.OK);
    }

    @RequestMapping(value = "/getRoles", method = RequestMethod.GET)
    public ResponseEntity<ResponseBase> getUserRoles(
            @RequestParam String userId,
            @RequestParam(defaultValue = "1") Integer pageNo,
            @RequestParam(defaultValue = "10") Integer pageSize) {

        RoleReps roleReps = roleService.selectRolesByUserId(pageNo, pageSize, userId);
        ResponseBase<RoleReps> responseBase = new ResponseBase<>();
        responseBase.success();
        responseBase.setRetBody(roleReps);
        return new ResponseEntity<>(responseBase, HttpStatus.OK);
    }

    @RequestMapping(value = "/findById", method = RequestMethod.GET)
    public ResponseEntity<ResponseBase> findById(@RequestParam String id) {
        RoleRep roleRep = roleService.findById(id);
        ResponseBase<RoleRep> responseBase = new ResponseBase<>();
        responseBase.success();
        responseBase.setRetBody(roleRep);
        return new ResponseEntity<>(responseBase, HttpStatus.OK);
    }

    @RequestMapping(value = "/getAreaRoles", method = RequestMethod.GET)
    public ResponseEntity<ResponseBase> getAreaRoles(
            @RequestParam String userId,
            @RequestParam(defaultValue = "1") Integer pageNo,
            @RequestParam(defaultValue = "10") Integer pageSize)
            throws IllegalAccessException, NoSuchMethodException, InvocationTargetException {
        Map<String, Object> userMap = userService.findAllById(userId);
        String regionType = userMap.get("type") + "";
        RoleReps roleReps = roleService.selectRolesByArea(pageNo, pageSize, userId, regionType);
        ResponseBase<RoleReps> responseBase = new ResponseBase<>();
        responseBase.success();
        responseBase.setRetBody(roleReps);
        return new ResponseEntity<>(responseBase, HttpStatus.OK);
    }
}
