package com.hywa.map.controller.map;

import com.hywa.map.common.utils.StringUtils;
import com.hywa.map.config.EnvProperties;
import com.hywa.map.config.interceptor.TokenManager;
import com.hywa.map.dao.mapper.ExpertInfoMapper;
import com.hywa.map.representation.*;
import com.hywa.map.service.dict.DictService;
import com.hywa.map.service.expert.AreaDataService;
import com.hywa.map.service.expert.DamService;
import com.hywa.map.service.map.MapModuleDataService;
import com.hywa.map.service.notice.NoticeService;
import io.swagger.annotations.Api;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import java.text.DecimalFormat;
import java.util.*;

/**
 * t_sys_branch  对应Controller 实现类
 *
 * @author hywa
 * @create by 2018-12-12 10:01:56
 */

@Slf4j
@Api(tags = "一张图")
@RestController
@RequestMapping("/map")
public class MapAdviseController {

    @Autowired
    private MapModuleDataService mapModuleDataService;
    @Autowired
    private DamService damService;
    @Autowired
    EnvProperties envProperties;
    @Resource
    private AreaDataService areaDataService;
    @Autowired
    private DictService dictService;
    @Autowired
    TokenManager tokenManager;
    @Autowired
    private NoticeService noticeService;
    @Autowired
    private ExpertInfoMapper expertInfoMapper;
    public static String[] areaArrays = {"冠山街道", "龙山镇", "醒狮镇", "谷脚镇", "湾滩河镇", "洗马镇", "龙里县"};
    public static String[] colorArrays = {"rgba(35, 104, 255, 1)", "rgba(9, 184, 209, 1)", "rgba(94, 139, 171, 1)", "rgba(172, 134, 240, 1)", "rgba(172, 134, 240, 1)",
            "rgba(0, 139, 139, 1)", "rgba(255, 255, 205, 1)", "rgba(255, 228, 181, 1)", "rgba(112, 128, 144, 1)", "rgba(0, 0, 128, 1)",
            "rgba(240, 255, 255, 1)", "rgba(230, 230, 250, 1)", "rgba(47, 79, 79, 1)", "rgba(106, 90, 205, 1)", "rgba(69, 139, 116, 1)",
            "rgba(0, 191, 255, 1)", "rgba(70, 130, 180, 1)", "rgba(0, 0, 255, 1)", "rgba(255, 248, 220, 1)", "rgba(255, 228, 225, 1)",
            "rgba(107, 142, 35, 1)", "rgba(189, 183, 107, 1)", "rgba(255, 215, 0, 1)", "rgba(238, 201, 0, 1)", "rgba(139, 101, 8, 1)",
            "rgba(0, 207, 236, 1)", "rgba(135, 206, 255, 1)", "rgba(198, 226, 255, 1)", "rgba(191, 239, 255, 1)", "rgba(180, 205, 205, 1)",
            "rgba(255, 225, 255, 1)", "rgba(255, 187, 255, 1)"};
    public static String[] legendArrays = {"北京", "天津", "河北", "重庆", "云南", "贵州"
            , "四川", "河南", "山东", "辽宁", "黑龙江", "内蒙古"
            , "吉林", "湖南", "安徽", "浙江", "江西", "湖北"
            , "江苏", "新疆", "甘肃", "山西", "青海", "陕西"
            , "宁夏", "广西", "福建", "广东", "海南", "上海"
            , "西藏", "台湾", "香港", "澳门", "南海诸岛"};

    public static DecimalFormat decimalFormat = new DecimalFormat("###################.###########");

    public ResponseEntity<ResponseBase> findSubDomainName(@RequestParam(required = false) String name) {
        ResponseBase<MapDomainNameReps> responseBase = new ResponseBase<>();
        name = StringUtils.isEmpty(name) ? "刺梨" : name;
        List<MapDomainNameRep> list = new ArrayList<>();
        AreaDataRep areaDataRep = areaDataService.queryDisplay("龙里县", "808b87b6e4ff403ca825f955031f6659");
        if (areaDataRep != null) {
            AreaDataDetailRep areaDataDetailRep = areaDataService.queryById(areaDataRep.getId());
            MapDomainNameRep nameRep = new MapDomainNameRep();
            nameRep.setIcon("icon-quanbushujumulu");
            nameRep.setId("全部");
            nameRep.setName("全部");
            list.add(nameRep);
            List<Map<String, String>> valList = areaDataDetailRep.getValList();
            for (int i = 0; i < valList.size(); i++) {
                MapDomainNameRep rep = new MapDomainNameRep();
                Map<String, String> item = valList.get(i);
                String subName = item.get("fieldVal2");
                if (StringUtils.isNotEmpty(subName) && item.get("fieldVal1").contains(name)) {
                    rep.setIcon("icon-moren_fenleiicon");
                    rep.setId(subName);
                    rep.setName(subName);
                    list.add(rep);
                }
            }
        }
        MapDomainNameReps reps = new MapDomainNameReps();
        reps.setId(name);
        reps.setList(list);
        responseBase.success().setRetBody(reps);
        return new ResponseEntity<>(responseBase, HttpStatus.OK);
    }

    /*
     * 获取产业销售区域分布
     */
    @RequestMapping(value = "/findDomainArea", method = RequestMethod.GET)
    public ResponseEntity<ResponseBase> findDomainArea(@RequestParam(required = false) String id,
                                                       @RequestParam(required = false) String domainName,
                                                       @RequestParam(required = false) String subDomainName) {
        ResponseBase<MapDomainAreaReps> responseBase = new ResponseBase<>();
        String fields = "";
        boolean isSignal = false;
        if ("全部".equals(subDomainName)) {
            ResponseEntity<ResponseBase> baseResponseEntity = findSubDomainName(domainName);
            ResponseBase base = baseResponseEntity.getBody();
            MapDomainNameReps mapDomainNameReps = (MapDomainNameReps) base.getRetBody();
            List<MapDomainNameRep> subName = mapDomainNameReps.getList();
            for (MapDomainNameRep item : subName) {
                fields = fields + item.getName() + "|";
            }
            fields = fields.substring(0, fields.length() - 1);
            if ("全部".equals(fields)) {
                fields = domainName;
                isSignal = true;
            } else {
                fields = fields.substring(3, fields.length());
            }
        } else {
            fields = subDomainName;
        }
        id = StringUtils.isEmpty(id) ? "龙里县" : id;
        //String[] areaNameArrays = new String[]{id};
        String[] areaNameArrays = (StringUtils.isEmpty(id) || "龙里县".equals(id)) ? areaArrays : (new String[]{id});
        String[] fieldArrays = fields.split("\\|");
        List<MapDomainSellRep> list = new ArrayList<>();
        double cnt1 = 0;
        double cnt2 = 0;
        List<Map<String, String>> allValList = new ArrayList<>();
        for (int i = 0; i < areaNameArrays.length; i++) {
            AreaDataRep areaDataRep = areaDataService.queryShow(areaNameArrays[i], "53e14d43e8154507b0ef2231f8a18da5");
            if (areaDataRep != null) {
                AreaDataDetailRep areaDataDetailRep = areaDataService.queryById(areaDataRep.getId());
                List<Map<String, String>> valList = areaDataDetailRep.getValList();
                for (Map<String, String> map : valList) {
                    allValList.add(map);
                }
            }
        }
        domainName = "蔬菜|水果|中药材|茶叶|油茶|食用菌".indexOf(domainName) >= 0 ? domainName + "产业" : domainName;
        List<Map<String, String>> sellList = new ArrayList<>();
        List<String> sellAreaList = new ArrayList<>();
        for (int j = 0; j < fieldArrays.length; j++) {
            for (Map<String, String> map : allValList) {
                if ("销售流向及占比".equals(map.get("fieldVal3")) && fieldArrays[j].equals(map.get("fieldVal1")) && (domainName.equals(map.get("fieldVal2")) || isSignal) && StringUtils.isNotEmpty(map.get("fieldVal4"))) {
                    String[] val4s = map.get("fieldVal4").split("\\,");
                    for (int k = 0; k < val4s.length; k++) {
                        Map<String, String> valMap = new HashMap<>();
                        valMap.put(val4s[k].split("\\=")[0], val4s[k].split("\\=")[1]);
                        sellList.add(valMap);
                        sellAreaList.add(val4s[k].split("\\=")[0]);
                    }
                }
            }
        }
        HashSet h = new HashSet(sellAreaList);
        sellAreaList.clear();
        sellAreaList.addAll(h);
        for (int m = 0; m < sellAreaList.size(); m++) {
            String area = sellAreaList.get(m);
            double cnt = 0;
            MapDomainSellRep rep = new MapDomainSellRep();
            for (Map<String, String> item : sellList) {
                for (String key : item.keySet()) {
                    String value = item.get(key);
                    if (key.equals(area)) {
                        cnt = cnt + Double.parseDouble(value);
                    }
                }
            }
            rep.setName(area);
            rep.setColor(colorArrays[m]);
            rep.setValue(decimalFormat.format(cnt));
            list.add(rep);
        }
        MapDomainAreaReps reps = new MapDomainAreaReps();
        reps.setId("");
        reps.setTitle("建议销售区域分布");
        reps.set_caveat("注意:城市对应值为图例序号减1");
        List<MapDomainAreaRep> legend = new ArrayList<>();
        MapDomainAreaRep areaRep = new MapDomainAreaRep();
        areaRep.setColor("rgba(74, 255, 0, 1)");
        areaRep.setName("销售区域");
        legend.add(areaRep);
        areaRep = new MapDomainAreaRep();
        areaRep.setColor("rgba(0, 109, 201, 0.73)");
        areaRep.setName("未销售区域");
        legend.add(areaRep);
        reps.setLegend(legend);
        List<MapMerchantRep> repList = new ArrayList<>();
        for (int i = 0; i < legendArrays.length; i++) {
            MapMerchantRep rep = new MapMerchantRep();
            rep.setName(legendArrays[i]);
            rep.setValue(sellAreaList.contains(legendArrays[i]) ? "0" : "1");
            repList.add(rep);
        }
        reps.setList(repList);
        responseBase.success().setRetBody(reps);
        return new ResponseEntity<>(responseBase, HttpStatus.OK);
    }

    /*
     * 获取产业市场需求量
     */
    @RequestMapping(value = "/findDomainNeed", method = RequestMethod.GET)
    public ResponseEntity<ResponseBase> findDomainNeed(@RequestParam(required = false) String id,
                                                       @RequestParam(required = false) String domainName,
                                                       @RequestParam(required = false) String subDomainName) {
        ResponseBase<MapDomainNeedReps> responseBase = new ResponseBase<>();
        String fields = "";
        boolean isSignal = false;
        if ("全部".equals(subDomainName)) {
            ResponseEntity<ResponseBase> baseResponseEntity = findSubDomainName(domainName);
            ResponseBase base = baseResponseEntity.getBody();
            MapDomainNameReps mapDomainNameReps = (MapDomainNameReps) base.getRetBody();
            List<MapDomainNameRep> subName = mapDomainNameReps.getList();
            for (MapDomainNameRep item : subName) {
                fields = fields + item.getName() + "|";
            }
            fields = fields.substring(0, fields.length() - 1);
            if ("全部".equals(fields)) {
                fields = domainName;
                isSignal = true;
            } else {
                fields = fields.substring(3, fields.length());
            }
        } else {
            fields = subDomainName;
        }
        id = StringUtils.isEmpty(id) ? "龙里县" : id;
        //String[] areaNameArrays = new String[]{id};
        String[] areaNameArrays = (StringUtils.isEmpty(id) || "龙里县".equals(id)) ? areaArrays : (new String[]{id});
        String[] fieldArrays = fields.split("\\|");
        List<MapDomainNeedRep> list = new ArrayList<>();
        double cnt1 = 0;
        double cnt2 = 0;
        List<Map<String, String>> allValList = new ArrayList<>();
        for (int i = 0; i < areaNameArrays.length; i++) {
            AreaDataRep areaDataRep = areaDataService.queryShow(areaNameArrays[i], "53e14d43e8154507b0ef2231f8a18da5");
            if (areaDataRep != null) {
                AreaDataDetailRep areaDataDetailRep = areaDataService.queryById(areaDataRep.getId());
                List<Map<String, String>> valList = areaDataDetailRep.getValList();
                for (Map<String, String> map : valList) {
                    allValList.add(map);
                }
            }
        }
        domainName = "蔬菜|水果|中药材|茶叶|油茶|食用菌".indexOf(domainName) >= 0 ? domainName + "产业" : domainName;
        List<Map<String, String>> sellList = new ArrayList<>();
        List<String> sellAreaList = new ArrayList<>();
        for (int j = 0; j < fieldArrays.length; j++) {
            for (Map<String, String> map : allValList) {
                if ("市场需求量".equals(map.get("fieldVal3")) && fieldArrays[j].equals(map.get("fieldVal1")) && (domainName.equals(map.get("fieldVal2")) || isSignal) && StringUtils.isNotEmpty(map.get("fieldVal4"))) {
                    String[] val4s = map.get("fieldVal4").split("\\,");
                    for (int k = 0; k < val4s.length; k++) {
                        Map<String, String> valMap = new HashMap<>();
                        valMap.put(val4s[k].split("\\=")[0], val4s[k].split("\\=")[1]);
                        sellList.add(valMap);
                        sellAreaList.add(val4s[k].split("\\=")[0]);
                    }
                }
            }
        }
        HashSet h = new HashSet(sellAreaList);
        sellAreaList.clear();
        sellAreaList.addAll(h);
        for (int m = 0; m < sellAreaList.size(); m++) {
            String area = sellAreaList.get(m);
            double cnt = 0;
            MapDomainNeedRep rep = new MapDomainNeedRep();
            for (Map<String, String> item : sellList) {
                for (String key : item.keySet()) {
                    String value = item.get(key);
                    if (key.equals(area)) {
                        cnt = cnt + Double.parseDouble(value);
                    }
                }
            }
            rep.setName(area);
            rep.setType("市场需求量");
            rep.setValue(cnt);
            list.add(rep);
        }
        MapDomainNeedReps reps = new MapDomainNeedReps();
        reps.setId("");
        reps.setTitle("市场需求量");
        reps.setSubtitle("副标题");
        List<MapDomainNeedRep> newList = new ArrayList<>();
        for (int i = 1; i < 13; i++) {
            String name = i + "";
            MapDomainNeedRep rep = new MapDomainNeedRep();
            rep.setType("市场需求量");
            rep.setName(name);
            double value = 0.0;
            for (MapDomainNeedRep item : list) {
                if (name.equals(item.getName())) {
                    value = item.getValue();
                    break;
                }
            }
            rep.setValue(value);
            newList.add(rep);
        }
        reps.setList(newList);
        responseBase.success().setRetBody(reps);
        return new ResponseEntity<>(responseBase, HttpStatus.OK);
    }

    /*
     * 获取产业价格指数
     */
    @RequestMapping(value = "/findDomainPrice", method = RequestMethod.GET)
    public ResponseEntity<ResponseBase> findDomainPrice(@RequestParam(required = false) String id,
                                                        @RequestParam(required = false) String domainName,
                                                        @RequestParam(required = false) String subDomainName) {
        ResponseBase<MapDomainNeedReps> responseBase = new ResponseBase<>();
        String fields = "";
        boolean isSignal = false;
        if ("全部".equals(subDomainName)) {
            ResponseEntity<ResponseBase> baseResponseEntity = findSubDomainName(domainName);
            ResponseBase base = baseResponseEntity.getBody();
            MapDomainNameReps mapDomainNameReps = (MapDomainNameReps) base.getRetBody();
            List<MapDomainNameRep> subName = mapDomainNameReps.getList();
            for (MapDomainNameRep item : subName) {
                fields = fields + item.getName() + "|";
            }
            fields = fields.substring(0, fields.length() - 1);
            if ("全部".equals(fields)) {
                fields = domainName;
                isSignal = true;
            } else {
                fields = fields.substring(3, fields.length());
            }
        } else {
            fields = subDomainName;
        }
        id = StringUtils.isEmpty(id) ? "龙里县" : id;
        //String[] areaNameArrays = new String[]{id};
        String[] areaNameArrays = (StringUtils.isEmpty(id) || "龙里县".equals(id)) ? areaArrays : (new String[]{id});
        String[] fieldArrays = fields.split("\\|");
        List<MapDomainNeedRep> list = new ArrayList<>();
        double cnt1 = 0;
        double cnt2 = 0;
        List<Map<String, String>> allValList = new ArrayList<>();
        for (int i = 0; i < areaNameArrays.length; i++) {
            AreaDataRep areaDataRep = areaDataService.queryShow(areaNameArrays[i], "53e14d43e8154507b0ef2231f8a18da5");
            if (areaDataRep != null) {
                AreaDataDetailRep areaDataDetailRep = areaDataService.queryById(areaDataRep.getId());
                List<Map<String, String>> valList = areaDataDetailRep.getValList();
                for (Map<String, String> map : valList) {
                    allValList.add(map);
                }
            }
        }
        domainName = "蔬菜|水果|中药材|茶叶|油茶|食用菌".indexOf(domainName) >= 0 ? domainName + "产业" : domainName;
        List<Map<String, String>> sellList = new ArrayList<>();
        List<String> sellAreaList = new ArrayList<>();
        int record = 0;
        for (int j = 0; j < fieldArrays.length; j++) {
            for (Map<String, String> map : allValList) {
                if ("平均价格".equals(map.get("fieldVal3")) && fieldArrays[j].equals(map.get("fieldVal1")) && (domainName.equals(map.get("fieldVal2")) || isSignal) && StringUtils.isNotEmpty(map.get("fieldVal4"))) {
                    String[] val4s = map.get("fieldVal4").split("\\,");
                    for (int k = 0; k < val4s.length; k++) {
                        Map<String, String> valMap = new HashMap<>();
                        valMap.put(val4s[k].split("\\=")[0], val4s[k].split("\\=")[1]);
                        sellList.add(valMap);
                        sellAreaList.add(val4s[k].split("\\=")[0]);
                    }
                    record = record + 1;
                }
            }
        }
        HashSet h = new HashSet(sellAreaList);
        sellAreaList.clear();
        sellAreaList.addAll(h);

        for (int m = 0; m < sellAreaList.size(); m++) {
            String area = sellAreaList.get(m);
            double cnt = 0;
            MapDomainNeedRep rep = new MapDomainNeedRep();
            for (Map<String, String> item : sellList) {
                for (String key : item.keySet()) {
                    String value = item.get(key);
                    if (key.equals(area)) {
                        cnt = cnt + Double.parseDouble(value);
                    }
                }
            }
            rep.setName(area);
            rep.setType("价格指数");
            rep.setValue(cnt);
            list.add(rep);
        }
        MapDomainNeedReps reps = new MapDomainNeedReps();
        reps.setId("");
        reps.setTitle("价格指数");
        reps.setSubtitle("副标题");
        List<MapDomainNeedRep> newList = new ArrayList<>();
        for (int i = 1; i < 13; i++) {
            String name = i + "";
            MapDomainNeedRep rep = new MapDomainNeedRep();
            rep.setType("价格指数");
            rep.setName(name);
            double value = 0.0;
            for (MapDomainNeedRep item : list) {
                if (name.equals(item.getName())) {
                    value = item.getValue();
                    break;
                }
            }
            rep.setValue(Double.parseDouble(String.format("%.2f", value / (record == 0 ? 1 : record))));
            newList.add(rep);
        }
        reps.setList(newList);
        responseBase.success().setRetBody(reps);
        return new ResponseEntity<>(responseBase, HttpStatus.OK);
    }

    /*
     * 获取产业扶贫列表
     */
    @RequestMapping(value = "/findHelpAll", method = RequestMethod.GET)
    public ResponseEntity<ResponseBase> findHelpAll(@RequestParam(defaultValue = "1") int pageNo,
                                                    @RequestParam(defaultValue = "10") int pageSize) {
        ResponseBase<MapHelpReps> responseBase = new ResponseBase<>();
        MapHelpRep mapHelpRep = new MapHelpRep();
        List<MapExpertTypeRep> list = new ArrayList<>();
        NoticeReps noticeReps = noticeService.selectAll(pageNo, pageSize, null, null, null, "9", null);
        if (noticeReps != null) {
            List<NoticeRep> noticeRepList = noticeReps.getList();
            for (NoticeRep item : noticeRepList) {
                MapExpertTypeRep mapExpertTypeRep = new MapExpertTypeRep();
                mapExpertTypeRep.setId(item.getId());
                mapExpertTypeRep.setName(item.getTitle());
                list.add(mapExpertTypeRep);
            }
            mapHelpRep.setList(list);
        }
        MapHelpReps reps = new MapHelpReps();
        reps.setData(mapHelpRep);
        responseBase.success().setRetBody(reps);
        return new ResponseEntity<>(responseBase, HttpStatus.OK);
    }

    /*
     * 获取产业扶贫列表
     */
    @RequestMapping(value = "/findHelpDetail", method = RequestMethod.GET)
    public ResponseEntity<ResponseBase> findHelpDetail(@RequestParam(required = false) String id) {
        ResponseBase<MapHelpDetailReps> responseBase = new ResponseBase<>();
        MapHelpDetailRep mapHelpDetailRep = new MapHelpDetailRep();
        List<MapHelpDetail> list = new ArrayList<>();
        NoticeRep noticeRep = noticeService.selectById(id);
        if (noticeRep != null) {
            List<FileReq> noticeRepList = noticeRep.getFiles();
            for (FileReq item : noticeRepList) {
                MapHelpDetail mapHelpDetail = new MapHelpDetail();
                mapHelpDetail.setSrc(item.getFileUrl());
                mapHelpDetail.setValue(item.getFileDesc());
                list.add(mapHelpDetail);
            }
            mapHelpDetailRep.setList(list);
        }
        MapHelpDetailReps reps = new MapHelpDetailReps();
        reps.setData(mapHelpDetailRep);
        responseBase.success().setRetBody(reps);
        return new ResponseEntity<>(responseBase, HttpStatus.OK);
    }
}

