package com.hywa.map.controller.map;

import com.hywa.map.common.log.Log;
import com.hywa.map.common.utils.DateUtils;
import com.hywa.map.common.utils.StringUtils;
import com.hywa.map.config.EnvProperties;
import com.hywa.map.config.interceptor.TokenManager;
import com.hywa.map.config.interceptor.TokenModel;
import com.hywa.map.config.redis.RedisUtil;
import com.hywa.map.representation.*;
import com.hywa.map.service.expert.AreaDataService;
import com.hywa.map.service.map.MapModuleDataService;
import com.hywa.map.timer.YsOpenPlatTimer;
import io.swagger.annotations.Api;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;

import static com.hywa.map.common.ConstantPool.SPLIT;

/**
 * t_sys_branch  对应Controller 实现类
 *
 * @author hywa
 * @create by 2018-12-12 10:01:56
 */

@Slf4j
@Api(tags = "一张图")
@RestController
@RequestMapping("/map")
public class MapController {

    @Autowired
    private MapModuleDataService mapModuleDataService;
    @Autowired
    EnvProperties envProperties;
    @Resource
    private RedisUtil redisUtil;
    @Autowired
    TokenManager tokenManager;
    @Autowired
    private AreaDataService areaDataService;

    @PostMapping
    @Transactional
    @Log(log = "一张图-新增大屏信息")
    public ResponseEntity<ResponseBase> saveMap(@RequestBody MapInfoReq reqInfo, HttpServletRequest req) {
        TokenModel accessToken = tokenManager.getToken(req.getHeader("accessToken"));
        //String userId = accessToken.getUserId().split(SPLIT)[1];
        String userId = "anonymous";
        try {
            mapModuleDataService.saveMap(reqInfo, userId);
        } catch (Exception e) {
            log.error("一张图-新增大屏信息异常，请求数据|" + reqInfo + "，异常信息|" + e);
            throw e;
        }
        return new ResponseEntity<>(new ResponseBase().success(), HttpStatus.OK);
    }

    @DeleteMapping
    @Log(log = "一张图-删除大屏信息")
    public ResponseEntity<ResponseBase> deleteMap(@RequestBody MapInfoReq reqInfo, HttpServletRequest req) {
        TokenModel accessToken = tokenManager.getToken(req.getHeader("accessToken"));
        //String userId = accessToken.getUserId().split(SPLIT)[1];
        String userId = "anonymous";
        try {
            mapModuleDataService.deleteMap(reqInfo, userId);
        } catch (Exception e) {
            log.error("一张图-删除大屏信息异常，请求数据|" + reqInfo + "，异常信息|" + e);
            throw e;
        }
        return new ResponseEntity<>(new ResponseBase().success(), HttpStatus.OK);
    }

    @PutMapping
    @Log(log = "一张图-更新大屏信息")
    public ResponseEntity<ResponseBase> updateMap(@RequestBody MapInfoReq reqInfo, HttpServletRequest req) {
        TokenModel accessToken = tokenManager.getToken(req.getHeader("accessToken"));
        //String userId = accessToken.getUserId().split(SPLIT)[1];
        String userId = "anonymous";
        try {
            mapModuleDataService.updateMap(reqInfo, userId);
        } catch (Exception e) {
            log.error("一张图-更新大屏信息异常，请求数据|" + reqInfo + "，异常信息|" + e);
            throw e;
        }
        return new ResponseEntity<>(new ResponseBase().success(), HttpStatus.OK);
    }

    @RequestMapping(value = "/updateState", method = RequestMethod.POST)
    @Transactional
    @Log(log = "一张图-更新大屏状态信息")
    public ResponseEntity<ResponseBase> updateMapState(@RequestBody MapInfoReq reqInfo, HttpServletRequest req) {
        TokenModel accessToken = tokenManager.getToken(req.getHeader("accessToken"));
        //String userId = accessToken.getUserId().split(SPLIT)[1];
        String userId = "anonymous";
        try {
            mapModuleDataService.updateMapState(reqInfo, userId);
        } catch (Exception e) {
            log.error("一张图-更新大屏状态信息异常，请求数据|" + reqInfo + "，异常信息|" + e);
            throw e;
        }
        return new ResponseEntity<>(new ResponseBase().success(), HttpStatus.OK);
    }

    @RequestMapping(value = "/findMapAll", method = RequestMethod.GET)
    public ResponseEntity<ResponseBase> findMapAll(@RequestParam(required = false) String mapName,
                                                   @RequestParam(defaultValue = "1") int pageNo,
                                                   @RequestParam(defaultValue = "100") int pageSize) {
        ResponseBase<MapInfoReps> responseBase = new ResponseBase<>();
        responseBase.success().setRetBody(mapModuleDataService.selectMapAll(pageNo, pageSize, mapName));
        return new ResponseEntity<>(responseBase, HttpStatus.OK);
    }

    @RequestMapping(value = "/findMapById", method = RequestMethod.GET)
    public ResponseEntity<ResponseBase> findMapById(@RequestParam(required = false) String id) {
        ResponseBase<MapInfoRep> responseBase = new ResponseBase<>();
        responseBase.success().setRetBody(mapModuleDataService.selectMapById(id));
        return new ResponseEntity<>(responseBase, HttpStatus.OK);
    }

    @RequestMapping(value = "/findMapTemplate", method = RequestMethod.GET)
    public ResponseEntity<ResponseBase> findGoodsType() {
        ResponseBase<ComboxReps> responseBase = new ResponseBase<>();
        responseBase.success().setRetBody(mapModuleDataService.selectMapTemplate());
        return new ResponseEntity<>(responseBase, HttpStatus.OK);
    }

    @RequestMapping(value = "/addModuleData", method = RequestMethod.POST)
    @Transactional
    @Log(log = "一张图-新增组件配置数据信息")
    public ResponseEntity<ResponseBase> saveOrUpdate(@RequestBody MapModuleDataReq reqInfo, HttpServletRequest req) {
        TokenModel accessToken = tokenManager.getToken(req.getHeader("accessToken"));
        //String userId = accessToken.getUserId().split(SPLIT)[1];
        String userId = "anonymous";
        try {
            mapModuleDataService.saveOrUpdate(reqInfo, userId);
        } catch (Exception e) {
            log.error("一张图-新增组件配置数据信息异常，请求数据|" + reqInfo + "，异常信息|" + e);
            throw e;
        }
        return new ResponseEntity<>(new ResponseBase().success(), HttpStatus.OK);
    }

    @RequestMapping(value = "/findModuleData", method = RequestMethod.GET)
    public ResponseEntity<ResponseBase<MapModuleDataRep>> findById(@RequestParam(required = false) String mapId,
                                                                   @RequestParam(required = false) String moduleCode,
                                                                   @RequestParam(required = false) String isShow) {
        ResponseBase<MapModuleDataRep> responseBase = new ResponseBase<>();
        responseBase.success().setRetBody(mapModuleDataService.selectModuleData(mapId, moduleCode, isShow));
        responseBase.success();
        return new ResponseEntity<>(responseBase, HttpStatus.OK);
    }

    @RequestMapping(value = "/findYsAccessToken", method = RequestMethod.GET)
    public ResponseEntity<ResponseBase> findYsAccessToken() {
        ResponseBase<YsTokenRep> responseBase = new ResponseBase<>();
        YsTokenRep rep = new YsTokenRep();
        String accessToken = redisUtil.get(YsOpenPlatTimer.ysOpenPlatAccessToken);
        rep.setAccessToken(accessToken.split("\\_")[1]);
        responseBase.success().setRetBody(rep);
        responseBase.success();
        return new ResponseEntity<>(responseBase, HttpStatus.OK);
    }

    @RequestMapping(value = "/findPrice", method = RequestMethod.GET)
    public ResponseEntity<ResponseBase> findPrice(@RequestParam(defaultValue = "1") int pageNo,
                                                  @RequestParam(defaultValue = "10") int pageSize,
                                                  @RequestParam(required = false) String beginDate,
                                                  @RequestParam(required = false) String endDate,
                                                  @RequestParam(required = false) String cateName,
                                                  @RequestParam(required = false) String areaName) {
        ResponseBase<AreaDataPriceReps> responseBase = new ResponseBase<>();
        beginDate = StringUtils.isEmpty(beginDate) ? "20000101" : beginDate;
        endDate = StringUtils.isEmpty(endDate) ? "99991231" : endDate;
        areaName = "1".equals(areaName) ? "贵阳农产品物流园" : "双龙农副产品交易中心";
        responseBase.success().setRetBody(areaDataService.queryPrice(pageNo, pageSize, beginDate, endDate, cateName, areaName));
        return new ResponseEntity<>(responseBase, HttpStatus.OK);
    }
}

