package com.hywa.map.controller.map;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.hywa.map.common.utils.DateUtils;
import com.hywa.map.common.utils.HttpUtils;
import com.hywa.map.common.utils.StringUtils;
import com.hywa.map.common.utils.UUIDUtils;
import com.hywa.map.config.EnvProperties;
import com.hywa.map.config.interceptor.TokenManager;
import com.hywa.map.dao.entity.AreaData;
import com.hywa.map.dao.entity.Combox;
import com.hywa.map.dao.entity.Dict;
import com.hywa.map.dao.entity.OrchardProduct;
import com.hywa.map.dao.mapper.ExpertInfoMapper;
import com.hywa.map.dao.mapper.OrchardInfoMapper;
import com.hywa.map.representation.*;
import com.hywa.map.service.dict.DictService;
import com.hywa.map.service.expert.AreaDataService;
import com.hywa.map.service.expert.DamService;
import com.hywa.map.service.map.MapModuleDataService;
import com.hywa.map.service.orchard.OrchardService;
import io.swagger.annotations.Api;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import java.text.DecimalFormat;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * t_sys_branch  对应Controller 实现类
 *
 * @author hywa
 * @create by 2018-12-12 10:01:56
 */

@Slf4j
@Api(tags = "一张图")
@RestController
@RequestMapping("/map")
public class MapDamController {

    @Autowired
    private MapModuleDataService mapModuleDataService;
    @Autowired
    private DamService damService;
    @Autowired
    EnvProperties envProperties;
    @Resource
    private AreaDataService areaDataService;
    @Autowired
    private DictService dictService;
    @Autowired
    TokenManager tokenManager;
    @Autowired
    private ExpertInfoMapper expertInfoMapper;
    @Autowired
    private OrchardInfoMapper orchardInfoMapper;
    @Autowired
    private OrchardService orchardService;
    public static String[] areaArrays = {"冠山街道", "龙山镇", "醒狮镇", "谷脚镇", "湾滩河镇", "洗马镇", "龙里县"};
    public static DecimalFormat decimalFormat = new DecimalFormat("###################.###########");
    public static HashMap<String, String> iconMap = new HashMap<String, String>() {{
        put("空气温度", "icon-wendu");
        put("空气湿度", "icon-icon-temperature");
        put("土壤温度", "icon-turangwendu");
        put("土壤湿度", "icon-turangshidu");
        put("土壤PH", "icon-shiguan");
        put("光照度", "icon-guangzhao");
        put("风速", "icon-fengsu");
        put("风向", "icon-tianqi-fengxiang");
        put("二氧化碳", "icon-co");
        put("降雨量", "icon-jiangyuliang");
        put("大气压力", "icon-daqiyali");
        put("PM2.5", "icon-PM");
        put("土壤盐分", "icon-turangyanfen");
    }};
    public static String custName = "湄潭县竹香米业有限责任公司";
    public static double custLng = 107.605340;
    public static double custLat = 27.884780;
    /*
     * 获取监控点列表信息
     */
    @RequestMapping(value = "/findMonitorPoint", method = RequestMethod.GET)
    public ResponseEntity<ResponseBase> findMonitorPoint(@RequestParam(required = false) String id) {
        ResponseBase<MapDamTypeReps> responseBase = new ResponseBase<>();
        List<MapDamTypeRep> list = new ArrayList<>();
        OrchardRep orchardRep = orchardService.selectById(id);
        if (orchardRep != null) {
            Map<String, String> queryParas = new HashMap<>();
            queryParas.put("enter", custName);
            queryParas.put("base", orchardRep.getOrchardName());
            String responseMsg = HttpUtils.get(envProperties.getIotServerUrl() + "oneMap/getPoint", queryParas);
            if (StringUtils.isNotEmpty(responseMsg)) {
                JSONObject jsonObject = JSONObject.parseObject(responseMsg);
                JSONObject retBody = jsonObject.getJSONObject("retBody");
                if (retBody != null) {
                    JSONArray data = retBody.getJSONArray("list");
                    for (int i = 0; i < data.size(); i++) {
                        JSONObject item = (JSONObject) data.get(i);
                        String name = item.getString("collection_point_name");
                        String monId = item.getString("id");
                        MapDamTypeRep rep = new MapDamTypeRep();
                        rep.setName(name);
                        rep.setMonId(monId);
                        list.add(rep);
                    }
                    log.info("获取物联网采集点成功，请求数据：id|" + id);
                }
            }
        }
        MapDamTypeReps reps = new MapDamTypeReps(list);
        reps.setId(id);
        responseBase.success().setRetBody(reps);
        return new ResponseEntity<>(responseBase, HttpStatus.OK);
    }

    /**
     * 根据风向度数取风向
     *
     * @param degree 风向度数
     * @return 中文风向
     */
    public String getWindDire(int degree) {
        String windDire = "东北";
        if ((degree >= 348 && degree <= 360) || (degree >= 0 && degree <= 11)) {
            windDire = "北";
        } else if (degree >= 12 && degree <= 79) {
            windDire = "东北";
        } else if (degree >= 80 && degree <= 101) {
            windDire = "东";
        } else if (degree >= 102 && degree <= 168) {
            windDire = "东南";
        } else if (degree >= 169 && degree <= 191) {
            windDire = "南";
        } else if (degree >= 192 && degree <= 258) {
            windDire = "西南";
        } else if (degree >= 259 && degree <= 281) {
            windDire = "西";
        } else if (degree >= 282 && degree <= 347) {
            windDire = "西北";
        }
        return windDire;
    }

    /*
     * 获取监控点数据信息
     */
    @RequestMapping(value = "/findMonitorData", method = RequestMethod.GET)
    public ResponseEntity<ResponseBase> findMonitorData(@RequestParam(required = false) String id,
                                                        @RequestParam(required = false) String monId) {
        ResponseBase<MapMonDataReps> responseBase = new ResponseBase<>();
        List<MapMonDataRep> list = new ArrayList<>();
        Map<String, String> queryParas = new HashMap<>();
        queryParas.put("point", monId);
        String responseMsg = HttpUtils.get(envProperties.getIotServerUrl() + "oneMap/devices", queryParas);
        if (StringUtils.isNotEmpty(responseMsg)) {
            JSONObject jsonObject = JSONObject.parseObject(responseMsg);
            JSONObject retBody = jsonObject.getJSONObject("retBody");
            if (retBody != null) {
                JSONArray data = retBody.getJSONArray("list");
                for (int i = 0; i < data.size(); i++) {
                    JSONObject item = (JSONObject) data.get(i);
                    String name = item.getString("sensor_name");
                    String value = item.getString("data_value");
                    String unit = item.getString("sensor_unit");
                    MapMonDataRep rep = new MapMonDataRep();
                    rep.setName(name);
                    rep.setValue("风向".equals(name) ? getWindDire(Integer.parseInt(value)) + "风" : value);
                    rep.setUnit(StringUtils.isEmpty(unit) ? "" : unit);
                    rep.setIcon(iconMap.get(name));
                    rep.setAuthCode(item.getString("authorize_code"));
                    rep.setSensorCode(item.getString("sensor_code"));
                    list.add(rep);
                }
                log.info("获取物联网采集点实时数据成功，请求数据：id|" + id + "，monId|" + monId);
            }
        }
        MapMonDataReps reps = new MapMonDataReps();
        reps.setId(id);
        reps.setList(list);
        reps.setMonId(monId);
        responseBase.success().setRetBody(reps);
        return new ResponseEntity<>(responseBase, HttpStatus.OK);
    }

    /*
     * 获取监控点24小时列表信息
     */
    @RequestMapping(value = "/findMonitorHourData", method = RequestMethod.GET)
    public ResponseEntity<ResponseBase> findMonitorHourData(@RequestParam(required = false) String authCode,
                                                            @RequestParam(required = false) String sensorCode) {
        ResponseBase<MapMonHourDataReps> responseBase = new ResponseBase<>();
        List<List<String>> chartData = new ArrayList<>();
        Map<String, String> queryParas = new HashMap<>();
        queryParas.put("authorize", authCode);
        queryParas.put("sensor_code", sensorCode);
        String responseMsg = HttpUtils.get(envProperties.getIotServerUrl() + "oneMap/iotData", queryParas);
        List<String> dataTime = new ArrayList<>();
        if (StringUtils.isNotEmpty(responseMsg)) {
            JSONObject jsonObject = JSONObject.parseObject(responseMsg);
            JSONObject retBody = jsonObject.getJSONObject("retBody");
            if (retBody != null) {
                JSONArray data = retBody.getJSONArray("list");
                List<String> values = new ArrayList<>();
                for (int i = 0; i < data.size(); i++) {
                    JSONObject item = (JSONObject) data.get(i);
                    String time = item.getString("time");
                    time = StringUtils.isEmpty(time) ? "00:00" : time.substring(11, 13) + ":00";
                    String value = item.getString("value");
                    values.add(value);
                    dataTime.add(time);
                }
                chartData.add(values);
                log.info("获取物联网采集点24小时数据成功，请求数据：authCode|" + authCode + "，sensorCode|" + sensorCode);
            }
        }
        MapMonHourDataReps reps = new MapMonHourDataReps();
        /*List<Integer> dataTime = new ArrayList<>();
        for (int i = 0; i < 24; i++) {
            dataTime.add(i);
        }*/
        MapMonHourDataRep dataRep = new MapMonHourDataRep();
        dataRep.setData(dataTime);
        reps.setX(dataRep);
        reps.setChartData(chartData);
        responseBase.success().setRetBody(reps);
        return new ResponseEntity<>(responseBase, HttpStatus.OK);
    }

    /*
     * 获取坝区详细信息
     */
    @RequestMapping(value = "/findDamInfo", method = RequestMethod.GET)
    public ResponseEntity<ResponseBase> findDamInfo(@RequestParam(required = false) String damId) {
        ResponseBase<MapGeogReps> responseBase = new ResponseBase<>();
        List<MapMerchantRep> list = new ArrayList<>();
        DamDetailInfoRep damDetailInfoRep = damService.queryById(damId);
        String address = "";
        String title = "";
        if (damDetailInfoRep != null) {
            MapMerchantRep rep = new MapMerchantRep();
            rep.setName("海拔（米）");
            rep.setValue(damDetailInfoRep.getElevation());
            list.add(rep);
            rep = new MapMerchantRep();
            rep.setName("种植面积(亩)");
            rep.setValue(damDetailInfoRep.getTotalArea());
            list.add(rep);
            rep = new MapMerchantRep();
            rep.setName("主要产品");
            rep.setValue(damDetailInfoRep.getPlantName());
            list.add(rep);
            rep = new MapMerchantRep();
            rep.setName("耕地产出值(元)");
            rep.setValue(damDetailInfoRep.getPlantVal());
            list.add(rep);
            rep = new MapMerchantRep();
            rep.setName("土地流转率");
            rep.setValue(damDetailInfoRep.getLandRate() + "%");
            list.add(rep);
            rep = new MapMerchantRep();
            rep.setName("农民收入贡献率");
            rep.setValue(damDetailInfoRep.getFarmerRate() + "%");
            list.add(rep);
            rep = new MapMerchantRep();
            rep.setName("产销对接率");
            rep.setValue(damDetailInfoRep.getMarketRate() + "%");
            list.add(rep);
            rep = new MapMerchantRep();
            rep.setName("产业参保率");
            rep.setValue(damDetailInfoRep.getInsureRate() + "%");
            list.add(rep);
            address = "地址：龙里县" + damDetailInfoRep.getAreaCodeName();
            title = damDetailInfoRep.getDamName();
        }
        MapGeogReps reps = new MapGeogReps(list);
        reps.setId("");
        reps.setTitle(title);
        reps.setIntroduce(address);
        responseBase.success().setRetBody(reps);
        return new ResponseEntity<>(responseBase, HttpStatus.OK);
    }

    /*
     * 获取坝区简介信息
     */
    @RequestMapping(value = "/findDamDesc", method = RequestMethod.GET)
    public ResponseEntity<ResponseBase> findDamDesc(@RequestParam(required = false) String damId) {
        ResponseBase<MapDamDescRep> responseBase = new ResponseBase<>();
        DamDetailInfoRep damDetailInfoRep = damService.queryById(damId);
        String info = "";
        if (damDetailInfoRep != null) {
            info = damDetailInfoRep.getDamDesc();
        }
        MapDamDescRep reps = new MapDamDescRep();
        reps.setId(damId);
        reps.setInfo(info);
        responseBase.success().setRetBody(reps);
        return new ResponseEntity<>(responseBase, HttpStatus.OK);
    }

    /*
     * 获取专家列表信息
     */
    @RequestMapping(value = "/findDamExpert", method = RequestMethod.GET)
    public ResponseEntity<ResponseBase> findDamExpert(@RequestParam(required = false) String damId,
                                                      @RequestParam(required = false) String typeId) {
        ResponseBase<MapExpertListReps> responseBase = new ResponseBase<>();
        List<MapExpertListRep> list = expertInfoMapper.selectByDamId(damId, typeId);
        for (MapExpertListRep item : list) {
            item.setImgUrl(envProperties.getComFileDownloadUrl() + item.getImgUrl());
        }
        MapExpertListReps reps = new MapExpertListReps(list);
        reps.setId(damId);
        responseBase.success().setRetBody(reps);
        return new ResponseEntity<>(responseBase, HttpStatus.OK);
    }

    /*
     * 获取园区种植统计信息
     */
    @RequestMapping(value = "/findOrchardStatis", method = RequestMethod.GET)
    public ResponseEntity<ResponseBase> findOrchardStatis(@RequestParam(required = false) String id) {
        ResponseBase<MapMerchantReps> responseBase = new ResponseBase<>();
        String[] fieldArrays = {"园区总数", "社员人数(人)", "总种植面积(亩)", "总年产量(公斤)"};
        List<MapMerchantRep> list = new ArrayList<>();
        MapOrchardStatisRep statisRep = orchardInfoMapper.selectStatis();
        if (statisRep != null) {
            DecimalFormat df = new DecimalFormat("###########0"); //四舍五入转换成整数
            for (int j = 0; j < fieldArrays.length; j++) {
                MapMerchantRep rep = new MapMerchantRep();
                if ("园区总数".equals(fieldArrays[j])) {
                    rep.setName("园区总数");
                    rep.setValue(statisRep.getOrchardCount());
                } else if ("社员人数(人)".equals(fieldArrays[j])) {
                    rep.setName("社员人数(人)");
                    rep.setValue(df.format(Double.parseDouble(statisRep.getWorkerCount())));
                } else if ("总种植面积(亩)".equals(fieldArrays[j])) {
                    rep.setName("总种植面积(亩)");
                    rep.setValue(df.format(Double.parseDouble(statisRep.getProdArea())));
                } else if ("总年产量(公斤)".equals(fieldArrays[j])) {
                    rep.setName("总年产量(公斤)");
                    rep.setValue(df.format(Double.parseDouble(statisRep.getProdVol())));
                }
                list.add(rep);
            }
        }
        MapMerchantReps reps = new MapMerchantReps(list);
        reps.setId("");
        reps.setTitle("竹香米业有限责任公司种植统计");
        responseBase.success().setRetBody(reps);
        return new ResponseEntity<>(responseBase, HttpStatus.OK);
    }

    /*
     * 获取园区列表
     */
    @RequestMapping(value = "/findOrchardAll", method = RequestMethod.GET)
    public ResponseEntity<ResponseBase> findOrchardAll(@RequestParam(required = false) String name,
                                                       @RequestParam(required = false) String id,
                                                       @RequestParam(defaultValue = "1") int pageNo,
                                                       @RequestParam(defaultValue = "200") int pageSize) {
        String title = "";
        ResponseBase<MapDamListReps> responseBase = new ResponseBase<>();
        OrchardReps orchardReps = orchardService.selectAll(pageNo, pageSize, name, null, null);
        List<MapDamListRep> list = new ArrayList<>();
        List<OrchardRep> orchardRep = orchardReps.getList();
        for (OrchardRep infoRep : orchardRep) {
            MapDamListRep mapDamRep = new MapDamListRep();
            mapDamRep.setId(infoRep.getId());
            //mapDamRep.setAreaId(infoRep.getTownName());
            mapDamRep.setName(infoRep.getOrchardName());
            String vol = infoRep.getProductVol();
            double totalVol = 0.0;
            if (StringUtils.isNotEmpty(vol)) {
                String[] vols = vol.split("\\,");
                for (int i = 0; i < vols.length; i++) {
                    totalVol = totalVol + Double.parseDouble(vols[i]);
                }
            }
            List<MapPriceValueRep> info = new ArrayList<>();
            MapPriceValueRep valueRep = new MapPriceValueRep();
            valueRep.setValue("产量：" + totalVol + "吨");
            info.add(valueRep);
            valueRep = new MapPriceValueRep();
            valueRep.setValue("规模：" + infoRep.getOrchardArea() + "亩");
            info.add(valueRep);
            mapDamRep.setUrl("");
            String border = infoRep.getOrchardBorder();
            String lon = "";
            String lat = "";
            if (StringUtils.isNotEmpty(border)) {
                JSONArray jsonArray = JSONArray.parseArray(border);
                for (int i = 0; i < jsonArray.size(); i++) {
                    JSONObject jsonObject = jsonArray.getJSONObject(i);
                    String position = jsonObject.getString("position");
                    if (StringUtils.isNotEmpty(position)) {
                        JSONObject pos = JSONObject.parseObject(position);
                        lon = pos.getString("lng");
                        lat = pos.getString("lat");
                    }
                }
            }
            List<String> lnglat = new ArrayList<>();
            lnglat.add(lon);
            lnglat.add(lat);
            if(StringUtils.isNotEmpty(lon) && StringUtils.isNotEmpty(lat)){
                valueRep = new MapPriceValueRep();
                valueRep.setValue("距公司：" + StringUtils.calcDist(custLat,custLng,Double.parseDouble(lat),Double.parseDouble(lon)) + "km");
                info.add(valueRep);
            }
            mapDamRep.setInfo(info);
            mapDamRep.setLnglat(lnglat);
            //mapDamRep.setLegend(1);
            list.add(mapDamRep);
        }
        MapDamListReps mapDamReps = new MapDamListReps(list);
        mapDamReps.setId("");
        mapDamReps.setTitle(title);
        mapDamReps.setTotal((orchardReps == null) ? 0 : orchardReps.getTotal());
        responseBase.success().setRetBody(mapDamReps);
        return new ResponseEntity<>(responseBase, HttpStatus.OK);
    }

    /*
     * 获取园区详细信息
     */
    @RequestMapping(value = "/findOrchardInfo", method = RequestMethod.GET)
    public ResponseEntity<ResponseBase> findOrchardInfo(@RequestParam(required = false) String id) {
        ResponseBase<MapGeogReps> responseBase = new ResponseBase<>();
        List<MapMerchantRep> list = new ArrayList<>();
        OrchardRep orchardRep = orchardService.selectById(id);
        String address = "";
        String title = "";
        if (orchardRep != null) {
            MapMerchantRep rep = new MapMerchantRep();
            rep.setName("坝区面积(亩)");
            rep.setValue(orchardRep.getOrchardArea());
            list.add(rep);
            rep = new MapMerchantRep();
            rep.setName("园区年产量(吨)");
            String vol = orchardRep.getProductVol();
            double totalVol = 0.0;
            if (StringUtils.isNotEmpty(vol)) {
                String[] vols = vol.split("\\,");
                for (int i = 0; i < vols.length; i++) {
                    totalVol = totalVol + Double.parseDouble(vols[i]);
                }
            }
            rep.setValue(totalVol + "");
            list.add(rep);
            rep = new MapMerchantRep();
            rep.setName("人员数(人)");
            rep.setValue(orchardRep.getWorkerCount());
            list.add(rep);
            rep = new MapMerchantRep();
            rep.setName("建园日期");
            rep.setValue(orchardRep.getBeginDate());
            list.add(rep);
            rep = new MapMerchantRep();
            rep.setName("海拔");
            rep.setValue(orchardRep.getElevation());
            list.add(rep);
            rep = new MapMerchantRep();
            rep.setName("负责人");
            rep.setValue(orchardRep.getLeaderName());
            list.add(rep);
            rep = new MapMerchantRep();
            rep.setName("联系方式");
            rep.setValue(orchardRep.getTelNo());
            list.add(rep);
            rep = new MapMerchantRep();
            rep.setName("地址");
            String provinceName = StringUtils.isEmpty(orchardRep.getProvinceName()) ? "" : orchardRep.getProvinceName();
            String cityName = StringUtils.isEmpty(orchardRep.getCityName()) ? "" : orchardRep.getCityName();
            String countyName = StringUtils.isEmpty(orchardRep.getCountyName()) ? "" : orchardRep.getCountyName();
            rep.setValue(provinceName + cityName + countyName);
            list.add(rep);
            title = orchardRep.getOrchardName();
        }
        MapGeogReps reps = new MapGeogReps(list);
        reps.setId("");
        reps.setTitle(title);
        reps.setIntroduce(orchardRep == null ? "" : orchardRep.getOrchardDesc());
        responseBase.success().setRetBody(reps);
        return new ResponseEntity<>(responseBase, HttpStatus.OK);
    }

    /*
     * 获取园区简介信息
     */
    @RequestMapping(value = "/findOrchardDesc", method = RequestMethod.GET)
    public ResponseEntity<ResponseBase> findOrchardDesc(@RequestParam(required = false) String id) {
        ResponseBase<MapDamDescRep> responseBase = new ResponseBase<>();
        OrchardRep orchardRep = orchardService.selectById(id);
        String info = "";
        if (orchardRep != null) {
            info = orchardRep.getOrchardDesc();
        }
        MapDamDescRep reps = new MapDamDescRep();
        reps.setId(id);
        reps.setInfo(info);
        responseBase.success().setRetBody(reps);
        return new ResponseEntity<>(responseBase, HttpStatus.OK);
    }

    /*
     * 获取园区产出物信息
     */
    @RequestMapping(value = "/findOrchardProd", method = RequestMethod.GET)
    public ResponseEntity<ResponseBase> findOrchardProd(@RequestParam(required = false) String id) {
        ResponseBase<MapOrchardProdReps> responseBase = new ResponseBase<>();
        OrchardRep orchardRep = orchardService.selectById(id);
        List<MapOrchardProdRep> list = new ArrayList<>();
        if (orchardRep != null) {
            List<OrchardProduct> products = orchardRep.getProducts();
            if (products != null) {
                for (OrchardProduct item : products) {
                    MapOrchardProdRep prodRep = new MapOrchardProdRep();
                    prodRep.setInfo("产出物：" + item.getProdName());
                    double area = (item.getProdArea() == null) ? 0 : Double.parseDouble(item.getProdArea());
                    double vol = (item.getProdVol() == null) ? 0 : Double.parseDouble(item.getProdVol());
                    prodRep.setInfo1("产出面积：" + decimalFormat.format(area) + "亩");
                    prodRep.setInfo2("预估产量：" + decimalFormat.format(vol) + "吨");
                    list.add(prodRep);
                }
            }
        }
        MapOrchardProdReps reps = new MapOrchardProdReps();
        reps.setList(list);
        responseBase.success().setRetBody(reps);
        return new ResponseEntity<>(responseBase, HttpStatus.OK);
    }

    /*
     * 获取园区地图编辑信息
     */
    @RequestMapping(value = "/findOrchardBorder", method = RequestMethod.GET)
    public ResponseEntity<ResponseBase> findOrchardBorder(@RequestParam(required = false) String id) {
        ResponseBase<MapDamBorderRep> responseBase = new ResponseBase<>();
        OrchardRep orchardRep = orchardService.selectById(id);
        String info = "";
        if (orchardRep != null) {
            info = orchardRep.getOrchardBorder();
        }
        MapDamBorderRep reps = new MapDamBorderRep();
        reps.setId(id);
        reps.setInfo(JSON.parseArray(info));
        responseBase.success().setRetBody(reps);
        return new ResponseEntity<>(responseBase, HttpStatus.OK);
    }
}

