package com.hywa.map.controller.map;

import com.hywa.map.common.utils.StringUtils;
import com.hywa.map.config.EnvProperties;
import com.hywa.map.config.interceptor.TokenManager;
import com.hywa.map.dao.mapper.ExpertInfoMapper;
import com.hywa.map.representation.*;
import com.hywa.map.service.dict.DictService;
import com.hywa.map.service.expert.AreaDataService;
import com.hywa.map.service.expert.DamService;
import com.hywa.map.service.map.MapModuleDataService;
import io.swagger.annotations.Api;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import java.text.DecimalFormat;
import java.util.*;

/**
 * t_sys_branch  对应Controller 实现类
 *
 * @author hywa
 * @create by 2018-12-12 10:01:56
 */

@Slf4j
@Api(tags = "一张图")
@RestController
@RequestMapping("/map")
public class MapDomainController {

    @Autowired
    private MapModuleDataService mapModuleDataService;
    @Autowired
    private DamService damService;
    @Autowired
    EnvProperties envProperties;
    @Resource
    private AreaDataService areaDataService;
    @Autowired
    private DictService dictService;
    @Autowired
    TokenManager tokenManager;
    @Autowired
    private ExpertInfoMapper expertInfoMapper;
    public static String[] areaArrays = {"冠山街道", "龙山镇", "醒狮镇", "谷脚镇", "湾滩河镇", "洗马镇", "龙里县"};
    public static String[] colorArrays = {"rgba(35, 104, 255, 1)", "rgba(9, 184, 209, 1)", "rgba(94, 139, 171, 1)", "rgba(172, 134, 240, 1)", "rgba(172, 134, 240, 1)",
            "rgba(0, 139, 139, 1)", "rgba(255, 255, 205, 1)", "rgba(255, 228, 181, 1)", "rgba(112, 128, 144, 1)", "rgba(0, 0, 128, 1)",
            "rgba(240, 255, 255, 1)", "rgba(230, 230, 250, 1)", "rgba(47, 79, 79, 1)", "rgba(106, 90, 205, 1)", "rgba(69, 139, 116, 1)",
            "rgba(0, 191, 255, 1)", "rgba(70, 130, 180, 1)", "rgba(0, 0, 255, 1)", "rgba(255, 248, 220, 1)", "rgba(255, 228, 225, 1)",
            "rgba(107, 142, 35, 1)", "rgba(189, 183, 107, 1)", "rgba(255, 215, 0, 1)", "rgba(238, 201, 0, 1)", "rgba(139, 101, 8, 1)",
            "rgba(0, 207, 236, 1)", "rgba(135, 206, 255, 1)", "rgba(198, 226, 255, 1)", "rgba(191, 239, 255, 1)", "rgba(180, 205, 205, 1)",
            "rgba(255, 225, 255, 1)", "rgba(255, 187, 255, 1)"};
    public static DecimalFormat decimalFormat = new DecimalFormat("###################.###########");

    /*
     * 获取8大产业信息信息
     */
    @RequestMapping(value = "/findDomainName", method = RequestMethod.GET)
    public ResponseEntity<ResponseBase> findDomainName(@RequestParam(required = false) String name) {
        ResponseBase<MapDomainNameReps> responseBase = new ResponseBase<>();
        name = StringUtils.isEmpty(name) ? "龙里县" : name;
        List<MapDomainNameRep> list = new ArrayList<>();
        AreaDataRep areaDataRep = areaDataService.queryDisplay(name, "808b87b6e4ff403ca825f955031f6659");
        if (areaDataRep != null) {
            AreaDataDetailRep areaDataDetailRep = areaDataService.queryById(areaDataRep.getId());
            List<Map<String, String>> valList = areaDataDetailRep.getValList();
            boolean flag1 = false;
            boolean flag2 = false;
            boolean flag3 = false;
            boolean flag4 = false;
            boolean flag5 = false;
            boolean flag6 = false;
            boolean flag7 = false;
            boolean flag8 = false;
            boolean flag9 = false;
            for (Map<String, String> item : valList) {
                MapDomainNameRep rep = new MapDomainNameRep();
                if (item.get("fieldVal1").contains("刺梨") && !flag1) {
                    rep.setIcon("icon-chudian_shuiguo");
                    rep.setId(item.get("fieldVal1"));
                    rep.setName(item.get("fieldVal1"));
                    flag1 = true;
                } else if (item.get("fieldVal1").contains("蔬菜") && !flag2) {
                    rep.setIcon("icon-shucai");
                    rep.setId(item.get("fieldVal1"));
                    rep.setName(item.get("fieldVal1"));
                    flag2 = true;
                } else if (item.get("fieldVal1").contains("水果") && !flag3) {
                    rep.setIcon("icon-shuiguo");
                    rep.setId(item.get("fieldVal1"));
                    rep.setName(item.get("fieldVal1"));
                    flag3 = true;
                } else if (item.get("fieldVal1").contains("中药材") && !flag4) {
                    rep.setIcon("icon-xianzhuanghaicao");
                    rep.setId(item.get("fieldVal1"));
                    rep.setName(item.get("fieldVal1"));
                    flag4 = true;
                } else if (item.get("fieldVal1").contains("茶叶") && !flag5) {
                    rep.setIcon("icon-chaye");
                    rep.setId(item.get("fieldVal1"));
                    rep.setName(item.get("fieldVal1"));
                    flag5 = true;
                } else if (item.get("fieldVal1").contains("油茶") && !flag6) {
                    rep.setIcon("icon-chahu");
                    rep.setId(item.get("fieldVal1"));
                    rep.setName(item.get("fieldVal1"));
                    flag6 = true;
                } else if (item.get("fieldVal1").contains("食用菌") && !flag7) {
                    rep.setIcon("icon--jungulei");
                    rep.setId(item.get("fieldVal1"));
                    rep.setName(item.get("fieldVal1"));
                    flag7 = true;
                } else if (item.get("fieldVal1").contains("生态畜牧业") && !flag8) {
                    rep.setIcon("icon-nongmuye");
                    rep.setId(item.get("fieldVal1"));
                    rep.setName(item.get("fieldVal1"));
                    flag8 = true;
                } else if (item.get("fieldVal1").contains("其他") && !flag9) {
                    rep.setIcon("icon-other");
                    rep.setId(item.get("fieldVal1"));
                    rep.setName(item.get("fieldVal1"));
                    flag9 = true;
                }
                if (StringUtils.isNotEmpty(rep.getIcon())) {
                    list.add(rep);
                }

            }
        }
        MapDomainNameReps reps = new MapDomainNameReps();
        reps.setId("");
        reps.setList(list);
        responseBase.success().setRetBody(reps);
        return new ResponseEntity<>(responseBase, HttpStatus.OK);
    }

    /*
     * 获取8大产业子类信息
     */
    @RequestMapping(value = "/findSubDomainName", method = RequestMethod.GET)
    public ResponseEntity<ResponseBase> findSubDomainName(@RequestParam(required = false) String name) {
        ResponseBase<MapDomainNameReps> responseBase = new ResponseBase<>();
        name = StringUtils.isEmpty(name) ? "刺梨" : name;
        List<MapDomainNameRep> list = new ArrayList<>();
        AreaDataRep areaDataRep = areaDataService.queryDisplay("龙里县", "808b87b6e4ff403ca825f955031f6659");
        if (areaDataRep != null) {
            AreaDataDetailRep areaDataDetailRep = areaDataService.queryById(areaDataRep.getId());
            MapDomainNameRep nameRep = new MapDomainNameRep();
            //nameRep.setIcon("icon-quanbushujumulu");
            nameRep.setIcon("");
            nameRep.setId("全部");
            nameRep.setName("全部");
            list.add(nameRep);
            List<Map<String, String>> valList = areaDataDetailRep.getValList();
            for (int i = 0; i < valList.size(); i++) {
                MapDomainNameRep rep = new MapDomainNameRep();
                Map<String, String> item = valList.get(i);
                String subName = item.get("fieldVal2");
                if (StringUtils.isNotEmpty(subName) && item.get("fieldVal1").contains(name)) {
                    //rep.setIcon("icon-moren_fenleiicon");
                    rep.setIcon("");
                    rep.setId(subName);
                    rep.setName(subName);
                    list.add(rep);
                }
            }
        }
        MapDomainNameReps reps = new MapDomainNameReps();
        reps.setId(name);
        reps.setList(list);
        responseBase.success().setRetBody(reps);
        return new ResponseEntity<>(responseBase, HttpStatus.OK);
    }

    /*
     * 获取产业现状统计类型列表
     */
    @RequestMapping(value = "/findDomainType", method = RequestMethod.GET)
    public ResponseEntity<ResponseBase> findIndustryType(@RequestParam(required = false) String id,
                                                         @RequestParam(required = false) String domainName,
                                                         @RequestParam(required = false) String subDomainName) {
        ResponseBase<MapIndustryReps> responseBase = new ResponseBase<>();
        domainName = StringUtils.isEmpty(domainName) ? "刺梨" : domainName;
        subDomainName = "全部".equals(subDomainName) ? domainName : subDomainName;
        List<MapIndustryRep> list = new ArrayList<>();
        String[] fieldArrays = {"种植面积|亩", "产量|吨", "产值|元"};
        if (domainName.contains("生态畜牧业")) {
            fieldArrays = new String[]{"养殖场|个", "产量|头", "产值|元"};
        }
        id = StringUtils.isEmpty(id) ? "龙里县" : id;
        for (int i = 0; i < fieldArrays.length; i++) {
            MapIndustryRep mapIndustryRep = new MapIndustryRep();
            String name = fieldArrays[i].split("\\|")[0];
            String unit = fieldArrays[i].split("\\|")[1];
            mapIndustryRep.setId(i + "");
            mapIndustryRep.setName(name);
            mapIndustryRep.setTitle(id + subDomainName + "统计");
            mapIndustryRep.setUnit("单位：" + unit);
            list.add(mapIndustryRep);
        }
        MapIndustryReps reps = new MapIndustryReps(list);
        reps.setId("");
        reps.setList(list);
        responseBase.success().setRetBody(reps);
        return new ResponseEntity<>(responseBase, HttpStatus.OK);
    }

    /*
     * 获取产业现状统计明细信息
     */
    @RequestMapping(value = "/findDomainDetail", method = RequestMethod.GET)
    public ResponseEntity<ResponseBase> findDomainDetail(@RequestParam(required = false) String id,
                                                         @RequestParam(required = false) String typeId,
                                                         @RequestParam(required = false) String domainName,
                                                         @RequestParam(required = false) String subDomainName) {
        ResponseBase<MapIndustryDetailReps> responseBase = new ResponseBase<>();
        String fields = "";
        boolean isSignal = false;
        if ("全部".equals(subDomainName)) {
            ResponseEntity<ResponseBase> baseResponseEntity = findSubDomainName(domainName);
            ResponseBase base = baseResponseEntity.getBody();
            MapDomainNameReps mapDomainNameReps = (MapDomainNameReps) base.getRetBody();
            List<MapDomainNameRep> subName = mapDomainNameReps.getList();
            for (MapDomainNameRep item : subName) {
                fields = fields + item.getName() + "|";
            }
            fields = fields.substring(0, fields.length() - 1);
            if ("全部".equals(fields)) {
                fields = domainName;
                isSignal = true;
            } else {
                fields = fields.substring(3, fields.length());
            }
        } else {
            fields = subDomainName;
        }
        String typeName = "";
        if ("0".equals(typeId)) {
            typeName = "生态畜牧业".equals(domainName) ? "养殖场" : "种植面积";
        } else if ("1".equals(typeId)) {
            typeName = "产量";
        } else if ("2".equals(typeId)) {
            typeName = "产值";
        }

        String[] areaNameArrays = (StringUtils.isEmpty(id) || "龙里县".equals(id)) ? areaArrays : (new String[]{id});
        id = StringUtils.isEmpty(id) ? "龙里县" : id;
        String[] fieldArrays = fields.split("\\|");
        List<MapIndustryDetailRep> list = new ArrayList<>();
        double cnt1 = 0;
        List<Map<String, String>> allValList = new ArrayList<>();
        for (int i = 0; i < areaNameArrays.length; i++) {
            AreaDataRep areaDataRep = areaDataService.queryShow(areaNameArrays[i], "894bb259e92c439cbeb2177974e27050");
            if (areaDataRep != null) {
                AreaDataDetailRep areaDataDetailRep = areaDataService.queryById(areaDataRep.getId());
                List<Map<String, String>> valList = areaDataDetailRep.getValList();
                for (Map<String, String> map : valList) {
                    allValList.add(map);
                }
            }
        }
        domainName = "蔬菜|水果|中药材|茶叶|油茶|食用菌".indexOf(domainName) >= 0 ? domainName + "产业" : domainName;
        for (int j = 0; j < fieldArrays.length; j++) {
            cnt1 = 0;
            for (Map<String, String> map : allValList) {
                if (fieldArrays[j].equals(map.get("fieldVal1")) && (domainName.equals(map.get("fieldVal2")) || isSignal) && typeName.equals(map.get("fieldVal3"))) {
                    cnt1 = cnt1 + Double.parseDouble(map.get("fieldVal4"));
                    continue;
                }
            }
            MapIndustryDetailRep rep = new MapIndustryDetailRep();
            rep.setViewName(fieldArrays[j]);
            rep.setViewVal(decimalFormat.format(cnt1));
            rep.setId(j + "");
            list.add(rep);
        }
        //20191210 根据需求修改为前10条 start
        List<MapIndustryDetailRep> topTenList = new ArrayList<>();
        if (list != null && list.size() > 0) {
            for (int k = 0; k < list.size(); k++) {
                if (k < 10) {
                    topTenList.add(list.get(k));
                }
            }
        }
        //20191210 根据需求修改为前10条 end
        MapIndustryDetailReps reps = new MapIndustryDetailReps(topTenList);
        reps.setId("");
        responseBase.success().setRetBody(reps);
        return new ResponseEntity<>(responseBase, HttpStatus.OK);
    }

    /*
     * 获取产业现状销售统计明细信息
     */
    @RequestMapping(value = "/findDomainSell", method = RequestMethod.GET)
    public ResponseEntity<ResponseBase> findDomainSell(@RequestParam(required = false) String id,
                                                       @RequestParam(required = false) String typeId,
                                                       @RequestParam(required = false) String domainName,
                                                       @RequestParam(required = false) String subDomainName) {
        ResponseBase<MapDomainSellReps> responseBase = new ResponseBase<>();
        String title = "1".equals(typeId) ? "销售渠道" : "销售流向";
        String fields = "";
        boolean isSignal = false;
        if ("全部".equals(subDomainName)) {
            ResponseEntity<ResponseBase> baseResponseEntity = findSubDomainName(domainName);
            ResponseBase base = baseResponseEntity.getBody();
            MapDomainNameReps mapDomainNameReps = (MapDomainNameReps) base.getRetBody();
            List<MapDomainNameRep> subName = mapDomainNameReps.getList();
            for (MapDomainNameRep item : subName) {
                fields = fields + item.getName() + "|";
            }
            fields = fields.substring(0, fields.length() - 1);
            if ("全部".equals(fields)) {
                fields = domainName;
                isSignal = true;
            } else {
                fields = fields.substring(3, fields.length());
            }
        } else {
            fields = subDomainName;
        }
        id = StringUtils.isEmpty(id) ? "龙里县" : id;
        //String[] areaNameArrays = new String[]{id};
        String[] areaNameArrays = (StringUtils.isEmpty(id) || "龙里县".equals(id)) ? areaArrays : (new String[]{id});
        String[] fieldArrays = fields.split("\\|");
        List<MapDomainSellRep> list = new ArrayList<>();
        double cnt1 = 0;
        double cnt2 = 0;
        List<Map<String, String>> allValList = new ArrayList<>();
        int areaCount = 0;
        for (int i = 0; i < areaNameArrays.length; i++) {
            AreaDataRep areaDataRep = areaDataService.queryShow(areaNameArrays[i], "f7251c484d3c4a4399beca397241f3e6");
            if (areaDataRep != null) {
                AreaDataDetailRep areaDataDetailRep = areaDataService.queryById(areaDataRep.getId());
                List<Map<String, String>> valList = areaDataDetailRep.getValList();
                for (Map<String, String> map : valList) {
                    allValList.add(map);
                }
                areaCount++;
            }
        }
        domainName = "蔬菜|水果|中药材|茶叶|油茶|食用菌".indexOf(domainName) >= 0 ? domainName + "产业" : domainName;
        List<Map<String, String>> sellList = new ArrayList<>();
        List<String> sellAreaList = new ArrayList<>();
        for (int j = 0; j < fieldArrays.length; j++) {
            for (Map<String, String> map : allValList) {
                if ("线上销售占比".equals(map.get("fieldVal3")) && fieldArrays[j].equals(map.get("fieldVal1")) && (domainName.equals(map.get("fieldVal2")) || isSignal)) {
                    cnt1 = cnt1 + Double.parseDouble(map.get("fieldVal4"));
                    continue;
                }
                if ("线下销售占比".equals(map.get("fieldVal3")) && fieldArrays[j].equals(map.get("fieldVal1")) && (domainName.equals(map.get("fieldVal2")) || isSignal)) {
                    cnt2 = cnt2 + Double.parseDouble(map.get("fieldVal4"));
                    continue;
                }
                if ("销售流向及占比".equals(map.get("fieldVal3")) && fieldArrays[j].equals(map.get("fieldVal1")) && (domainName.equals(map.get("fieldVal2")) || isSignal)) {
                    String[] val4s = map.get("fieldVal4").split("\\#");
                    for (int k = 0; k < val4s.length; k++) {
                        Map<String, String> valMap = new HashMap<>();
                        valMap.put(val4s[k].split("\\=")[0], val4s[k].split("\\=")[1]);
                        sellList.add(valMap);
                        sellAreaList.add(val4s[k].split("\\=")[0]);
                    }
                    continue;
                }
            }
        }
        String onlineRate = String.format("%.2f", ((cnt1 / fieldArrays.length)) / ((areaCount == 0) ? 1 : areaCount));
        String offRate = String.format("%.2f", ((cnt2 / fieldArrays.length)) / ((areaCount == 0) ? 1 : areaCount));
        if ("1".equals(typeId)) {
            MapDomainSellRep rep = new MapDomainSellRep();
            rep.setName("线上销售占比");
            rep.setColor("rgba(35, 104, 255, 1)");
            rep.setValue(cnt1 == 0 ? "0" : decimalFormat.format(Double.parseDouble(onlineRate)));
            list.add(rep);
            rep = new MapDomainSellRep();
            rep.setName("线下销售占比");
            rep.setColor("rgba(0, 207, 236, 1)");
            rep.setValue(cnt2 == 0 ? "0" : decimalFormat.format(Double.parseDouble(offRate)));
            list.add(rep);
        } else {
            //获取销售流向 start
            allValList.clear();
            for (int i = 0; i < areaNameArrays.length; i++) {
                AreaDataRep areaDataRep = areaDataService.queryShow(areaNameArrays[i], "53e14d43e8154507b0ef2231f8a18da5");
                if (areaDataRep != null) {
                    AreaDataDetailRep areaDataDetailRep = areaDataService.queryById(areaDataRep.getId());
                    List<Map<String, String>> valList = areaDataDetailRep.getValList();
                    for (Map<String, String> map : valList) {
                        allValList.add(map);
                    }
                }
            }
            sellList.clear();
            sellAreaList.clear();
            for (int j = 0; j < fieldArrays.length; j++) {
                for (Map<String, String> map : allValList) {
                    if ("销售流向及占比".equals(map.get("fieldVal3")) && fieldArrays[j].equals(map.get("fieldVal1")) && (domainName.equals(map.get("fieldVal2")) || isSignal) && StringUtils.isNotEmpty(map.get("fieldVal4"))) {
                        String[] val4s = map.get("fieldVal4").split("\\,");
                        for (int k = 0; k < val4s.length; k++) {
                            Map<String, String> valMap = new HashMap<>();
                            valMap.put(val4s[k].split("\\=")[0], val4s[k].split("\\=")[1]);
                            sellList.add(valMap);
                            sellAreaList.add(val4s[k].split("\\=")[0]);
                        }
                    }
                }
            }
            //获取销售流向 end
            HashSet h = new HashSet(sellAreaList);
            sellAreaList.clear();
            sellAreaList.addAll(h);
            for (int m = 0; m < sellAreaList.size(); m++) {
                String area = sellAreaList.get(m);
                double cnt = 0;
                MapDomainSellRep rep = new MapDomainSellRep();
                for (Map<String, String> item : sellList) {
                    for (String key : item.keySet()) {
                        String value = item.get(key);
                        if (key.equals(area)) {
                            cnt = cnt + Double.parseDouble(value);
                        }
                    }
                }
                rep.setName(area);
                rep.setColor(colorArrays[m]);
                rep.setValue(decimalFormat.format(cnt));
                list.add(rep);
            }
        }
        MapDomainSellReps reps = new MapDomainSellReps(list);
        reps.setId("");
        reps.setSubtitle(title);
        responseBase.success().setRetBody(reps);
        return new ResponseEntity<>(responseBase, HttpStatus.OK);
    }

    /*
     * 获取产业现状图列信息
     */
    @RequestMapping(value = "/findDomainLegend", method = RequestMethod.GET)
    public ResponseEntity<ResponseBase> findDomainLegend(@RequestParam(required = false) String id,
                                                         @RequestParam(required = false) String typeId,
                                                         @RequestParam(required = false) String domainName,
                                                         @RequestParam(required = false) String subDomainName) {
        ResponseBase<MapDomainLegendReps> responseBase = new ResponseBase<>();
        subDomainName = StringUtils.isEmpty(subDomainName) ? "全部" : subDomainName;
        String fields = "";
        boolean isSignal = false;
        if ("全部".equals(subDomainName)) {
            ResponseEntity<ResponseBase> baseResponseEntity = findSubDomainName(domainName);
            ResponseBase base = baseResponseEntity.getBody();
            MapDomainNameReps mapDomainNameReps = (MapDomainNameReps) base.getRetBody();
            List<MapDomainNameRep> subName = mapDomainNameReps.getList();
            for (MapDomainNameRep item : subName) {
                fields = fields + item.getName() + "|";
            }
            fields = fields.substring(0, fields.length() - 1);
            if ("全部".equals(fields)) {
                fields = domainName;
                isSignal = true;
            } else {
                fields = fields.substring(3, fields.length());
            }
        } else {
            fields = subDomainName;
        }
        String typeName = "";
        if ("0".equals(typeId)) {
            typeName = "生态畜牧业".equals(domainName) ? "养殖场" : "种植面积";
        } else if ("1".equals(typeId)) {
            typeName = "产量";
        } else if ("2".equals(typeId)) {
            typeName = "产值";
        }

        String[] areaNameArrays = (StringUtils.isEmpty(id) || "龙里县".equals(id)) ? areaArrays : (new String[]{id});
        id = StringUtils.isEmpty(id) ? "龙里县" : id;
        String[] fieldArrays = fields.split("\\|");
        List<MapIndustryDetailRep> list = new ArrayList<>();
        double cnt1 = 0;
        List<Map<String, String>> allValList = new ArrayList<>();
        List<MapDomainCoverRep> covers = new ArrayList<>();

        domainName = "蔬菜|水果|中药材|茶叶|油茶|食用菌".indexOf(domainName) >= 0 ? domainName + "产业" : domainName;
        for (int i = 0; i < areaNameArrays.length; i++) {
            list.clear();
            AreaDataRep areaDataRep = areaDataService.queryShow(areaNameArrays[i], "894bb259e92c439cbeb2177974e27050");
            if (areaDataRep != null) {
                AreaDataDetailRep areaDataDetailRep = areaDataService.queryById(areaDataRep.getId());
                List<Map<String, String>> valList = areaDataDetailRep.getValList();
                for (int j = 0; j < fieldArrays.length; j++) {
                    cnt1 = 0;
                    for (Map<String, String> map : valList) {
                        if (fieldArrays[j].equals(map.get("fieldVal1")) && (domainName.equals(map.get("fieldVal2")) || isSignal) && typeName.equals(map.get("fieldVal3"))) {
                            cnt1 = cnt1 + Double.parseDouble(map.get("fieldVal4"));
                            continue;
                        }
                    }
                    MapIndustryDetailRep rep = new MapIndustryDetailRep();
                    rep.setViewName(fieldArrays[j]);
                    rep.setViewVal(cnt1 + "");
                    rep.setId(j + "");
                    list.add(rep);
                }
                MapDomainCoverRep coverRep = new MapDomainCoverRep();
                double value = 0;
                for (MapIndustryDetailRep item : list) {
                    value = value + Double.parseDouble(item.getViewVal());
                }
                coverRep.setValue((decimalFormat.format(value)) + ("生态畜牧业".equals(domainName) ? "个" : "亩"));
                coverRep.setId("");
                coverRep.setType("Marker");
                coverRep.setLegend("1");
                MapDomainPosRep posRep = new MapDomainPosRep();
                posRep.setLat(areaDataRep.getPosLat());
                posRep.setLng(areaDataRep.getPosLon());
                posRep.setP(areaDataRep.getPosLat());
                posRep.setQ(areaDataRep.getPosLon());
                coverRep.setPosition(posRep);
                MapDomainStyleRep styleRep = new MapDomainStyleRep();
                styleRep.setFillColor("#9C427E");
                styleRep.setStrokeColor("#9C427E");
                coverRep.setStyle(styleRep);
                covers.add(coverRep);
            }
        }
        MapDomainLegendReps reps = new MapDomainLegendReps();
        reps.setId("");
        reps.set_caveat("提示：目前地图配置使用高德地图API。（图例内容：图标、图片地址、文字）");
        reps.setAmapkey("");
        MapDomainMapRep mapRep = new MapDomainMapRep();
        mapRep.setMapStyle("");
        mapRep.setZoom(10);
        List<Double> center = new ArrayList<>();
        center.add(106.979524);
        center.add(26.453154);
        mapRep.setCenter(center);
        reps.setMap(mapRep);
        List<MapDomainLegendRep> legend = new ArrayList<>();
        MapDomainLegendRep legendRep = new MapDomainLegendRep();
        legendRep.setColor("#9C427E");
        String name = "全部".equals(subDomainName) ? domainName : subDomainName;
        String title = "生态畜牧业".equals(domainName) ? "养殖场分布(个)" : "种植面积分布(亩)";
        legendRep.setName(name + title);
        legendRep.setIcon("");
        legend.add(legendRep);
        reps.setLegend(legend);
        //统计各乡镇信息

        reps.setCovers(covers);
        responseBase.success().setRetBody(reps);
        return new ResponseEntity<>(responseBase, HttpStatus.OK);
    }
}

