package com.hywa.map.controller.map;

import com.hywa.map.config.interceptor.TokenManager;
import com.hywa.map.config.interceptor.TokenModel;
import com.hywa.map.representation.MapPwdReps;
import com.hywa.map.representation.MapPwdReq;
import com.hywa.map.representation.ResponseBase;
import com.hywa.map.service.map.MapPwdService;
import io.swagger.annotations.Api;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;

import static com.hywa.map.common.ConstantPool.SPLIT;
import static com.hywa.map.common.enums.CommonEnum.SCREEN_ERROR;

/**
 * t_map_pwd  对应Controller 实现类
 *
 * @author hywa
 * @create by 2019-12-11 13:28:52
 */
@Slf4j
@Api(tags = "口令管理")
@RestController
@RequestMapping("/map")
public class MapPwdController {

    @Autowired
    private MapPwdService mapPwdService;
    @Autowired
    TokenManager tokenManager;

    /*
     * 获取所有口令
     */
    @RequestMapping(value = "/findPwd", method = RequestMethod.GET)
    public ResponseEntity<ResponseBase> findPwd(@RequestParam(defaultValue = "1") int pageNo,
                                                @RequestParam(defaultValue = "10") int pageSize) {
        ResponseBase<MapPwdReps> responseBase = new ResponseBase<>();
        responseBase.success().setRetBody(mapPwdService.selectAll(pageNo, pageSize));
        return new ResponseEntity<>(responseBase, HttpStatus.OK);
    }

    /*
     * 更新口令
     */
    @RequestMapping(value = "/updatePwd", method = RequestMethod.POST)
    public ResponseEntity<ResponseBase> updatePwd(@RequestBody MapPwdReq reqInfo, HttpServletRequest req) {
        TokenModel accessToken = tokenManager.getToken(req.getHeader("accessToken"));
        String userId = accessToken.getUserId().split(SPLIT)[1];
        try {
            mapPwdService.update(reqInfo, userId);
        } catch (Exception e) {
            log.error("大屏口令-更新口令信息异常，请求数据：reqInfo|" + reqInfo + "，异常信息|" + e);
            throw e;
        }
        return new ResponseEntity<>(new ResponseBase().success(), HttpStatus.OK);
    }

    /*
     * 验证口令
     */
    @RequestMapping(value = "/checkPwd", method = RequestMethod.POST)
    public ResponseEntity<ResponseBase> checkPwd(@RequestBody MapPwdReq reqInfo) {
        boolean result = mapPwdService.checkPwd(reqInfo.getPwdType() + "", reqInfo.getPwd());
        if (result) {
            return new ResponseEntity<>(new ResponseBase().success(), HttpStatus.OK);
        } else {
            return new ResponseEntity<>(new ResponseBase().failure(SCREEN_ERROR.getIndex(), SCREEN_ERROR.getValue()), HttpStatus.OK);
        }
    }
}

