package com.hywa.map.controller.map;

import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.hywa.map.common.utils.DateUtils;
import com.hywa.map.common.utils.ListUtils;
import com.hywa.map.common.utils.StringUtils;
import com.hywa.map.config.EnvProperties;
import com.hywa.map.config.interceptor.TokenManager;
import com.hywa.map.dao.entity.AreaData;
import com.hywa.map.dao.entity.SysArea;
import com.hywa.map.dao.mapper.AreaDataMapper;
import com.hywa.map.dao.mapper.ExpertInfoMapper;
import com.hywa.map.dao.mapper.TopicInfoMapper;
import com.hywa.map.representation.*;
import com.hywa.map.service.dict.DictService;
import com.hywa.map.service.expert.AreaDataService;
import com.hywa.map.service.expert.DamService;
import com.hywa.map.service.map.MapModuleDataService;
import io.swagger.annotations.Api;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Map;

/**
 * t_sys_branch  对应Controller 实现类
 *
 * @author hywa
 * @create by 2018-12-12 10:01:56
 */

@Slf4j
@Api(tags = "一张图")
@RestController
@RequestMapping("/map")
public class MapStatisController {

    @Autowired
    private MapModuleDataService mapModuleDataService;
    @Autowired
    private DamService damService;
    @Autowired
    EnvProperties envProperties;
    @Resource
    private AreaDataService areaDataService;
    @Autowired
    private DictService dictService;
    @Autowired
    TokenManager tokenManager;
    @Autowired
    private ExpertInfoMapper expertInfoMapper;
    @Autowired
    private TopicInfoMapper topicInfoMapper;
    @Autowired
    private AreaDataMapper areaDataMapper;

    public static String[] areaArrays = {"冠山街道", "龙山镇", "醒狮镇", "谷脚镇", "湾滩河镇", "洗马镇", "龙里县"};
    public static DecimalFormat decimalFormat = new DecimalFormat("###################.###########");

    /*
     * 获取所有坝区
     */
    @RequestMapping(value = "/findDamAll", method = RequestMethod.GET)
    public ResponseEntity<ResponseBase> findDamAll(@RequestParam(required = false) String damName,
                                                   @RequestParam(required = false) String id,
                                                   @RequestParam(defaultValue = "1") int pageNo,
                                                   @RequestParam(defaultValue = "200") int pageSize) {
        String title = (StringUtils.isEmpty(id) || "龙里县".equals(id)) ? "坝区专题" : id;
        String newAreaName = (StringUtils.isEmpty(id) || "龙里县".equals(id)) ? null : id;
        ResponseBase<MapDamReps> responseBase = new ResponseBase<>();
        DamInfoReps damInfoReps = damService.queryAll(pageNo, pageSize, null, damName, "0", newAreaName);
        List<MapDamRep> list = new ArrayList<>();
        List<DamInfoRep> damInfoRep = damInfoReps.getList();
        for (DamInfoRep infoRep : damInfoRep) {
            MapDamRep mapDamRep = new MapDamRep();
            mapDamRep.setId(infoRep.getId());
            mapDamRep.setAreaId(infoRep.getAreaCodeName());
            mapDamRep.setName(infoRep.getDamName());
            mapDamRep.setInfo("总面积：" + infoRep.getTotalArea() + "（亩）");
            mapDamRep.setInfo1("主要种植产品：" + infoRep.getPlantName());
            mapDamRep.setPosLon(infoRep.getPosLon());
            mapDamRep.setPosLat(infoRep.getPosLat());
            list.add(mapDamRep);
        }
        MapDamReps mapDamReps = new MapDamReps(list);
        mapDamReps.setId("");
        mapDamReps.setTitle(title);
        responseBase.success().setRetBody(mapDamReps);
        return new ResponseEntity<>(responseBase, HttpStatus.OK);
    }

    /*
     * 获取灾害预警信息
     */
    @RequestMapping(value = "/findWarning", method = RequestMethod.GET)
    public ResponseEntity<ResponseBase> findWarning(@RequestParam(required = false) String id) {
        ResponseBase<MapWarningReps> responseBase = new ResponseBase<>();
        id = StringUtils.isEmpty(id) ? "龙里县" : id;
        AreaDataRep areaDataRep = areaDataService.queryShow(id, "7f81d26c471d48dcaf27e1355096ed4e");
        List<MapWarningRep> list = new ArrayList<>();
        String des = "";
        if (areaDataRep != null) {
            AreaDataDetailRep areaDataDetailRep = areaDataService.queryById(areaDataRep.getId());
            List<Map<String, String>> valList = areaDataDetailRep.getValList();
            MapWarningRep mapWarningRep = new MapWarningRep();
            mapWarningRep.setId(areaDataRep.getId());
            List<String> value = new ArrayList<>();
            value.add(areaDataRep.getPosLon());
            value.add(areaDataRep.getPosLat());
            value.add("2");
            mapWarningRep.setValue(value);
            for (int i = 0; i < valList.size(); i++) {
                if (i == 0) {
                    des = valList.get(i).get("fieldVal2");
                } else {
                    mapWarningRep.setSignaltype(valList.get(i).get("fieldVal2"));
                }
            }
            list.add(mapWarningRep);
        }
        MapWarningReps mapWarningReps = new MapWarningReps(list);
        mapWarningReps.setId("");
        mapWarningReps.setTitle(id + "灾害预警");
        mapWarningReps.setDes(des);
        responseBase.success().setRetBody(mapWarningReps);
        return new ResponseEntity<>(responseBase, HttpStatus.OK);
    }

    /*
     * 获取招商情况信息
     */
    @RequestMapping(value = "/findMerchant", method = RequestMethod.GET)
    public ResponseEntity<ResponseBase> findMerchant(@RequestParam(required = false) String id) {
        ResponseBase<MapMerchantReps> responseBase = new ResponseBase<>();
        String[] areaNameArrays = (StringUtils.isEmpty(id) || "龙里县".equals(id)) ? areaArrays : (new String[]{id});
        id = StringUtils.isEmpty(id) ? "龙里县" : id;
        String[] fieldArrays = {"招商项目数", "合同引资", "到位资金", "组织企业", "总投资"};
        List<MapMerchantRep> list = new ArrayList<>();
        double merchant = 0;
        double contract = 0;
        double money = 0;
        double group = 0;
        double total = 0;
        for (int i = 0; i < areaNameArrays.length; i++) {
            AreaDataRep areaDataRep = areaDataService.queryShow(areaNameArrays[i], "4e04fbce696d4032afa0ecfa45861636");
            if (areaDataRep != null) {
                AreaDataDetailRep areaDataDetailRep = areaDataService.queryById(areaDataRep.getId());
                List<Map<String, String>> valList = areaDataDetailRep.getValList();
                for (Map<String, String> map : valList) {
                    if ("招商项目数".equals(map.get("fieldVal1"))) {
                        merchant = merchant + Double.parseDouble(map.get("fieldVal2"));
                    } else if ("合同引资".equals(map.get("fieldVal1"))) {
                        contract = contract + Double.parseDouble(map.get("fieldVal2"));
                    } else if ("到位资金".equals(map.get("fieldVal1"))) {
                        money = money + Double.parseDouble(map.get("fieldVal2"));
                    } else if ("组织企业".equals(map.get("fieldVal1"))) {
                        group = group + Double.parseDouble(map.get("fieldVal2"));
                    } else if ("总投资".equals(map.get("fieldVal1"))) {
                        total = total + Double.parseDouble(map.get("fieldVal2"));
                    }
                }
            }
        }
        for (int j = 0; j < fieldArrays.length; j++) {
            MapMerchantRep rep = new MapMerchantRep();
            if ("招商项目数".equals(fieldArrays[j])) {
                rep.setName("招商项目(个)");
                rep.setValue(decimalFormat.format(merchant));
            } else if ("合同引资".equals(fieldArrays[j])) {
                rep.setName("合同引资（亿元）");
                rep.setValue(decimalFormat.format(contract));
            } else if ("到位资金".equals(fieldArrays[j])) {
                rep.setName("到位资金（亿元）");
                rep.setValue(decimalFormat.format(money));
            } else if ("组织企业".equals(fieldArrays[j])) {
                rep.setName("组织企业（家）");
                rep.setValue(decimalFormat.format(group));
            } else if ("总投资".equals(fieldArrays[j])) {
                rep.setName("总投资（亿元）");
                rep.setValue(decimalFormat.format(total));
            }
            list.add(rep);
            if ("招商项目数".equals(fieldArrays[j])) {
                MapMerchantRep nullRep = new MapMerchantRep();
                nullRep.setName("");
                nullRep.setValue("");
                list.add(nullRep);
            }
        }
        MapMerchantReps reps = new MapMerchantReps(list);
        reps.setId("");
        reps.setTitle(id + "招商情况信息");
        responseBase.success().setRetBody(reps);
        return new ResponseEntity<>(responseBase, HttpStatus.OK);
    }

    /*
     * 获取专家类型列表
     */
    @RequestMapping(value = "/findExpertType", method = RequestMethod.GET)
    public ResponseEntity<ResponseBase> findExpertType() {
        ResponseBase<MapExpertTypeReps> responseBase = new ResponseBase<>();
        List<MapExpertTypeRep> list = new ArrayList<>();
        DictReps dictReps = dictService.selectVal("ZJLX");
        List<DictRep> dictRepList = dictReps.getList();
        for (DictRep item : dictRepList) {
            MapExpertTypeRep mapExpertTypeRep = new MapExpertTypeRep();
            mapExpertTypeRep.setId(item.getId());
            mapExpertTypeRep.setName(item.getDictValue() + "统计");
            list.add(mapExpertTypeRep);
        }
        MapExpertTypeReps reps = new MapExpertTypeReps(list);
        reps.setId("");
        reps.setList(list);
        responseBase.success().setRetBody(reps);
        return new ResponseEntity<>(responseBase, HttpStatus.OK);
    }

    /*
     * 获取专家统计信息
     */
    @RequestMapping(value = "/findExpertStatis", method = RequestMethod.GET)
    public ResponseEntity<ResponseBase> findExpertStatis(@RequestParam(required = false) String typeId) {
        ResponseBase<MapExpertStatisReps> responseBase = new ResponseBase<>();
        List<MapExpertStatisRep> list = expertInfoMapper.selectStatis(typeId);
        DictRep dictRep = dictService.selectValById(typeId);
        int cnt = 0;
        for (MapExpertStatisRep item : list) {
            cnt = cnt + Integer.parseInt(item.getValue());
        }
        MapExpertStatisReps reps = new MapExpertStatisReps(list);
        reps.setTitle(dictRep.getDictValue() + "统计");
        reps.setSubtitle("专家人数" + cnt + "人");
        responseBase.success().setRetBody(reps);
        return new ResponseEntity<>(responseBase, HttpStatus.OK);
    }

    /*
     * 获取专家列表信息
     */
    @RequestMapping(value = "/findExpertList", method = RequestMethod.GET)
    public ResponseEntity<ResponseBase> findExpertList(@RequestParam(required = false) String typeId,
                                                       @RequestParam(required = false) String id) {
        ResponseBase<MapExpertListReps> responseBase = new ResponseBase<>();
        List<MapExpertListRep> list = expertInfoMapper.selectByTypeId(typeId, id);
        for (MapExpertListRep item : list) {
            item.setImgUrl(envProperties.getComFileDownloadUrl() + item.getImgUrl());
        }
        MapExpertListReps reps = new MapExpertListReps(list);
        reps.setId(id);
        responseBase.success().setRetBody(reps);
        return new ResponseEntity<>(responseBase, HttpStatus.OK);
    }

    /*
     * 获取坝区统计信息
     */
    @RequestMapping(value = "/findDamStatis", method = RequestMethod.GET)
    public ResponseEntity<ResponseBase> findDamStatis(@RequestParam(required = false) String id) {
        ResponseBase<MapDamStatisReps> responseBase = new ResponseBase<>();
        String title = (StringUtils.isEmpty(id) || "龙里县".equals(id)) ? "坝区总体统计" : (id + "坝区统计");
        String[] areaNameArrays = (StringUtils.isEmpty(id) || "龙里县".equals(id)) ? areaArrays : (new String[]{id});
        id = StringUtils.isEmpty(id) ? "" : id;
        String[] fieldArrays = {"总种植面积（亩）", "耕地出产值（元）", "土地流转率", "农民收入贡献率", "产销对接率", "产业参保率"};
        List<MapMerchantRep> list = new ArrayList<>();
        double totalArea = 0;
        double plantVal = 0;
        double landRate = 0;
        double farmerRate = 0;
        double marketRate = 0;
        double insureRate = 0;
        int total = 0;
        for (int i = 0; i < areaNameArrays.length; i++) {
            DamInfoReps damInfoReps = damService.queryAll(1, 200, null, null, "0", areaNameArrays[i]);
            if (damInfoReps != null) {
                List<DamInfoRep> valList = damInfoReps.getList();
                total = total + valList.size();
                for (DamInfoRep map : valList) {
                    String totalAreaStr = StringUtils.isEmpty(map.getTotalArea()) ? "0" : map.getTotalArea();
                    String plantValStr = StringUtils.isEmpty(map.getPlantVal()) ? "0" : map.getPlantVal();
                    String landRateStr = (StringUtils.isEmpty(map.getLandRate()) ? "0" : map.getLandRate()).replaceAll("%", "");
                    String farmerRateStr = (StringUtils.isEmpty(map.getFarmerRate()) ? "0" : map.getFarmerRate()).replaceAll("%", "");
                    String marketRateStr = (StringUtils.isEmpty(map.getMarketRate()) ? "0" : map.getMarketRate()).replaceAll("%", "");
                    String insureRateStr = (StringUtils.isEmpty(map.getInsureRate()) ? "0" : map.getInsureRate()).replaceAll("%", "");

                    totalArea = totalArea + Double.parseDouble(totalAreaStr);
                    plantVal = plantVal + Double.parseDouble(plantValStr);
                    landRate = landRate + Double.parseDouble(landRateStr);
                    farmerRate = farmerRate + Double.parseDouble(farmerRateStr);
                    marketRate = marketRate + Double.parseDouble(marketRateStr);
                    insureRate = insureRate + Double.parseDouble(insureRateStr);
                }
            }
        }
        for (int j = 0; j < fieldArrays.length; j++) {
            MapMerchantRep rep = new MapMerchantRep();
            if ("总种植面积（亩）".equals(fieldArrays[j])) {
                rep.setName(fieldArrays[j]);
                rep.setValue(decimalFormat.format(totalArea));
            } else if ("耕地出产值（元）".equals(fieldArrays[j])) {
                rep.setName(fieldArrays[j]);
                rep.setValue(decimalFormat.format(plantVal));
            } else if ("土地流转率".equals(fieldArrays[j])) {
                rep.setName(fieldArrays[j]);
                String landRateTemp = String.format("%.2f", (landRate / total));
                landRateTemp = decimalFormat.format(landRate == 0 ? 0.0 : Double.parseDouble(landRateTemp));
                rep.setValue(landRateTemp + "%");

            } else if ("农民收入贡献率".equals(fieldArrays[j])) {
                rep.setName(fieldArrays[j]);
                String farmerRateTemp = String.format("%.2f", (farmerRate / total));
                farmerRateTemp = decimalFormat.format(farmerRate == 0 ? 0.0 : Double.parseDouble(farmerRateTemp));
                rep.setValue(farmerRateTemp + "%");
            } else if ("产销对接率".equals(fieldArrays[j])) {
                rep.setName(fieldArrays[j]);
                String marketRateTemp = String.format("%.2f", (marketRate / total));
                marketRateTemp = decimalFormat.format(marketRate == 0 ? 0.0 : Double.parseDouble(marketRateTemp));
                rep.setValue(marketRateTemp + "%");
            } else if ("产业参保率".equals(fieldArrays[j])) {
                rep.setName(fieldArrays[j]);
                String insureRateTemp = String.format("%.2f", (insureRate / total));
                insureRateTemp = decimalFormat.format(insureRate == 0 ? 0.0 : Double.parseDouble(insureRateTemp));
                rep.setValue(insureRateTemp + "%");
            }
            list.add(rep);
        }
        MapDamStatisReps reps = new MapDamStatisReps(list);
        reps.setId(id);
        reps.setTitle(title);
        reps.setTotalName("坝区个数(个)");
        reps.setTotal(total + "");
        responseBase.success().setRetBody(reps);
        return new ResponseEntity<>(responseBase, HttpStatus.OK);
    }

    /*
     * 获取产业发展统计类型列表
     */
    @RequestMapping(value = "/findIndustryType", method = RequestMethod.GET)
    public ResponseEntity<ResponseBase> findIndustryType(@RequestParam(required = false) String id) {
        ResponseBase<MapIndustryReps> responseBase = new ResponseBase<>();
        List<MapIndustryRep> list = new ArrayList<>();
        String[] fieldArrays = {"种植面积|亩", "产量|吨", "产值|元"};
        id = StringUtils.isEmpty(id) ? "龙里县" : id;
        for (int i = 0; i < fieldArrays.length; i++) {
            MapIndustryRep mapIndustryRep = new MapIndustryRep();
            String name = fieldArrays[i].split("\\|")[0];
            String unit = fieldArrays[i].split("\\|")[1];
            mapIndustryRep.setId(i + "");
            mapIndustryRep.setName(name);
            mapIndustryRep.setTitle(id + name + "统计");
            mapIndustryRep.setUnit("单位：" + unit);
            list.add(mapIndustryRep);
        }
        MapIndustryReps reps = new MapIndustryReps(list);
        reps.setId("");
        reps.setList(list);
        responseBase.success().setRetBody(reps);
        return new ResponseEntity<>(responseBase, HttpStatus.OK);
    }

    /*
     * 获取产业发展统计明细信息
     */
    /*@RequestMapping(value = "/findIndustryDetail", method = RequestMethod.GET)
    public ResponseEntity<ResponseBase> findIndustryDetail(@RequestParam(required = false) String id,
                                                           @RequestParam(required = false) String typeId) {
        ResponseBase<MapIndustryDetailReps> responseBase = new ResponseBase<>();
        String typeName = "";
        if ("0".equals(typeId)) {
            typeName = "种植面积";
        } else if ("1".equals(typeId)) {
            typeName = "产量";
        } else if ("2".equals(typeId)) {
            typeName = "产值";
        }
        String[] areaNameArrays = (StringUtils.isEmpty(id) || "龙里县".equals(id)) ? areaArrays : (new String[]{id});
        id = StringUtils.isEmpty(id) ? "龙里县" : id;
        String[] fieldArrays = {"刺梨", "蔬菜", "水果", "中药材", "茶叶", "油茶", "食用菌", "生态畜牧业", "其他产业"};
        List<MapIndustryDetailRep> list = new ArrayList<>();
        //刺梨
        double cnt1 = 0;
        //蔬菜
        double cnt2 = 0;
        //水果
        double cnt3 = 0;
        //中药材
        double cnt4 = 0;
        //茶叶
        double cnt5 = 0;
        //油茶
        double cnt6 = 0;
        //食用菌
        double cnt7 = 0;
        //生态畜牧业
        double cnt8 = 0;
        //其他产业
        double cnt9 = 0;
        for (int i = 0; i < areaNameArrays.length; i++) {
            AreaDataRep areaDataRep = areaDataService.queryShow(areaNameArrays[i], "07889bd64d7841a09de0e5fe9b8ae34e");
            if (areaDataRep != null) {
                AreaDataDetailRep areaDataDetailRep = areaDataService.queryById(areaDataRep.getId());
                List<Map<String, String>> valList = areaDataDetailRep.getValList();
                for (Map<String, String> map : valList) {
                    if ("刺梨".equals(map.get("fieldVal1")) && typeName.equals(map.get("fieldVal2"))) {
                        cnt1 = cnt1 + Double.parseDouble(map.get("fieldVal3"));
                        continue;
                    } else if ("蔬菜".equals(map.get("fieldVal1")) && typeName.equals(map.get("fieldVal2"))) {
                        cnt2 = cnt2 + Double.parseDouble(map.get("fieldVal3"));
                        continue;
                    } else if ("水果".equals(map.get("fieldVal1")) && typeName.equals(map.get("fieldVal2"))) {
                        cnt3 = cnt3 + Double.parseDouble(map.get("fieldVal3"));
                        continue;
                    } else if ("中药材".equals(map.get("fieldVal1")) && typeName.equals(map.get("fieldVal2"))) {
                        cnt4 = cnt4 + Double.parseDouble(map.get("fieldVal3"));
                        continue;
                    } else if ("茶叶".equals(map.get("fieldVal1")) && typeName.equals(map.get("fieldVal2"))) {
                        cnt5 = cnt5 + Double.parseDouble(map.get("fieldVal3"));
                        continue;
                    } else if ("油茶".equals(map.get("fieldVal1")) && typeName.equals(map.get("fieldVal2"))) {
                        cnt6 = cnt6 + Double.parseDouble(map.get("fieldVal3"));
                        continue;
                    } else if ("食用菌".equals(map.get("fieldVal1")) && typeName.equals(map.get("fieldVal2"))) {
                        cnt7 = cnt7 + Double.parseDouble(map.get("fieldVal3"));
                        continue;
                    } else if ("生态畜牧业".equals(map.get("fieldVal1"))) {
                        String newTypeName = "种植面积".equals(typeName) ? "养殖场数" : typeName;
                        if (newTypeName.equals(map.get("fieldVal2"))) {
                            cnt8 = cnt8 + Double.parseDouble(map.get("fieldVal3"));
                            continue;
                        }
                    } else if ("其他产业".equals(map.get("fieldVal1")) && typeName.equals(map.get("fieldVal2"))) {
                        cnt9 = cnt9 + Double.parseDouble(map.get("fieldVal3"));
                        continue;
                    }
                }
            }
        }
        for (int j = 0; j < fieldArrays.length; j++) {
            MapIndustryDetailRep rep = new MapIndustryDetailRep();
            if ("刺梨".equals(fieldArrays[j])) {
                rep.setViewName(fieldArrays[j]);
                rep.setViewVal(decimalFormat.format(cnt1));
                rep.setId(j + "");
            } else if ("蔬菜".equals(fieldArrays[j])) {
                rep.setViewName(fieldArrays[j]);
                rep.setViewVal(decimalFormat.format(cnt2));
                rep.setId(j + "");
            } else if ("水果".equals(fieldArrays[j])) {
                rep.setViewName(fieldArrays[j]);
                rep.setViewVal(decimalFormat.format(cnt3));
                rep.setId(j + "");
            } else if ("中药材".equals(fieldArrays[j])) {
                rep.setViewName(fieldArrays[j]);
                rep.setViewVal(decimalFormat.format(cnt4));
                rep.setId(j + "");
            } else if ("茶叶".equals(fieldArrays[j])) {
                rep.setViewName(fieldArrays[j]);
                rep.setViewVal(decimalFormat.format(cnt5));
                rep.setId(j + "");
            } else if ("油茶".equals(fieldArrays[j])) {
                rep.setViewName(fieldArrays[j]);
                rep.setViewVal(decimalFormat.format(cnt6));
                rep.setId(j + "");
            } else if ("食用菌".equals(fieldArrays[j])) {
                rep.setViewName(fieldArrays[j]);
                rep.setViewVal(decimalFormat.format(cnt7));
                rep.setId(j + "");
            } else if ("生态畜牧业".equals(fieldArrays[j])) {
                rep.setViewName(fieldArrays[j]);
                rep.setViewVal(decimalFormat.format(cnt8));
                rep.setId(j + "");
            } else if ("其他产业".equals(fieldArrays[j])) {
                rep.setViewName(fieldArrays[j]);
                rep.setViewVal(decimalFormat.format(cnt9));
                rep.setId(j + "");
            }
            list.add(rep);
        }
        MapIndustryDetailReps reps = new MapIndustryDetailReps(list);
        reps.setId("");
        responseBase.success().setRetBody(reps);
        return new ResponseEntity<>(responseBase, HttpStatus.OK);
    }*/

    /*
     * 获取产业发展统计明细信息
     */
    @RequestMapping(value = "/findIndustryDetail", method = RequestMethod.GET)
    public ResponseEntity<ResponseBase> findIndustryDetail(@RequestParam(required = false) String id,
                                                           @RequestParam(required = false) String typeId) {
        ResponseBase<MapIndustryDetailReps> responseBase = new ResponseBase<>();
        String typeName = "";
        if ("0".equals(typeId)) {
            typeName = "种植面积";
        } else if ("1".equals(typeId)) {
            typeName = "产量";
        } else if ("2".equals(typeId)) {
            typeName = "产值";
        }
        String[] areaNameArrays = (StringUtils.isEmpty(id) || "龙里县".equals(id)) ? areaArrays : (new String[]{id});
        id = StringUtils.isEmpty(id) ? "龙里县" : id;
        String[] fieldArrays = {"刺梨", "蔬菜", "水果", "中药材", "茶叶", "油茶", "食用菌", "生态畜牧业", "其他产业"};
        List<MapIndustryDetailRep> list = new ArrayList<>();
        //刺梨
        double cnt1 = 0;
        //蔬菜
        double cnt2 = 0;
        //水果
        double cnt3 = 0;
        //中药材
        double cnt4 = 0;
        //茶叶
        double cnt5 = 0;
        //油茶
        double cnt6 = 0;
        //食用菌
        double cnt7 = 0;
        //生态畜牧业
        double cnt8 = 0;
        //其他产业
        double cnt9 = 0;
        for (int i = 0; i < areaNameArrays.length; i++) {
            AreaDataRep areaDataRep = areaDataService.queryShow(areaNameArrays[i], "894bb259e92c439cbeb2177974e27050");
            if (areaDataRep != null) {
                AreaDataDetailRep areaDataDetailRep = areaDataService.queryById(areaDataRep.getId());
                List<Map<String, String>> valList = areaDataDetailRep.getValList();
                for (Map<String, String> map : valList) {
                    if ("刺梨".equals(map.get("fieldVal1")) && typeName.equals(map.get("fieldVal3"))) {
                        cnt1 = cnt1 + Double.parseDouble(map.get("fieldVal4"));
                        continue;
                    } else if ("蔬菜".equals(map.get("fieldVal2")) && typeName.equals(map.get("fieldVal3"))) {
                        cnt2 = cnt2 + Double.parseDouble(map.get("fieldVal4"));
                        continue;
                    } else if ("水果".equals(map.get("fieldVal2")) && (!"刺梨".equals(map.get("fieldVal1"))) && typeName.equals(map.get("fieldVal3"))) {
                        cnt3 = cnt3 + Double.parseDouble(map.get("fieldVal4"));
                        continue;
                    } else if ("中药材".equals(map.get("fieldVal2")) && typeName.equals(map.get("fieldVal3"))) {
                        cnt4 = cnt4 + Double.parseDouble(map.get("fieldVal4"));
                        continue;
                    } else if ("茶叶".equals(map.get("fieldVal2")) && typeName.equals(map.get("fieldVal3"))) {
                        cnt5 = cnt5 + Double.parseDouble(map.get("fieldVal4"));
                        continue;
                    } else if ("油茶".equals(map.get("fieldVal2")) && typeName.equals(map.get("fieldVal3"))) {
                        cnt6 = cnt6 + Double.parseDouble(map.get("fieldVal4"));
                        continue;
                    } else if ("食用菌".equals(map.get("fieldVal2")) && typeName.equals(map.get("fieldVal3"))) {
                        cnt7 = cnt7 + Double.parseDouble(map.get("fieldVal4"));
                        continue;
                    } else if ("生态畜牧业".equals(map.get("fieldVal2"))) {
                        String newTypeName = "种植面积".equals(typeName) ? "养殖场" : typeName;
                        if (newTypeName.equals(map.get("fieldVal3"))) {
                            cnt8 = cnt8 + Double.parseDouble(map.get("fieldVal4"));
                            continue;
                        }
                    } else if ("其他产业".equals(map.get("fieldVal2")) && typeName.equals(map.get("fieldVal3"))) {
                        cnt9 = cnt9 + Double.parseDouble(map.get("fieldVal4"));
                        continue;
                    }
                }
            }
        }
        for (int j = 0; j < fieldArrays.length; j++) {
            MapIndustryDetailRep rep = new MapIndustryDetailRep();
            if ("刺梨".equals(fieldArrays[j])) {
                rep.setViewName(fieldArrays[j]);
                rep.setViewVal(decimalFormat.format(cnt1));
                rep.setId(j + "");
            } else if ("蔬菜".equals(fieldArrays[j])) {
                rep.setViewName(fieldArrays[j]);
                rep.setViewVal(decimalFormat.format(cnt2));
                rep.setId(j + "");
            } else if ("水果".equals(fieldArrays[j])) {
                rep.setViewName(fieldArrays[j]);
                rep.setViewVal(decimalFormat.format(cnt3));
                rep.setId(j + "");
            } else if ("中药材".equals(fieldArrays[j])) {
                rep.setViewName(fieldArrays[j]);
                rep.setViewVal(decimalFormat.format(cnt4));
                rep.setId(j + "");
            } else if ("茶叶".equals(fieldArrays[j])) {
                rep.setViewName(fieldArrays[j]);
                rep.setViewVal(decimalFormat.format(cnt5));
                rep.setId(j + "");
            } else if ("油茶".equals(fieldArrays[j])) {
                rep.setViewName(fieldArrays[j]);
                rep.setViewVal(decimalFormat.format(cnt6));
                rep.setId(j + "");
            } else if ("食用菌".equals(fieldArrays[j])) {
                rep.setViewName(fieldArrays[j]);
                rep.setViewVal(decimalFormat.format(cnt7));
                rep.setId(j + "");
            } else if ("生态畜牧业".equals(fieldArrays[j])) {
                rep.setViewName(fieldArrays[j]);
                rep.setViewVal(decimalFormat.format(cnt8));
                rep.setId(j + "");
            } else if ("其他产业".equals(fieldArrays[j])) {
                rep.setViewName(fieldArrays[j]);
                rep.setViewVal(decimalFormat.format(cnt9));
                rep.setId(j + "");
            }
            list.add(rep);
        }
        MapIndustryDetailReps reps = new MapIndustryDetailReps(list);
        reps.setId("");
        responseBase.success().setRetBody(reps);
        return new ResponseEntity<>(responseBase, HttpStatus.OK);
    }

    /*
     * 获取地理情况信息
     */
    @RequestMapping(value = "/findGeog", method = RequestMethod.GET)
    public ResponseEntity<ResponseBase> findGeog(@RequestParam(required = false) String id) {
        ResponseBase<MapGeogReps> responseBase = new ResponseBase<>();
        id = StringUtils.isEmpty(id) ? "龙里县" : id;
        List<MapMerchantRep> list = new ArrayList<>();
        String desc = "";
        AreaDataRep areaDataRep = areaDataService.queryShow(id, "ef1ec2914a7d434497cdc73e1e172e41");
        if (areaDataRep != null) {
            AreaDataDetailRep areaDataDetailRep = areaDataService.queryById(areaDataRep.getId());
            List<Map<String, String>> valList = areaDataDetailRep.getValList();
            for (Map<String, String> map : valList) {
                MapMerchantRep rep = new MapMerchantRep();
                if ("总人口".equals(map.get("fieldVal1"))) {
                    rep.setName("总人口（人）");
                    rep.setValue(map.get("fieldVal2"));
                } else if ("地理位置".equals(map.get("fieldVal1"))) {
                    rep.setName("地理位置");
                    rep.setValue(map.get("fieldVal2"));
                } else if ("平均海拔".equals(map.get("fieldVal1"))) {
                    rep.setName("平均海拔");
                    rep.setValue(map.get("fieldVal2"));
                } else if ("年平均气温".equals(map.get("fieldVal1"))) {
                    rep.setName("年平均气温（℃）");
                    rep.setValue(map.get("fieldVal2"));
                } else if ("面积".equals(map.get("fieldVal1"))) {
                    rep.setName("面积（平方公里）");
                    rep.setValue(map.get("fieldVal2"));
                } else if ("耕地面积".equals(map.get("fieldVal1"))) {
                    rep.setName("耕地面积（亩）");
                    rep.setValue(map.get("fieldVal2"));
                } else if ("简介".equals(map.get("fieldVal1"))) {
                    desc = map.get("fieldVal2");
                    continue;
                }
                list.add(rep);
            }
        }
        MapGeogReps reps = new MapGeogReps(list);
        reps.setId("");
        reps.setTitle(id);
        reps.setIntroduce(desc);
        responseBase.success().setRetBody(reps);
        return new ResponseEntity<>(responseBase, HttpStatus.OK);
    }

    /*
     * 获取价格类型列表
     */
    @RequestMapping(value = "/findPriceType", method = RequestMethod.GET)
    public ResponseEntity<ResponseBase> findPriceType() {
        ResponseBase<MapExpertTypeReps> responseBase = new ResponseBase<>();
        List<MapExpertTypeRep> list = new ArrayList<>();
        DictReps dictReps = dictService.selectVal("NCPJG");
        List<DictRep> dictRepList = dictReps.getList();
        for (DictRep item : dictRepList) {
            MapExpertTypeRep mapExpertTypeRep = new MapExpertTypeRep();
            mapExpertTypeRep.setId(item.getId());
            mapExpertTypeRep.setName(item.getDictValue());
            list.add(mapExpertTypeRep);
        }
        MapExpertTypeReps reps = new MapExpertTypeReps(list);
        reps.setId("");
        reps.setList(list);
        responseBase.success().setRetBody(reps);
        return new ResponseEntity<>(responseBase, HttpStatus.OK);
    }

    /*
     * 获取价格列表信息
     */
    /*@RequestMapping(value = "/findPriceList", method = RequestMethod.GET)
    public ResponseEntity<ResponseBase> findPriceList(@RequestParam(required = false) String name) {
        ResponseBase<MapPriceReps> responseBase = new ResponseBase<>();
        name = StringUtils.isEmpty(name) ? "双龙农副产品交易中心" : name;
        List<MapPriceHeadRep> headList = new ArrayList<>();
        List<List<MapPriceValueRep>> list = new ArrayList<>();
        AreaDataRep areaDataRep = areaDataService.queryDisplay(name, "dbe4a7c915b44bca80e9fa0c93c4847d");
        if (areaDataRep != null) {
            String[] field = areaDataRep.getDataField().split("\\|");
            for (int i = 0; i < field.length; i++) {
                MapPriceHeadRep headRep = new MapPriceHeadRep();
                headRep.setName(field[i]);
                headList.add(headRep);
            }
            AreaDataDetailRep areaDataDetailRep = areaDataService.queryById(areaDataRep.getId());
            List<Map<String, String>> valList = areaDataDetailRep.getValList();

            for (Map<String, String> map : valList) {
                List<MapPriceValueRep> valueReps = new ArrayList<>();
                MapPriceValueRep rep = new MapPriceValueRep();
                String val1 = map.get("fieldVal1");
                try{
                    val1 = StringUtils.isEmpty(val1) ? val1 : val1.substring(0, 10);
                }catch (Exception e){
                    String dataTime=areaDataRep.getDataTime();
                    val1=dataTime.substring(0,4)+"-"+dataTime.substring(4,6)+"-"+dataTime.substring(6,8);
                }
                rep.setValue(val1);
                valueReps.add(rep);
                rep = new MapPriceValueRep();
                rep.setValue(map.get("fieldVal2"));
                valueReps.add(rep);
                rep = new MapPriceValueRep();
                rep.setValue(map.get("fieldVal3"));
                valueReps.add(rep);
                list.add(valueReps);
            }
        }
        MapPriceReps reps = new MapPriceReps();
        reps.setId("");
        reps.setTabHeader(headList);
        reps.setList(list);
        responseBase.success().setRetBody(reps);
        return new ResponseEntity<>(responseBase, HttpStatus.OK);
    }*/

    /*
     * 获取价格列表信息
     */
    @RequestMapping(value = "/findPriceList", method = RequestMethod.GET)
    public ResponseEntity<ResponseBase> findPriceList(@RequestParam(required = false) String name) {
        ResponseBase<MapPriceReps> responseBase = new ResponseBase<>();
        name = StringUtils.isEmpty(name) ? "双龙农副产品交易中心" : name;
        List<MapPriceHeadRep> headList = new ArrayList<>();
        List<List<MapPriceValueRep>> list = new ArrayList<>();
        AreaDataRep areaDataRep = areaDataService.queryDisplay(name, "dbe4a7c915b44bca80e9fa0c93c4847d");
        if (areaDataRep != null) {
            String displayTime = DateUtils.getCurrentDate();
            String[] field = areaDataRep.getDataField().split("\\|");
            for (int i = 0; i < field.length; i++) {
                MapPriceHeadRep headRep = new MapPriceHeadRep();
                headRep.setName(field[i]);
                headList.add(headRep);
            }
            MapPriceHeadRep priceHeadRep = new MapPriceHeadRep();
            priceHeadRep.setName("升降百分比");
            headList.add(priceHeadRep);
            AreaDataPriceReps areaDataPriceReps = areaDataService.queryPriceRate(1, 500, areaDataRep.getDataTime(), areaDataRep.getDataTime(), null, name, areaDataRep.getId());
            List<AreaDataPriceRep> valList = areaDataPriceReps.getList();

            for (AreaDataPriceRep map : valList) {
                List<MapPriceValueRep> valueReps = new ArrayList<>();
                MapPriceValueRep rep = new MapPriceValueRep();
                String val1 = map.getTradeDate();
                try {
                    val1 = StringUtils.isEmpty(val1) ? val1 : val1.substring(0, 10);
                } catch (Exception e) {
                    String dataTime = areaDataRep.getDataTime();
                    val1 = dataTime.substring(0, 4) + "-" + dataTime.substring(4, 6) + "-" + dataTime.substring(6, 8);
                }
                //rep.setValue(displayTime);
                rep.setValue(val1);
                valueReps.add(rep);
                rep = new MapPriceValueRep();
                rep.setValue(map.getCateName());
                valueReps.add(rep);
                rep = new MapPriceValueRep();
                rep.setValue(map.getPrice());
                valueReps.add(rep);
                String riseRate = map.getRiseRate();
                rep = new MapPriceValueRep();
                double rateSign = Double.parseDouble(riseRate);
                int signFlag = 0;
                if (rateSign == 0.00) {
                    signFlag = 0;
                } else if (rateSign > 0.00) {
                    signFlag = 1;
                } else if (rateSign < 0.00) {
                    signFlag = 2;
                }
                rep.setValue(signFlag + "");
                valueReps.add(rep);
                list.add(valueReps);
            }
        }
        MapPriceReps reps = new MapPriceReps();
        reps.setId("");
        reps.setTabHeader(headList);
        reps.setList(list);
        responseBase.success().setRetBody(reps);
        return new ResponseEntity<>(responseBase, HttpStatus.OK);
    }

    /*
     * 获取收购商类型列表
     */
    @RequestMapping(value = "/findPurchaseType", method = RequestMethod.GET)
    public ResponseEntity<ResponseBase> findPurchaseType() {
        ResponseBase<MapExpertTypeReps> responseBase = new ResponseBase<>();
        List<MapExpertTypeRep> list = new ArrayList<>();
        DictReps dictReps = dictService.selectVal("SGSPM");
        List<DictRep> dictRepList = dictReps.getList();
        for (DictRep item : dictRepList) {
            MapExpertTypeRep mapExpertTypeRep = new MapExpertTypeRep();
            mapExpertTypeRep.setId(item.getId());
            mapExpertTypeRep.setName(item.getDictValue());
            list.add(mapExpertTypeRep);
        }
        MapExpertTypeReps reps = new MapExpertTypeReps(list);
        reps.setId("");
        reps.setList(list);
        responseBase.success().setRetBody(reps);
        return new ResponseEntity<>(responseBase, HttpStatus.OK);
    }

    /*
     * 获取收购商列表信息
     */
    @RequestMapping(value = "/findPurchaseList", method = RequestMethod.GET)
    public ResponseEntity<ResponseBase> findPurchaseList(@RequestParam(required = false) String name) {
        ResponseBase<MapPriceReps> responseBase = new ResponseBase<>();
        name = StringUtils.isEmpty(name) ? "收购商数量排名" : name;
        List<MapPriceHeadRep> headList = new ArrayList<>();
        List<List<MapPriceValueRep>> list = new ArrayList<>();
        AreaDataRep areaDataRep = areaDataService.queryDisplay(name, "56a998b77085432084e3c6e9e5871565");
        if (areaDataRep != null) {
            String[] field = areaDataRep.getDataField().split("\\|");
            for (int i = 0; i < field.length; i++) {
                MapPriceHeadRep headRep = new MapPriceHeadRep();
                headRep.setName(field[i]);
                headList.add(headRep);
            }
            AreaDataDetailRep areaDataDetailRep = areaDataService.queryById(areaDataRep.getId());
            List<Map<String, String>> valList = areaDataDetailRep.getValList();
            ListUtils.sortMapByKey(valList, "fieldVal3", 2);
            for (Map<String, String> map : valList) {
                List<MapPriceValueRep> valueReps = new ArrayList<>();
                MapPriceValueRep rep = new MapPriceValueRep();
                rep.setValue(map.get("fieldVal1"));
                valueReps.add(rep);
                rep = new MapPriceValueRep();
                rep.setValue(map.get("fieldVal2"));
                valueReps.add(rep);
                rep = new MapPriceValueRep();
                rep.setValue(map.get("fieldVal3"));
                valueReps.add(rep);
                list.add(valueReps);
            }
        }
        MapPriceReps reps = new MapPriceReps();
        reps.setId("");
        reps.setTabHeader(headList);
        reps.setList(list);
        responseBase.success().setRetBody(reps);
        return new ResponseEntity<>(responseBase, HttpStatus.OK);
    }

    /*
     * 获取专题列表信息
     */
    @RequestMapping(value = "/findTopicList", method = RequestMethod.GET)
    public ResponseEntity<ResponseBase> findTopicList(@RequestParam(defaultValue = "1") int pageNo,
                                                      @RequestParam(defaultValue = "10") int pageSize) {
        ResponseBase<MapTopicStatisReps> responseBase = new ResponseBase<>();
        PageHelper.startPage(pageNo, pageSize, true);
        List<MapTopicStatisRep> list = topicInfoMapper.selectStatis();
        MapTopicStatisReps reps = new MapTopicStatisReps(new PageInfo<>(list).getTotal(), list);
        responseBase.success().setRetBody(reps);
        return new ResponseEntity<>(responseBase, HttpStatus.OK);
    }

    /*
     * 获取专题图列信息
     */
    @RequestMapping(value = "/findTopicFigure", method = RequestMethod.GET)
    public ResponseEntity<ResponseBase> findTopicFigure(@RequestParam(required = false) String id) {
        ResponseBase<MapTopicFigureReps> responseBase = new ResponseBase<>();
        TopicDetailRep topicDetailRep = topicInfoMapper.selectById(id);
        List<MapTopicFigureRep> list = new ArrayList<>();
        if (topicDetailRep != null) {
            String type = topicDetailRep.getShowType() + "";
            String address = topicDetailRep.getAddress();
            AreaDataRep areaData = areaDataMapper.selectById(topicDetailRep.getSourceId());
            String[] fields = areaData.getDataField().split("\\|");
            int pos = 0;
            for (int i = 0; i < fields.length; i++) {
                if (address.equals(fields[i])) {
                    pos = i;
                    break;
                }
            }
            List<AreaData> areaDatas = areaDataMapper.selectDetailByDataId(areaData.getId());
            List<String> addressName = new ArrayList<>();
            for (AreaData item : areaDatas) {
                String[] dataVal = item.getDataField().split("\\|");
                try {
                    addressName.add(dataVal[pos]);
                } catch (Exception e) {
                    log.info("专题图列源数据解析失败，数据：data|" + item.getDataField());
                }
            }
            HashSet h = new HashSet(addressName);
            addressName.clear();
            addressName.addAll(h);
            for (String name : addressName) {
                int value = 0;
                for (AreaData item : areaDatas) {
                    String[] dataVal = item.getDataField().split("\\|");
                    try {
                        if (name.equals(dataVal[pos])) {
                            value++;
                        }
                    } catch (Exception e) {

                    }
                }
                //获取经纬度start
                String lon = "";
                String lat = "";
                int posProvince = 0;
                int posCity = 0;
                int posCounty = 0;
                for (int i = 0; i < fields.length; i++) {
                    if ("省".equals(fields[i])) {
                        posProvince = i;
                    }
                    if ("市".equals(fields[i])) {
                        posCity = i;
                    }
                    if ("县".equals(fields[i])) {
                        posCounty = i;
                    }
                }
                for (AreaData item : areaDatas) {
                    String[] dataVal = item.getDataField().split("\\|");
                    String porvinceName = null;
                    String cityName = null;
                    String countyName = null;
                    try {
                        if (name.equals(dataVal[pos])) {
                            if ("省".equals(address)) {
                                porvinceName = null;
                                cityName = null;
                                countyName = null;
                            } else if ("市".equals(address)) {
                                porvinceName = dataVal[posProvince].replaceAll("省", "");
                                cityName = null;
                                countyName = null;
                            } else if ("县".equals(address)) {
                                porvinceName = dataVal[posProvince].replaceAll("省", "");
                                cityName = dataVal[posCity].replaceAll("市", "");
                                countyName = null;
                            } else {
                                porvinceName = dataVal[posProvince].replaceAll("省", "");
                                cityName = dataVal[posCity].replaceAll("市", "");
                                countyName = dataVal[posCounty];
                            }
                            String posAliasName = name.replaceAll("省", "").replaceAll("市", "");
                            SysArea sysArea = areaDataMapper.selectPosByName(name, posAliasName, porvinceName, cityName, countyName);
                            lon = sysArea.getPosLon();
                            lat = sysArea.getPosLat();
                            break;
                        }
                    } catch (Exception e) {

                    }
                }
                //获取经纬度end
                MapTopicFigureRep rep = new MapTopicFigureRep();
                rep.setType(type);
                rep.setValue(value + "");
                rep.setLon(lon);
                rep.setLat(lat);
                if (StringUtils.isNotEmpty(lon)) {
                    list.add(rep);
                }
            }
            //处理显示类型为分布点的时候start
            if ("2".equals(type)) {
                List<MapTopicFigureRep> newList = new ArrayList<>();
                for (MapTopicFigureRep item : list) {
                    int count = Integer.parseInt(item.getValue());
                    double newLon = Double.parseDouble(item.getLon());
                    double newLat = Double.parseDouble(item.getLat());
                    double scale = 0.0;
                    if ("省".equals(address)) {
                        scale = 0.5;
                    } else if ("市".equals(address)) {
                        scale = 0.4;
                    } else if ("县".equals(address)) {
                        scale = 0.2;
                    } else {
                        scale = 0.04;
                    }
                    for (int i = 0; i < count; i++) {
                        String lonLat = StringUtils.randomLonLat(newLon - scale, newLon, newLat - scale, newLat, "all");
                        MapTopicFigureRep newRep = new MapTopicFigureRep();
                        newRep.setType(type);
                        newRep.setValue("1");
                        newRep.setLon(lonLat.split("\\|")[0]);
                        newRep.setLat(lonLat.split("\\|")[1]);
                        newList.add(newRep);
                    }
                }
                list.clear();
                list.addAll(newList);
            }
            //处理显示类型为分布点的时候end
        }

        MapTopicFigureReps reps = new MapTopicFigureReps(new PageInfo<>(list).getTotal(), list);
        responseBase.success().setRetBody(reps);
        return new ResponseEntity<>(responseBase, HttpStatus.OK);
    }

    /*
     * 获取专题分布信息
     */
    @RequestMapping(value = "/findTopicDistri", method = RequestMethod.GET)
    public ResponseEntity<ResponseBase> findTopicDistri(@RequestParam(required = false) String id) {
        ResponseBase<MapMerchantReps> responseBase = new ResponseBase<>();
        TopicDetailRep topicDetailRep = topicInfoMapper.selectById(id);
        List<MapMerchantRep> list = new ArrayList<>();
        if (topicDetailRep != null) {
            String address = topicDetailRep.getAddress();
            AreaDataRep areaData = areaDataMapper.selectById(topicDetailRep.getSourceId());
            String[] fields = areaData.getDataField().split("\\|");
            int pos = 0;
            for (int i = 0; i < fields.length; i++) {
                if (address.equals(fields[i])) {
                    pos = i;
                    break;
                }
            }
            List<AreaData> areaDatas = areaDataMapper.selectDetailByDataId(areaData.getId());
            List<String> addressName = new ArrayList<>();
            for (AreaData item : areaDatas) {
                String[] dataVal = item.getDataField().split("\\|");
                try {
                    addressName.add(dataVal[pos]);
                } catch (Exception e) {
                    log.info("专题分布源数据解析失败，数据：data|" + item.getDataField());
                }
            }
            HashSet h = new HashSet(addressName);
            addressName.clear();
            addressName.addAll(h);
            for (String name : addressName) {
                int value = 0;
                for (AreaData item : areaDatas) {
                    String[] dataVal = item.getDataField().split("\\|");
                    try{
                        if (name.equals(dataVal[pos])) {
                            value++;
                        }
                    }catch (Exception e){

                    }
                }
                MapMerchantRep rep = new MapMerchantRep();
                rep.setValue(value + "");
                rep.setName(name);
                list.add(rep);
            }
        }
        MapMerchantReps reps = new MapMerchantReps();
        reps.setId("");
        reps.setList(list);
        reps.setTitle((topicDetailRep == null) ? "" : topicDetailRep.getTopicName() + "统计");
        responseBase.success().setRetBody(reps);
        return new ResponseEntity<>(responseBase, HttpStatus.OK);
    }
}

