package com.hywa.map.controller.notice;

import com.hywa.map.common.log.Log;
import com.hywa.map.config.EnvProperties;
import com.hywa.map.config.interceptor.TokenManager;
import com.hywa.map.config.interceptor.TokenModel;
import com.hywa.map.representation.NoticeRep;
import com.hywa.map.representation.NoticeReps;
import com.hywa.map.representation.NoticeReq;
import com.hywa.map.representation.ResponseBase;
import com.hywa.map.service.notice.NoticeService;
import io.swagger.annotations.Api;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;

import static com.hywa.map.common.ConstantPool.SPLIT;

/**
 * t_web_notice  对应Controller 实现类
 *
 * @author hywa
 * @create by 2019-02-18 14:53:53
 */
@Slf4j
@Api(tags = "新闻通知公告信息")
@RestController
@RequestMapping("/black")
public class BlackController {
    @Autowired
    private NoticeService noticeService;
    @Autowired
    EnvProperties envProperties;

    @Autowired
    TokenManager tokenManager;

    @PostMapping
    @Transactional
    @Log(log = "红黑榜发布管理-新增红黑榜信息")
    public ResponseEntity<ResponseBase> save(@RequestBody NoticeReq reqInfo, HttpServletRequest req) {
        TokenModel accessToken = tokenManager.getToken(req.getHeader("accessToken"));
        String userId = accessToken.getUserId().split(SPLIT)[1];
        try {
            noticeService.saveBlack(reqInfo, userId);
        } catch (Exception e) {
            log.error("新增红黑榜信息异常，请求数据|" + reqInfo + "，异常信息|" + e);
            throw e;
        }
        return new ResponseEntity<>(new ResponseBase().success(), HttpStatus.OK);
    }

    @DeleteMapping
    @Log(log = "红黑榜发布管理-删除红黑榜信息")
    public ResponseEntity<ResponseBase> delete(@RequestBody NoticeReq reqInfo, HttpServletRequest req) {
        TokenModel accessToken = tokenManager.getToken(req.getHeader("accessToken"));
        String userId = accessToken.getUserId().split(SPLIT)[1];
        try {
            noticeService.delete(reqInfo, userId);
        } catch (Exception e) {
            log.error("删除红黑榜信息异常，请求数据|" + reqInfo + "，异常信息|" + e);
            throw e;
        }
        return new ResponseEntity<>(new ResponseBase().success(), HttpStatus.OK);
    }

    @RequestMapping(value = "/findById", method = RequestMethod.GET)
    public ResponseEntity<ResponseBase<NoticeRep>> findById(@RequestParam(required = false) String id) {
        ResponseBase<NoticeRep> responseBase = new ResponseBase<>();
        responseBase.success().setRetBody(noticeService.selectById(id));
        responseBase.success();
        return new ResponseEntity<>(responseBase, HttpStatus.OK);
    }


    @PutMapping
    @Log(log = "红黑榜发布管理-编辑红黑榜信息")
    public ResponseEntity<ResponseBase> update(@RequestBody NoticeReq reqInfo, HttpServletRequest req) {
        TokenModel accessToken = tokenManager.getToken(req.getHeader("accessToken"));
        String userId = accessToken.getUserId().split(SPLIT)[1];
        try {
            noticeService.updateBlack(reqInfo, userId);
        } catch (Exception e) {
            log.error("编辑红黑榜信息异常，请求数据|" + reqInfo + "，异常信息|" + e);
            throw e;
        }
        return new ResponseEntity<>(new ResponseBase().success(), HttpStatus.OK);
    }

    @RequestMapping(value = "/publish", method = RequestMethod.POST)
    @Log(log = "红黑榜发布管理-批量发布红黑榜信息")
    public ResponseEntity<ResponseBase> publish(@RequestBody NoticeReq reqInfo, HttpServletRequest req) {
        TokenModel accessToken = tokenManager.getToken(req.getHeader("accessToken"));
        String userId = accessToken.getUserId().split(SPLIT)[1];
        try {
            noticeService.publish(reqInfo, userId);
        } catch (Exception e) {
            log.error("批量发布红黑榜信息异常，请求数据|" + reqInfo + "，异常信息|" + e);
            throw e;
        }
        return new ResponseEntity<>(new ResponseBase().success(), HttpStatus.OK);
    }
    @RequestMapping(value = "/publishSingle", method = RequestMethod.POST)
    @Log(log = "红黑榜发布管理-发布红黑榜信息")
    public ResponseEntity<ResponseBase> publishSingle(@RequestBody NoticeReq reqInfo, HttpServletRequest req) {
        TokenModel accessToken = tokenManager.getToken(req.getHeader("accessToken"));
        String userId = accessToken.getUserId().split(SPLIT)[1];
        try {
            noticeService.publishSingle(reqInfo, userId);
        } catch (Exception e) {
            log.error("发布红黑榜信息异常，请求数据|" + reqInfo + "，异常信息|" + e);
            throw e;
        }
        return new ResponseEntity<>(new ResponseBase().success(), HttpStatus.OK);
    }
    @RequestMapping(value = "/findAll", method = RequestMethod.GET)
    public ResponseEntity<ResponseBase> findByParam(@RequestParam(defaultValue = "1") int pageNo,
                                                    @RequestParam(defaultValue = "10") int pageSize,
                                                    @RequestParam(required = false) String queryParam,
                                                    @RequestParam(required = false) String beginDate,
                                                    @RequestParam(required = false) String endDate,
                                                    @RequestParam(required = false) String noticeType,
                                                    @RequestParam(required = false) String state) {
        ResponseBase<NoticeReps> responseBase = new ResponseBase<>();
        responseBase.success().setRetBody(noticeService.selectBlackAll(pageNo, pageSize, queryParam,beginDate,endDate,noticeType,state));
        return new ResponseEntity<>(responseBase, HttpStatus.OK);
    }
}

