package com.hywa.map.controller.orchard;

import com.alibaba.fastjson.JSON;
import com.hywa.map.common.log.Log;
import com.hywa.map.common.utils.DateUtils;
import com.hywa.map.common.utils.HttpUtils;
import com.hywa.map.config.EnvProperties;
import com.hywa.map.config.interceptor.TokenManager;
import com.hywa.map.config.interceptor.TokenModel;
import com.hywa.map.representation.*;
import com.hywa.map.service.orchard.OrchardService;
import io.swagger.annotations.Api;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;

import java.util.HashMap;
import java.util.Map;

import static com.hywa.map.common.ConstantPool.SPLIT;

/**
 * t_orchard_info  对应Controller 实现类
 *
 * @author hywa
 * @create by 2019-07-17 13:44:47
 */
@Slf4j
@Api(tags = "园区管理")
@RestController
@RequestMapping("/orchard")
public class OrchardInfoController {
    @Autowired
    TokenManager tokenManager;
    @Autowired
    private OrchardService orchardService;

    @PostMapping
    @Transactional
    @Log(log = "园区管理-新增园区信息")
    public ResponseEntity<ResponseBase> save(@RequestBody OrchardReq reqInfo, HttpServletRequest req) {
        TokenModel accessToken = tokenManager.getToken(req.getHeader("accessToken"));
        String userId = accessToken.getUserId().split(SPLIT)[1];
        OrchardSaveRep orchardSaveRep = new OrchardSaveRep();
        try {
            orchardSaveRep = orchardService.save(reqInfo, userId);
        } catch (Exception e) {
            log.error("新增园区信息异常，请求数据|" + reqInfo + "，异常信息|" + e);
            throw e;
        }
        ResponseBase<OrchardSaveRep> responseBase = new ResponseBase<>();
        responseBase.success().setRetBody(orchardSaveRep);
        return new ResponseEntity<>(responseBase, HttpStatus.OK);
    }

    @PutMapping
    @Log(log = "园区管理-编辑园区信息")
    public ResponseEntity<ResponseBase> update(@RequestBody OrchardReq reqInfo, HttpServletRequest req) {
        TokenModel accessToken = tokenManager.getToken(req.getHeader("accessToken"));
        String userId = accessToken.getUserId().split(SPLIT)[1];
        try {
            orchardService.update(reqInfo, userId);
        } catch (Exception e) {
            log.error("编辑园区信息异常，请求数据|" + reqInfo + "，异常信息|" + e);
            throw e;
        }
        return new ResponseEntity<>(new ResponseBase().success(), HttpStatus.OK);
    }

    @DeleteMapping
    @Log(log = "园区管理-删除园区信息")
    public ResponseEntity<ResponseBase> delete(@RequestBody OrchardReq reqInfo, HttpServletRequest req) {
        TokenModel accessToken = tokenManager.getToken(req.getHeader("accessToken"));
        String userId = accessToken.getUserId().split(SPLIT)[1];
        try {
            orchardService.delete(reqInfo, userId);
        } catch (Exception e) {
            log.error("删除园区信息异常，请求数据|" + reqInfo + "，异常信息|" + e);
            throw e;
        }
        return new ResponseEntity<>(new ResponseBase().success(), HttpStatus.OK);
    }

    @RequestMapping(value = "/findAgriType", method = RequestMethod.GET)
    public ResponseEntity<ResponseBase> findAgriType(@RequestParam(required = false) String id) {
        ResponseBase<ComboxReps> responseBase = new ResponseBase<>();
        responseBase.success().setRetBody(orchardService.selectAgriType());
        return new ResponseEntity<>(responseBase, HttpStatus.OK);
    }

    @RequestMapping(value = "/findProdType", method = RequestMethod.GET)
    public ResponseEntity<ResponseBase> findProdType(@RequestParam(required = false) String id) {
        ResponseBase<ComboxReps> responseBase = new ResponseBase<>();
        responseBase.success().setRetBody(orchardService.selectProdType());
        return new ResponseEntity<>(responseBase, HttpStatus.OK);
    }

    @RequestMapping(value = "/findAll", method = RequestMethod.GET)
    public ResponseEntity<ResponseBase> findByParam(@RequestParam(defaultValue = "1") int pageNo,
                                                    @RequestParam(defaultValue = "10") int pageSize,
                                                    @RequestParam(required = false) String queryParam,
                                                    @RequestParam(required = false) String induType,
                                                    @RequestParam(required = false) String productName) {
        ResponseBase<OrchardReps> responseBase = new ResponseBase<>();
        responseBase.success().setRetBody(orchardService.selectAll(pageNo, pageSize, queryParam, induType, productName));
        return new ResponseEntity<>(responseBase, HttpStatus.OK);
    }

    @RequestMapping(value = "/findById", method = RequestMethod.GET)
    public ResponseEntity<ResponseBase<OrchardRep>> findById(@RequestParam(required = false) String id) {
        ResponseBase<OrchardRep> responseBase = new ResponseBase<>();
        responseBase.success().setRetBody(orchardService.selectById(id));
        responseBase.success();
        return new ResponseEntity<>(responseBase, HttpStatus.OK);
    }
}

