package com.hywa.map.controller.statis;

import com.hywa.map.config.EnvProperties;
import com.hywa.map.config.interceptor.TokenManager;
import com.hywa.map.representation.*;
import com.hywa.map.service.statis.AgriDataService;
import com.hywa.map.service.statis.AgriGroupService;
import io.swagger.annotations.Api;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

/**
 * t_sys_branch  对应Controller 实现类
 *
 * @author hywa
 * @create by 2018-12-12 10:01:56
 */

@Slf4j
@Api(tags = "农业态势统计")
@RestController
@RequestMapping("/statis")
public class AgriStatisController {

    @Autowired
    private AgriGroupService agriGroupService;
    @Autowired
    private AgriDataService agriDataService;
    @Autowired
    EnvProperties envProperties;
    @Autowired
    TokenManager tokenManager;

    @RequestMapping(value = "/findGroupAll", method = RequestMethod.GET)
    public ResponseEntity<ResponseBase> selectGroupAll(@RequestParam(defaultValue = "1") int pageNo,
                                                       @RequestParam(defaultValue = "10") int pageSize) {
        ResponseBase<AgriGroupReps> responseBase = new ResponseBase<>();
        responseBase.success().setRetBody(agriGroupService.selectGroupAll(pageNo, pageSize));
        return new ResponseEntity<>(responseBase, HttpStatus.OK);
    }

    @RequestMapping(value = "/findGroupItem", method = RequestMethod.GET)
    public ResponseEntity<ResponseBase> selectGroupItem(@RequestParam(required = false) String id) {
        ResponseBase<AgriGroupItemReps> responseBase = new ResponseBase<>();
        responseBase.success().setRetBody(agriGroupService.selectGroupItem(id));
        return new ResponseEntity<>(responseBase, HttpStatus.OK);
    }

    @RequestMapping(value = "/findProfile", method = RequestMethod.GET)
    public ResponseEntity<ResponseBase> selectProfile(@RequestParam(required = false) String areaId) {
        ResponseBase<AgriProfileReps> responseBase = new ResponseBase<>();
        responseBase.success().setRetBody(agriGroupService.selectProfile(areaId));
        return new ResponseEntity<>(responseBase, HttpStatus.OK);
    }

    @RequestMapping(value = "/findAgriData", method = RequestMethod.GET)
    public ResponseEntity<ResponseBase> selectAgriData(@RequestParam(required = false) String type,
                                                       @RequestParam(required = false) String areaId) {
        ResponseBase<AgriFisheryRep> responseBase = new ResponseBase<>();
        responseBase.success().setRetBody(agriDataService.selectAgriData(type, areaId));
        return new ResponseEntity<>(responseBase, HttpStatus.OK);
    }

    @RequestMapping(value = "/findTownAgriData", method = RequestMethod.GET)
    public ResponseEntity<ResponseBase> selectTownAgriData(@RequestParam(required = false) String type,
                                                       @RequestParam(required = false) String areaId) {
        ResponseBase<AgriTownDataRep> responseBase = new ResponseBase<>();
        responseBase.success().setRetBody(agriDataService.selectTownAgriData(areaId));
        return new ResponseEntity<>(responseBase, HttpStatus.OK);
    }

    @RequestMapping(value = "/findGroupDetail", method = RequestMethod.GET)
    public ResponseEntity<ResponseBase> findGroupDetail(@RequestParam(required = false) String id,
                                                        @RequestParam(required = false) String areaId) {
        ResponseBase<AgriGroupDetailRep> responseBase = new ResponseBase<>();
        responseBase.success().setRetBody(agriGroupService.selectGroupDetail(id, areaId));
        return new ResponseEntity<>(responseBase, HttpStatus.OK);
    }
}

