package com.hywa.map.controller.statis;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.hywa.map.common.utils.DateUtils;
import com.hywa.map.common.utils.StringUtils;
import com.hywa.map.config.EnvProperties;
import com.hywa.map.config.interceptor.TokenManager;
import com.hywa.map.dao.entity.OrchardProduct;
import com.hywa.map.representation.*;
import com.hywa.map.service.orchard.OrchardService;
import com.hywa.map.service.statis.AgriDataService;
import io.swagger.annotations.Api;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

/**
 * t_sys_branch  对应Controller 实现类
 *
 * @author hywa
 * @create by 2018-12-12 10:01:56
 */

@Slf4j
@Api(tags = "园区态势统计")
@RestController
@RequestMapping("/statis")
public class OrchardStatisController {

    @Autowired
    private OrchardService orchardService;
    @Autowired
    private AgriDataService agriDataService;
    @Autowired
    EnvProperties envProperties;
    @Autowired
    TokenManager tokenManager;

    @RequestMapping(value = "/findOrchardAll", method = RequestMethod.GET)
    public ResponseEntity<ResponseBase> findOrchardAll(@RequestParam(defaultValue = "1") int pageNo,
                                                       @RequestParam(defaultValue = "10") int pageSize,
                                                       @RequestParam(required = false) String queryParam,
                                                       @RequestParam(required = false) String induType,
                                                       @RequestParam(required = false) String productName) {
        ResponseBase<OrchardReps> responseBase = new ResponseBase<>();
        OrchardReps orchardReps = orchardService.selectAll(pageNo, pageSize, queryParam, induType, productName);
        List<OrchardRep> orchardRep = orchardReps.getList();
        for (OrchardRep item : orchardRep) {
            item.setProductName("产品：" + item.getProductName().replaceAll(",", "、"));
            item.setOrchardArea("规模：" + item.getOrchardArea() + "m²");
        }
        responseBase.success().setRetBody(orchardReps);
        return new ResponseEntity<>(responseBase, HttpStatus.OK);
    }

    @RequestMapping(value = "/findGroupData", method = RequestMethod.GET)
    public ResponseEntity<ResponseBase> selectGroupData(@RequestParam(required = false) String id) {
        ResponseBase<AgriFisheryRep> responseBase = new ResponseBase<>();
        responseBase.success().setRetBody(agriDataService.selectGroupData(id));
        return new ResponseEntity<>(responseBase, HttpStatus.OK);
    }

    @RequestMapping(value = "/findOrchardById", method = RequestMethod.GET)
    public ResponseEntity<ResponseBase<OrchardViewRep>> findById(@RequestParam(required = false) String id) {
        ResponseBase<OrchardViewRep> responseBase = new ResponseBase<>();
        OrchardRep orchardRep = orchardService.selectById(id);
        JSONObject jsonObj = (JSONObject) JSON.toJSON(orchardRep);
        OrchardViewRep orchardViewRep = JSON.toJavaObject(jsonObj, OrchardViewRep.class);
        orchardViewRep.setOrchardNameTitle("园区名称");
        orchardViewRep.setOrchardCodeTitle("园区编码");
        orchardViewRep.setOrchardAreaTitle("园区规模(平方米)");
        orchardViewRep.setInduTypeNameTitle("行业类型");
        orchardViewRep.setWorkerCountTitle("人员数(人)");
        orchardViewRep.setBeginDateTitle("建园日期：");
        orchardViewRep.setBeginDate(DateUtils.formatChineseDate(orchardViewRep.getBeginDate()));
        orchardViewRep.setLeaderNameTitle("负责人：");
        orchardViewRep.setTelNoTitle("联系方式：");
        orchardViewRep.setAddressTitle("地址：");
        String provinceName = StringUtils.isEmpty(orchardViewRep.getProvinceName()) ? "" : orchardViewRep.getProvinceName();
        String cityName = StringUtils.isEmpty(orchardViewRep.getCityName()) ? "" : orchardViewRep.getCityName();
        String countyName = StringUtils.isEmpty(orchardViewRep.getCountyName()) ? "" : orchardViewRep.getCountyName();
        String townName = StringUtils.isEmpty(orchardViewRep.getTownName()) ? "" : orchardViewRep.getTownName();
        String villageName = StringUtils.isEmpty(orchardViewRep.getVillageName()) ? "" : orchardViewRep.getVillageName();
        orchardViewRep.setAddress(provinceName + cityName + countyName + townName + villageName + orchardViewRep.getAddress());
        orchardViewRep.setElevationTitle("海拔：");
        orchardViewRep.setOrchardDesc("园区简介："+orchardViewRep.getOrchardDesc());
        List<OrchardProduct> products = orchardViewRep.getProducts();
        for (OrchardProduct item : products) {
            item.setProdName("产出物："+item.getProdName());
            item.setProdArea("产出面积："+item.getProdArea()+"亩");
            item.setProdVol("预估产量："+item.getProdVol()+"kg");
        }
        orchardViewRep.setLongitudeTitle("经度");
        orchardViewRep.setLatitudeTitle("纬度");
        responseBase.success().setRetBody(orchardViewRep);
        responseBase.success();
        return new ResponseEntity<>(responseBase, HttpStatus.OK);
    }

    @RequestMapping(value = "/findOrchardMonitor", method = RequestMethod.GET)
    public ResponseEntity<ResponseBase> findOrchardMonitor(@RequestParam(required = false) String id) {
        ResponseBase<ComboxReps> responseBase = new ResponseBase<>();
        responseBase.success().setRetBody(orchardService.selectOrchardMonitor());
        return new ResponseEntity<>(responseBase, HttpStatus.OK);
    }

    @RequestMapping(value = "/findOrchardMonitorDetail", method = RequestMethod.GET)
    public ResponseEntity<ResponseBase> findOrchardMonitorDetail(@RequestParam(required = false) String orchardId,
                                                                 @RequestParam(required = false) String monitorId) {
        ResponseBase<CommonViewReps> responseBase = new ResponseBase<>();
        responseBase.success().setRetBody(orchardService.selectOrchardMonitorDetail(orchardId, monitorId));
        return new ResponseEntity<>(responseBase, HttpStatus.OK);
    }

    @RequestMapping(value = "/findOrchardWeather", method = RequestMethod.GET)
    public ResponseEntity<ResponseBase> findOrchardWeather(@RequestParam(required = false) String orchardId,
                                                                 @RequestParam(required = false) String monitorId) {
        ResponseBase<TempHourRep> responseBase = new ResponseBase<>();
        responseBase.success().setRetBody(orchardService.selectOrchardWeather(orchardId, monitorId));
        return new ResponseEntity<>(responseBase, HttpStatus.OK);
    }
}

