package com.hywa.map.controller.topic;

import com.hywa.map.common.exception.GlobalException;
import com.hywa.map.common.log.Log;
import com.hywa.map.common.utils.FileUtils;
import com.hywa.map.config.EnvProperties;
import com.hywa.map.config.interceptor.TokenManager;
import com.hywa.map.config.interceptor.TokenModel;
import com.hywa.map.representation.*;
import com.hywa.map.service.topic.TopicInfoService;
import io.swagger.annotations.Api;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;

import static com.hywa.map.common.ConstantPool.SPLIT;
import static com.hywa.map.common.enums.CommonEnum.ILLEGAL_PARAMETER_ERROR;

/**
 * t_topic_info  对应Controller 实现类
 *
 * @author hywa
 * @create by 2019-05-10 13:50:25
 */
@Slf4j
@Api(tags = "专题管理")
@RestController
@RequestMapping("/topic")
public class TopicInfoController {

    @Autowired
    private TopicInfoService topicInfoService;
    @Autowired
    EnvProperties envProperties;
    @Autowired
    TokenManager tokenManager;

    @PostMapping
    @Transactional
    @Log(log = "专题管理-新增专题基本信息")
    public ResponseEntity<ResponseBase> save(@RequestBody TopicInfoReq reqInfo, HttpServletRequest req) {
        TokenModel accessToken = tokenManager.getToken(req.getHeader("accessToken"));
        String userId = accessToken.getUserId().split(SPLIT)[1];
        TopicSaveRep topicSaveRep = new TopicSaveRep();
        try {
            topicSaveRep = topicInfoService.saveTopicInfo(reqInfo, userId);
        } catch (Exception e) {
            log.error("新增专题基本信息异常，请求数据|" + reqInfo + "，异常信息|" + e);
            throw e;
        }
        ResponseBase<TopicSaveRep> responseBase = new ResponseBase<>();
        responseBase.success().setRetBody(topicSaveRep);
        return new ResponseEntity<>(responseBase, HttpStatus.OK);
    }

    @PutMapping
    @Log(log = "专题管理-编辑专题基本信息")
    public ResponseEntity<ResponseBase> update(@RequestBody TopicInfoReq reqInfo, HttpServletRequest req) {
        TokenModel accessToken = tokenManager.getToken(req.getHeader("accessToken"));
        String userId = accessToken.getUserId().split(SPLIT)[1];
        try {
            topicInfoService.updateTopicInfo(reqInfo, userId);
        } catch (Exception e) {
            log.error("编辑专题基本信息异常，请求数据|" + reqInfo + "，异常信息|" + e);
            throw e;
        }
        return new ResponseEntity<>(new ResponseBase().success(), HttpStatus.OK);
    }

    @DeleteMapping
    @Log(log = "专题管理-删除专题基本信息")
    public ResponseEntity<ResponseBase> delete(@RequestBody TopicInfoReq reqInfo, HttpServletRequest req) {
        TokenModel accessToken = tokenManager.getToken(req.getHeader("accessToken"));
        String userId = accessToken.getUserId().split(SPLIT)[1];
        try {
            topicInfoService.deleteTopicInfo(reqInfo, userId);
        } catch (Exception e) {
            log.error("删除专题基本信息异常，请求数据|" + reqInfo + "，异常信息|" + e);
            throw e;
        }
        return new ResponseEntity<>(new ResponseBase().success(), HttpStatus.OK);
    }

    @RequestMapping(value = "/findTopicInfoById", method = RequestMethod.GET)
    public ResponseEntity<ResponseBase> queryTopicInfoById(@RequestParam(required = false) String id) {
        ResponseBase<TopicDetailRep> responseBase = new ResponseBase<>();
        responseBase.success().setRetBody(topicInfoService.selectTopicInfoById(id));
        return new ResponseEntity<>(responseBase, HttpStatus.OK);
    }

    @RequestMapping(value = "/findTopicInfo", method = RequestMethod.GET)
    public ResponseEntity<ResponseBase> queryTopicInfo(@RequestParam(required = false) String topicName,
                                                       @RequestParam(defaultValue = "1") int pageNo,
                                                       @RequestParam(defaultValue = "10") int pageSize) {
        ResponseBase<TopicInfoReps> responseBase = new ResponseBase<>();
        responseBase.success().setRetBody(topicInfoService.selectTopicInfo(pageNo, pageSize, topicName));
        return new ResponseEntity<>(responseBase, HttpStatus.OK);
    }

    @RequestMapping(value = "/findTopicTemplate", method = RequestMethod.GET)
    public ResponseEntity<ResponseBase> queryTopicTemplate() {
        ResponseBase<TopicTemplateReps> responseBase = new ResponseBase<>();
        responseBase.success().setRetBody(topicInfoService.selectTopicTemplate());
        return new ResponseEntity<>(responseBase, HttpStatus.OK);
    }

    @RequestMapping(value = "/findTopicModel", method = RequestMethod.GET)
    public ResponseEntity<ResponseBase> queryTopicModel(@RequestParam(required = false) String id) {
        ResponseBase<ViewTemplateReps> responseBase = new ResponseBase<>();
        responseBase.success().setRetBody(topicInfoService.selectTopicModel(id));
        return new ResponseEntity<>(responseBase, HttpStatus.OK);
    }

    @RequestMapping(value = "/findTopicDb", method = RequestMethod.GET)
    public ResponseEntity<ResponseBase> queryTopicDb(@RequestParam(defaultValue = "1") int pageNo,
                                                     @RequestParam(defaultValue = "10") int pageSize) {
        ResponseBase<TopicDbReps> responseBase = new ResponseBase<>();
        responseBase.success().setRetBody(topicInfoService.selectDb(pageNo, pageSize));
        return new ResponseEntity<>(responseBase, HttpStatus.OK);
    }

    @RequestMapping(value = "/findTopicDbItem", method = RequestMethod.GET)
    public ResponseEntity<ResponseBase> queryTopicDbItem(@RequestParam(defaultValue = "1") int pageNo,
                                                         @RequestParam(defaultValue = "10") int pageSize,
                                                         @RequestParam(required = false) String id) {
        ResponseBase<TopicDbItemReps> responseBase = new ResponseBase<>();
        responseBase.success().setRetBody(topicInfoService.selectDbItem(pageNo, pageSize, id));
        return new ResponseEntity<>(responseBase, HttpStatus.OK);
    }

    @RequestMapping(value = "/findTopicDbField", method = RequestMethod.GET)
    public ResponseEntity<ResponseBase> queryTopicDbField(@RequestParam(required = false) String id) {
        ResponseBase<TopicDbFieldReps> responseBase = new ResponseBase<>();
        responseBase.success().setRetBody(topicInfoService.selectDbField(id));
        return new ResponseEntity<>(responseBase, HttpStatus.OK);
    }

    @RequestMapping(value = "/findTopicTemplateField", method = RequestMethod.GET)
    public ResponseEntity<ResponseBase> queryTopicTemplateField(@RequestParam(required = false) String id) {
        ResponseBase<TopicTemplateFieldReps> responseBase = new ResponseBase<>();
        responseBase.success().setRetBody(topicInfoService.selectTemplateField(id));
        return new ResponseEntity<>(responseBase, HttpStatus.OK);
    }

}

