package com.hywa.map.controller.tracing;

import com.hywa.map.config.EnvProperties;
import com.hywa.map.config.interceptor.TokenManager;
import com.hywa.map.representation.*;
import com.hywa.map.service.tracing.TracingService;
import io.swagger.annotations.Api;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;


@Slf4j
@Api(tags = "可追溯系统")
@RestController
@RequestMapping("tracing")
public class TracingController {

    @Autowired
    EnvProperties envProperties;

    @Autowired
    TokenManager tokenManager;

    @Autowired
    private TracingService tracingService;

    /*
     * 获取果树分类
     */
    @RequestMapping(value = "/getFruitType", method = RequestMethod.GET)
    public ResponseEntity<ResponseBase> getFruitType() {
        ResponseBase<SysDictReps> responseBase = new ResponseBase<>();
        responseBase.success().setRetBody(tracingService.getFruitType());
        return new ResponseEntity<>(responseBase, HttpStatus.OK);
    }
    /*
     * 获取果树分类下的果树列表
     */
    @RequestMapping(value = "/getFruitTree", method = RequestMethod.GET)
    public ResponseEntity<ResponseBase> getFruitTree(@RequestParam(required = false) String id) {
        ResponseBase<SysDictReps> responseBase = new ResponseBase<>();
        responseBase.success().setRetBody(tracingService.getFruitTree(id));
        return new ResponseEntity<>(responseBase, HttpStatus.OK);
    }
    /*
     * 获取农事类型
     */
    @RequestMapping(value = "/getFarmType", method = RequestMethod.GET)
    public ResponseEntity<ResponseBase> getFarmType() {
        ResponseBase<SysDictReps> responseBase = new ResponseBase<>();
        responseBase.success().setRetBody(tracingService.getFarmType());
        return new ResponseEntity<>(responseBase, HttpStatus.OK);
    }
    /*
     * 获取种植方式
     */
    @RequestMapping(value = "/getPlantingType", method = RequestMethod.GET)
    public ResponseEntity<ResponseBase> getPlantingType() {
        ResponseBase<SysDictReps> responseBase = new ResponseBase<>();
        responseBase.success().setRetBody(tracingService.getPlantingType());
        return new ResponseEntity<>(responseBase, HttpStatus.OK);
    }
    /*
     * 获取包装规格
     */
    @RequestMapping(value = "/getPackageType", method = RequestMethod.GET)
    public ResponseEntity<ResponseBase> getPackageType() {
        ResponseBase<SysDictReps> responseBase = new ResponseBase<>();
        responseBase.success().setRetBody(tracingService.getPackageType());
        return new ResponseEntity<>(responseBase, HttpStatus.OK);
    }
    /*
     * 获取计量单位
     */
    @RequestMapping(value = "/getDosageUnit", method = RequestMethod.GET)
    public ResponseEntity<ResponseBase> getDosageUnit() {
        ResponseBase<SysDictReps> responseBase = new ResponseBase<>();
        responseBase.success().setRetBody(tracingService.getDosageUnit());
        return new ResponseEntity<>(responseBase, HttpStatus.OK);
    }
    /*
     * 获取灾害类型
     */
    @RequestMapping(value = "/getDisasterType", method = RequestMethod.GET)
    public ResponseEntity<ResponseBase> getDisasterType() {
        ResponseBase<SysDictReps> responseBase = new ResponseBase<>();
        responseBase.success().setRetBody(tracingService.getDisasterType());
        return new ResponseEntity<>(responseBase, HttpStatus.OK);
    }
}


