package com.hywa.map.controller.web;

import com.hywa.map.config.EnvProperties;
import com.hywa.map.config.interceptor.TokenManager;
import com.hywa.map.representation.*;
import com.hywa.map.service.notice.NoticeService;
import io.swagger.annotations.Api;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

/**
 * t_mon_farmer  对应Controller 实现类
 *
 * @author hywa
 * @create by 2019-02-22 09:08:36
 */
@Slf4j
@Api(tags = "门户网站")
@RestController
@RequestMapping("/web")
public class WebController {
    @Autowired
    private NoticeService noticeService;
    @Autowired
    EnvProperties envProperties;

    @Autowired
    TokenManager tokenManager;
    @RequestMapping(value = "/notice/findAll", method = RequestMethod.GET)
    public ResponseEntity<ResponseBase> findNoticeAll(@RequestParam(defaultValue = "1") int pageNo,
                                                    @RequestParam(defaultValue = "10") int pageSize,
                                                    @RequestParam(required = false) String queryParam,
                                                    @RequestParam(required = false) String beginDate,
                                                    @RequestParam(required = false) String endDate,
                                                    @RequestParam(required = false) String noticeType,
                                                    @RequestParam(required = false) String state) {
        ResponseBase<NoticeReps> responseBase = new ResponseBase<>();
        responseBase.success().setRetBody(noticeService.selectAll(pageNo, pageSize, queryParam, beginDate, endDate, noticeType, state));
        return new ResponseEntity<>(responseBase, HttpStatus.OK);
    }

    @RequestMapping(value = "/notice/findById", method = RequestMethod.GET)
    public ResponseEntity<ResponseBase<NoticeRep>> findNoticeById(@RequestParam(required = false) String id) {
        ResponseBase<NoticeRep> responseBase = new ResponseBase<>();
        responseBase.success().setRetBody(noticeService.selectById(id));
        responseBase.success();
        return new ResponseEntity<>(responseBase, HttpStatus.OK);
    }

    @RequestMapping(value = "/black/findAll", method = RequestMethod.GET)
    public ResponseEntity<ResponseBase> findBlackAll(@RequestParam(defaultValue = "1") int pageNo,
                                                    @RequestParam(defaultValue = "10") int pageSize,
                                                    @RequestParam(required = false) String queryParam,
                                                    @RequestParam(required = false) String beginDate,
                                                    @RequestParam(required = false) String endDate,
                                                    @RequestParam(required = false) String noticeType,
                                                    @RequestParam(required = false) String state) {
        ResponseBase<NoticeReps> responseBase = new ResponseBase<>();
        responseBase.success().setRetBody(noticeService.selectBlackAll(pageNo, pageSize, queryParam,beginDate,endDate,noticeType,state));
        return new ResponseEntity<>(responseBase, HttpStatus.OK);
    }

    @RequestMapping(value = "/black/findById", method = RequestMethod.GET)
    public ResponseEntity<ResponseBase<NoticeRep>> findBlackById(@RequestParam(required = false) String id) {
        ResponseBase<NoticeRep> responseBase = new ResponseBase<>();
        responseBase.success().setRetBody(noticeService.selectById(id));
        responseBase.success();
        return new ResponseEntity<>(responseBase, HttpStatus.OK);
    }
}

