package com.hywa.map.representation;

import com.hywa.map.common.enums.CommonEnum;
import com.hywa.map.dao.entity.User;

import java.io.Serializable;
import java.util.List;
import javax.validation.constraints.NotNull;

import lombok.ToString;

@ToString
public class UserRep implements Serializable {

    private static final long serialVersionUID = -4963266899668807475L;

    private String userId;

    @NotNull(message = "用户名不能为空")
    private String name;

    private String phone;

    private String sex;

    private String jobTitle;

    //@NotNull(message = "请绑定区域")
    private String regionId;

    private String workUnit;

    //最小限制1,你写10
    private Integer age;

    private String pwd;

    private String accessToken;

    private String realName;//用户姓名
    private String status;//状态 0-正常 1-停用
    private String userType;//用户类型 00-省管理员 10-市管理员 20-县管理员 01-省操作员 11-市操作员 21-县操作员
    private String remark;//备注
    private String roleName;//角色
    private String areaName;//区域
    private String loginTime;//最后登录时间
    private List<SysRep> sysReps;

    private boolean informationSet;
    private String branchId;//机构编码
    private String branchName;//机构名称
    public UserRep() {
    }

    public UserRep(String userId, String name, String phone,
                   Short sex, String jobTitle, String workUnit,
                   Integer age, String regionId,String realName,
                   String roleName,String areaName) {
        this.setUserId(userId);
        this.setName(name);
        this.setPhone(phone);
        this.setSex(sex == null ? null : CommonEnum.getSex(sex));
        this.setJobTitle(jobTitle);
        this.setWorkUnit(workUnit);
        this.setAge(age);
        this.setRegionId(regionId);
        this.setRealName(realName);
        this.setRoleName(roleName);
        this.setAreaName(areaName);
    }

    public UserRep(User user) {
        this.setUserId(user.getId());
        this.setName(user.getName());
        this.setPhone(user.getTelephone());
        this.setSex(CommonEnum.getSex(user.getSex()));
        this.setJobTitle(user.getJob());
        this.setWorkUnit(user.getWorkUnit());
        this.setAge(user.getAge());
        this.setRealName(user.getRealName());
        this.setRoleName(user.getRoleName());
        this.setAreaName(user.getAreaName());
        this.setLoginTime(user.getLoginTime());
        this.setStatus(user.getStatus());
        this.setBranchId(user.getBranchId());
        this.setBranchName(user.getBranchName());
    }

    public UserRep(User user, String regionId) {
        this(user);
        this.setRegionId(regionId);
    }

    public List<SysRep> getSysReps() {
        return sysReps;
    }

    public void setSysReps(List<SysRep> sysReps) {
        this.sysReps = sysReps;
    }

    public String getRegionId() {
        return regionId;
    }

    public void setRegionId(String regionId) {
        this.regionId = regionId;
    }

    public String getPwd() {
        return pwd;
    }

    public void setPwd(String pwd) {
        this.pwd = pwd;
    }

    public Integer getAge() {
        return age;
    }

    public void setAge(Integer age) {
        this.age = age;
    }

    public String getWorkUnit() {
        return workUnit;
    }

    public void setWorkUnit(String workUnit) {
        this.workUnit = workUnit;
    }

    public String getUserId() {
        return userId;
    }

    public void setUserId(String userId) {
        this.userId = userId;
    }

    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    public String getPhone() {
        return phone;
    }

    public void setPhone(String phone) {
        this.phone = phone;
    }

    public String getSex() {
        return sex;
    }

    public void setSex(String sex) {
        this.sex = sex;
    }

    public String getJobTitle() {
        return jobTitle;
    }

    public void setJobTitle(String jobTitle) {
        this.jobTitle = jobTitle;
    }

    public void setAccessToken(String token) {
        this.accessToken = token;
    }

    public String getAccessToken() {
        return accessToken;
    }

    public String getRealName() {
        return realName;
    }

    public void setRealName(String realName) {
        this.realName = realName;
    }

    public String getStatus() {
        return status;
    }

    public void setStatus(String status) {
        this.status = status;
    }

    public String getUserType() {
        return userType;
    }

    public void setUserType(String userType) {
        this.userType = userType;
    }

    public String getRemark() {
        return remark;
    }

    public void setRemark(String remark) {
        this.remark = remark;
    }

    public String getRoleName() {
        return roleName;
    }

    public void setRoleName(String roleName) {
        this.roleName = roleName;
    }

    public String getAreaName() {
        return areaName;
    }

    public void setAreaName(String areaName) {
        this.areaName = areaName;
    }

    public String getLoginTime() {
        return loginTime;
    }

    public void setLoginTime(String loginTime) {
        this.loginTime = loginTime;
    }

    public boolean isInformationSet() {
        return informationSet;
    }

    public void setInformationSet(boolean informationSet) {
        this.informationSet = informationSet;
    }

    public String getBranchId() {
        return branchId;
    }

    public void setBranchId(String branchId) {
        this.branchId = branchId;
    }

    public String getBranchName() {
        return branchName;
    }

    public void setBranchName(String branchName) {
        this.branchName = branchName;
    }
}


