package com.hywa.map.service.branch.impl;

import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.hywa.map.common.ConstantPool;
import com.hywa.map.common.exception.GlobalException;
import com.hywa.map.common.utils.*;
import com.hywa.map.config.EnvProperties;
import com.hywa.map.dao.entity.Branch;
import com.hywa.map.dao.mapper.BranchMapper;
import com.hywa.map.representation.*;
import com.hywa.map.service.branch.BranchService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import static com.hywa.map.common.enums.CommonEnum.*;

/**
 * branch  对应ServiceImpl 实现类
 *
 * @author hywa
 * @create by 2019-02-14 09:13:52
 */
@Slf4j
@Service
public class BranchServiceImpl implements BranchService {

    @Autowired
    private BranchMapper branchMapper;
    @Autowired
    EnvProperties envProperties;

    /*
     * 保存信息
     */
    @Override
    public int save(BranchReq reqInfo, String userId) {
        String menuName = "组织机构-新增组织机构信息";
        if (StringUtils.isEmpty(reqInfo.getBranchName())) {
            throw new GlobalException(ILLEGAL_PARAMETER_ERROR.getIndex(), ILLEGAL_PARAMETER_ERROR.getValue() + "，机构名称不能为空", null, menuName);
        }
        if (branchMapper.countByName(reqInfo.getBranchName(), null) > 0) {
            throw new GlobalException(BRANCH_NAME_FOUND.getIndex(), BRANCH_NAME_FOUND.getValue(), null, menuName);
        }
        Branch entity = packEntity(reqInfo, userId);
        int count = branchMapper.add(entity);
        log.info("新增组织机构信息完成，请求数据|" + reqInfo + "，操作结果|" + count);
        return count;
    }

    @Override
    public int update(BranchReq reqInfo, String userId) {
        String menuName = "组织机构-编辑组织机构信息";
        if (StringUtils.isEmpty(reqInfo.getId())) {
            throw new GlobalException(ILLEGAL_PARAMETER_ERROR.getIndex(), ILLEGAL_PARAMETER_ERROR.getValue() + "，机构id不能为空", null, menuName);
        }
        if (branchMapper.countByName(reqInfo.getBranchName(), reqInfo.getId()) > 0) {
            throw new GlobalException(BRANCH_NAME_FOUND.getIndex(), BRANCH_NAME_FOUND.getValue(), null, menuName);
        }
        Branch entity = new Branch();
        entity.setId(reqInfo.getId());
        entity.setBranchName(reqInfo.getBranchName());
        entity.setLeader(reqInfo.getLeader());
        entity.setTelNo(reqInfo.getTelNo());
        entity.setRemark(reqInfo.getRemark());
        entity.setUpdateDate(new Date());
        entity.setUpdateUser(userId);
        int count = branchMapper.update(entity);
        log.info("编辑组织机构信息完成，请求数据|" + reqInfo + "，操作结果|" + count);
        return count;
    }

    @Override
    public int delete(BranchReq reqInfo, String userId) {
        String menuName = "组织机构-删除组织机构信息";
        if (StringUtils.isEmpty(reqInfo.getId())) {
            throw new GlobalException(ILLEGAL_PARAMETER_ERROR.getIndex(), ILLEGAL_PARAMETER_ERROR.getValue() + "，机构id不能为空", null, menuName);
        }
        if (branchMapper.countById(reqInfo.getId()) > 0) {
            throw new GlobalException(BRANCH_USED.getIndex(), BRANCH_USED.getValue(), null, menuName);
        }
        if (branchMapper.countUserById(reqInfo.getId()) > 0) {
            throw new GlobalException(BRANCH_USER_USED.getIndex(), BRANCH_USER_USED.getValue(), null, menuName);
        }
        int count = branchMapper.put(reqInfo.getId(), userId);
        log.info("删除组织机构信息完成，请求数据|" + reqInfo + "，操作结果|" + count);
        return count;
    }

    @Override
    public BranchReps selectAll() {
        List<Branch> list = branchMapper.selectAll();
        List<BranchRep> branchReps = new ArrayList<>();
        branchReps = getTreeRep(list, "#");
        log.info("查询组织机构信息完成，请求数据|" + "");
        return new BranchReps((long) branchReps.size(), branchReps);
    }

    @Override
    public BranchRep selectById(String id) {
        String menuName = "组织机构-查询组织机构信息";
        if (StringUtils.isEmpty(id)) {
            throw new GlobalException(ILLEGAL_PARAMETER_ERROR.getIndex(), ILLEGAL_PARAMETER_ERROR.getValue() + "，机构id不能为空", null, menuName);
        }
        Branch branch = branchMapper.selectById(id);
        if (branch == null) {
            return null;
        }
        BranchRep branchRep = new BranchRep(branch);
        return branchRep;
    }

    @Override
    public BranchReps selectByParam(int pageNo, int pageSize, String queryParam) {
        PageHelper.startPage(pageNo, pageSize, true);
        List<Branch> list = branchMapper.select(queryParam);
        List<BranchRep> branchRepList = new ArrayList<>();
        for (Branch branch : list) {
            BranchRep branchRep = new BranchRep(branch);
            branchRepList.add(branchRep);
        }
        log.info("查询组织机构信息完成，请求数据|" + queryParam);
        return new BranchReps(new PageInfo<>(list).getTotal(), branchRepList);
    }

    /*
     * 统一组装实体
     */
    public Branch packEntity(BranchReq branchReq, String userId) {
        Branch entity = new Branch();
        entity.setId(UUIDUtils.randomUUID());
        entity.setBranchName(branchReq.getBranchName());
        entity.setParentId(StringUtils.isEmpty(branchReq.getParentId()) ? "#" : branchReq.getParentId());
        entity.setLeader(branchReq.getLeader());
        entity.setTelNo(branchReq.getTelNo());
        entity.setRemark(branchReq.getRemark());
        entity.setIsDel(ConstantPool.NOT_DEL);
        entity.setCreateDate(new Date());
        entity.setCreateUser(userId);
        return entity;
    }

    /*
     * 获取子节点
     */
    public List<BranchRep> getTreeRep(List<Branch> list, String parentId) {
        List<BranchRep> branchReps = new ArrayList<>();
        for (Branch branch : list) {
            if (parentId.equals(branch.getParentId())) {
                BranchRep branchRep = new BranchRep(branch);
                List<BranchRep> allBranchRep;
                allBranchRep = getTreeRep(list, branch.getId());
                branchRep.setChildren(allBranchRep);
                branchReps.add(branchRep);
            }
        }
        return branchReps;
    }

}

