package com.hywa.map.service.common.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.hywa.map.common.enums.WeatherConts;
import com.hywa.map.common.exception.GlobalException;
import com.hywa.map.common.utils.*;
import com.hywa.map.dao.mapper.WeatherMapper;
import com.hywa.map.representation.*;
import com.hywa.map.service.common.WeatherService;
import com.hywa.map.service.orchard.impl.OrchardServiceImpl;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.util.*;

import static com.hywa.map.common.enums.CommonEnum.ILLEGAL_PARAMETER_ERROR;

/**
 * dap_weather  对应ServiceImpl 实现类
 *
 * @author
 * @create by 2018-08-06 15:02:49
 */
@Service
public class WeatherServiceImpl implements WeatherService {

    @Autowired
    private WeatherMapper weatherMapper;
    @Value("${lbs.amap.com.dev.key}")
    private String lbsDevKey;
    @Value("${lbs.amap.com.dev.url}")
    private String lbsDevUrl;
    private static Log log = LogFactory.getLog(WeatherServiceImpl.class);

    @Override
    public WeatherForecast getWeather(String cityCode) {
        String menuName = "态势统计-查询天气信息";
        if (StringUtils.isEmpty(cityCode)) {
            throw new GlobalException(ILLEGAL_PARAMETER_ERROR.getIndex(), ILLEGAL_PARAMETER_ERROR.getValue() + "，城市不能为空", null, menuName);
        }
        WeatherForecast weatherForecast = new WeatherForecast();
        TodayWeather todayWeather = new TodayWeather();
        List<CastWeather> castWeatherList = new ArrayList<CastWeather>();
        String currDate = DateUtils.getCurrentDate();
        String adCode = null;
        //1、获取高德区域代码
        adCode = cityCode;//目前高德城市代码和本地城市代码相同
        //2、发送高德接口获取天气预报信息
        String weatherInfo = sendWeatherHttp(adCode, "all");
        //3、解析天气预报信息
        if (!StringUtils.isEmpty(weatherInfo)) {
            JSONObject weatherObj = JSON.parseObject(weatherInfo);
            JSONArray weatherArray = JSON.parseArray(weatherObj.getString("forecasts"));
            for (Iterator iterator = weatherArray.iterator(); iterator.hasNext(); ) {
                JSONObject forecastObj = (JSONObject) iterator.next();
                JSONArray castArray = JSON.parseArray(forecastObj.getString("casts"));
                for (Iterator it = castArray.iterator(); it.hasNext(); ) {
                    JSONObject castObj = (JSONObject) it.next();
                    if (currDate.equals(castObj.getString("date"))) {
                        todayWeather.setCityName(forecastObj.getString("city"));
                        todayWeather.setCurrDate(DateUtils.getMDChina(currDate));
                        todayWeather.setWeek(DateUtils.getWeek(currDate));
                        todayWeather.setLuanr("农历" + LunarCalendar.getMDLunar(currDate));
                        todayWeather.setMaxTemp(castObj.getString("daytemp"));
                        todayWeather.setMinTemp(castObj.getString("nighttemp"));
                        //todayWeather.setTemperature();
                        //todayWeather.setWeather(castObj.getString("dayweather"));
                    } else {
                        CastWeather castWeather = new CastWeather();
                        castWeather.setCastDate(DateUtils.getMDChina(castObj.getString("date")));
                        castWeather.setCastWeek(DateUtils.getWeek(castObj.getString("date")));
                        castWeather.setCastMaxTemp(castObj.getString("daytemp"));
                        castWeather.setCastMinTemp(castObj.getString("nighttemp"));
                        String dayWeather = castObj.getString("dayweather");
                        dayWeather = WeatherConts.weatherMap.get(dayWeather);
                        castWeather.setCastWeaDesc(dayWeather == null ? "多云" : dayWeather);
                        castWeather.setCastWindDire(castObj.getString("daywind") + "风");
                        castWeatherList.add(castWeather);
                    }
                }
                break;
            }
        }

        //4、查询数据库获取当天天气信息
        try {
            Map<String, String> selectMap = new HashMap<String, String>();
            selectMap.put("adCode", cityCode);
            selectMap.put("beginDate", currDate + " 00:00:00");
            selectMap.put("endDate", currDate + " 23:59:59");
            List<Map<String, Object>> wList = weatherMapper.selectCurrWeather(selectMap);
            Iterator<Map<String, Object>> wIt = wList.iterator();
            while (wIt.hasNext()) {
                Map<String, Object> wMap = wIt.next();
                String weather = (String) wMap.get("weather");
                int wTemp = (int) wMap.get("temperature");
                //todayWeather.setTemperature(wTemp + "°");
                todayWeather.setTemperature(wTemp + "");
                weather = WeatherConts.weatherMap.get(weather);
                todayWeather.setWeather(weather == null ? "多云" : weather);
                String windDire = (String) wMap.get("wind_dire");
                String windPower = (String) wMap.get("wind_power");
                windPower = WeatherConts.windpowerMap.get(windPower);
                todayWeather.setWindDirePower(windDire + "风" + (windPower == null ? "轻风" : windPower));
                break;
            }
        } catch (Exception e) {
            log.error("查询天气信息，城市：" + cityCode + "|获取当天天气信息异常：" + e);
        }
        //5、处理最高温度 最低温度 实时温度的关系
        /*String maxTemp = todayWeather.getMaxTemp().replaceAll("°", "");
        String minTemp = todayWeather.getMinTemp().replaceAll("°", "");
        String temperature = todayWeather.getTemperature().replaceAll("°", "");
        int temper = Integer.parseInt(temperature);
        if (temper > Integer.parseInt(maxTemp)) {
            todayWeather.setMaxTemp(temperature);
        }
        if (temper < Integer.parseInt(minTemp)) {
            todayWeather.setMinTemp(temperature);
        }*/
        //6、返回天气实体
        weatherForecast.setCityCode(cityCode);
        weatherForecast.setTodayWeather(todayWeather);
        weatherForecast.setCastWeather(castWeatherList);
        log.info("查询天气信息，城市：" + cityCode + "|查询结果：" + weatherForecast);
        return weatherForecast;
    }

    @Override
    public TempHour getHourWeather(String cityCode) {
        String menuName = "态势统计-查询24小时天气信息";
        if (StringUtils.isEmpty(cityCode)) {
            throw new GlobalException(ILLEGAL_PARAMETER_ERROR.getIndex(), ILLEGAL_PARAMETER_ERROR.getValue() + "，城市不能为空", null, menuName);
        }
        TempHour tempHour = new TempHour();
        try {
            double[] tempH = new double[24];//温度小时
            //获取24小时天气信息 start
            String currDate = DateUtils.getCurrentDate();
            String preDate = DateUtils.getDiffDay(-1);
            Map<String, String> selectMap = new HashMap<String, String>();
            selectMap.put("adCode", cityCode);
            selectMap.put("beginDate", preDate + " 00:00:00");
            selectMap.put("endDate", currDate + " 23:59:59");
            List<Map<String, Object>> wList = weatherMapper.selectCurrWeather(selectMap);
            Iterator<Map<String, Object>> wIt = wList.iterator();
            while (wIt.hasNext()) {
                Map<String, Object> wMap = wIt.next();
                int temp = (int) wMap.get("temperature");
                //int humidity = (int) wMap.get("humidity");
                Date createDate = (Date) wMap.get("create_date");
                int hour = DateUtils.getHour(createDate);
                tempH[hour] = temp;//温度小时
            }
            //获取24小时天气信息 start
            List<Environ> tempHourList = new ArrayList<Environ>();//温度小时
            OrchardServiceImpl orchardServiceImpl = new OrchardServiceImpl();
            orchardServiceImpl.genEnvironH(tempHourList, tempH);
            tempHour.setTempH(tempHourList);
            tempHour.setMaxValue(StringUtils.getMax(tempH));
            tempHour.setMinValue(StringUtils.getMin(tempH));
        } catch (Exception e) {
            log.error("查询最近24小时天气信息，城市：" + cityCode + "|获取当天天气信息异常：" + e);
        }
        log.info("查询最近24小时天气信息，城市：" + cityCode + "|返回结果：" + tempHour.toString().length());
        return tempHour;
    }

    /**
     * 发送高德http接口获取天气信息
     *
     * @param city       高德区域编码
     * @param extensions 气象类型 base:返回实况天气 all:返回预报天气
     * @return 天气详情
     */
    public String sendWeatherHttp(String city, String extensions) {
        Map<String, String> queryParas = new HashMap<String, String>();
        queryParas.put("city", city);
        queryParas.put("key", lbsDevKey);
        queryParas.put("extensions", extensions);
        String weatherInfo = null;
        try {
            weatherInfo = HttpUtils.get(lbsDevUrl, queryParas);
            //weatherInfo = HttpClientUtils.get(lbsDevUrl, queryParas);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return weatherInfo;
    }
}

