package com.hywa.map.service.csv.impl;

import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.hywa.map.common.ConstantPool;
import com.hywa.map.common.exception.GlobalException;
import com.hywa.map.common.utils.PinYinUtil;
import com.hywa.map.common.utils.StringUtils;
import com.hywa.map.common.utils.TextFileUtils;
import com.hywa.map.common.utils.UUIDUtils;
import com.hywa.map.config.EnvProperties;
import com.hywa.map.dao.entity.CsvFile;
import com.hywa.map.dao.mapper.CsvMapper;
import com.hywa.map.representation.CsvReps;
import com.hywa.map.service.csv.CsvService;
import com.opencsv.CSVReader;
import com.opencsv.CSVReaderBuilder;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.InputStreamReader;
import java.util.*;

import static com.hywa.map.common.enums.CommonEnum.ILLEGAL_PARAMETER_ERROR;

@Slf4j
@Service
@Transactional
public class CsvServiceImpl implements CsvService {

    @Autowired
    private CsvMapper csvMapper;
    @Autowired
    EnvProperties envProperties;

    @Override
    public CsvReps findAll(int pageNo, int pageSize, String tableName) {
        PageHelper.startPage(pageNo, pageSize, true);
        String menuName = "CSV数据文件-查询数据信息";
        if (StringUtils.isEmpty(tableName) || StringUtils.checkSql(tableName)) {
            throw new GlobalException(ILLEGAL_PARAMETER_ERROR.getIndex(), ILLEGAL_PARAMETER_ERROR.getValue() + "，表名为空或非法", null, menuName);
        }
        //刷新csv数据表
        String flushSql = "flush table " + tableName;
        csvMapper.createTable(flushSql);
        List<Map<String, String>> repList = csvMapper.selectByTableName(tableName);
        if (repList == null) {
            return new CsvReps();
        }
        for (Map<String, String> map : repList) {
            Set<String> keySet = map.keySet();
            for (String key : keySet) {
                String value = map.get(key);
                map.put(key, value.replaceAll("this is null", "").replaceAll("##comma##", ",").replaceAll("##n##", "\r\n"));
            }
        }
        List<Map<String, String>> field = csvMapper.selectFieldByTableName(tableName);
        CsvReps csvReps = new CsvReps(new PageInfo<>(repList).getTotal(), repList);
        csvReps.setFieldDesc(field);
        return csvReps;
    }

    @Override
    public String doCsvFile(String filePath, String fileName, String userId) {
        //1、读取csv数据
        ArrayList<String> list = new ArrayList<>();
        try (CSVReader csvReader = new CSVReaderBuilder(new BufferedReader(new InputStreamReader(new FileInputStream(new File(filePath + fileName)), "gbk"))).build()) {
            Iterator<String[]> iterator = csvReader.iterator();
            while (iterator.hasNext()) {
                String newLine = "";
                String[] line = iterator.next();
                for (int i = 0; i < line.length; i++) {
                    line[i] = StringUtils.isEmpty(line[i]) ? "this is null" : line[i];
                    newLine = newLine + "," + line[i].replaceAll(",", "##comma##").replaceAll("\n", "##n##");
                }
                list.add(newLine.substring(1));
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        //2、保存csv文件与csv数据表的对应关系
        String oldFileName = fileName;
        fileName = fileName.substring(0, fileName.indexOf("."));
        int randomInt = new Random().nextInt(9999);
        String tableName = "t_csv_" + PinYinUtil.toFirstChar(fileName) + "_" + randomInt;
        CsvFile csvFile = csvMapper.selectFile(null, oldFileName);
        String csvId = null;
        if (csvFile == null) {
            CsvFile entity = packCsvFileEntity(oldFileName, tableName, userId);
            csvMapper.addFile(entity);
            csvId = entity.getId();
        } else {
            csvMapper.updateFile(csvFile.getId(), userId);
            tableName = csvFile.getTableName();
            String dropSql = "DROP TABLE " + tableName;
            csvMapper.createTable(dropSql);
            csvId = csvFile.getId();
        }
        //3、创建csv数据表
        String ddlSql = "CREATE TABLE `" + tableName + "` (";
        String column = list.get(0);
        String[] columnArray = column.split("\\,");
        for (int i = 0; i < columnArray.length; i++) {
            ddlSql = ddlSql + "`col" + (i + 1) + "` text NOT NULL COMMENT '" + columnArray[i] + "',";
        }
        ddlSql = ddlSql.substring(0, ddlSql.length() - 1) + ") ENGINE=CSV  DEFAULT CHARSET=utf8 COMMENT='" + fileName + "'";
        csvMapper.createTable(ddlSql);
        //4、释放csv数据表
        String flushSql = "flush table " + tableName;
        csvMapper.createTable(flushSql);
        //5、保存csv数据文件到数据库
        list.remove(0);
        TextFileUtils.writeFile(list, envProperties.getMysqlDataFilePath() + tableName + ".CSV", false);
        return csvId;
    }

    /*
     * 统一组装实体
     */
    public CsvFile packCsvFileEntity(String fileName, String tableName, String userId) {
        CsvFile entity = new CsvFile();
        entity.setId(UUIDUtils.randomUUID());
        entity.setFileName(fileName);
        entity.setTableName(tableName);
        entity.setIsDel(ConstantPool.NOT_DEL);
        entity.setCreateDate(new Date());
        entity.setCreateUser(userId);
        return entity;
    }
}
