package com.hywa.map.service.csv.impl;

import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.hywa.map.common.ConstantPool;
import com.hywa.map.common.exception.GlobalException;
import com.hywa.map.common.utils.*;
import com.hywa.map.config.EnvProperties;
import com.hywa.map.dao.entity.CsvFile;
import com.hywa.map.dao.entity.DbCsv;
import com.hywa.map.dao.entity.DbLib;
import com.hywa.map.dao.entity.DbSource;
import com.hywa.map.dao.mapper.CsvMapper;
import com.hywa.map.dao.mapper.DbSourceMapper;
import com.hywa.map.representation.CsvReps;
import com.hywa.map.representation.DbRep;
import com.hywa.map.representation.DbReps;
import com.hywa.map.representation.DbReq;
import com.hywa.map.service.csv.DbService;
import com.opencsv.CSVReader;
import com.opencsv.CSVReaderBuilder;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.InputStreamReader;
import java.util.*;

import static com.hywa.map.common.enums.CommonEnum.*;
import static com.hywa.map.common.enums.CommonEnum.BRANCH_USER_USED;

@Slf4j
@Service
@Transactional
public class DbServiceImpl implements DbService {

    @Autowired
    private DbSourceMapper dbSourceMapper;
    @Autowired
    EnvProperties envProperties;

    @Override
    public int save(DbReq reqInfo, String userId) {
        //1.新增数据源
        String menuName = "我的数据源-新增数据源信息";
        String sourceType = reqInfo.getSourceType();
        if (StringUtils.isEmpty(reqInfo.getSourceName())) {
            throw new GlobalException(ILLEGAL_PARAMETER_ERROR.getIndex(), ILLEGAL_PARAMETER_ERROR.getValue() + "，数据名称不能为空", null, menuName);
        }
        if (StringUtils.isEmpty(sourceType)) {
            throw new GlobalException(ILLEGAL_PARAMETER_ERROR.getIndex(), ILLEGAL_PARAMETER_ERROR.getValue() + "，数据类型不能为空", null, menuName);
        }
        if (dbSourceMapper.countByName(reqInfo.getSourceName(), null) > 0) {
            throw new GlobalException(DB_SOURCE_NAME_FOUND.getIndex(), DB_SOURCE_NAME_FOUND.getValue(), null, menuName);
        }
        DbSource sourceEntity = packSourceEntity(reqInfo, userId);
        int count = dbSourceMapper.addSource(sourceEntity);
        //2.新增数据库或csv 0-csv 1-数据库
        reqInfo.setId(sourceEntity.getId());
        if ("0".equals(sourceType)) {
            DbCsv csvEntity = packCsvEntity(reqInfo, userId);
            dbSourceMapper.addCsv(csvEntity);
        } else if ("1".equals(sourceType)) {
            DbLib libEntity = packLibEntity(reqInfo, userId);
            dbSourceMapper.addLib(libEntity);
        } else {

        }
        log.info("新增数据源信息完成，请求数据|" + reqInfo + "，操作结果|" + count);
        return count;
    }

    @Override
    public int update(DbReq reqInfo, String userId) {
        //1.编辑数据源
        String menuName = "我的数据源-编辑数据源信息";
        String sourceType = reqInfo.getSourceType();
        if (StringUtils.isEmpty(reqInfo.getId())) {
            throw new GlobalException(ILLEGAL_PARAMETER_ERROR.getIndex(), ILLEGAL_PARAMETER_ERROR.getValue() + "，id不能为空", null, menuName);
        }
        if (StringUtils.isEmpty(reqInfo.getSourceName())) {
            throw new GlobalException(ILLEGAL_PARAMETER_ERROR.getIndex(), ILLEGAL_PARAMETER_ERROR.getValue() + "，数据名称不能为空", null, menuName);
        }
        if (StringUtils.isEmpty(sourceType)) {
            throw new GlobalException(ILLEGAL_PARAMETER_ERROR.getIndex(), ILLEGAL_PARAMETER_ERROR.getValue() + "，数据类型不能为空", null, menuName);
        }
        if (dbSourceMapper.countByName(reqInfo.getSourceName(), reqInfo.getId()) > 0) {
            throw new GlobalException(DB_SOURCE_NAME_FOUND.getIndex(), DB_SOURCE_NAME_FOUND.getValue(), null, menuName);
        }
        DbSource sourceEntity = packSourceEntity(reqInfo, userId);
        sourceEntity.setId(reqInfo.getId());
        sourceEntity.setUpdateDate(new Date());
        sourceEntity.setUpdateUser(userId);
        int count = dbSourceMapper.updateSource(sourceEntity);
        //2.编辑数据库或csv 0-csv 1-数据库
        if ("0".equals(sourceType)) {
            DbCsv csvEntity = packCsvEntity(reqInfo, userId);
            dbSourceMapper.updateCsv(csvEntity);
        } else if ("1".equals(sourceType)) {
            DbLib libEntity = packLibEntity(reqInfo, userId);
            dbSourceMapper.updateLib(libEntity);
        } else {

        }
        log.info("编辑数据源信息完成，请求数据|" + reqInfo + "，操作结果|" + count);
        return count;
    }

    @Override
    public int delete(DbReq reqInfo, String userId) {
        String menuName = "我的数据源-删除数据源信息";
        if (StringUtils.isEmpty(reqInfo.getId())) {
            throw new GlobalException(ILLEGAL_PARAMETER_ERROR.getIndex(), ILLEGAL_PARAMETER_ERROR.getValue() + "，id不能为空", null, menuName);
        }
        if (dbSourceMapper.countRefById(reqInfo.getId()) > 0) {
            throw new GlobalException(DB_SOURCE_USED.getIndex(), DB_SOURCE_USED.getValue(), null, menuName);
        }
        int count = dbSourceMapper.put(reqInfo.getId(), userId);
        log.info("删除数据源信息完成，请求数据|" + reqInfo + "，操作结果|" + count);
        return count;
    }

    @Override
    public DbReps selectAll(int pageNo, int pageSize, String sourceType, String sourceName) {
        PageHelper.startPage(pageNo, pageSize, true);
        List<DbRep> list = dbSourceMapper.select(sourceType, sourceName);
        log.info("查询数据源列表信息完成，请求数据：pageNo|" + pageNo + "，pageSize|" + pageSize + "，sourceType|" + sourceType + "，sourceName|" + sourceName);
        return new DbReps(new PageInfo<>(list).getTotal(), list);
    }

    @Override
    public DbRep selectById(String id) {
        String menuName = "我的数据源-根据id查询数据源信息";
        if (StringUtils.isEmpty(id)) {
            throw new GlobalException(ILLEGAL_PARAMETER_ERROR.getIndex(), ILLEGAL_PARAMETER_ERROR.getValue() + "，id不能为空", null, menuName);
        }
        DbRep dbRep = dbSourceMapper.selectById(id);
        if (dbRep == null) {
            return null;
        }
        //附件
        if ("0".equals(dbRep.getSourceType()) && StringUtils.isNotEmpty(dbRep.getCsvName())) {
            String csvUrl = envProperties.getLocalHttp() + envProperties.getCsvDownloadUrl() + dbRep.getCsvName();
            dbRep.setCsvUrl(csvUrl);
        } else {
            String key = envProperties.getThreeDesKey();
            dbRep.setLibIp(ThreeDESUtil.decode3Des(key, dbRep.getLibIp()));
            dbRep.setUserName(ThreeDESUtil.decode3Des(key, dbRep.getUserName()));
            dbRep.setUserPwd(ThreeDESUtil.decode3Des(key, dbRep.getUserPwd()));
            dbRep.setLibPort(ThreeDESUtil.decode3Des(key, dbRep.getLibPort()));
            dbRep.setLibName(ThreeDESUtil.decode3Des(key, dbRep.getLibName()));
        }
        return dbRep;
    }

    /*
     * 统一组装数据源实体
     */
    public DbSource packSourceEntity(DbReq reqInfo, String userId) {
        DbSource entity = new DbSource();
        entity.setId(UUIDUtils.randomUUID());
        entity.setSourceName(reqInfo.getSourceName());
        entity.setSourceType(reqInfo.getSourceType());
        entity.setRemark(reqInfo.getRemark());
        entity.setIsRef(0);
        entity.setIsDel(ConstantPool.NOT_DEL);
        entity.setCreateDate(new Date());
        entity.setCreateUser(userId);
        return entity;
    }

    /*
     * 统一组装数据库实体
     */
    public DbLib packLibEntity(DbReq reqInfo, String userId) {
        String key = envProperties.getThreeDesKey();
        DbLib entity = new DbLib();
        entity.setId(UUIDUtils.randomUUID());
        entity.setSourceId(reqInfo.getId());
        entity.setLibIp(ThreeDESUtil.encode3Des(key, reqInfo.getLibIp()));
        entity.setUserName(ThreeDESUtil.encode3Des(key, reqInfo.getUserName()));
        entity.setUserPwd(ThreeDESUtil.encode3Des(key, reqInfo.getUserPwd()));
        entity.setLibPort(ThreeDESUtil.encode3Des(key, reqInfo.getLibPort()));
        entity.setLibName(ThreeDESUtil.encode3Des(key, reqInfo.getLibName()));
        return entity;
    }

    /*
     * 统一组装csv文件实体
     */
    public DbCsv packCsvEntity(DbReq reqInfo, String userId) {
        DbCsv entity = new DbCsv();
        entity.setId(UUIDUtils.randomUUID());
        entity.setSourceId(reqInfo.getId());
        entity.setCsvId(reqInfo.getCsvId());
        return entity;
    }

}
