package com.hywa.map.service.depart.impl;

import com.hywa.map.common.ConstantPool;
import com.hywa.map.common.exception.GlobalException;
import com.hywa.map.common.utils.StringUtils;
import com.hywa.map.common.utils.UUIDUtils;
import com.hywa.map.config.EnvProperties;
import com.hywa.map.dao.entity.*;
import com.hywa.map.dao.mapper.DepartMapper;
import com.hywa.map.representation.*;
import com.hywa.map.service.depart.DepartService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import static com.hywa.map.common.enums.CommonEnum.ILLEGAL_PARAMETER_ERROR;

/**
 * t_sys_depart  对应ServiceImpl 实现类
 *
 * @author hywa
 * @create by 2019-02-14 15:50:31
 */
@Slf4j
@Service
public class DepartServiceImpl implements DepartService {

    @Autowired
    private DepartMapper departMapper;
    @Autowired
    EnvProperties envProperties;

    /*
     * 保存信息
     */
    @Override
    public int save(DepartReq reqInfo, String userId) {
        String menuName = "单位基本信息-保存单位基本信息";
        if (StringUtils.isEmpty(reqInfo.getObjName())) {
            throw new GlobalException(ILLEGAL_PARAMETER_ERROR.getIndex(), ILLEGAL_PARAMETER_ERROR.getValue() + "，单位名称不能为空", null, menuName);
        }
        Depart entity = packEntity(reqInfo, userId);
        departMapper.delete(null);
        int count = departMapper.add(entity);
        List<Contact> contacts = new ArrayList<>();
        List<ContactReq> contactReqs = reqInfo.getContact();
        if (contactReqs != null) {
            contactReqs.forEach(item -> {
                contacts.add(new Contact(entity.getId(), item.getContactName(), item.getContactTel(),0));
            });
            departMapper.deleteContact(null);
            departMapper.insertContact(contacts);
        }
        log.info("保存单位基本信息完成，请求数据|" + reqInfo + "，操作结果|" + count);
        return count;
    }

    @Override
    public DepartRep select() {
        Depart depart = departMapper.select();
        if (depart == null) {
            return null;
            //return new DepartRep();
        }
        DepartRep departRep = new DepartRep(depart);
        List<ContactReq> contact = new ArrayList<>();
        List<Contact> list = departMapper.selectContact(depart.getId());
        for (Contact item : list) {
            ContactReq contactReq = new ContactReq();
            contactReq.setContactName(item.getContactName());
            contactReq.setContactTel(item.getContactTel());
            contact.add(contactReq);
        }
        departRep.setContact(contact);
        log.info("查询单位基本信息完成，请求数据|" + "");
        return departRep;
    }

    @Override
    public ProvinceReps selectProvince() {
        List<Province> provinces = departMapper.selectProvince();
        return new ProvinceReps(provinces.size(), provinces);
    }

    @Override
    public CityReps selectCity(String provinceId) {
        if(StringUtils.isEmpty(provinceId)){
            return null;
        }
        List<City> citys = departMapper.selectCity(provinceId);
        return new CityReps(citys.size(), citys);
    }

    @Override
    public CountyReps selectCounty(String cityId) {
        if(StringUtils.isEmpty(cityId)){
            return null;
        }
        List<County> countys = departMapper.selectCounty(cityId);
        return new CountyReps(countys.size(), countys);
    }

    @Override
    public TownReps selectTown(String countyId) {
        if(StringUtils.isEmpty(countyId)){
            return null;
        }
        List<Town> towns = departMapper.selectTown(countyId);
        return new TownReps(towns.size(), towns);
    }

    @Override
    public VillageReps selectVillage(String townId) {
        if(StringUtils.isEmpty(townId)){
            return null;
        }
        List<Village> towns = departMapper.selectVillage(townId);
        return new VillageReps(towns.size(), towns);
    }

    /*
     * 统一组装实体
     */
    public Depart packEntity(DepartReq reqInfo, String userId) {
        Depart entity = new Depart();
        entity.setId(UUIDUtils.randomUUID());
        entity.setObjName(reqInfo.getObjName());
        entity.setLevel(reqInfo.getLevel());
        entity.setProvinceId(reqInfo.getProvinceId());
        entity.setCityId(reqInfo.getCityId());
        entity.setCountyId(reqInfo.getCountyId());
        entity.setTownId(reqInfo.getTownId());
        entity.setIsDel(ConstantPool.NOT_DEL);
        entity.setCreateDate(new Date());
        entity.setCreateUser(userId);
        return entity;
    }

}

