package com.hywa.map.service.dict.impl;

import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.hywa.map.common.exception.GlobalException;
import com.hywa.map.common.utils.StringUtils;
import com.hywa.map.common.utils.UUIDUtils;
import com.hywa.map.dao.entity.Dict;
import com.hywa.map.dao.mapper.DictMapper;
import com.hywa.map.representation.*;
import com.hywa.map.service.dict.DictService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;

import static com.hywa.map.common.enums.CommonEnum.*;

/**
 * t_sys_dict  对应ServiceImpl 实现类
 *
 * @author hywa
 * @create by 2019-02-15 09:56:54
 */
@Slf4j
@Service
public class DictServiceImpl implements DictService {

    @Autowired
    private DictMapper dictMapper;

    @Override
    public int save(DictReq reqInfo, String userId) {
        String menuName = "数据字典-新增数据字典信息";
        if (StringUtils.isEmpty(reqInfo.getDictType())) {
            throw new GlobalException(ILLEGAL_PARAMETER_ERROR.getIndex(), ILLEGAL_PARAMETER_ERROR.getValue() + "，字典类型不能为空", null, menuName);
        }
        if (StringUtils.isEmpty(reqInfo.getDictCode())) {
            throw new GlobalException(ILLEGAL_PARAMETER_ERROR.getIndex(), ILLEGAL_PARAMETER_ERROR.getValue() + "，字典编码不能为空", null, menuName);
        }
        if (dictMapper.countByType(reqInfo.getDictType(), null) > 0) {
            throw new GlobalException(DICT_TYPE_FOUND.getIndex(), DICT_TYPE_FOUND.getValue(), null, menuName);
        }
        if (dictMapper.countByCode(reqInfo.getDictCode(), null) > 0) {
            throw new GlobalException(DICT_CODE_FOUND.getIndex(), DICT_CODE_FOUND.getValue(), null, menuName);
        }
        Dict entity = packEntity(reqInfo, userId);
        int count = dictMapper.add(entity);
        log.info("新增数据字典信息完成，请求数据|" + reqInfo + "，操作结果|" + count);
        return count;
    }

    @Override
    public int update(DictReq reqInfo, String userId) {
        String menuName = "数据字典-修改数据字典信息";
        if (StringUtils.isEmpty(reqInfo.getDictType())) {
            throw new GlobalException(ILLEGAL_PARAMETER_ERROR.getIndex(), ILLEGAL_PARAMETER_ERROR.getValue() + "，字典类型不能为空", null, menuName);
        }
        if (StringUtils.isEmpty(reqInfo.getDictCode())) {
            throw new GlobalException(ILLEGAL_PARAMETER_ERROR.getIndex(), ILLEGAL_PARAMETER_ERROR.getValue() + "，字典编码不能为空", null, menuName);
        }
        if (dictMapper.countByType(reqInfo.getDictType(), reqInfo.getId()) > 0) {
            throw new GlobalException(DICT_TYPE_FOUND.getIndex(), DICT_TYPE_FOUND.getValue(), null, menuName);
        }
        if (dictMapper.countByCode(reqInfo.getDictCode(), reqInfo.getId()) > 0) {
            throw new GlobalException(DICT_CODE_FOUND.getIndex(), DICT_CODE_FOUND.getValue(), null, menuName);
        }
        Dict entity = new Dict();
        entity.setId(reqInfo.getId());
        entity.setDictType(reqInfo.getDictType());
        entity.setDictCode(reqInfo.getDictCode());
        entity.setRemark(reqInfo.getRemark());
        int count = dictMapper.update(entity);
        log.info("编辑数据字典信息完成，请求数据|" + reqInfo + "，操作结果|" + count);
        return count;
    }

    @Override
    public DictReps select(int pageNo, int pageSize, String queryParam) {
        PageHelper.startPage(pageNo, pageSize, true);
        List<Dict> list = dictMapper.select(queryParam);
        List<DictRep> repList = new ArrayList<>();
        for (Dict item : list) {
            DictRep dictRep = new DictRep(item);
            repList.add(dictRep);
        }
        log.info("查询数据字典信息完成，请求数据|" + queryParam);
        return new DictReps(new PageInfo<>(list).getTotal(), repList);
    }

    @Override
    public DictRep selectById(String id) {
        String menuName = "数据字典-查询数据字典信息";
        if (StringUtils.isEmpty(id)) {
            throw new GlobalException(ILLEGAL_PARAMETER_ERROR.getIndex(), ILLEGAL_PARAMETER_ERROR.getValue() + "，机构id不能为空", null, menuName);
        }
        Dict dict = dictMapper.selectById(id);
        if (dict == null) {
            return null;
        }
        DictRep dictRep = new DictRep(dict);
        return dictRep;
    }

    @Override
    public int delete(DictReq reqInfo, String userId) {
        String menuName = "数据字典-删除数据字典信息";
        if (StringUtils.isEmpty(reqInfo.getId())) {
            throw new GlobalException(ILLEGAL_PARAMETER_ERROR.getIndex(), ILLEGAL_PARAMETER_ERROR.getValue() + "，字典类型id不能为空", null, menuName);
        }
        if (dictMapper.countByDictId(reqInfo.getId()) > 0) {
            throw new GlobalException(DICT_TYPE_USED.getIndex(), DICT_TYPE_USED.getValue(), null, menuName);
        }
        int count = dictMapper.delete(reqInfo.getId());
        log.info("删除数据字典信息完成，请求数据|" + reqInfo + "，操作结果|" + count);
        return count;
    }

    @Override
    public DictReps selectVal(int pageNo, int pageSize, String dictId, String queryParam) {
        String menuName = "数据字典-查询数据字典信息";
        if (StringUtils.isEmpty(dictId)) {
            throw new GlobalException(ILLEGAL_PARAMETER_ERROR.getIndex(), ILLEGAL_PARAMETER_ERROR.getValue() + "，字典类型id不能为空", null, menuName);
        }
        PageHelper.startPage(pageNo, pageSize, true);
        List<Dict> list = dictMapper.selectVal(dictId, queryParam);
        List<DictRep> repList = new ArrayList<>();
        for (Dict item : list) {
            DictRep dictRep = new DictRep(item);
            dictRep.setDictId(dictId);
            repList.add(dictRep);
        }
        log.info("查询数据字典信息完成，请求数据|" + queryParam);
        return new DictReps(new PageInfo<>(list).getTotal(), repList);
    }

    @Override
    public int saveVal(DictReq reqInfo, String userId) {
        String menuName = "数据字典-新增字典值信息";
        if (StringUtils.isEmpty(reqInfo.getDictId())) {
            throw new GlobalException(ILLEGAL_PARAMETER_ERROR.getIndex(), ILLEGAL_PARAMETER_ERROR.getValue() + "，字典类型id不能为空", null, menuName);
        }
        if (StringUtils.isEmpty(reqInfo.getDictValue())) {
            throw new GlobalException(ILLEGAL_PARAMETER_ERROR.getIndex(), ILLEGAL_PARAMETER_ERROR.getValue() + "，字典值不能为空", null, menuName);
        }
        if (StringUtils.isEmpty(reqInfo.getDictSeq())) {
            throw new GlobalException(ILLEGAL_PARAMETER_ERROR.getIndex(), ILLEGAL_PARAMETER_ERROR.getValue() + "，字典序号不能为空", null, menuName);
        }
        if (dictMapper.countByVal(reqInfo.getDictValue(), reqInfo.getDictId(), null) > 0) {
            throw new GlobalException(DICT_VALUE_FOUND.getIndex(), DICT_VALUE_FOUND.getValue(), null, menuName);
        }
        if (dictMapper.countBySeq(reqInfo.getDictSeq(), reqInfo.getDictId(), null) > 0) {
            throw new GlobalException(DICT_SEQ_FOUND.getIndex(), DICT_SEQ_FOUND.getValue(), null, menuName);
        }
        Dict entity = packEntity(reqInfo);
        int count = dictMapper.insertVal(entity);
        log.info("新增字典值信息完成，请求数据|" + reqInfo + "，操作结果|" + count);
        return count;
    }

    @Override
    public DictRep selectValById(String id) {
        String menuName = "数据字典-查询字典值信息";
        if (StringUtils.isEmpty(id)) {
            throw new GlobalException(ILLEGAL_PARAMETER_ERROR.getIndex(), ILLEGAL_PARAMETER_ERROR.getValue() + "，字典值id不能为空", null, menuName);
        }
        Dict dict = dictMapper.selectValById(id);
        if (dict == null) {
            return null;
        }
        DictRep dictRep = new DictRep(dict);
        return dictRep;
    }

    @Override
    public int updateVal(DictReq reqInfo, String userId) {
        String menuName = "数据字典-修改字典值信息";
        if (StringUtils.isEmpty(reqInfo.getId())) {
            throw new GlobalException(ILLEGAL_PARAMETER_ERROR.getIndex(), ILLEGAL_PARAMETER_ERROR.getValue() + "，字典值id不能为空", null, menuName);
        }
        if (StringUtils.isEmpty(reqInfo.getDictId())) {
            throw new GlobalException(ILLEGAL_PARAMETER_ERROR.getIndex(), ILLEGAL_PARAMETER_ERROR.getValue() + "，字典类型id不能为空", null, menuName);
        }
        if (StringUtils.isEmpty(reqInfo.getDictValue())) {
            throw new GlobalException(ILLEGAL_PARAMETER_ERROR.getIndex(), ILLEGAL_PARAMETER_ERROR.getValue() + "，字典值不能为空", null, menuName);
        }
        if (StringUtils.isEmpty(reqInfo.getDictSeq())) {
            throw new GlobalException(ILLEGAL_PARAMETER_ERROR.getIndex(), ILLEGAL_PARAMETER_ERROR.getValue() + "，字典值序号不能为空", null, menuName);
        }
        if (dictMapper.countByVal(reqInfo.getDictValue(), reqInfo.getDictId(), reqInfo.getId()) > 0) {
            throw new GlobalException(DICT_VALUE_FOUND.getIndex(), DICT_VALUE_FOUND.getValue(), null, menuName);
        }
        if (dictMapper.countBySeq(reqInfo.getDictSeq(), reqInfo.getDictId(), reqInfo.getId()) > 0) {
            throw new GlobalException(DICT_SEQ_FOUND.getIndex(), DICT_SEQ_FOUND.getValue(), null, menuName);
        }
        Dict entity = new Dict();
        entity.setId(reqInfo.getId());
        entity.setDictName(reqInfo.getDictValue());
        entity.setDictSeq(Integer.parseInt(reqInfo.getDictSeq()));
        entity.setRemark(reqInfo.getRemark());
        int count = dictMapper.updateVal(entity);
        log.info("修改字典值信息完成，请求数据|" + reqInfo + "，操作结果|" + count);
        return count;
    }

    @Override
    public int deleteVal(DictReq reqInfo, String userId) {
        String menuName = "数据字典-删除字典值信息";
        if (StringUtils.isEmpty(reqInfo.getId())) {
            throw new GlobalException(ILLEGAL_PARAMETER_ERROR.getIndex(), ILLEGAL_PARAMETER_ERROR.getValue() + "，字典值id不能为空", null, menuName);
        }
        int count = dictMapper.deleteVal(reqInfo.getId());
        log.info("删除字典值信息完成，请求数据|" + reqInfo + "，操作结果|" + count);
        return count;
    }

    @Override
    public DictReps selectVal(String dictCode) {
        String menuName = "数据字典-根据编码查询字典值信息";
        if (StringUtils.isEmpty(dictCode)) {
            throw new GlobalException(ILLEGAL_PARAMETER_ERROR.getIndex(), ILLEGAL_PARAMETER_ERROR.getValue() + "，字典编码不能为空", null, menuName);
        }
        List<Dict> list = dictMapper.selectValByCode(dictCode);
        List<DictRep> repList = new ArrayList<>();
        for (Dict item : list) {
            DictRep dictRep = new DictRep(item);
            repList.add(dictRep);
        }
        log.info("根据编码查询字典值信息完成，请求数据|" + dictCode);
        return new DictReps((long) repList.size(), repList);
    }

    /*
     * 统一组装实体
     */
    public Dict packEntity(DictReq reqInfo, String userId) {
        Dict entity = new Dict();
        entity.setId(UUIDUtils.randomUUID());
        entity.setDictType(reqInfo.getDictType());
        entity.setDictCode(reqInfo.getDictCode());
        entity.setRemark(reqInfo.getRemark());
        return entity;
    }

    /*
     * 统一组装实体
     */
    public Dict packEntity(DictReq reqInfo) {
        Dict entity = new Dict();
        entity.setId(UUIDUtils.randomUUID());
        entity.setDictId(reqInfo.getDictId());
        entity.setDictName(reqInfo.getDictValue());
        entity.setDictSeq(Integer.parseInt(reqInfo.getDictSeq()));
        entity.setRemark(reqInfo.getRemark());
        return entity;
    }
}

