package com.hywa.map.service.expert.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.parser.Feature;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.hywa.map.common.ConstantPool;
import com.hywa.map.common.exception.GlobalException;
import com.hywa.map.common.utils.*;
import com.hywa.map.config.EnvProperties;
import com.hywa.map.dao.entity.AreaData;
import com.hywa.map.dao.entity.Combox;
import com.hywa.map.dao.entity.Dict;
import com.hywa.map.dao.entity.SysArea;
import com.hywa.map.dao.mapper.AreaDataMapper;
import com.hywa.map.dao.mapper.DictMapper;
import com.hywa.map.representation.*;
import com.hywa.map.service.expert.AreaDataService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import java.io.File;
import java.util.*;

import static com.hywa.map.common.enums.CommonEnum.AREA_DATA_USED;
import static com.hywa.map.common.enums.CommonEnum.FILE_TEMPLATE_ERROR;
import static com.hywa.map.common.enums.CommonEnum.ILLEGAL_PARAMETER_ERROR;

/**
 * 专家系统  对应ServiceImpl 实现类
 *
 * @author hywa
 * @create by 2018-12-13 13:41:59
 */
@Slf4j
@Service
public class AreaDataServiceImpl implements AreaDataService {

    @Autowired
    private AreaDataMapper areaDataMapper;
    @Autowired
    private DictMapper dictMapper;

    @Autowired
    EnvProperties envProperties;

    @Override
    public void importXls(MultipartFile file, String userId) {
        String menuName = "区域数据管理-导入excel数据";
        try {
            //1.获取excel数据
            List<String[]> list = null;
            try {
                list = PoiUtils.readExcel(file, 0, 0);
            } catch (Exception e) {
                e.printStackTrace();
            }
            //2.解析list数据：前4行属于基本信息保存在t_area_data
            //其余行属于具体数据值保存在t_area_data_detail
            String uuid = UUIDUtils.randomUUID();
            String dataName = "";
            String areaName = "";
            String dataType = "";
            String dataTime = "";
            String dataField = "";
            String excelName = "";
            if (list != null && list.size() > 0) {
                List<Combox> datas = new ArrayList<>();
                for (int i = 0; i < list.size(); i++) {
                    String[] label = list.get(i);
                    if (i == 0) {
                        dataName = label[0];
                    } else if (i == 1) {
                        for (int j = 0; j < label.length; j++) {
                            dataField = dataField + label[j] + "|";
                        }
                        dataField = dataField.substring(0, dataField.length() - 1);
                    } else {
                        String dataVal = "";
                        for (int k = 0; k < label.length; k++) {
                            dataVal = dataVal + ("请进行填报".equals(label[k]) ? "" : label[k]) + "|";
                        }
                        dataVal = dataVal.substring(0, dataVal.length() - 1);
                        Combox cb = new Combox();
                        cb.setId(uuid);
                        cb.setName(dataVal);
                        datas.add(cb);
                    }
                }
                //3.基本信息保存在t_area_data
                AreaData areaData = packAreaDataEntity(dataName, areaName, dataType, dataTime, dataField, userId);
                areaData.setId(uuid);
                areaDataMapper.add(areaData);
                //4.数据值保存在t_area_data_detail
                areaDataMapper.insertAreaDataBatch(datas);
            }
        } catch (Exception e) {
            if (e instanceof GlobalException) {
                throw e;
            }
            throw new GlobalException(FILE_TEMPLATE_ERROR.getIndex(), FILE_TEMPLATE_ERROR.getValue(), null, menuName);
        }
    }

    @Override
    public AreaDataReps queryAll(int pageNo, int pageSize, String areaName, String dataType, String dataName, String dataTime) {
        PageHelper.startPage(pageNo, pageSize, true);
        dataTime = StringUtils.isEmpty(dataTime) ? dataTime : dataTime.replaceAll("-", "");
        List<AreaDataRep> list = areaDataMapper.selectAll(areaName, dataType, dataName, dataTime);
        for (AreaDataRep areaDataRep : list) {
            areaDataRep.setDataTime(areaDataRep.getDataTime());
            areaDataRep.setIsShowName(areaDataRep.getIsShow() == 0 ? "否" : "是");
        }
        log.info("查询所有区域数据信息完成，请求数据|");
        return new AreaDataReps(new PageInfo<>(list).getTotal(), list);
    }

    @Override
    public AreaDataDetailRep queryById(String id) {
        String menuName = "区域数据管理-根据id查询区域数据详细信息";
        if (StringUtils.isEmpty(id)) {
            throw new GlobalException(ILLEGAL_PARAMETER_ERROR.getIndex(), ILLEGAL_PARAMETER_ERROR.getValue() + "，id不能为空", null, menuName);
        }
        AreaDataDetailRep rep = new AreaDataDetailRep();
        AreaDataRep areaData = areaDataMapper.selectById(id);
        rep.setId(areaData.getId());
        rep.setDataName(areaData.getDataName());
        rep.setAreaName(areaData.getAreaName());
        rep.setDataType(areaData.getDataType());
        rep.setDataTypeName(areaData.getDataTypeName());
        rep.setDataTime(areaData.getDataTime());
        rep.setIsShow(areaData.getIsShow());
        rep.setIsShowName(areaData.getIsShow() == 0 ? "否" : "是");
        rep.setCreateDate(areaData.getCreateDate());
        rep.setCreateUser(areaData.getCreateUser());
        List<Map<String, String>> fieldList = new ArrayList<>();
        List<Map<String, String>> valList = new ArrayList<>();
        String[] dataField = areaData.getDataField().split("\\|");
        for (int i = 0; i < dataField.length; i++) {
            Map field = new HashMap();
            field.put("fieldName", dataField[i]);
            fieldList.add(field);
        }
        List<AreaData> areaDatas = areaDataMapper.selectDetailByDataId(areaData.getId());
        for (AreaData item : areaDatas) {
            String[] dataVal = item.getDataField().split("\\|");
            Map val = new LinkedHashMap();
            for (int i = 0; i < dataField.length; i++) {
                String tempVal = "";
                try {
                    tempVal = dataVal[i];
                } catch (Exception e) {

                }
                val.put("fieldVal" + (i), tempVal);
            }
            HashMap valMap = JSON.parseObject(JSON.toJSONString(val), LinkedHashMap.class, Feature.OrderedField);
            valList.add(valMap);
        }
        rep.setFieldList(fieldList);
        rep.setValList(valList);
        log.info("根据id查询区域数据详细信息完成，请求数据：id|" + id);
        return rep;
    }

    @Override
    public String exportXls(String id) {
        AreaDataDetailRep areaDataDetailRep = queryById(id);
        List<Map<String, String>> fieldList = areaDataDetailRep.getFieldList();
        List<Map<String, String>> valList = areaDataDetailRep.getValList();
        String fileName = areaDataDetailRep.getDataName() + ".xls";
        String tableTitle = areaDataDetailRep.getDataName();
        exportExcel(fieldList, valList, tableTitle, fileName, null);
        return fileName;
    }

    @Override
    public int delete(DamInfoReq reqInfo, String userId) {
        String menuName = "区域数据管理-删除区域数据信息";
        if (StringUtils.isEmpty(reqInfo.getId())) {
            throw new GlobalException(ILLEGAL_PARAMETER_ERROR.getIndex(), ILLEGAL_PARAMETER_ERROR.getValue() + "，id不能为空", null, menuName);
        }
        int topic = areaDataMapper.countBySourceId(reqInfo.getId());
        if (topic >= 1) {
            throw new GlobalException(AREA_DATA_USED.getIndex(), AREA_DATA_USED.getValue(), null, menuName);
        }
        int count = areaDataMapper.put(reqInfo.getId(), userId);
        log.info("区域数据管理-删除区域数据信息完成，请求数据：reqInfo|" + reqInfo + "，操作结果|" + count);
        return count;
    }

    @Override
    public int updateIsShow(DamInfoReq reqInfo, String userId) {
        String menuName = "区域数据管理-是否应用于大屏";
        if (StringUtils.isEmpty(reqInfo.getId())) {
            throw new GlobalException(ILLEGAL_PARAMETER_ERROR.getIndex(), ILLEGAL_PARAMETER_ERROR.getValue() + "，id不能为空", null, menuName);
        }
        if (StringUtils.isEmpty(reqInfo.getState())) {
            throw new GlobalException(ILLEGAL_PARAMETER_ERROR.getIndex(), ILLEGAL_PARAMETER_ERROR.getValue() + "，状态不能为空", null, menuName);
        }
        AreaDataRep areaData = areaDataMapper.selectById(reqInfo.getId());
        int count = areaDataMapper.updateIsShow(reqInfo.getId(), userId, reqInfo.getState());
        AreaData entity = new AreaData();
        entity.setId(areaData.getId());
        entity.setUpdateUser(userId);
        entity.setAreaName(areaData.getAreaName());
        entity.setDataType(areaData.getDataType());
        entity.setDataTime(areaData.getDataTime());
        areaDataMapper.updateOther(entity);
        log.info("区域数据管理-是否应用于大屏完成，请求数据：reqInfo|" + reqInfo + "，操作结果|" + count);
        return count;
    }

    @Override
    public AreaDataRep queryShow(String areaName, String dataType) {
        AreaDataRep areaDataRep = areaDataMapper.selectShow(areaName, dataType);
        log.info("查询所有区域数据信息，用于大屏显示完成，请求数据：areaName|" + areaName + "，dataType|" + dataType);
        return areaDataRep;
    }

    @Override
    public AreaDataRep queryDisplay(String areaName, String dataType) {
        AreaDataRep areaDataRep = areaDataMapper.selectDisplay(areaName, dataType);
        log.info("查询所有区域数据信息，用于大屏显示完成(适用于没有区域id的情况)，请求数据：areaName|" + areaName + "，dataType|" + dataType);
        return areaDataRep;
    }

    @Override
    public AreaDataPriceReps queryPrice(int pageNo, int pageSize, String beginDate, String endDate, String cateName, String areaName) {
        String menuName = "区域数据管理-查询品种价格信息";
        if (StringUtils.isEmpty(beginDate)) {
            throw new GlobalException(ILLEGAL_PARAMETER_ERROR.getIndex(), ILLEGAL_PARAMETER_ERROR.getValue() + "，开始时间不能为空", null, menuName);
        }
        if (StringUtils.isEmpty(endDate)) {
            throw new GlobalException(ILLEGAL_PARAMETER_ERROR.getIndex(), ILLEGAL_PARAMETER_ERROR.getValue() + "，结束时间不能为空", null, menuName);
        }
        beginDate = beginDate.replaceAll("-", "");
        endDate = endDate.replaceAll("-", "");
        PageHelper.startPage(pageNo, pageSize, true);
        List<AreaDataPriceRep> list = areaDataMapper.selectPrice(beginDate, endDate, cateName, areaName);
        for (AreaDataPriceRep item : list) {
            item.setPriceUnit("斤");
            String tradeDate = item.getTradeDate();
            item.setTradeDate(tradeDate.substring(0, 4) + "-" + tradeDate.substring(4, 6) + "-" + tradeDate.substring(6, 8));
            double price = Double.parseDouble(item.getPrice()) / 2;
            double lastPrice = StringUtils.isEmpty(item.getRiseRate()) ? 0 : Double.parseDouble(item.getRiseRate()) / 2;
            String riseRate = StringUtils.isEmpty(item.getRiseRate()) ? "0" : String.format("%.2f", ((price - lastPrice) / lastPrice) * 100);
            riseRate = "Infinity".equals(riseRate) ? "0" : riseRate;
            item.setPrice(String.format("%.2f", price));
            item.setRiseRate(("0".equals(riseRate) || "0.00".equals(riseRate)) ? "0" : riseRate);
            item.setSourceName("贵阳农产品物流园".equals(areaName) ? "扎佐物流园" : "双龙物流园");
            item.setId(UUIDUtils.randomUUID());
        }
        log.info("查询品种价格信息，用于益农社系统显示完成，请求数据：pageNo|" + pageNo + "，pageSize|" + pageSize + "，beginDate|" + beginDate + "，endDate|" + endDate + "，cateName|" + cateName + "，areaName|" + areaName);
        return new AreaDataPriceReps(new PageInfo<>(list).getTotal(), list);
    }

    @Override
    public SysAreaReps queryAreaBySn(String sn) {
        return null;
    }

    @Override
    public AreaDataPriceReps queryPriceRate(int pageNo, int pageSize, String beginDate, String endDate, String cateName, String areaName, String dataId) {
        String menuName = "区域数据管理-查询品种价格信息";
        if (StringUtils.isEmpty(beginDate)) {
            throw new GlobalException(ILLEGAL_PARAMETER_ERROR.getIndex(), ILLEGAL_PARAMETER_ERROR.getValue() + "，开始时间不能为空", null, menuName);
        }
        if (StringUtils.isEmpty(endDate)) {
            throw new GlobalException(ILLEGAL_PARAMETER_ERROR.getIndex(), ILLEGAL_PARAMETER_ERROR.getValue() + "，结束时间不能为空", null, menuName);
        }
        beginDate = beginDate.replaceAll("-", "");
        endDate = endDate.replaceAll("-", "");
        PageHelper.startPage(pageNo, pageSize, true);
        List<AreaDataPriceRep> list = areaDataMapper.selectPriceRate(beginDate, endDate, cateName, areaName,dataId);
        for (AreaDataPriceRep item : list) {
            item.setPriceUnit("公斤");
            String tradeDate = item.getTradeDate();
            //item.setTradeDate(tradeDate.substring(0, 4) + "-" + tradeDate.substring(4, 6) + "-" + tradeDate.substring(6, 8));
            double price = Double.parseDouble(item.getPrice());
            double lastPrice = StringUtils.isEmpty(item.getRiseRate()) ? 0 : Double.parseDouble(item.getRiseRate());
            String riseRate = StringUtils.isEmpty(item.getRiseRate()) ? "0" : String.format("%.2f", ((price - lastPrice) / lastPrice) * 100);
            riseRate = "Infinity".equals(riseRate) ? "0" : riseRate;
            //item.setPrice(String.format("%.2f", price));
            item.setRiseRate(("0".equals(riseRate) || "0.00".equals(riseRate)) ? "0" : riseRate);
            item.setSourceName(areaName);
            item.setId(UUIDUtils.randomUUID());
        }
        log.info("查询品种价格信息，用于前端大屏幕系统显示完成，请求数据：pageNo|" + pageNo + "，pageSize|" + pageSize + "，beginDate|" + beginDate + "，endDate|" + endDate + "，cateName|" + cateName + "，areaName|" + areaName);
        return new AreaDataPriceReps(new PageInfo<>(list).getTotal(), list);
    }

    /*
     * 生成结果文件
     */
    public void exportExcel(List<Map<String, String>> fieldList, List valList, String tableTitle, String fileName, String redCell) {
        LinkedHashMap<String, String> filedNames = new LinkedHashMap<>();
        for (int i = 0; i < fieldList.size(); i++) {
            Map<String, String> map = fieldList.get(i);
            filedNames.put("fieldVal" + (i), map.get("fieldName"));
        }
        File file = ExcelUtils.exportExcelForFile(valList, tableTitle, tableTitle, filedNames, envProperties.getComFilePath(), fileName, redCell);
    }

    /*
     * 统一组装实体
     */
    public AreaData packAreaDataEntity(String dataName, String areaName, String dataType, String dataTime, String dataField, String userId) {
        AreaData entity = new AreaData();
        entity.setId(UUIDUtils.randomUUID());
        entity.setDataName(dataName);
        entity.setAreaName(areaName);
        entity.setDataType(dataType);
        entity.setDataTime(dataTime);
        entity.setIsShow(0);
        entity.setDataField(dataField);
        entity.setIsDel(ConstantPool.NOT_DEL);
        entity.setCreateDate(new Date());
        entity.setCreateUser(userId);
        return entity;
    }

}

