package com.hywa.map.service.expert.impl;

import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.hywa.map.common.ConstantPool;
import com.hywa.map.common.exception.GlobalException;
import com.hywa.map.common.utils.StringUtils;
import com.hywa.map.common.utils.UUIDUtils;
import com.hywa.map.config.EnvProperties;
import com.hywa.map.dao.entity.DamInfo;
import com.hywa.map.dao.mapper.AgriDataMapper;
import com.hywa.map.dao.mapper.DamInfoMapper;
import com.hywa.map.dao.mapper.SysDictMapper;
import com.hywa.map.representation.*;
import com.hywa.map.service.expert.DamService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.List;

import static com.hywa.map.common.enums.CommonEnum.ILLEGAL_PARAMETER_ERROR;

/**
 * 专家系统  对应ServiceImpl 实现类
 *
 * @author hywa
 * @create by 2018-12-13 13:41:59
 */
@Slf4j
@Service
public class DamServiceImpl implements DamService {

    @Autowired
    private DamInfoMapper damInfoMapper;

    @Autowired
    EnvProperties envProperties;

    @Override
    public DamInfoReps queryAll(int pageNo, int pageSize, String areaCode, String damName, String state, String areaName) {
        PageHelper.startPage(pageNo, pageSize, true);
        List<DamInfoRep> list = damInfoMapper.selectAll(areaCode, damName,state,areaName);
        log.info("查询所有坝区信息完成，请求数据|");
        return new DamInfoReps(new PageInfo<>(list).getTotal(), list);
    }

    @Override
    public DamDetailInfoRep queryById(String id) {
        String menuName = "坝区管理-根据id查询坝区信息";
        if (StringUtils.isEmpty(id)) {
            throw new GlobalException(ILLEGAL_PARAMETER_ERROR.getIndex(), ILLEGAL_PARAMETER_ERROR.getValue() + "，id不能为空", null, menuName);
        }
        DamDetailInfoRep rep = damInfoMapper.selectById(id);
        log.info("根据id查询坝区信息完成，请求数据：id|id");
        return rep;
    }

    @Override
    public DamDetailInfoRep save(DamInfoReq reqInfo, String userId) {
        String menuName = "坝区管理-新增坝区信息";
        /*if (StringUtils.isEmpty(reqInfo.getDamName())) {
            throw new GlobalException(ILLEGAL_PARAMETER_ERROR.getIndex(), ILLEGAL_PARAMETER_ERROR.getValue() + "，坝区名称不能为空", null, menuName);
        }*/
        DamInfo entity = packDamInfoEntity(reqInfo, userId);
        int count = damInfoMapper.add(entity);
        DamDetailInfoRep rep = new DamDetailInfoRep();
        rep.setId(entity.getId());
        log.info("坝区管理-新增坝区信息完成，请求数据：reqInfo|" + reqInfo + "，操作结果|" + count);
        return rep;
    }

    @Override
    public int update(DamInfoReq reqInfo, String userId) {
        String menuName = "坝区管理-修改坝区信息";
        if (StringUtils.isEmpty(reqInfo.getId())) {
            throw new GlobalException(ILLEGAL_PARAMETER_ERROR.getIndex(), ILLEGAL_PARAMETER_ERROR.getValue() + "，id不能为空", null, menuName);
        }
        if (StringUtils.isEmpty(reqInfo.getDamName())) {
            throw new GlobalException(ILLEGAL_PARAMETER_ERROR.getIndex(), ILLEGAL_PARAMETER_ERROR.getValue() + "，坝区名称不能为空", null, menuName);
        }
        DamInfo entity = packDamInfoEntity(reqInfo, userId);
        entity.setId(reqInfo.getId());
        entity.setUpdateDate(new Date());
        entity.setUpdateUser(userId);
        int count = damInfoMapper.update(entity);
        log.info("坝区管理-修改坝区信息完成，请求数据：reqInfo|" + reqInfo + "，操作结果|" + count);
        return count;
    }

    @Override
    public int delete(DamInfoReq reqInfo, String userId) {
        String menuName = "坝区管理-删除坝区信息";
        if (StringUtils.isEmpty(reqInfo.getId())) {
            throw new GlobalException(ILLEGAL_PARAMETER_ERROR.getIndex(), ILLEGAL_PARAMETER_ERROR.getValue() + "，id不能为空", null, menuName);
        }
        int count = damInfoMapper.put(reqInfo.getId(),userId);
        log.info("坝区管理-删除坝区信息完成，请求数据：reqInfo|" + reqInfo + "，操作结果|" + count);
        return count;
    }

    @Override
    public int updateState(DamInfoReq reqInfo, String userId) {
        String menuName = "坝区管理-是否启用";
        if (StringUtils.isEmpty(reqInfo.getId())) {
            throw new GlobalException(ILLEGAL_PARAMETER_ERROR.getIndex(), ILLEGAL_PARAMETER_ERROR.getValue() + "，id不能为空", null, menuName);
        }
        if (StringUtils.isEmpty(reqInfo.getState())) {
            throw new GlobalException(ILLEGAL_PARAMETER_ERROR.getIndex(), ILLEGAL_PARAMETER_ERROR.getValue() + "，状态不能为空", null, menuName);
        }
        //String state = "0".equals(reqInfo.getState()) ? "1" : "0";
        int count = damInfoMapper.updateState(reqInfo.getId(),userId,reqInfo.getState());
        log.info("坝区管理-是否启用完成，请求数据：reqInfo|" + reqInfo + "，操作结果|" + count);
        return count;
    }

    /*
     * 统一组装实体
     */
    public DamInfo packDamInfoEntity(DamInfoReq reqInfo, String userId) {
        DamInfo entity = new DamInfo();
        entity.setId(UUIDUtils.randomUUID());
        entity.setDamName(reqInfo.getDamName());
        entity.setAreaCode(reqInfo.getAreaCode());
        entity.setElevation(reqInfo.getElevation());
        entity.setTotalArea(reqInfo.getTotalArea());
        entity.setPlantName(reqInfo.getPlantName());
        entity.setPlantVal(reqInfo.getPlantVal());
        entity.setLandRate(reqInfo.getLandRate());
        entity.setFarmerRate(reqInfo.getFarmerRate());
        entity.setMarketRate(reqInfo.getMarketRate());
        entity.setInsureRate(reqInfo.getInsureRate());
        entity.setDamDesc(reqInfo.getDamDesc());
        entity.setState("0");
        entity.setIsDel(ConstantPool.NOT_DEL);
        entity.setCreateDate(new Date());
        entity.setCreateUser(userId);
        return entity;
    }
}

