package com.hywa.map.service.expert.impl;

import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.hywa.map.common.ConstantPool;
import com.hywa.map.common.exception.GlobalException;
import com.hywa.map.common.utils.StringUtils;
import com.hywa.map.common.utils.UUIDUtils;
import com.hywa.map.config.EnvProperties;
import com.hywa.map.dao.entity.ExpertInfo;
import com.hywa.map.dao.entity.SysArea;
import com.hywa.map.dao.mapper.AgriDataMapper;
import com.hywa.map.dao.mapper.ExpertInfoMapper;
import com.hywa.map.dao.mapper.SysDictMapper;
import com.hywa.map.representation.*;
import com.hywa.map.service.expert.ExpertService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import static com.hywa.map.common.enums.CommonEnum.ILLEGAL_PARAMETER_ERROR;

/**
 * 专家系统  对应ServiceImpl 实现类
 *
 * @author hywa
 * @create by 2018-12-13 13:41:59
 */
@Slf4j
@Service
public class ExpertServiceImpl implements ExpertService {

    @Autowired
    private SysDictMapper sysdictMapper;
    @Autowired
    private AgriDataMapper agriDataMapper;
    @Autowired
    private ExpertInfoMapper expertInfoMapper;

    @Autowired
    EnvProperties envProperties;

    @Override
    public SysAreaReps queryAreaBySn(String sn) {
        List<SysArea> list = agriDataMapper.selectSubAreaBySn(sn);
        List<SysAreaRep> repList = new ArrayList<>();
        for (SysArea item : list) {
            SysAreaRep rep = new SysAreaRep(item);
            repList.add(rep);
        }
        log.info("查询所属区域信息完成，请求数据：sn|" + sn);
        return new SysAreaReps((long) repList.size(), repList);
    }

    @Override
    public int save(ExpertInfoReq reqInfo,String userId) {
        String menuName = "专家团队管理-新增专家信息";
        if (StringUtils.isEmpty(reqInfo.getExpertName())) {
            throw new GlobalException(ILLEGAL_PARAMETER_ERROR.getIndex(), ILLEGAL_PARAMETER_ERROR.getValue() + "，专家姓名不能为空", null, menuName);
        }
        ExpertInfo entity = packExpertInfoEntity(reqInfo, userId);
        int count = expertInfoMapper.add(entity);
        log.info("专家团队管理-新增专家信息完成，请求数据：reqInfo|" + reqInfo + "，操作结果|" + count);
        return count;
    }

    @Override
    public int update(ExpertInfoReq reqInfo,String userId) {
        String menuName = "专家团队管理-修改专家信息";
        if (StringUtils.isEmpty(reqInfo.getId())) {
            throw new GlobalException(ILLEGAL_PARAMETER_ERROR.getIndex(), ILLEGAL_PARAMETER_ERROR.getValue() + "，id不能为空", null, menuName);
        }
        if (StringUtils.isEmpty(reqInfo.getExpertName())) {
            throw new GlobalException(ILLEGAL_PARAMETER_ERROR.getIndex(), ILLEGAL_PARAMETER_ERROR.getValue() + "，专家姓名不能为空", null, menuName);
        }
        ExpertInfo entity = packExpertInfoEntity(reqInfo, userId);
        entity.setId(reqInfo.getId());
        entity.setUpdateDate(new Date());
        entity.setUpdateUser(userId);
        int count = expertInfoMapper.update(entity);
        log.info("专家团队管理-修改专家信息完成，请求数据：reqInfo|" + reqInfo + "，操作结果|" + count);
        return count;
    }

    @Override
    public int delete(ExpertInfoReq reqInfo,String userId) {
        String menuName = "专家团队管理-删除专家信息";
        if (StringUtils.isEmpty(reqInfo.getId())) {
            throw new GlobalException(ILLEGAL_PARAMETER_ERROR.getIndex(), ILLEGAL_PARAMETER_ERROR.getValue() + "，id不能为空", null, menuName);
        }
        int count = expertInfoMapper.put(reqInfo.getId(),userId);
        log.info("专家团队管理-删除专家信息完成，请求数据：reqInfo|" + reqInfo + "，操作结果|" + count);
        return count;
    }

    @Override
    public int updateState(ExpertInfoReq reqInfo,String userId) {
        String menuName = "专家团队管理-是否启用";
        if (StringUtils.isEmpty(reqInfo.getId())) {
            throw new GlobalException(ILLEGAL_PARAMETER_ERROR.getIndex(), ILLEGAL_PARAMETER_ERROR.getValue() + "，id不能为空", null, menuName);
        }
        if (StringUtils.isEmpty(reqInfo.getState())) {
            throw new GlobalException(ILLEGAL_PARAMETER_ERROR.getIndex(), ILLEGAL_PARAMETER_ERROR.getValue() + "，状态不能为空", null, menuName);
        }
        //String state = "0".equals(reqInfo.getState()) ? "1" : "0";
        int count = expertInfoMapper.updateState(reqInfo.getId(),userId,reqInfo.getState());
        log.info("专家团队管理-是否启用完成，请求数据：reqInfo|" + reqInfo + "，操作结果|" + count);
        return count;
    }

    @Override
    public ExpertInfoReps queryAll(int pageNo, int pageSize, String areaCode,String expertName,String typeId) {
        PageHelper.startPage(pageNo, pageSize, true);
        List<ExpertInfoRep> list = expertInfoMapper.selectAll(areaCode,expertName,typeId);
        log.info("查询所有专家信息完成，请求数据：areaCode|"+areaCode+"，expertName|"+expertName);
        return new ExpertInfoReps(new PageInfo<>(list).getTotal(), list);
    }

    @Override
    public ExpertInfoRep queryById(String id) {
        String menuName = "专家团队管理-根据id查询专家信息";
        if (StringUtils.isEmpty(id)) {
            throw new GlobalException(ILLEGAL_PARAMETER_ERROR.getIndex(), ILLEGAL_PARAMETER_ERROR.getValue() + "，id不能为空", null, menuName);
        }
        ExpertInfoRep rep = expertInfoMapper.selectById(id);
        rep.setImgUrl(envProperties.getComFileDownloadUrl()+rep.getImg());
        log.info("根据id查询专家信息完成，请求数据：id|id");
        return rep;
    }

    /*
     * 统一组装实体
     */
    public ExpertInfo packExpertInfoEntity(ExpertInfoReq reqInfo, String userId) {
        ExpertInfo entity = new ExpertInfo();
        entity.setId(UUIDUtils.randomUUID());
        entity.setExpertName(reqInfo.getExpertName());
        entity.setTypeId(reqInfo.getTypeId());
        entity.setAreaCode(reqInfo.getAreaCode());
        entity.setDamId(reqInfo.getDamId());
        entity.setJob(reqInfo.getJob());
        entity.setMobile(reqInfo.getMobile());
        entity.setDomain(reqInfo.getDomain());
        entity.setPersonDesc(reqInfo.getPersonDesc());
        entity.setImg(reqInfo.getImg());
        entity.setState("0");
        entity.setIsDel(ConstantPool.NOT_DEL);
        entity.setCreateDate(new Date());
        entity.setCreateUser(userId);
        return entity;
    }
}

