package com.hywa.map.service.login.impl;

import com.hywa.map.common.enums.CommonEnum;
import com.hywa.map.common.exception.GlobalException;
import com.hywa.map.dao.entity.*;
import com.hywa.map.dao.mapper.AreaMenuMapper;
import com.hywa.map.representation.*;
import com.hywa.map.service.login.AreaMenuService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;

@Slf4j
@Service
@Transactional
public class AreaMenuServiceImpl implements AreaMenuService {

    @Autowired
    private AreaMenuMapper areaMenuMapper;

    @Override
    @Transactional
    public void save(AreaMenuReq areaMenuReq) {
    	if(!nameIsEmpty(areaMenuReq.getRegionId())) {
    		throw new GlobalException(CommonEnum.AREA_ID_REPEATED.getIndex(), CommonEnum.AREA_ID_REPEATED.getValue());
    	}
        List<AreaMenuKey> areaMenus = new ArrayList<>();
        areaMenuReq.getMenuReps().forEach(menu->{
            areaMenus.add(new AreaMenuKey(areaMenuReq.getRegionId(),areaMenuReq.getType(),String.valueOf(menu)));
        });
        areaMenuMapper.insertBatch(areaMenus);
    }

    @Override
    @Transactional
    public void update(AreaMenuReq areaMenuReq) {
        if(nameIsEmpty(areaMenuReq.getRegionId())) {
            throw new GlobalException(CommonEnum.AREA_ID_NOT_FOUND.getIndex(), CommonEnum.AREA_ID_NOT_FOUND.getValue());
        }
        areaMenuMapper.deleteByRegionId(areaMenuReq.getRegionId());
        List<AreaMenuKey> areaMenus = new ArrayList<>();
        areaMenuReq.getMenuReps().forEach(menu->{
            areaMenus.add(new AreaMenuKey(areaMenuReq.getRegionId(),areaMenuReq.getType(),String.valueOf(menu)));
        });
        areaMenuMapper.insertBatch(areaMenus);
    }

    @Override
    @Transactional
    public void delete(String regionId) {
        areaMenuMapper.deleteByRegionId(regionId);
    }

    @Override
    public List<MenuRep> selectAll(String userId,String userName,String regionId) {
        /*PageHelper.startPage(pageNo, pageSize, true);
        List<AreaMenu> areaMenus = areaMenuMapper.selectAll(regionId);
        List<AreaMenuRep> areaMenuReps = new ArrayList<>();
        areaMenus.forEach(menu -> areaMenuReps.add(new AreaMenuRep(menu)));
        return new AreaMenuReps(areaMenuReps, new PageInfo<>(areaMenus).getTotal());*/
        List<Menu> menus = null;
        if("admin".equals(userName)){
            menus = areaMenuMapper.selectAll(regionId);
        }else {
            menus = areaMenuMapper.selectByRegionId(userId,regionId);
        }
        List<String> regionList = new ArrayList<>();
        for (Menu menu : menus) {
            String regionCode = menu.getRegionId();
            regionList.add(regionCode);
        }
        HashSet h = new HashSet(regionList);
        regionList.clear();
        regionList.addAll(h);
        List<MenuRep> menuReps = new ArrayList<>();
        for(String parentRegion:regionList){
            List<MenuRep> menuList = new ArrayList<>();
            menuList = getAllMenuArea(menus, "#",parentRegion);
            menuReps.addAll(menuList);
        }
        //List<MenuRep> menuReps = getAllMenuArea(menus, "#");
        return menuReps;
    }

    @Override
    public List<MenuRep> selectByRegionId(String userId, String regionId) {
        //此方法暂时未使用
        List<Menu> menus = areaMenuMapper.selectByRegionId(userId,regionId);
        List<String> regionList = new ArrayList<>();
        for (Menu menu : menus) {
            String regionCode = menu.getRegionId();
            regionList.add(regionCode);
        }
        HashSet h = new HashSet(regionList);
        regionList.clear();
        regionList.addAll(h);
        List<MenuRep> menuReps = new ArrayList<>();
        for(String parentRegion:regionList){
            List<MenuRep> menuList = new ArrayList<>();
            menuList = getAllMenuArea(menus, "#",parentRegion);
            menuReps.addAll(menuList);
        }
        return menuReps;
        //return getAllMenuArea(menus, "#","");
    }


    public boolean nameIsEmpty(String regionId) {
		return areaMenuMapper.countByRegionId(regionId) > 0 ? false : true;
	}

    public List<MenuRep> getAllMenuArea(List<Menu> menuList, String parentId,String regionId) {
        List<MenuRep> menuReps = new ArrayList<>();
        for (Menu menu : menuList) {
            if (menu == null) {
                continue;
            }
            if (menu.getParentId().equals(parentId) && menu.getRegionId().equals(regionId)) {
                MenuRep menuRep = new MenuRep(menu);
                List<MenuRep> allMenuRep;
                    allMenuRep = getAllMenuArea(menuList, menu.getId(),menu.getRegionId());
                menuRep.setChildMenus(allMenuRep);
                menuReps.add(menuRep);
            }
        }
        return menuReps;
    }
    
}
