package com.hywa.map.service.login.impl;

import static com.hywa.map.common.enums.CommonEnum.ROLE_NO_FOUND;

import com.hywa.map.common.ConstantPool;
import com.hywa.map.common.enums.CommonEnum;
import com.hywa.map.common.exception.GlobalException;
import com.hywa.map.common.utils.StringUtils;
import com.hywa.map.common.utils.UUIDUtils;
import com.hywa.map.config.EnvProperties;
import com.hywa.map.dao.entity.Menu;
import com.hywa.map.dao.entity.MenuExample;
import com.hywa.map.dao.entity.MenuExample.Criteria;
import com.hywa.map.dao.entity.Role;
import com.hywa.map.dao.entity.RoleMenuExample;
import com.hywa.map.dao.entity.RoleMenuKey;
import com.hywa.map.dao.entity.RoleSysKey;
import com.hywa.map.dao.mapper.MenuMapper;
import com.hywa.map.dao.mapper.RoleMenuMapper;
import com.hywa.map.representation.*;
import com.hywa.map.service.login.MenuService;
import com.hywa.map.service.login.RoleService;

import java.util.*;

import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

@Service
public class MenuServiceImpl implements MenuService {

    @Autowired
    private MenuMapper menuMapper;

    @Autowired
    private RoleMenuMapper roleMenuMapper;

    private static final String PARENT_ID = "#";

    @Autowired
    EnvProperties envProperties;
    @Autowired
    private RoleService roleService;

    @Override
    public int save(MenuRep menuRep) {
        if (!StringUtils.isEmpty(menuRep.getName()) && !nameIsEmpty(menuRep.getName(), null)) {
            throw new GlobalException(CommonEnum.MENU_NAME_REPEATED.getIndex(),
                    CommonEnum.MENU_NAME_REPEATED.getValue());
        }
        Menu menu = new Menu();
        String menuId = UUIDUtils.randomUUID();
        menu.setId(menuId);
        menu.setCreateTime(new Date());
        menu.setCreateUser(menuRep.getCreateUser());
        menu.setName(menuRep.getName());
        menu.setSort((short) (findSortByParentId(menuRep.getParentId()) + 1));
        menu.setStatus(menuRep.getStatus());
        menu.setParentId(menuRep.getParentId());
        menu.setDescription(menuRep.getDescription());
        menu.setIcon(menuRep.getIcon());
        menu.setType(menuRep.getType());
        RoleMenuKey roleMenuKey = new RoleMenuKey(envProperties.getAdminRoleId(), menuId);
        roleMenuMapper.insert(roleMenuKey);
        return menuMapper.insertSelective(menu);
    }

    @Override
    public int update(MenuRep menuRep) {
        if (!StringUtils.isEmpty(menuRep.getName()) && !nameIsEmpty(menuRep.getName(),
                menuRep.getId())) {
            throw new GlobalException(CommonEnum.MENU_NAME_REPEATED.getIndex(),
                    CommonEnum.MENU_NAME_REPEATED.getValue());
        }
        Menu menu = new Menu();
        BeanUtils.copyProperties(menuRep, menu, "createTime", "createUser");
        menu.setUpdateTime(new Date());
        return menuMapper.updateByPrimaryKeySelective(menu);
    }

    @Override
    public int delete(String id) {
        RoleMenuExample roleMenuExample = new RoleMenuExample();
        roleMenuExample.createCriteria().andMenuIdEqualTo(id)
                .andRoleIdNotEqualTo(envProperties.getAdminRoleId());
        if (!CollectionUtils.isEmpty(roleMenuMapper.selectByExample(roleMenuExample))) {
            throw new GlobalException(CommonEnum.TYPE_REFERENCED.getIndex(),
                    CommonEnum.TYPE_REFERENCED.getValue());
        }
        return menuMapper.deleteByPrimaryKey(id);
    }

    @Override
    public List<MenuRep> getChildren(String userId, String parentMenuId) {
        List<Menu> menuList = menuMapper.selectByUserIdAndParentId(userId, parentMenuId);
        List<MenuRep> menuReps = new ArrayList<>();
        menuList.forEach(menu -> menuReps.add(new MenuRep(menu)));
        return menuReps;
    }

    @Override
    public List<MenuRep> getMenuTreeByUserId(String userId, String requestMode) {
        List<Menu> menuList = menuMapper.selectMenusByUserId(userId);
        return getAllMenuRep(menuList, PARENT_ID, requestMode);
    }

    @Override
    public List<MenuRep> selectMenusByRoleId(String roleId, String userId, String userName) {
        List<Menu> menuList = null;
        if ("admin".equals(userName)) {
            menuList = menuMapper.selectByRoleId(roleId);
        } else {
            menuList = menuMapper.selectByUserIdRoleId(roleId, userId);
        }

        return getAllMenuRep(menuList, PARENT_ID, "0");
    }

    @Override
    public List<MenuRep> getMenuAllTree(String roleId, String type) {
        List<Menu> menus = menuMapper.selectAllOrRoleId(roleId, type);
        return getAllMenuRep(menus, PARENT_ID, ConstantPool.REQUEST_MODE_ALL);
    }

    public List<MenuRep> getAllMenuRep(List<Menu> menuList, String parentId, String requestMode) {
        List<MenuRep> menuReps = new ArrayList<>();
        for (Menu menu : menuList) {
            if (menu == null) {
                continue;
            }

            if (menu.getParentId().equals(parentId)) {
                MenuRep menuRep = new MenuRep(menu);
                List<MenuRep> allMenuRep;
                if (ConstantPool.REQUEST_MODE_ALL.equals(requestMode)) {
                    allMenuRep = getAllMenuRep(menuList, menu.getId(), requestMode);
                } else {
                    allMenuRep = findChildrenMenus(menuList, menu.getId());
                }
                menuRep.setChildMenus(allMenuRep);
                menuReps.add(menuRep);
            }
        }
        return menuReps;
    }

    private List<MenuRep> findChildrenMenus(List<Menu> menuList, String parentId) {
        List<MenuRep> menuRepList = new ArrayList<>();
        for (Menu menu : menuList) {
            if (menu == null) {
                continue;
            }
            if (menu.getParentId().equals(parentId)) {
                MenuRep menuRep1 = new MenuRep(menu);
                menuRepList.add(menuRep1);
            }
        }
        return menuRepList;
    }

    @Override
    public MenuReps search(int pageNo, int pageSize, String parentId, String id, String menuName) {
        //PageHelper.startPage(pageNo, pageSize, true);
        MenuExample example = new MenuExample();
        Criteria criteria = example.createCriteria();
        criteria.andIsDelEqualTo(ConstantPool.NOT_DEL);
//		criteria.andStatusEqualTo((short) 0);
        if (StringUtils.isNotEmpty(id)) {
            criteria.andIdEqualTo(id);
        } else {
            criteria.andTypeEqualTo((short) 0);
        }
        if (StringUtils.isNotEmpty(menuName)) {
            criteria.andNameLike(menuName);
        }
        if (StringUtils.isNotEmpty(parentId)) {
            criteria.andIdNotEqualTo(parentId);
            criteria.andParentIdNotEqualTo(parentId);
        }
        example.setOrderByClause("sort DESC");
        List<Menu> menus = menuMapper.selectByExample(example);
        List<MenuRep> list = new ArrayList<>();
        menus.forEach(menu -> list.add(new MenuRep(menu)));
        return new MenuReps(list, list.size());
    }

    @Override
    public short findSortByParentId(String parentId) {
        MenuExample example = new MenuExample();
        example.createCriteria().andParentIdEqualTo(parentId);
        List<Menu> menus = menuMapper.selectByExample(example);
        menus.sort((o1, o2) -> o2.getSort() - o1.getSort());
        return CollectionUtils.isEmpty(menus) ? 0
                : (null == menus.get(0).getSort() ? 0 : menus.get(0).getSort());
    }

    @Override
    public boolean nameIsEmpty(String username, String id) {
        MenuExample menuExample = new MenuExample();
        MenuExample.Criteria criteria = menuExample.createCriteria();
        criteria.andNameEqualTo(username);
        criteria.andIsDelEqualTo(ConstantPool.NOT_DEL);
        criteria.andTypeEqualTo((short) 0);
        if (StringUtils.isNotEmpty(id)) {
            criteria.andIdNotEqualTo(id);
        }
        return menuMapper.countByExample(menuExample) > 0 ? false : true;
    }

    @Override
    public List<MenuRep> getAreaMenuTree(String userId, String type) {
        List<Menu> menus = menuMapper.selectAreaMenu(userId, type);
        return getAllMenuRep(menus, PARENT_ID, ConstantPool.REQUEST_MODE_ALL);
    }

    @Override
    public List<MenuRep> getMenuAllTreeByRegionId(String roleId, String type, String regionId) {
        List<Menu> menus = menuMapper.selectAllOrRoleIdByRegionId(roleId, type, regionId);
        return getAllMenuRep(menus, PARENT_ID, ConstantPool.REQUEST_MODE_ALL);
    }

    @Override
    public List<MenuRep> getAreaMenuTreeByRegionId(String userId, String type, String regionId) {
        List<Menu> menus = menuMapper.selectAreaMenuByRegionId(userId, type, regionId);
        return getAllMenuRep(menus, PARENT_ID, ConstantPool.REQUEST_MODE_ALL);
    }

    @Override
    public MenuSysReps getMenuByRole(String roleId) {
        Role role = roleService.selectByPrimaryKey(roleId);
        if (role == null) {
            throw new GlobalException(ROLE_NO_FOUND.getIndex(),
                    ROLE_NO_FOUND.getValue());
        }
        List<Menu> menuAll = new ArrayList<>();
        menuAll = menuMapper.selectAll();
        List<SysRep> sysReps = roleMenuMapper.selectSysByRole(roleId);
        List<Menu> menuList = new ArrayList<>();
        menuList = menuMapper.selectMenuByRoleId(roleId);
        //根据sysid获取系统菜单
        List<Map<String, String>> sysList = new ArrayList<>();
        for (Menu menu : menuList) {
            Map map = new HashMap();
            String sysCode = menu.getSysId();
            String sysName = menu.getSysName();
            map.put("sysCode", sysCode);
            map.put("sysName", sysName);
            sysList.add(map);
        }
        HashSet h = new HashSet(sysList);
        sysList.clear();
        sysList.addAll(h);
        MenuSysReps menuSysReps = new MenuSysReps();

        List<MenuSysRep> menuSysRepList = new ArrayList<>();
        for (Map<String, String> sysId : sysList) {
            boolean isOpen = false;
            for (SysRep sysRep : sysReps) {
                if (sysRep.getSysId().equals(sysId.get("sysCode"))) {
                    isOpen = true;
                    break;
                }
            }
            List<MenuRepNew> menuRep = new ArrayList<>();
            menuRep = getSysMenu(menuList, "#", sysId.get("sysCode"), isOpen);
            MenuSysRep menuSysRep = new MenuSysRep();
            menuSysRep.setSysId(sysId.get("sysCode"));
            menuSysRep.setSysName(sysId.get("sysName"));
            menuSysRep.setList(menuRep);
            //menuSysRep.setOpen(isOpen ? 1 : 0);
            menuSysRep.setOpen(1);
            //筛选出选中的菜单start
            List<String> checkedList = new ArrayList<>();
            for (Menu menu : menuList) {
                if (menu.getSysId().equals(sysId.get("sysCode"))) {
                    int isChecked = menu.getIsChecked();
                    if (isChecked == 1) {
                        String menuId = menu.getId();
                        boolean flag = false;
                        for (Menu item : menuAll) {
                            if (menuId.equals(item.getParentId())) {
                                flag = true;
                                break;
                            }
                        }
                        if (!flag) {
                            checkedList.add(menuId);
                        }
                    }
                }
            }
            menuSysRep.setCheckedList(checkedList);
            //筛选出选中的菜单end
            menuSysRepList.add(menuSysRep);
        }

		/*if (menuSysRepList.size() == 0 || menuSysRepList == null) {
			throw new GlobalException(CommonEnum.MENU_NO_FOUND.getIndex(), CommonEnum.MENU_NO_FOUND.getValue());
		}*/
        //判断子菜单是否全部选择start
        /*for (MenuSysRep menuSysRep : menuSysRepList) {
            List<MenuRepNew> menuRepNewList = menuSysRep.getList();
            for (MenuRepNew oneMenuRep : menuRepNewList) {
                List<MenuRepNew> twoMenuRep = oneMenuRep.getChildren();
                if (twoMenuRep != null && twoMenuRep.size() > 0 ) {
                    oneMenuRep.setChecked(2);
                    for (MenuRepNew threeMenuRep : twoMenuRep) {
                        List<MenuRepNew> fourMenuRep = threeMenuRep.getChildren();
                        if (fourMenuRep != null && fourMenuRep.size() > 0) {
                            threeMenuRep.setChecked(2);
                            for (MenuRepNew fiveMenuRep : fourMenuRep) {
                                List<MenuRepNew> sixMenuRep = fiveMenuRep.getChildren();
                                if (sixMenuRep != null && sixMenuRep.size() > 0) {
                                    fiveMenuRep.setChecked(2);
                                    for (MenuRepNew senvenMenuRep : sixMenuRep) {
                                        List<MenuRepNew> eightMenuRep = senvenMenuRep.getChildren();
                                        if (eightMenuRep != null && eightMenuRep.size() > 0) {
                                            senvenMenuRep.setChecked(2);
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }*/
        //判断子菜单是否全部选择end
        menuSysReps.setRoleId(roleId);
        menuSysReps.setTotal(menuSysRepList.size());
        menuSysReps.setList(menuSysRepList);
        return menuSysReps;
    }

    @Override
    public void saveRoleMenu(MenuReq menuReq) {
        Role role = roleService.selectByPrimaryKey(menuReq.getRoleId());
        if (role == null) {
            throw new GlobalException(ROLE_NO_FOUND.getIndex(),
                    ROLE_NO_FOUND.getValue(), null, "角色管理-保存权限");
        }
        //test start
        /*List<Menu> menuList = new ArrayList<>();
        menuList = menuMapper.selectAll();
        String tt = "37b5eb0f2b744653b9b9a22f9d6cbd90";
        List<String> newList = new ArrayList<>();
        for (tt = "84cde0bed53a48408f9e6014f72a0e9c";!"#".equals(tt);){
            for(Menu menu:menuList){
                if(tt.equals(menu.getId())){
                    newList.add(menu.getParentId());
                    tt = menu.getParentId();
                    break;
                }
            }
        }
        System.out.println("hello==="+ Arrays.toString(newList.toArray()));*/
        //test end
        //增加角色菜单关联表
        roleMenuMapper.deleteByRoleId(menuReq.getRoleId());
        List<Object> menuList = menuReq.getMenuReq();
        if (!(menuList == null || menuList.size() == 0)) {
            List<RoleMenuKey> roleMenus = new ArrayList<>();
            menuList.forEach(menu -> {
                roleMenus.add(new RoleMenuKey(menuReq.getRoleId(), String.valueOf(menu)));
            });
            roleMenuMapper.insertBatch(roleMenus);
        }
        //增加角色系统关联表
        roleMenuMapper.deleteSysByRoleId(menuReq.getRoleId());
        List<Object> sysList = menuReq.getSysReq();
        if (!(sysList == null || sysList.size() == 0)) {
            List<RoleSysKey> roleSys = new ArrayList<>();
            sysList.forEach(sys -> {
                roleSys.add(new RoleSysKey(menuReq.getRoleId(), String.valueOf(sys)));
            });
            roleMenuMapper.insertRoleSysBatch(roleSys);
        }

    }

    @Override
    public List<MenuRep> getMenuTreeBySysId(String userId, String sysId) {
        List<Menu> menuList = menuMapper.selectMenusBySysId(userId, sysId);
        return getAllMenuRep(menuList, PARENT_ID, "0");
    }

    @Override
    public ButtonReps selectButtonByMenuId(String menuId, String userId) {
        List<Menu> menuList = menuMapper.selectButtonByMenuId(menuId, userId);
        ButtonReps buttonReps = new ButtonReps();
        buttonReps.setList(menuList);
        buttonReps.setTotal(menuList.size());
        buttonReps.setMenuId(menuId);
        return buttonReps;
    }

    @Override
    public List<MenuRep> findMenuAllTree() {
        MenuExample example = new MenuExample();
        example.createCriteria().
                andIsDelEqualTo(ConstantPool.NOT_DEL).
                andStatusEqualTo((short) 0).
                andTypeEqualTo((short) 0);
        List<Menu> menus = menuMapper.selectByExample(example);
        return getAllMenuRep(menus, PARENT_ID, ConstantPool.REQUEST_MODE_ALL);
    }

    public List<MenuRepNew> getSysMenu(List<Menu> menuList, String parentId, String sysId,
                                       boolean isOpen) {
        List<MenuRepNew> menuReps = new ArrayList<>();
        for (Menu menu : menuList) {
            if (menu == null) {
                continue;
            }
            if (menu.getParentId().equals(parentId) && menu.getSysId().equals(sysId)) {
                MenuRepNew menuRep = new MenuRepNew(menu);
                if (menu.getType() == 0) {
                    menuRep.setIcon("&#xe62d;");
                } else if (menu.getType() == 1) {
                    menuRep.setIcon("&#xe760;");
                }
                //menuRep.setDisabled(!isOpen);
                //目前只有一个系统,所有菜单均可点击
                menuRep.setDisabled(false);
                List<MenuRepNew> allMenuRep;
                allMenuRep = getSysMenu(menuList, menu.getId(), menu.getSysId(), isOpen);
                menuRep.setChildren(allMenuRep);
                menuReps.add(menuRep);
            }
        }
        return menuReps;
    }

    @Override
    public ButtonRep getUserButtons(String userId, String parentId) {
        List<String> buttonList = menuMapper.selectUserButton(parentId, userId);
        ButtonRep buttonRep = new ButtonRep();
        for (String buttonName : buttonList) {
            if (buttonName.contains("增加") || buttonName.contains("添加") || buttonName.contains("新增")
                    || buttonName.contains("新建")) {
                buttonRep.setAdd(true);
            }

            if (buttonName.contains("改") || buttonName.contains("编辑")) {
                buttonRep.setUpdate(true);
            }

            if (buttonName.contains("删")) {
                buttonRep.setDelete(true);
            }

            if (buttonName.contains("导入")) {
                buttonRep.setInput(true);
            }

            if (buttonName.contains("导出") || buttonName.contains("下载")) {
                buttonRep.setOutput(true);
            }
        }
        return buttonRep;
    }

    @Override
    public MenuReps searchSubMenu() {
        List<Menu> menus = menuMapper.selectSubMenu();
        List<MenuRep> list = new ArrayList<>();
        menus.forEach(menu -> list.add(new MenuRep(menu)));
        return new MenuReps(list, list.size());
    }

    @Override
    public List<MenuItemRep> getUserMenuTreeByUserId(String userId, String sysId) {
        List<Menu> menuList = menuMapper.selectMenuItemByUserId(userId, sysId);
        List<MenuItemRep> menuItemReps = getAllMenuRep(menuList, PARENT_ID);
        return menuItemReps;
    }

    public List<MenuItemRep> getAllMenuRep(List<Menu> menuList, String parentId) {
        List<MenuItemRep> menuReps = new ArrayList<>();
        for (Menu menu : menuList) {
            if (menu == null) {
                continue;
            }
            if (menu.getParentId().equals(parentId) && menu.getType() == 0) {
                MenuItemRep menuRep = new MenuItemRep(menu);
                List<MenuItemRep> allMenuRep;
                allMenuRep = getAllMenuRep(menuList, menu.getId());
                menuRep.setChildrenMenus(allMenuRep);
                List<String> buttons = new ArrayList<>();
                String buttonName = menu.getButtonName();
                if (buttonName != null) {
                    String[] button = buttonName.split("\\,");
                    for (String name : button) {
                        if (name.contains("增加") || name.contains("添加") || name.contains("新增")
                                || name.contains("新建") || name.contains("保存")) {
                            buttons.add("save");
                        }
                        if (name.contains("改") || name.contains("编辑")) {
                            buttons.add("update");
                        }
                        if (name.contains("删")) {
                            buttons.add("delete");
                        }
                        if (name.contains("导入")) {
                            buttons.add("import");
                        }
                        if (name.contains("导出") || name.contains("下载")) {
                            buttons.add("export");
                        }
                        if (name.contains("解除")) {
                            buttons.add("relieve");
                        }
                        if (name.contains("恢复")) {
                            buttons.add("recover");
                        }
                        if (name.contains("撤回")) {
                            buttons.add("recall");
                        }
                        if (name.contains("发布")) {
                            buttons.add("publish");
                        }
                        if (name.contains("批量撤回")) {
                            buttons.add("batchRecall");
                        }
                        if (name.contains("批量发布")) {
                            buttons.add("batchPublish");
                        }
                        if (name.contains("重置密码")) {
                            buttons.add("resetPwd");
                        }
                        if (name.contains("应用")) {
                            buttons.add("apply");
                        }
                    }
                    menuRep.setAuth(buttons);
                }

                menuReps.add(menuRep);
            }
        }
        return menuReps;
    }

}
