package com.hywa.map.service.login.impl;

import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.hywa.map.common.ConstantPool;
import com.hywa.map.common.enums.CommonEnum;
import com.hywa.map.common.exception.GlobalException;
import com.hywa.map.common.utils.StringUtils;
import com.hywa.map.common.utils.UUIDUtils;
import com.hywa.map.dao.entity.Role;
import com.hywa.map.dao.entity.RoleExample;
import com.hywa.map.dao.mapper.RoleMapper;
import com.hywa.map.dao.mapper.RoleMenuMapper;
import com.hywa.map.dao.mapper.UserRoleMapper;
import com.hywa.map.representation.RoleRep;
import com.hywa.map.representation.RoleReps;
import com.hywa.map.service.login.RoleService;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import static com.hywa.map.common.enums.CommonEnum.ILLEGAL_PARAMETER_ERROR;

@Slf4j
@Service
@Transactional
public class RoleServiceImpl implements RoleService {

    @Autowired
    private RoleMapper roleMapper;

    @Autowired
    private RoleMenuMapper roleMenuMapper;

    @Autowired
    private UserRoleMapper userRoleMapper;

    @Override
    @Transactional
    public void save(RoleRep roleRep) {
    	if(!nameIsEmpty(roleRep.getName(),null)) {
    		throw new GlobalException(CommonEnum.ROLE_NAME_REPEATED.getIndex(), CommonEnum.ROLE_NAME_REPEATED.getValue(),null,"角色管理-新增");
    	}
        Role role = new Role();
        BeanUtils.copyProperties(roleRep, role, "code", "id");
        role.setCreateTime(new Date());
        role.setId(UUIDUtils.randomUUID());
        role.setIsDel(ConstantPool.NOT_DEL);
        role.setStatus(0);
        roleMapper.insert(role);

        /*List<RoleMenuKey> roleMenus = new ArrayList<>();
        roleRep.getMenuReps().forEach(menu->{
        	roleMenus.add(new RoleMenuKey(role.getId(),String.valueOf(menu)));
        });
        roleMenuMapper.insertBatch(roleMenus);*/
    }

    @Override
    @Transactional
    public int update(RoleRep roleRep) {
        /*if(roleRep.getMenuReps()==null) {
            throw new GlobalException(CommonEnum.ROLE_MENU_NOT_FOUND.getIndex(), CommonEnum.ROLE_MENU_NOT_FOUND.getValue());
        }
        roleMenuMapper.deleteByRoleId(roleRep.getId());
        List<RoleMenuKey> roleMenus = new ArrayList<>();
        roleRep.getMenuReps().forEach(menu->{
        	roleMenus.add(new RoleMenuKey(roleRep.getId(),String.valueOf(menu)));
        });
        if(!CollectionUtils.isEmpty(roleMenus)) {
        	roleMenuMapper.insertBatch(roleMenus);
        }*/
        Role role = new Role();
        role.setId(roleRep.getId());
        role.setUpdateTime(new Date());
        role.setUpdateUser(roleRep.getUpdateUser());
        role.setDescription(roleRep.getDescription());
        role.setName(roleRep.getName());
        String status = roleRep.getStatus()+"";
        if("2".equals(status) || "0".equals(status)){
            role.setName(null);
            role.setDescription(null);
            role.setStatus(roleRep.getStatus());
        } else {
            if (StringUtils.isEmpty(roleRep.getName())) {
                throw new GlobalException(ILLEGAL_PARAMETER_ERROR.getIndex(),
                        ILLEGAL_PARAMETER_ERROR.getValue() + ",角色名称为空",null,"角色管理-修改");
            }
            if (StringUtils.isEmpty(roleRep.getDescription())) {
                throw new GlobalException(ILLEGAL_PARAMETER_ERROR.getIndex(),
                        ILLEGAL_PARAMETER_ERROR.getValue() + ",角色描述为空",null,"角色管理-修改");
            }
            if(!nameIsEmpty(roleRep.getName(),roleRep.getId())) {
                throw new GlobalException(CommonEnum.ROLE_NAME_REPEATED.getIndex(), CommonEnum.ROLE_NAME_REPEATED.getValue(),null,"角色管理-修改");
            }
        }
        return roleMapper.updateByPrimaryKeySelective(role);
    }

    @Override
    @Transactional
    public void delete(String id) {
        if(!userIsEmpty(id)) {
            throw new GlobalException(CommonEnum.ROLE_USER_FOUND.getIndex(), CommonEnum.ROLE_USER_FOUND.getValue(),null,"角色管理-删除");
        }
        roleMapper.deleteByPrimaryKey(id);
        roleMenuMapper.deleteByRoleId(id);
        userRoleMapper.deleteByRoleId(id);
        roleMenuMapper.deleteSysByRoleId(id);
    }

    @Override
    public RoleReps selectAll(String name,String flag) {
        List<Role> roles = null;
        String status = StringUtils.isEmpty(flag) ? null : "2";
        roles = roleMapper.selectAll(name,status);
        List<RoleRep> roleReps = new ArrayList<>();
        roles.forEach(role -> roleReps.add(new RoleRep(role)));
        return new RoleReps(roleReps, new PageInfo<>(roles).getTotal());
    }

    @Override
    public RoleReps selectRolesByUserId(int pageNo, int pageSize, String userId) {
        PageHelper.startPage(pageNo, pageSize, true);
        List<Role> roles = roleMapper.selectByUserId(userId);
        List<RoleRep> roleReps = new ArrayList<>();
        roles.forEach(role -> roleReps.add(new RoleRep(role)));
        return new RoleReps(roleReps, new PageInfo<>(roles).getTotal());
    }
    
    @Override
	public boolean nameIsEmpty(String name,String id) {
		RoleExample roleExample = new RoleExample();
		RoleExample.Criteria criteria = roleExample.createCriteria();
		criteria.andNameEqualTo(name);
		criteria.andIsDelEqualTo(ConstantPool.NOT_DEL);
		if(StringUtils.isNotEmpty(id)) {
			criteria.andIdNotEqualTo(id);
		}
		return roleMapper.countByExample(roleExample) > 0 ? false : true;
	/*	UserExample userExample = new UserExample();
		UserExample.Criteria criteria = userExample.createCriteria();
		criteria.andNameEqualTo(username);
		criteria.andIsDelEqualTo(ConstantPool.NOT_DEL);
		if(StringUtils.isNotEmpty(id)) {
			criteria.andIdNotEqualTo(id);
		}
		return userMapper.countByExample(userExample) > 0 ? false : true;*/
	}

	@Override
	public RoleRep findById(String id) {
		Role role=roleMapper.findById(id);
		return null==role?null:new RoleRep(role);
	}

    @Override
    public RoleReps selectRolesByArea(int pageNo, int pageSize, String userId,String regionType) {
        //PageHelper.startPage(pageNo, pageSize, true);
        List<Role> roles = null;
        if("1".equals(regionType)){
            roles = roleMapper.selectProvinceRole(userId);
        } else if("2".equals(regionType)){
            roles = roleMapper.selectCityRole(userId);
        } else {
            roles = roleMapper.selectCountyRole(userId);
        }
        List<RoleRep> roleReps = new ArrayList<>();
        roles.forEach(role -> roleReps.add(new RoleRep(role)));
        return new RoleReps(roleReps, new PageInfo<>(roles).getTotal());
    }

    @Override
    public Role selectByPrimaryKey(String id) {
        return roleMapper.selectByPrimaryKey(id);
    }

    public boolean userIsEmpty(String roleId) {
        return roleMapper.countByRoleId(roleId) > 0 ? false : true;
    }

}
