package com.hywa.map.service.login.impl;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import com.hywa.map.common.utils.DateUtils;
import com.hywa.map.common.utils.UUIDUtils;
import com.hywa.map.config.EnvProperties;
import com.hywa.map.dao.entity.*;
import com.hywa.map.dao.mapper.*;
import com.hywa.map.representation.*;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.hywa.map.common.ConstantPool;
import com.hywa.map.common.enums.CommonEnum;
import com.hywa.map.common.exception.GlobalException;
import com.hywa.map.common.utils.StringUtils;
import com.hywa.map.representation.ext.UserRepExt;
import com.hywa.map.service.login.UserAreaService;
import com.hywa.map.service.login.UserRoleService;
import com.hywa.map.service.login.UserService;

import static com.hywa.map.common.enums.CommonEnum.*;

@Slf4j
@Service
@Transactional
public class UserServiceImpl implements UserService {

    @Autowired
    UserMapper userMapper;

    @Autowired
    RoleMapper roleMapper;

    @Autowired
    UserAreaMapper userAreaMapper;

    @Autowired
    private StringRedisTemplate redis;

    @Autowired
    UserRoleMapper userRoleMapper;

    @Autowired
    private UserAreaService userAreaService;

    @Autowired
    UserRoleService userRoleService;

    @Autowired
    private MenuMapper menuMapper;
    @Autowired
    EnvProperties envProperties;
    @Autowired
    UserAdviceMapper userAdviceMapper;


    @Override
    public void save(UserRepExt userRep) {
        User user = new User(userRep);
        if (!StringUtils.isEmpty(userRep.getName()) && !nameIsEmpty(userRep.getName(), null)) {
            throw new GlobalException(USER_NAME_REPEATED.getIndex(), USER_NAME_REPEATED.getValue(), user.getCreateUser(), "账户管理-新增账户");
        }
        if (!StringUtils.isEmpty(userRep.getPhone()) && !StringUtils.isCellPhone(userRep.getPhone())) {
            throw new GlobalException(TELEPHONE_FORMAL_ERROR.getIndex(), TELEPHONE_FORMAL_ERROR.getValue(), user.getCreateUser(), "账户管理-新增账户");
        }
        if (!CollectionUtils.isEmpty(userRep.getRoles())) {
            List<UserRoleKey> userRoleKeys = new ArrayList<>();
            userRep.getRoles().forEach(roles -> userRoleKeys.add(new UserRoleKey(user.getId(), roles)));
            userRoleService.insertBatch(userRoleKeys);
        }
        userMapper.insert(user);
        userRep.setUserId(user.getId());
        String regionId;
        short type;
        if (StringUtils.isNotEmpty(userRep.getCountyId())) {
            type = (short) 3;
            regionId = userRep.getCountyId();
        } else if (StringUtils.isNotEmpty(userRep.getCityId())) {
            type = (short) 2;
            regionId = userRep.getCityId();
        } else {
            type = (short) 1;
            regionId = userRep.getProvinceId();
        }
        userAreaMapper.insert(new UserArea(user.getId(), regionId, type));
    }

    @Override
    public UserRep findById(String userId) {
        User user = userMapper.selectByPrimaryKey(userId);
        String regionId = userAreaService.findRegionId(user.getId());

        return new UserRep(user.getId(), user.getName(), user.getTelephone(), user.getSex(), user.getJob(),
                user.getWorkUnit(), user.getAge(), regionId, user.getRealName(),
                user.getRoleName(), user.getAreaName());
    }

    @Override
    public UserReps findUsers(int pageNo, int pageSize, String name, String region, String workUnit, String userName, String userId) {
        PageHelper.startPage(pageNo, pageSize, true);
        PageHelper.orderBy("create_time desc");
        List<User> users = null;
        users = userMapper.selectUserBySelf(name, null);
        PageInfo<User> page = new PageInfo<>(users);
        List<UserRep> userRepList = new ArrayList<>();
        for (User user : users) {
            UserRep userRep = new UserRep(user);
            userRepList.add(userRep);
        }
        return new UserReps(page.getTotal(), userRepList);
    }

    @Override
    public User findByName(String userName) {
        return userMapper.selectByUserName(userName);
    }

    @Override
    public UserRep userLogin(String userName, String psw) {
        String menuName="用户登录-登录";
        User user = userMapper.selectByUserName(userName);
        if (user == null) {
            // 用户未注册
            throw new GlobalException(USER_UNREGISTERED_ERROR.getIndex(), USER_UNREGISTERED_ERROR.getValue(), userName, menuName);
        }
        user.setAreaName(userMapper.finAreaNameById(user.getId()));
        if (user.getPassword().equals(StringUtils.md5(psw))) {
            if (user.getIsDel().equals(ConstantPool.DEL)) {
                // 用户已经注销，请联系管理员
                throw new GlobalException(USER_CANCEL.getIndex(), USER_CANCEL.getValue(), userName, menuName);
            } else {
                String regionId = null;
                try {
                    regionId = userAreaService.findRegionId(user.getId());
                } catch (Exception e) {
                    // 此账户未绑定区域，请联系管理员！
                    throw new GlobalException(UNBOUND_REGION_ERROR.getIndex(), UNBOUND_REGION_ERROR.getValue(), userName, menuName);
                }
                UserRep userRep = new UserRep(user.getId(), user.getName(), user.getTelephone(), user.getSex(),
                        user.getJob(), user.getWorkUnit(), user.getAge(), regionId, user.getRealName(),
                        user.getRoleName(), user.getAreaName());
                return userRep;
            }
        } else {
            // 密码错误
            throw new GlobalException(PASSWORD_ERROR.getIndex(), PASSWORD_ERROR.getValue(), userName, menuName);
        }
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean delete(String userId) {
        if (StringUtils.isEmpty(userId)) {
            throw new GlobalException(ILLEGAL_PARAMETER_ERROR.getIndex(), "userId不能为空", null, "账户管理-删除账户");
        }
        if (!(userMapper.deleteByUserId(userId) > 0)) {
            return false;
        }
        userAreaMapper.deleteByUserId(userId);
        userRoleMapper.deleteByUserId(userId);
        return true;
    }

    @Override
    public boolean nameIsEmpty(String username, String id) {
        UserExample userExample = new UserExample();
        UserExample.Criteria criteria = userExample.createCriteria();
        criteria.andNameEqualTo(username);
        criteria.andIsDelEqualTo(ConstantPool.NOT_DEL);
        if (StringUtils.isNotEmpty(id)) {
            criteria.andIdNotEqualTo(id);
        }
        return userMapper.countByExample(userExample) > 0 ? false : true;
    }

    @Override
    public void updateUserRole(String userId, List<String> roleReps) {
        if (StringUtils.isEmpty(userId)) {
            throw new GlobalException(ILLEGAL_PARAMETER_ERROR.getIndex(), "userId不能为空", null, "账户管理-修改账户");
        }
        userRoleMapper.deleteByUserId(userId);
        List<UserRoleKey> userRoleKeys = new ArrayList<>();
        for (String roleId : roleReps) {
            UserRoleKey userRoleKey = new UserRoleKey();
            userRoleKey.setRoleId(roleId);
            userRoleKey.setUserId(userId);
            userRoleKeys.add(userRoleKey);
        }
        userRoleMapper.insertBatch(userRoleKeys);
    }

    @Override
    public boolean phoneIsEmpty(String phone) {
        UserExample userExample = new UserExample();
        UserExample.Criteria criteria = userExample.createCriteria();
        criteria.andTelephoneEqualTo(phone);
        criteria.andIsDelEqualTo(ConstantPool.NOT_DEL);
        return userMapper.countByExample(userExample) > 0 ? false : true;
    }

    @Override
    public Map<String, Object> findAllById(String id) {
        Map<String, Object> map = new HashMap<>();
        map = userMapper.selectByIdUserInfo(id);

        if (CollectionUtils.isEmpty(map)) {
            return null;
        }
        if ((StringUtils.isNotEmpty((String) map.get("region_id")) && null != map.get("type"))) {
            map.putAll(userMapper.selectByIdUserArea((String) map.get("region_id"), (int) map.get("type")));
        }
        return reMap(map);
    }

    // 转换格式
    private Map<String, Object> reMap(Map<String, Object> map) {
        Map<String, Object> reField = new HashMap<String, Object>();// 返回字段
        reField.put("id", map.get("id"));// 主键
        reField.put("name", map.get("name"));// 姓名
        reField.put("sex", 1 == (int) map.get("sex") ? "男" : "女");// 1男 2女
        reField.put("phone", map.get("telephone"));// 电话
        reField.put("account", map.get("account"));// MD5加密
        reField.put("email", map.get("email"));// 邮箱号码
        reField.put("updateUser", map.get("update_user"));// 更新人
        reField.put("createUser", map.get("create_user"));// 创建人
        reField.put("isDel", map.get("is_del"));// 0 未删 1 删除
        reField.put("jobTitle", map.get("job"));// 职务
        reField.put("type", map.get("type"));// 0 未删 1 删除
        reField.put("region", map.get("region_id"));//
        reField.put("workUnit", map.get("work_unit"));// 工作单位
        reField.put("age", map.get("age"));// 年龄
        reField.put("countyId", map.get("countyId"));// 区县id
        reField.put("cityId", map.get("cityId"));// 市id
        reField.put("provinceId", map.get("provinceId"));// 省id
        reField.put("countyName", map.get("countyName"));// 区县名
        reField.put("cityName", map.get("cityName"));// 市名
        reField.put("provinceName", map.get("provinceName"));// 省名
        reField.put("roles", map.get("roles"));// 角色id
        reField.put("realName", map.get("real_name"));// 姓名
        reField.put("spotId", map.get("spotId"));// 收购点id
        reField.put("branchId", map.get("branch_id"));// 机构id
        String icon = (String) map.get("icon");
        String fileUrl=null;
        if (StringUtils.isNotEmpty(icon)) {
            icon = icon.substring(icon.lastIndexOf("/") + 1);
            fileUrl = envProperties.getLocalHttp() + envProperties.getComFileDownloadUrl() + icon;
        }
        reField.put("icon", fileUrl);// 用户头像URL
        return reField;// 返回字段
    }

    @Override
    public void update(UserRepExt userRep, String userId) {
        if (!StringUtils.isEmpty(userRep.getName()) && !nameIsEmpty(userRep.getName(), userRep.getUserId())) {
            throw new GlobalException(USER_NAME_REPEATED.getIndex(), USER_NAME_REPEATED.getValue(), null, "账户管理-修改账户");
        }
        if (!StringUtils.isEmpty(userRep.getPhone()) && !StringUtils.isCellPhone(userRep.getPhone())) {
            throw new GlobalException(TELEPHONE_FORMAL_ERROR.getIndex(), TELEPHONE_FORMAL_ERROR.getValue(), null, "账户管理-修改账户");
        }
        User user = new User();
        user.setJob(userRep.getJobTitle());
        if (!StringUtils.isEmpty(userRep.getSex())) {
            user.setSex(CommonEnum.getIndex(userRep.getSex()));
        }
        user.setName(userRep.getName());
        user.setAge(userRep.getAge());
        user.setWorkUnit(userRep.getWorkUnit());
        user.setId(userRep.getUserId());
        user.setUpdateTime(new Date());
        user.setTelephone(userRep.getPhone());
        user.setUpdateUser(userId);
        user.setRealName(userRep.getRealName());
        user.setStatus(userRep.getStatus());
        user.setUserType(userRep.getUserType());
        user.setRemark(userRep.getRemark());
        user.setPassword(userRep.getPwd());
        user.setBranchId(userRep.getBranchId());

        userMapper.updateByPrimaryKey(user);
        updateUserArea(userRep);
        updateUserRole(userRep.getUserId(), userRep.getRoles());
    }

    private void updateUserArea(UserRepExt userRep) {
        short type;
        String regionId;

        UserArea record = new UserArea();
        record.setUserId(userRep.getUserId());

        if (StringUtils.isNotEmpty(userRep.getCountyId())) {
            type = (short) 3;
            regionId = userRep.getCountyId();
        } else if (StringUtils.isNotEmpty(userRep.getCityId())) {
            type = (short) 2;
            regionId = userRep.getCityId();
        } else {
            type = (short) 1;
            regionId = userRep.getProvinceId();
        }

        record.setType(type);
        record.setRegionId(regionId);
        userAreaMapper.updateByPrimaryKey(record);
    }

    @Override
    public void changePwd(NewUserInfoRep newUserInfoRep) {
        User user = userMapper.selectByPrimaryKey(newUserInfoRep.getId());
        if (null == user) {
            throw new GlobalException(USER_INFO_ERROR.getIndex(), USER_INFO_ERROR.getValue(), null, "系统设置-修改密码");
        }
        if (!user.getPassword().equals(StringUtils.md5(newUserInfoRep.getPassword()))) {
            throw new GlobalException(PASSWORD_ERROR.getIndex(), PASSWORD_ERROR.getValue(), null, "系统设置-修改密码");
        }
        User newUser = new User();
        newUser.setId(user.getId());
        newUser.setPassword(StringUtils.md5(newUserInfoRep.getNewpassword()));
        userMapper.updateByPrimaryKeySelective(newUser);
    }

    @Override
    public void setAuthority(String userId) {
        //获取用户权限存放于redis start
        List<Menu> menuList = menuMapper.selectMenusByUserId(userId);
        String userAuthKey = userId + "_authority";
        redis.opsForSet().getOperations().delete(userAuthKey);
        for (Menu menu : menuList) {
            String method = "";
            String menuName = menu.getName();
            if (menuName.indexOf("保存") >= 0 || menuName.indexOf("新增") >= 0 || menuName.indexOf("增加") >= 0 || menuName.indexOf("添加") >= 0 || menuName.indexOf("导入") >= 0 || menuName.indexOf("举报") >= 0 || menuName.indexOf("发布") >= 0) {
                method = "POST";
            } else if (menuName.indexOf("修改") >= 0 || menuName.indexOf("更新") >= 0 || menuName.indexOf("变更") >= 0) {
                method = "PUT";
            } else if (menuName.indexOf("删除") >= 0 || menuName.indexOf("去除") >= 0 || menuName.indexOf("删掉") >= 0) {
                method = "DELETE";
            } else {
                method = "GET";
            }
            String description = menu.getDescription();
            description = description.endsWith("/") ? description.substring(0, description.length() - 1) : description;
            redis.opsForSet().add(userAuthKey, description + "|" + method);
        }
        //获取用户权限存放于redis end
    }

    @Override
    public boolean checkArea(String userId, String regionId) {
        Map<String, Object> userMap = findAllById(userId);
        String regionType = userMap.get("type") + "";
        String areaId = userMap.get("region") + "";
        String userName = (String) userMap.get("name");
        if ("admin".equals(userName) || areaId.equals(regionId)) {
            return true;
        }
        if ("1".equals(regionType)) {
            return userMapper.isBelongProvince(areaId, regionId) > 0 ? true : false;
        } else if ("2".equals(regionType)) {
            return userMapper.isBelongCity(areaId, regionId) > 0 ? true : false;
        } else {
            return false;
        }
    }

    @Override
    public boolean checkAreaRole(String roleId, String regionId) {
        return userMapper.matchAreaRole(roleId, regionId) > 0 ? true : false;
    }

    @Override
    public List<SysRep> findSysInfoByUserId(String userId, String userName) {
        return userMapper.findSysInfoByUserId(userId, userName);
    }

    @Override
    public List<SysRep> findSysInfoAllByUserId(String userId, String userName) {
        return userMapper.findSysInfoAllByUserId(userId, userName);
    }

    @Override
    public void disableUser(Map map) {
        userMapper.disableUserById(map);
    }


    @Override
    public boolean isInformationSet() {
        return userMapper.isInformationSet() > 0;
    }

    @Override
    public void insertIcon(String userId, String icon) {
        if (StringUtils.isEmpty(icon)) {
            throw new GlobalException(USER_ICON_NO_FOUND.getIndex(), USER_ICON_NO_FOUND.getValue(), null, "个人信息-上传头像");
        }
        userMapper.deleteIcon(userId);
        UserIcon entity = new UserIcon();
        entity.setUserId(userId);
        entity.setContent(icon);
        userMapper.insertIcon(entity);
    }

    @Override
    public void updatePush(SetReq setReq) {
        if (StringUtils.isEmpty(setReq.getIsPush())) {
            throw new GlobalException(ILLEGAL_PARAMETER_ERROR.getIndex(), ILLEGAL_PARAMETER_ERROR.getValue() + "，消息通知推送标志不能为空", null, "系统设置-消息通知推送");
        }
        userMapper.updatePush(setReq.getId(), setReq.getIsPush());
        log.info("设置消息通知推送完成，请求数据：" + setReq);
    }

    @Override
    public void updatePersonInfo(UserRepExt userRep, String userId) {
        if (!StringUtils.isEmpty(userRep.getPhone()) && !StringUtils.isCellPhone(userRep.getPhone())) {
            throw new GlobalException(TELEPHONE_FORMAL_ERROR.getIndex(), TELEPHONE_FORMAL_ERROR.getValue(), null, "账户管理-修改账户");
        }
        User user = new User();
        if (!StringUtils.isEmpty(userRep.getSex())) {
            user.setSex(CommonEnum.getIndex(userRep.getSex()));
        }
        user.setId(userId);
        user.setRealName(userRep.getRealName());
        user.setTelephone(userRep.getPhone());

        userMapper.updatePersonInfo(user);
    }

    @Override
    public void resetPwd(NewUserInfoRep newUserInfoRep) {
        User user = userMapper.selectByPrimaryKey(newUserInfoRep.getId());
        if (null == user) {
            throw new GlobalException(USER_INFO_ERROR.getIndex(), USER_INFO_ERROR.getValue(), null, "系统设置-修改密码");
        }
        User newUser = new User();
        newUser.setId(user.getId());
        newUser.setPassword(StringUtils.md5(newUserInfoRep.getNewpassword()));
        userMapper.updateByPrimaryKeySelective(newUser);
        log.info("用户管理-重置密码完成，请求数据：id|" + user.getId());
    }
}
