package com.hywa.map.service.map.impl;

import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.hywa.map.common.ConstantPool;
import com.hywa.map.common.exception.GlobalException;
import com.hywa.map.common.utils.StringUtils;
import com.hywa.map.common.utils.UUIDUtils;
import com.hywa.map.config.EnvProperties;
import com.hywa.map.dao.entity.Combox;
import com.hywa.map.dao.entity.MapInfo;
import com.hywa.map.dao.entity.MapModuleData;
import com.hywa.map.dao.mapper.MapInfoMapper;
import com.hywa.map.dao.mapper.MapModuleDataMapper;
import com.hywa.map.representation.*;
import com.hywa.map.service.map.MapModuleDataService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import static com.hywa.map.common.enums.CommonEnum.ILLEGAL_PARAMETER_ERROR;
import static com.hywa.map.common.enums.CommonEnum.MAP_NAME_FOUND;

/**
 * t_map_module_data  对应ServiceImpl 实现类
 *
 * @author hywa
 * @create by 2019-09-17 13:35:20
 */
@Slf4j
@Service
public class MapModuleDataServiceImpl implements MapModuleDataService {

    @Autowired
    private MapModuleDataMapper mapModuledataMapper;
    @Autowired
    private MapInfoMapper mapInfoMapper;
    @Autowired
    EnvProperties envProperties;

    @Override
    public int saveMap(MapInfoReq reqInfo, String userId) {
        String menuName = "一张图-新增大屏信息";
        if (StringUtils.isEmpty(reqInfo.getMapName())) {
            throw new GlobalException(ILLEGAL_PARAMETER_ERROR.getIndex(), ILLEGAL_PARAMETER_ERROR.getValue() + "，大屏名称不能为空", null, menuName);
        }
        if (StringUtils.isEmpty(reqInfo.getTemplateId())) {
            throw new GlobalException(ILLEGAL_PARAMETER_ERROR.getIndex(), ILLEGAL_PARAMETER_ERROR.getValue() + "，套用框架不能为空", null, menuName);
        }
        if (mapInfoMapper.countByName(reqInfo.getMapName(), null) > 0) {
            throw new GlobalException(MAP_NAME_FOUND.getIndex(), MAP_NAME_FOUND.getValue(), null, menuName);
        }
        MapInfo entity = packMapInfoEntity(reqInfo, userId);
        int count = mapInfoMapper.add(entity);
        log.info("一张图-新增大屏信息完成，请求数据|" + reqInfo + "，操作结果|" + count);
        return count;
    }

    @Override
    public int deleteMap(MapInfoReq reqInfo, String userId) {
        String menuName = "一张图-删除大屏信息";
        if (StringUtils.isEmpty(reqInfo.getId())) {
            throw new GlobalException(ILLEGAL_PARAMETER_ERROR.getIndex(), ILLEGAL_PARAMETER_ERROR.getValue() + "，id不能为空", null, menuName);
        }
        int count = mapInfoMapper.put(reqInfo.getId(), userId);
        //mapModuledataMapper.deleteByMapId(reqInfo.getId());
        log.info("一张图-删除大屏信息完成，请求数据|" + reqInfo + "，操作结果|" + count);
        return count;
    }

    @Override
    public int updateMap(MapInfoReq reqInfo, String userId) {
        String menuName = "一张图-更新大屏信息";
        if (StringUtils.isEmpty(reqInfo.getId())) {
            throw new GlobalException(ILLEGAL_PARAMETER_ERROR.getIndex(), ILLEGAL_PARAMETER_ERROR.getValue() + "，id不能为空", null, menuName);
        }
        if (StringUtils.isEmpty(reqInfo.getMapName())) {
            throw new GlobalException(ILLEGAL_PARAMETER_ERROR.getIndex(), ILLEGAL_PARAMETER_ERROR.getValue() + "，大屏名称不能为空", null, menuName);
        }
        if (StringUtils.isEmpty(reqInfo.getTemplateId())) {
            throw new GlobalException(ILLEGAL_PARAMETER_ERROR.getIndex(), ILLEGAL_PARAMETER_ERROR.getValue() + "，套用框架不能为空", null, menuName);
        }
        if (mapInfoMapper.countByName(reqInfo.getMapName(), reqInfo.getId()) > 0) {
            throw new GlobalException(MAP_NAME_FOUND.getIndex(), MAP_NAME_FOUND.getValue(), null, menuName);
        }
        MapInfo entity = packMapInfoEntity(reqInfo, userId);
        entity.setId(reqInfo.getId());
        entity.setUpdateDate(new Date());
        entity.setUpdateUser(userId);
        int count = mapInfoMapper.update(entity);
        log.info("一张图-更新大屏信息完成，请求数据|" + reqInfo + "，操作结果|" + count);
        return count;
    }

    @Override
    public int updateMapState(MapInfoReq reqInfo, String userId) {
        String menuName = "一张图-更新大屏状态信息";
        if (StringUtils.isEmpty(reqInfo.getId())) {
            throw new GlobalException(ILLEGAL_PARAMETER_ERROR.getIndex(), ILLEGAL_PARAMETER_ERROR.getValue() + "，id不能为空", null, menuName);
        }
        if (StringUtils.isEmpty(reqInfo.getState())) {
            throw new GlobalException(ILLEGAL_PARAMETER_ERROR.getIndex(), ILLEGAL_PARAMETER_ERROR.getValue() + "，状态不能为空", null, menuName);
        }
        int count = mapInfoMapper.updateState(reqInfo.getId(), userId, reqInfo.getState());
        log.info("一张图-更新大屏状态信息完成，请求数据|" + reqInfo + "，操作结果|" + count);
        return count;
    }

    @Override
    public MapInfoReps selectMapAll(int pageNo, int pageSize, String mapName) {
        PageHelper.startPage(pageNo, pageSize, true);
        List<MapInfo> list = mapInfoMapper.select(mapName);
        List<MapInfoRep> repList = new ArrayList<>();
        for (MapInfo item : list) {
            item.setImg(envProperties.getLocalHttp() + envProperties.getPubFileDownloadUrl() + item.getImg());
            MapInfoRep rep = new MapInfoRep(item);
            repList.add(rep);
        }
        log.info("查询大屏列表信息完成，请求数据：mapName|" + mapName);
        return new MapInfoReps(new PageInfo<>(list).getTotal(), repList);
    }

    @Override
    public MapInfoRep selectMapById(String id) {
        String menuName = "一张图-根据id查询大屏信息";
        if (StringUtils.isEmpty(id)) {
            throw new GlobalException(ILLEGAL_PARAMETER_ERROR.getIndex(), ILLEGAL_PARAMETER_ERROR.getValue() + "，id不能为空", null, menuName);
        }
        MapInfo entity = mapInfoMapper.selectById(id);
        if (entity == null) {
            return new MapInfoRep();
        }
        MapInfoRep rep = new MapInfoRep(entity);
        return rep;
    }

    @Override
    public ComboxReps selectMapTemplate() {
        List<Combox> list = mapInfoMapper.selectTemplate();
        if (list == null) {
            return null;
        }
        List<ComboxRep> repList = new ArrayList<>();
        for (int i = 0; i < list.size(); i++) {
            Combox combox = list.get(i);
            ComboxRep repObj = new ComboxRep();
            repObj.setId(combox.getId());
            repObj.setName(combox.getName());
            repList.add(repObj);
        }
        return new ComboxReps((long) repList.size(), repList);
    }

    @Override
    public int saveOrUpdate(MapModuleDataReq reqInfo, String userId) {
        String menuName = "一张图-新增组件配置数据信息";
        if (StringUtils.isEmpty(reqInfo.getModuleCode())) {
            throw new GlobalException(ILLEGAL_PARAMETER_ERROR.getIndex(), ILLEGAL_PARAMETER_ERROR.getValue() + "，组件编码不能为空", null, menuName);
        }
        if (StringUtils.isEmpty(reqInfo.getDataType() + "")) {
            throw new GlobalException(ILLEGAL_PARAMETER_ERROR.getIndex(), ILLEGAL_PARAMETER_ERROR.getValue() + "，数据类型不能为空", null, menuName);
        }
        MapModuleData entity = packMapModuleDataEntity(reqInfo, userId);
        int count = 0;
        if (mapModuledataMapper.countByMapId(reqInfo.getMapId(),reqInfo.getModuleCode()) > 0) {
            count = mapModuledataMapper.updateByModuleCodeAndMapId(entity);
        } else {
            count = mapModuledataMapper.add(entity);
        }
        log.info("一张图-新增组件配置数据信息完成，请求数据|" + reqInfo + "，操作结果|" + count);
        return count;
    }

    @Override
    public MapModuleDataRep selectModuleData(String mapId,String moduleCode,String isShow) {
        String menuName = "一张图-查询组件配置数据信息";
        if (StringUtils.isEmpty(moduleCode)) {
            throw new GlobalException(ILLEGAL_PARAMETER_ERROR.getIndex(), ILLEGAL_PARAMETER_ERROR.getValue() + "，组件编码不能为空", null, menuName);
        }
        /*if("0".equals(isShow)){
            return new MapModuleDataRep();
        }*/
        MapModuleData mapModuleData = mapModuledataMapper.selectByModuleCode(mapId,moduleCode);
        if (mapModuleData == null) {
            return new MapModuleDataRep();
        }
        MapModuleDataRep mapModuleDataRep = new MapModuleDataRep(mapModuleData);
        log.info("一张图-查询组件配置数据信息完成，请求数据|" + moduleCode);
        return mapModuleDataRep;
    }

    /*
     * 统一组装大屏实体
     */
    public MapInfo packMapInfoEntity(MapInfoReq reqInfo, String userId) {
        MapInfo entity = new MapInfo();
        entity.setId(UUIDUtils.randomUUID());
        entity.setMapName(reqInfo.getMapName());
        entity.setTemplateId(reqInfo.getTemplateId());
        entity.setMapDesc(reqInfo.getMapDesc());
        entity.setIsDel(ConstantPool.NOT_DEL);
        entity.setCreateDate(new Date());
        entity.setCreateUser(userId);
        return entity;
    }

    /*
     * 统一组装组件实体
     */
    public MapModuleData packMapModuleDataEntity(MapModuleDataReq reqInfo, String userId) {
        MapModuleData entity = new MapModuleData();
        entity.setModuleCode(reqInfo.getModuleCode());
        entity.setDataType(reqInfo.getDataType());
        entity.setSourceName(reqInfo.getSourceName());
        entity.setContent(reqInfo.getContent());
        entity.setMapId(reqInfo.getMapId());
        return entity;
    }
}

