package com.hywa.map.service.map.impl;

import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.hywa.map.common.exception.GlobalException;
import com.hywa.map.common.utils.StringUtils;
import com.hywa.map.dao.entity.MapPwd;
import com.hywa.map.dao.mapper.MapPwdMapper;
import com.hywa.map.representation.MapPwdRep;
import com.hywa.map.representation.MapPwdReps;
import com.hywa.map.representation.MapPwdReq;
import com.hywa.map.service.map.MapPwdService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

import static com.hywa.map.common.enums.CommonEnum.ILLEGAL_PARAMETER_ERROR;

/**
 * t_map_pwd  对应ServiceImpl 实现类
 *
 * @author hywa
 * @create by 2019-12-11 13:28:52
 */
@Slf4j
@Service
public class MapPwdServiceImpl implements MapPwdService {

    @Autowired
    private MapPwdMapper mapPwdMapper;


    @Override
    public MapPwdReps selectAll(int pageNo, int pageSize) {
        PageHelper.startPage(pageNo, pageSize, true);
        List<MapPwdRep> list = mapPwdMapper.selectAll();
        log.info("查询大屏口令信息完成，请求数据：|");
        return new MapPwdReps(new PageInfo<>(list).getTotal(), list);
    }

    @Override
    public int update(MapPwdReq reqInfo, String userId) {
        String menuName = "大屏口令-更新口令信息";
        if (StringUtils.isEmpty(reqInfo.getId() + "")) {
            throw new GlobalException(ILLEGAL_PARAMETER_ERROR.getIndex(), ILLEGAL_PARAMETER_ERROR.getValue() + "，id不能为空", null, menuName);
        }
        if (StringUtils.isEmpty(reqInfo.getState())) {
            throw new GlobalException(ILLEGAL_PARAMETER_ERROR.getIndex(), ILLEGAL_PARAMETER_ERROR.getValue() + "，状态不能为空", null, menuName);
        }
        if (StringUtils.isEmpty(reqInfo.getPwd())) {
            throw new GlobalException(ILLEGAL_PARAMETER_ERROR.getIndex(), ILLEGAL_PARAMETER_ERROR.getValue() + "，口令不能为空", null, menuName);
        }
        MapPwd entity = new MapPwd();
        entity.setId(reqInfo.getId());
        entity.setUpdateUser(userId);
        entity.setState(reqInfo.getState());
        entity.setPwd(reqInfo.getPwd());
        int count = mapPwdMapper.update(entity);
        log.info("大屏口令-更新口令信息完成，请求数据|" + reqInfo + "，操作结果|" + count);
        return count;
    }

    @Override
    public boolean checkPwd(String pwdType, String pwd) {
        String menuName = "大屏口令-验证口令信息";
        if (StringUtils.isEmpty(pwdType)) {
            throw new GlobalException(ILLEGAL_PARAMETER_ERROR.getIndex(), ILLEGAL_PARAMETER_ERROR.getValue() + "，口令类型不能为空", null, menuName);
        }
        if (StringUtils.isEmpty(pwd)) {
            throw new GlobalException(ILLEGAL_PARAMETER_ERROR.getIndex(), ILLEGAL_PARAMETER_ERROR.getValue() + "，口令不能为空", null, menuName);
        }
        MapPwd mapPwd = mapPwdMapper.selectByType(pwdType);
        if (mapPwd == null) {
            return false;
        }
        if ("1".equals(mapPwd.getState())) {
            return true;
        }
        return pwd.equals(mapPwd.getPwd());
    }
}

