package com.hywa.map.service.notice.impl;

import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.hywa.map.common.ConstantPool;
import com.hywa.map.common.exception.GlobalException;
import com.hywa.map.common.utils.StringUtils;
import com.hywa.map.common.utils.UUIDUtils;
import com.hywa.map.config.EnvProperties;
import com.hywa.map.dao.entity.ExamAttach;
import com.hywa.map.dao.entity.Notice;
import com.hywa.map.dao.mapper.NoticeMapper;
import com.hywa.map.representation.*;
import com.hywa.map.service.notice.NoticeService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import static com.hywa.map.common.enums.CommonEnum.*;

/**
 * t_web_notice  对应ServiceImpl 实现类
 *
 * @author hywa
 * @create by 2019-02-18 14:53:53
 */
@Slf4j
@Service
public class NoticeServiceImpl implements NoticeService {

    @Autowired
    private NoticeMapper noticeMapper;
    @Autowired
    EnvProperties envProperties;

    /*
     * 保存信息
     */
    @Override
    public int save(NoticeReq reqInfo, String userId) {
        String menuName = "新闻通知公告管理-新增新闻通知公告信息";
        //1.新增基本信息
        if (StringUtils.isEmpty(reqInfo.getTitle())) {
            throw new GlobalException(ILLEGAL_PARAMETER_ERROR.getIndex(), ILLEGAL_PARAMETER_ERROR.getValue() + "，标题不能为空", null, menuName);
        }
        /*if (StringUtils.isEmpty(reqInfo.getContent())) {
            throw new GlobalException(ILLEGAL_PARAMETER_ERROR.getIndex(), ILLEGAL_PARAMETER_ERROR.getValue() + "，内容不能为空", null, menuName);
        }*/
        if (noticeMapper.countByTitleName(reqInfo.getTitle(), null) > 0) {
            throw new GlobalException(NOTICE_NAME_FOUND.getIndex(), NOTICE_NAME_FOUND.getValue(), null, menuName);
        }
        Notice entity = packEntity(reqInfo, userId);
        int count = noticeMapper.add(entity);
        //2.新增产品附件
        List<FileReq> files = reqInfo.getFiles();
        if (files != null && files.size() > 0) {
            for (FileReq item : files) {
                // 获取文件名
                String name = item.getFileName();
                String remark = item.getFileDesc();
                ExamAttach entityAttach = packAttachEntity(entity.getId(), name, remark);
                noticeMapper.insertAttach(entityAttach);
            }
        }
        log.info("新增新闻通知公告信息完成，请求数据|" + reqInfo + "，操作结果|" + count);
        return count;
    }

    @Override
    public int update(NoticeReq reqInfo, String userId) {
        String menuName = "新闻通知公告管理-编辑新闻通知公告信息";
        if (StringUtils.isEmpty(reqInfo.getId())) {
            throw new GlobalException(ILLEGAL_PARAMETER_ERROR.getIndex(), ILLEGAL_PARAMETER_ERROR.getValue() + "，id不能为空", null, menuName);
        }
        if (StringUtils.isEmpty(reqInfo.getTitle())) {
            throw new GlobalException(ILLEGAL_PARAMETER_ERROR.getIndex(), ILLEGAL_PARAMETER_ERROR.getValue() + "，标题不能为空", null, menuName);
        }
        /*if (StringUtils.isEmpty(reqInfo.getContent())) {
            throw new GlobalException(ILLEGAL_PARAMETER_ERROR.getIndex(), ILLEGAL_PARAMETER_ERROR.getValue() + "，内容不能为空", null, menuName);
        }*/
        if (noticeMapper.countByTitleName(reqInfo.getTitle(), reqInfo.getId()) > 0) {
            throw new GlobalException(NOTICE_NAME_FOUND.getIndex(), NOTICE_NAME_FOUND.getValue(), null, menuName);
        }
        Notice entity = new Notice();
        entity.setId(reqInfo.getId());
        entity.setTitle(reqInfo.getTitle());
        entity.setContent(reqInfo.getContent());
        entity.setNoticeType(reqInfo.getNoticeType());
        entity.setUpdateDate(new Date());
        entity.setUpdateUser(userId);
        entity.setState(reqInfo.getState());
        entity.setOrigin(reqInfo.getOrigin());
        int count = noticeMapper.update(entity);
        //2.新增产品附件
        noticeMapper.deleteAttachByNoticeId(entity.getId());
        List<FileReq> files = reqInfo.getFiles();
        if (files != null && files.size() > 0) {
            for (FileReq item : files) {
                // 获取文件名
                String name = item.getFileName();
                String remark = item.getFileDesc();
                ExamAttach entityAttach = packAttachEntity(entity.getId(), name, remark);
                noticeMapper.insertAttach(entityAttach);
            }
        }
        log.info("编辑新闻通知公告信息完成，请求数据|" + reqInfo + "，操作结果|" + count);
        return count;
    }

    @Override
    public int delete(NoticeReq reqInfo, String userId) {
        String menuName = "新闻通知公告管理-删除新闻通知公告信息";
        if (StringUtils.isEmpty(reqInfo.getId())) {
            throw new GlobalException(ILLEGAL_PARAMETER_ERROR.getIndex(), ILLEGAL_PARAMETER_ERROR.getValue() + "，id不能为空", null, menuName);
        }
        int count = noticeMapper.put(reqInfo.getId(), userId);
        log.info("删除新闻通知公告信息完成，请求数据|" + reqInfo + "，操作结果|" + count);
        return count;
    }

    @Override
    public int publish(NoticeReq reqInfo, String userId) {
        String menuName = "新闻通知公告管理-批量发布新闻通知公告信息";
        List<String> ids = reqInfo.getIds();
        if (ids == null) {
            throw new GlobalException(ILLEGAL_PARAMETER_ERROR.getIndex(), ILLEGAL_PARAMETER_ERROR.getValue() + "，id不能为空", null, menuName);
        }
        List<Notice> list = new ArrayList<>();
        ids.forEach(item -> {
            list.add(new Notice(item));
        });
        int count = noticeMapper.updateStateBatch(reqInfo.getState(), userId, list);
        log.info("批量发布新闻通知公告信息完成，请求数据|" + reqInfo + "，操作结果|" + count);
        return count;
    }

    @Override
    public int publishSingle(NoticeReq reqInfo, String userId) {
        String menuName = "新闻通知公告管理-发布新闻通知公告信息";
        String id = reqInfo.getId();
        if (id == null) {
            throw new GlobalException(ILLEGAL_PARAMETER_ERROR.getIndex(), ILLEGAL_PARAMETER_ERROR.getValue() + "，id不能为空", null, menuName);
        }
        int count = noticeMapper.updateStateSingle(reqInfo.getState(), userId, id);
        log.info("发布新闻通知公告信息完成，请求数据|" + reqInfo + "，操作结果|" + count);
        return count;
    }

    @Override
    public NoticeRep selectById(String id) {
        String menuName = "新闻通知公告管理-查询新闻通知公告信息";
        if (StringUtils.isEmpty(id)) {
            throw new GlobalException(ILLEGAL_PARAMETER_ERROR.getIndex(), ILLEGAL_PARAMETER_ERROR.getValue() + "，id不能为空", null, menuName);
        }
        Notice entity = noticeMapper.selectById(id);
        if (entity == null) {
            return null;
        }
        NoticeRep noticeRep = new NoticeRep(entity);
        //附件
        List<ExamAttach> attachList = noticeMapper.selectAttachByNoticeId(entity.getId());
        List<FileReq> attachUrl = new ArrayList<>();
        if (attachList != null) {
            for (ExamAttach list : attachList) {
                FileReq fileReq = new FileReq();
                fileReq.setFileName(list.getContent());
                fileReq.setFileUrl(envProperties.getComFileDownloadUrl() + list.getContent());
                fileReq.setFileDesc(list.getRemark());
                attachUrl.add(fileReq);
            }
            noticeRep.setFiles(attachUrl);
        }
        return noticeRep;
    }

    @Override
    public NoticeReps selectAll(int pageNo, int pageSize, String queryParam, String beginDate, String endDate, String noticeType, String state) {
        PageHelper.startPage(pageNo, pageSize, true);
        String dateFlag = (StringUtils.isEmpty(beginDate) && StringUtils.isEmpty(endDate)) ? null : "1";
        beginDate = StringUtils.isEmpty(beginDate) ? "1900-01-01 00:00:00" : beginDate + " 00:00:00";
        endDate = StringUtils.isEmpty(endDate) ? "9999-12-12 23:59:59" : endDate + " 23:59:59";
        List<Notice> list = noticeMapper.select(queryParam, beginDate, endDate, noticeType, state, dateFlag);
        List<NoticeRep> repList = new ArrayList<>();
        for (Notice item : list) {
            NoticeRep repObj = new NoticeRep(item);
            repList.add(repObj);
        }
        log.info("查询新闻通知公告信息完成，请求数据：queryParam|" + queryParam + "，beginDate|" + beginDate + "，endDate|" + endDate + "，noticeType|" + noticeType + "，state|" + state);
        return new NoticeReps(new PageInfo<>(list).getTotal(), repList);
    }

    @Override
    public int saveBlack(NoticeReq reqInfo, String userId) {
        String menuName = "红黑榜发布管理-新增红黑榜信息";
        if (StringUtils.isEmpty(reqInfo.getTitle())) {
            throw new GlobalException(ILLEGAL_PARAMETER_ERROR.getIndex(), ILLEGAL_PARAMETER_ERROR.getValue() + "，标题不能为空", null, menuName);
        }
        if (StringUtils.isEmpty(reqInfo.getContent())) {
            throw new GlobalException(ILLEGAL_PARAMETER_ERROR.getIndex(), ILLEGAL_PARAMETER_ERROR.getValue() + "，内容不能为空", null, menuName);
        }
        if (noticeMapper.countByTitleName(reqInfo.getTitle(), null) > 0) {
            throw new GlobalException(NOTICE_NAME_FOUND.getIndex(), NOTICE_NAME_FOUND.getValue(), null, menuName);
        }
        Notice entity = packBlackEntity(reqInfo, userId);
        int count = noticeMapper.add(entity);
        log.info("新增红黑榜信息完成，请求数据|" + reqInfo + "，操作结果|" + count);
        return count;
    }

    @Override
    public int updateBlack(NoticeReq reqInfo, String userId) {
        String menuName = "红黑榜发布管理-编辑红黑榜信息";
        if (StringUtils.isEmpty(reqInfo.getId())) {
            throw new GlobalException(ILLEGAL_PARAMETER_ERROR.getIndex(), ILLEGAL_PARAMETER_ERROR.getValue() + "，id不能为空", null, menuName);
        }
        if (StringUtils.isEmpty(reqInfo.getTitle())) {
            throw new GlobalException(ILLEGAL_PARAMETER_ERROR.getIndex(), ILLEGAL_PARAMETER_ERROR.getValue() + "，标题不能为空", null, menuName);
        }
        if (StringUtils.isEmpty(reqInfo.getContent())) {
            throw new GlobalException(ILLEGAL_PARAMETER_ERROR.getIndex(), ILLEGAL_PARAMETER_ERROR.getValue() + "，内容不能为空", null, menuName);
        }
        if (noticeMapper.countByTitleName(reqInfo.getTitle(), reqInfo.getId()) > 0) {
            throw new GlobalException(NOTICE_NAME_FOUND.getIndex(), NOTICE_NAME_FOUND.getValue(), null, menuName);
        }
        Notice entity = new Notice();
        entity.setId(reqInfo.getId());
        entity.setTitle(reqInfo.getTitle());
        entity.setContent(reqInfo.getContent());
        entity.setNoticeType((short) 9);
        entity.setSubType(reqInfo.getSubType());
        entity.setBatchNo(reqInfo.getBatchNo());
        entity.setUpdateDate(new Date());
        entity.setUpdateUser(userId);
        entity.setState(reqInfo.getState());
        entity.setOrigin(reqInfo.getOrigin());
        int count = noticeMapper.update(entity);
        log.info("编辑红黑榜信息完成，请求数据|" + reqInfo + "，操作结果|" + count);
        return count;
    }

    @Override
    public NoticeReps selectBlackAll(int pageNo, int pageSize, String queryParam, String beginDate, String endDate, String noticeType, String state) {
        PageHelper.startPage(pageNo, pageSize, true);
        String dateFlag = (StringUtils.isEmpty(beginDate) && StringUtils.isEmpty(endDate)) ? null : "1";
        beginDate = StringUtils.isEmpty(beginDate) ? "1900-01-01 00:00:00" : beginDate + " 00:00:00";
        endDate = StringUtils.isEmpty(endDate) ? "9999-12-12 23:59:59" : endDate + " 23:59:59";
        List<Notice> list = noticeMapper.selectBlack(queryParam, beginDate, endDate, noticeType, state, dateFlag);
        List<NoticeRep> repList = new ArrayList<>();
        for (Notice item : list) {
            NoticeRep repObj = new NoticeRep(item);
            repList.add(repObj);
        }
        log.info("查询红黑榜信息完成，请求数据：queryParam|" + queryParam + "，beginDate|" + beginDate + "，endDate|" + endDate + "，noticeType|" + noticeType + "，state|" + state);
        return new NoticeReps(new PageInfo<>(list).getTotal(), repList);
    }

    /*
     * 统一组装实体
     */
    public Notice packEntity(NoticeReq reqInfo, String userId) {
        Notice entity = new Notice();
        entity.setId(UUIDUtils.randomUUID());
        entity.setTitle(reqInfo.getTitle());
        entity.setContent(reqInfo.getContent());
        entity.setNoticeType(reqInfo.getNoticeType());
        entity.setState(reqInfo.getState());
        entity.setIsDel(ConstantPool.NOT_DEL);
        entity.setCreateDate(new Date());
        entity.setCreateUser(userId);
        entity.setOrigin(reqInfo.getOrigin());
        return entity;
    }

    /*
     * 统一组装实体
     */
    public Notice packBlackEntity(NoticeReq reqInfo, String userId) {
        Notice entity = new Notice();
        entity.setId(UUIDUtils.randomUUID());
        entity.setTitle(reqInfo.getTitle());
        entity.setContent(reqInfo.getContent());
        entity.setNoticeType((short) 9);
        entity.setSubType(reqInfo.getSubType());
        entity.setBatchNo(reqInfo.getBatchNo());
        entity.setState(reqInfo.getState());
        entity.setIsDel(ConstantPool.NOT_DEL);
        entity.setCreateDate(new Date());
        entity.setCreateUser(userId);
        entity.setOrigin(reqInfo.getOrigin());
        return entity;
    }

    /*
     * 统一组装实体附件信息
     */
    public ExamAttach packAttachEntity(String prodId, String content, String remark) {
        ExamAttach entity = new ExamAttach();
        entity.setProdId(prodId);
        entity.setContent(content);
        entity.setRemark(remark);
        return entity;
    }
}

