package com.hywa.map.service.orchard;

import com.hywa.map.representation.*;

/**
 * 果园管理接口类
 *
 * @author hywa
 * @create by 2018-08-06 15:02:49
 */
public interface OrchardService {
    /**
     * 保存果园信息
     *
     * @param reqInfo 请求数据
     * @param userId 操作人员id
     * @return 园区id
     */
    OrchardSaveRep save(OrchardReq reqInfo, String userId);
    /**
     * 获取gis果园列表
     *
     * @param posType 行政级别 2-省 3-市 4-县 5-村
     * @param id 专题id
     * @return 园区数据列表
     */
    GisParkReps queryGisPark(String posType, String id);
    /**
     * 编辑果园信息
     *
     * @param reqInfo 请求数据
     * @param userId 操作人员id
     * @return 园区id
     */
    void update(OrchardReq reqInfo, String userId);
    /**
     * 删除果园信息
     *
     * @param reqInfo 请求数据
     * @param userId 操作人员id
     * @return 园区id
     */
    void delete(OrchardReq reqInfo, String userId);
    /**
     * 查询涉及农业
     *
     * @param
     * @param
     * @return 涉及农业类型
     */
    ComboxReps selectAgriType();
    /**
     * 查询产出物
     *
     * @param
     * @param
     * @return 产出物类型
     */
    ComboxReps selectProdType();
    /**
     * 查询园区列表
     *
     * @param queryParam 园区编码或名称
     * @param induType 行业类型
     * @param productName 产出物
     * @return list列表
     */
    OrchardReps selectAll(int pageNo, int pageSize, String queryParam, String induType, String productName);
    /**
     * 根据id查询园区详细信息
     *
     * @param  id 园区id
     * @return OrchardRep
     */
    OrchardRep selectById(String id);
    /**
     * 查询果园监测点列表
     *
     * @param
     * @param
     * @return 涉及农业类型
     */
    ComboxReps selectOrchardMonitor();
    /**
     * 查询果园监测点详细信息
     *
     * @param orchardId 园区id
     * @param monitorId 监测点id
     * @return CommonViewReps
     */
    CommonViewReps selectOrchardMonitorDetail(String orchardId, String monitorId);
    /**
     * 查询果园监测点24小时天气信息
     *
     * @param orchardId 园区id
     * @param monitorId 监测点id
     * @return CommonViewReps
     */
    TempHourRep selectOrchardWeather(String orchardId, String monitorId);
}

