package com.hywa.map.service.orchard.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.hywa.map.common.ConstantPool;
import com.hywa.map.common.exception.GlobalException;
import com.hywa.map.common.utils.DateUtils;
import com.hywa.map.common.utils.HttpUtils;
import com.hywa.map.common.utils.StringUtils;
import com.hywa.map.common.utils.UUIDUtils;
import com.hywa.map.config.EnvProperties;
import com.hywa.map.dao.entity.*;
import com.hywa.map.dao.mapper.*;
import com.hywa.map.representation.*;
import com.hywa.map.service.orchard.OrchardService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.text.SimpleDateFormat;
import java.util.*;

import static com.hywa.map.common.enums.CommonEnum.*;

/**
 * 园区管理接口实现类
 *
 * @author hywa
 * @create by 2018-08-06 15:02:49
 */
@Slf4j
@Service
public class OrchardServiceImpl implements OrchardService {

    @Autowired
    private OrchardInfoMapper orchardInfoMapper;
    @Autowired
    private OrchardProductMapper orchardProductMapper;
    @Autowired
    private OrchardLocateMapper orchardLocateMapper;
    @Autowired
    private GisParkMapper gisParkMapper;
    @Autowired
    private WeatherMapper weatherMapper;
    @Autowired
    EnvProperties envProperties;

    @Override
    public OrchardSaveRep save(OrchardReq reqInfo, String userId) {
        String menuName = "园区管理-新增园区信息";
        //1.新增基本信息
        String id = reqInfo.getId();
        OrchardInfo entity = packOrchardInfoEntity(reqInfo, userId);
        if (StringUtils.isNotEmpty(id)) {
            orchardInfoMapper.delete(id);
            orchardProductMapper.delete(id);
            orchardLocateMapper.delete(id);
            entity.setId(id);
        }
        if (StringUtils.isEmpty(reqInfo.getOrchardCode())) {
            throw new GlobalException(ILLEGAL_PARAMETER_ERROR.getIndex(), ILLEGAL_PARAMETER_ERROR.getValue() + "，园区编码不能为空", null, menuName);
        }
        if (StringUtils.isEmpty(reqInfo.getOrchardName())) {
            throw new GlobalException(ILLEGAL_PARAMETER_ERROR.getIndex(), ILLEGAL_PARAMETER_ERROR.getValue() + "，园区名称不能为空", null, menuName);
        }
        if (orchardInfoMapper.countByOrchardCode(reqInfo.getOrchardCode(), null) > 0) {
            throw new GlobalException(ORCHARD_CODE_FOUND.getIndex(), ORCHARD_CODE_FOUND.getValue(), null, menuName);
        }
        if (orchardInfoMapper.countByOrchardName(reqInfo.getOrchardName(), null) > 0) {
            throw new GlobalException(ORCHARD_NAME_FOUND.getIndex(), ORCHARD_NAME_FOUND.getValue(), null, menuName);
        }
        int count = orchardInfoMapper.add(entity);
        //2.新增产出物信息
        List<OrchardProduct> products = reqInfo.getProducts();
        if (!CollectionUtils.isEmpty(products)) {
            for (OrchardProduct item : products) {
                item.setOrchardId(entity.getId());
                item.setProdArea(StringUtils.isEmpty(item.getProdArea()) ? "0" : item.getProdArea());
                item.setProdVol(StringUtils.isEmpty(item.getProdVol()) ? "0" : item.getProdVol());
            }
            orchardProductMapper.insertBatch(products);
        }
        //3.新增地理配置信息
        OrchardLocate orchardLocate = packOrchardLocateEntity(reqInfo, entity.getId());
        orchardLocateMapper.add(orchardLocate);
        log.info("新增园区信息完成，请求数据|" + reqInfo + "，操作结果|" + count);
        OrchardSaveRep rep = new OrchardSaveRep();
        rep.setId(entity.getId());
        //发送物联网保存
        try {
            Map<String, String> queryParas = new HashMap<>();
            queryParas.put("id", "");
            queryParas.put("baseName", reqInfo.getOrchardName());
            queryParas.put("baseAbridge", reqInfo.getOrchardCode());
            queryParas.put("baseUser", reqInfo.getLeaderName());
            queryParas.put("basePhone", reqInfo.getTelNo());
            queryParas.put("baseScale", reqInfo.getOrchardArea());
            queryParas.put("baseOutput", "");
            queryParas.put("baseSynopsis", reqInfo.getOrchardDesc());
            queryParas.put("industryId", reqInfo.getInduType());
            queryParas.put("customerId", "");
            queryParas.put("provinceId", reqInfo.getProvinceId());
            queryParas.put("cityId", reqInfo.getCityId());
            queryParas.put("areaId", reqInfo.getCountyId());
            queryParas.put("townId", reqInfo.getTownId());
            queryParas.put("villageId", reqInfo.getVillageId());
            queryParas.put("address", reqInfo.getAddress());
            queryParas.put("remark", "");
            queryParas.put("isDel", "0");
            //queryParas.put("createId", userId);
            //queryParas.put("createTime", DateUtils.getCurrentDate());
            queryParas.put("orchard_border", reqInfo.getOrchardBorder());
            queryParas.put("oneMap", entity.getId());
            queryParas.put("products", JSON.toJSONString(reqInfo.getProducts()));
            String params = JSON.toJSONString(queryParas);
            String responseMsg = HttpUtils.post(envProperties.getIotServerUrl() + "devid/orchard", params);
            log.error("发送物联网系统新增园区信息完成，请求数据|" + reqInfo + "，响应信息|" + responseMsg);
        } catch (Exception e) {
            log.error("发送物联网系统新增园区信息异常，请求数据|" + reqInfo + "，异常信息|" + e);
        }
        return rep;
    }

    @Override
    public GisParkReps queryGisPark(String posType, String id) {
        String menuName = "Gis地图-查询园区统计信息";
        if (StringUtils.isEmpty(posType)) {
            throw new GlobalException(ILLEGAL_PARAMETER_ERROR.getIndex(), ILLEGAL_PARAMETER_ERROR.getValue() + "，查询类型不能为空", null, menuName);
        }
        List<GisParkRep> list = gisParkMapper.select(posType, id);
        log.info("查询gis园区统计信息完成，请求数据：posType|" + posType + "，id|" + id);
        return new GisParkReps((long) list.size(), list);
    }

    @Override
    public void update(OrchardReq reqInfo, String userId) {
        String menuName = "园区管理-编辑园区信息";
        //1.新增基本信息
        String id = reqInfo.getId();
        if (StringUtils.isEmpty(id)) {
            throw new GlobalException(ILLEGAL_PARAMETER_ERROR.getIndex(), ILLEGAL_PARAMETER_ERROR.getValue() + "，id不能为空", null, menuName);
        }
        OrchardInfo entity = packOrchardInfoEntity(reqInfo, userId);
        entity.setId(id);
        entity.setUpdateDate(new Date());
        entity.setUpdateUser(userId);
        if (StringUtils.isEmpty(reqInfo.getOrchardCode())) {
            throw new GlobalException(ILLEGAL_PARAMETER_ERROR.getIndex(), ILLEGAL_PARAMETER_ERROR.getValue() + "，园区编码不能为空", null, menuName);
        }
        if (StringUtils.isEmpty(reqInfo.getOrchardName())) {
            throw new GlobalException(ILLEGAL_PARAMETER_ERROR.getIndex(), ILLEGAL_PARAMETER_ERROR.getValue() + "，园区名称不能为空", null, menuName);
        }
        if (orchardInfoMapper.countByOrchardCode(reqInfo.getOrchardCode(), id) > 0) {
            throw new GlobalException(ORCHARD_CODE_FOUND.getIndex(), ORCHARD_CODE_FOUND.getValue(), null, menuName);
        }
        if (orchardInfoMapper.countByOrchardName(reqInfo.getOrchardName(), id) > 0) {
            throw new GlobalException(ORCHARD_NAME_FOUND.getIndex(), ORCHARD_NAME_FOUND.getValue(), null, menuName);
        }
        int count = orchardInfoMapper.update(entity);
        //2.新增产出物信息
        orchardProductMapper.delete(id);
        List<OrchardProduct> products = reqInfo.getProducts();
        if (!CollectionUtils.isEmpty(products)) {
            for (OrchardProduct item : products) {
                item.setOrchardId(entity.getId());
                item.setProdArea(StringUtils.isEmpty(item.getProdArea()) ? "0" : item.getProdArea());
                item.setProdVol(StringUtils.isEmpty(item.getProdVol()) ? "0" : item.getProdVol());
            }
            orchardProductMapper.insertBatch(products);
        }
        //3.新增地理配置信息
        OrchardLocate orchardLocate = packOrchardLocateEntity(reqInfo, entity.getId());
        orchardLocateMapper.delete(id);
        orchardLocateMapper.add(orchardLocate);
        //发送物联网保存
        try {
            Map<String, String> queryParas = new HashMap<>();
            queryParas.put("id", "");
            queryParas.put("baseName", reqInfo.getOrchardName());
            queryParas.put("baseAbridge", reqInfo.getOrchardCode());
            queryParas.put("baseUser", reqInfo.getLeaderName());
            queryParas.put("basePhone", reqInfo.getTelNo());
            queryParas.put("baseScale", reqInfo.getOrchardArea());
            queryParas.put("baseOutput", "");
            queryParas.put("baseSynopsis", reqInfo.getOrchardDesc());
            queryParas.put("industryId", reqInfo.getInduType());
            queryParas.put("customerId", "");
            queryParas.put("provinceId", reqInfo.getProvinceId());
            queryParas.put("cityId", reqInfo.getCityId());
            queryParas.put("areaId", reqInfo.getCountyId());
            queryParas.put("townId", reqInfo.getTownId());
            queryParas.put("villageId", reqInfo.getVillageId());
            queryParas.put("address", reqInfo.getAddress());
            queryParas.put("remark", "");
            queryParas.put("isDel", "0");
            //queryParas.put("createId", userId);
            //queryParas.put("createTime", DateUtils.getCurrentDate());
            queryParas.put("orchard_border", reqInfo.getOrchardBorder());
            queryParas.put("oneMap", entity.getId());
            queryParas.put("products", JSON.toJSONString(reqInfo.getProducts()));
            String params = JSON.toJSONString(queryParas);
            String responseMsg = HttpUtils.post(envProperties.getIotServerUrl() + "devid/orchardUp", params);
            log.error("发送物联网系统编辑园区信息完成，请求数据|" + reqInfo + "，响应信息|" + responseMsg);
        } catch (Exception e) {
            log.error("发送物联网系统编辑园区信息异常，请求数据|" + reqInfo + "，异常信息|" + e);
        }
        log.info("编辑园区信息完成，请求数据|" + reqInfo + "，操作结果|" + count);
    }

    @Override
    public void delete(OrchardReq reqInfo, String userId) {
        String menuName = "园区管理-删除园区信息";
        //1.新增基本信息
        String id = reqInfo.getId();
        if (StringUtils.isEmpty(id)) {
            throw new GlobalException(ILLEGAL_PARAMETER_ERROR.getIndex(), ILLEGAL_PARAMETER_ERROR.getValue() + "，id不能为空", null, menuName);
        }
        orchardProductMapper.delete(id);
        orchardLocateMapper.delete(id);
        int count = orchardInfoMapper.put(id);
        //发送物联网保存
        try {
            Map<String, String> queryParas = new HashMap<>();
            queryParas.put("oneMap", id);
            String responseMsg = HttpUtils.get(envProperties.getIotServerUrl() + "devid/orchardDe", queryParas);
            log.error("发送物联网系统删除园区信息完成，请求数据|" + reqInfo + "，响应信息|" + responseMsg);
        } catch (Exception e) {
            log.error("发送物联网系统删除园区信息异常，请求数据|" + reqInfo + "，异常信息|" + e);
        }
        log.info("删除园区信息完成，请求数据|" + reqInfo + "，操作结果|" + count);
    }

    @Override
    public ComboxReps selectAgriType() {
        List<ComboxRep> repList = new ArrayList<>();
        String tradeType = "农业|林业|畜牧业|渔业|农林牧渔服务业";
        String[] tradeTypeArray = tradeType.split("\\|");
        for (int i = 0; i < tradeTypeArray.length; i++) {
            ComboxRep repObj = new ComboxRep();
            repObj.setId("0" + (i + 1));
            repObj.setName(tradeTypeArray[i]);
            repList.add(repObj);
        }
        return new ComboxReps((long) repList.size(), repList);
    }

    @Override
    public ComboxReps selectProdType() {
        List<Combox> list = orchardProductMapper.selectProdType();
        if (list == null) {
            return null;
        }
        List<ComboxRep> repList = new ArrayList<>();
        for (int i = 0; i < list.size(); i++) {
            ComboxRep repObj = new ComboxRep(list.get(i));
            repObj.setId((i + 1) + "");
            repList.add(repObj);
        }
        return new ComboxReps((long) repList.size(), repList);
    }

    @Override
    public OrchardReps selectAll(int pageNo, int pageSize, String queryParam, String induType, String productName) {
        PageHelper.startPage(pageNo, pageSize, true);
        List<OrchardRep> list = orchardInfoMapper.select(queryParam, induType, productName, null);
        for (OrchardRep item : list) {
            if ("01".equals(item.getInduType())) {
                item.setInduTypeName("农业");
            } else if ("02".equals(item.getInduType())) {
                item.setInduTypeName("林业");
            } else if ("03".equals(item.getInduType())) {
                item.setInduTypeName("畜牧业");
            } else if ("04".equals(item.getInduType())) {
                item.setInduTypeName("渔业");
            } else if ("05".equals(item.getInduType())) {
                item.setInduTypeName("农林牧渔服务业");
            }
        }
        log.info("查询园区列表信息完成，请求数据：queryParam|" + queryParam + "，induType|" + induType + "，productName|" + productName);
        return new OrchardReps(new PageInfo<>(list).getTotal(), list);
    }

    @Override
    public OrchardRep selectById(String id) {
        List<OrchardRep> list = orchardInfoMapper.select(null, null, null, id);
        if (list == null || list.size() <= 0) {
            return null;
        }
        for (OrchardRep item : list) {
            if ("01".equals(item.getInduType())) {
                item.setInduTypeName("农业");
            } else if ("02".equals(item.getInduType())) {
                item.setInduTypeName("林业");
            } else if ("03".equals(item.getInduType())) {
                item.setInduTypeName("畜牧业");
            } else if ("04".equals(item.getInduType())) {
                item.setInduTypeName("渔业");
            } else if ("05".equals(item.getInduType())) {
                item.setInduTypeName("农林牧渔服务业");
            }
        }
        OrchardRep orchardRep = list.get(0);
        List<OrchardProduct> products = orchardProductMapper.selectByOrchardId(id);
        orchardRep.setProducts(products);
        OrchardLocate orchardLocate = orchardLocateMapper.selectByOrchardId(id);
        if (orchardLocate != null) {
            String border = orchardLocate.getOrchardBorder();
            orchardRep.setOrchardBorder(border);
        }
        log.info("根据id查询园区详细信息完成，请求数据：id|" + id);
        return orchardRep;
    }

    @Override
    public ComboxReps selectOrchardMonitor() {
        List<ComboxRep> repList = new ArrayList<>();
        String tradeType = "监测点1|监测点2|监测点3|监测点4";
        String[] tradeTypeArray = tradeType.split("\\|");
        for (int i = 0; i < tradeTypeArray.length; i++) {
            ComboxRep repObj = new ComboxRep();
            repObj.setId((i + 1) + "");
            repObj.setName(tradeTypeArray[i]);
            repList.add(repObj);
        }
        return new ComboxReps((long) repList.size(), repList);
    }

    @Override
    public CommonViewReps selectOrchardMonitorDetail(String orchardId, String monitorId) {
        List<CommonView> repList = new ArrayList<>();
        String detailInfo = "土壤温度:23.1℃|土壤PH值:9.2|土壤湿度:10rh|降雨量:0mm|空气湿度:68.6rh|CO2:430ppm|空气温度:30℃|PM2.5:29μg/m³|光照度:8778Lx|土壤盐分:4730|风向:西南风|风速:7.8m/s|大气压力:86.62kPa|海拔高度:1587m";
        String[] detailArray = detailInfo.split("\\|");
        for (int i = 0; i < detailArray.length; i++) {
            String[] viewArray = detailArray[i].split("\\:");
            CommonView repObj = new CommonView();
            repObj.setViewName(viewArray[0]);
            repObj.setViewVal(viewArray[1]);
            repList.add(repObj);
        }
        return new CommonViewReps((long) repList.size(), repList);
    }

    @Override
    public TempHourRep selectOrchardWeather(String orchardId, String monitorId) {
        TempHourRep tempHour = new TempHourRep();
        tempHour.setTitle("24小时空气温度变化");
        try {
            double[] tempH = new double[24];//温度小时
            //获取24小时天气信息 start
            String currDate = DateUtils.getCurrentDate();
            String preDate = DateUtils.getDiffDay(-1);
            Map<String, String> selectMap = new HashMap<String, String>();
            selectMap.put("adCode", "522730");
            selectMap.put("beginDate", preDate + " 00:00:00");
            selectMap.put("endDate", currDate + " 23:59:59");
            List<Map<String, Object>> wList = weatherMapper.selectCurrWeather(selectMap);
            Iterator<Map<String, Object>> wIt = wList.iterator();
            while (wIt.hasNext()) {
                Map<String, Object> wMap = wIt.next();
                int temp = (int) wMap.get("temperature");
                //int humidity = (int) wMap.get("humidity");
                Date createDate = (Date) wMap.get("create_date");
                int hour = DateUtils.getHour(createDate);
                tempH[hour] = temp;//温度小时
            }
            //获取24小时天气信息 start
            List<Environ> tempHourList = new ArrayList<Environ>();//温度小时
            OrchardServiceImpl orchardServiceImpl = new OrchardServiceImpl();
            orchardServiceImpl.genEnvironH(tempHourList, tempH);
            tempHour.setTempH(tempHourList);
            tempHour.setMaxValue(StringUtils.getMax(tempH) + "℃");
            tempHour.setMinValue(StringUtils.getMin(tempH) + "℃");
        } catch (Exception e) {
            log.error("查询园区24小时天气信息，orchardId|" + orchardId + "，monitorId|" + monitorId + "，获取当天天气信息异常：" + e);
        }
        log.info("查询园区24小时天气信息，orchardId|" + orchardId + "，monitorId|" + monitorId + "，返回结果：" + tempHour.toString().length());
        return tempHour;
    }

    /*
     * 统一组装实体
     */
    public OrchardInfo packOrchardInfoEntity(OrchardReq reqInfo, String userId) {
        OrchardInfo entity = new OrchardInfo();
        entity.setId(UUIDUtils.randomUUID());
        entity.setOrchardCode(reqInfo.getOrchardCode());
        entity.setOrchardName(reqInfo.getOrchardName());
        entity.setOrchardArea(reqInfo.getOrchardArea());
        entity.setInduType(reqInfo.getInduType());
        entity.setWorkerCount(reqInfo.getWorkerCount());
        entity.setBeginDate(StringUtils.isNotEmpty(reqInfo.getBeginDate()) ? DateUtils.parseDate(reqInfo.getBeginDate()) : null);
        entity.setEndDate(StringUtils.isNotEmpty(reqInfo.getEndDate()) ? DateUtils.parseDate(reqInfo.getEndDate()) : null);
        entity.setLeaderName(reqInfo.getLeaderName());
        entity.setTelNo(reqInfo.getTelNo());
        entity.setElevation(reqInfo.getElevation());
        entity.setOrchardDesc(reqInfo.getOrchardDesc());
        entity.setIsDel(ConstantPool.NOT_DEL);
        entity.setCreateDate(new Date());
        entity.setCreateUser(userId);
        return entity;
    }

    /*
     * 统一组装实体
     */
    public OrchardLocate packOrchardLocateEntity(OrchardReq reqInfo, String orchardId) {
        OrchardLocate entity = new OrchardLocate();
        entity.setOrchardId(orchardId);
        entity.setAddress(reqInfo.getAddress());
        entity.setProvinceId(reqInfo.getProvinceId());
        entity.setCityId(reqInfo.getCityId());
        entity.setCountyId(reqInfo.getCountyId());
        entity.setTownId(reqInfo.getTownId());
        entity.setVillageId(reqInfo.getVillageId());
        entity.setLongitude(reqInfo.getLongitude());
        entity.setLatitude(reqInfo.getLatitude());
        entity.setOrchardBorder(reqInfo.getOrchardBorder());
        return entity;
    }

    /**
     * 封装物理环境信息24小时
     *
     * @param environHour 需要封装变量
     * @param arr         double数组参数
     * @return List<Environ>
     */
    public List<Environ> genEnvironH(List<Environ> environHour, double[] arr) {
        for (int i = 0; i < 24; i++) {
            Calendar calendar = Calendar.getInstance();
            calendar.set(Calendar.HOUR_OF_DAY, calendar.get(Calendar.HOUR_OF_DAY) - i);
            SimpleDateFormat df = new SimpleDateFormat("HH");
            Environ environ = new Environ();
            String hour = df.format(calendar.getTime());
            environ.setTimeLine(hour + ":00");
            environ.setCoorValue(arr[Integer.parseInt(hour)]);
            environ.setUnit("℃");
            environHour.add(environ);
        }
        return environHour;
    }


}

