package com.hywa.map.service.statis.impl;

import com.hywa.map.common.exception.GlobalException;
import com.hywa.map.common.utils.StringUtils;
import com.hywa.map.dao.entity.*;
import com.hywa.map.dao.mapper.*;
import com.hywa.map.representation.*;
import com.hywa.map.service.statis.AgriDataService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;

import static com.hywa.map.common.enums.CommonEnum.ILLEGAL_PARAMETER_ERROR;

/**
 * t_agri_group  对应ServiceImpl 实现类
 *
 * @author hywa
 * @create by 2019-05-05 13:53:05
 */
@Slf4j
@Service
public class AgriDataServiceImpl implements AgriDataService {

    @Autowired
    private AgriDataMapper agriDataMapper;
    @Autowired
    private AgriDataDetailMapper agriDataDetailMapper;
    @Autowired
    private AgriGroupMapper agriGroupMapper;


    @Override
    public AgriFisheryRep selectAgriData(String type, String areaId) {
        String menuName = "态势统计-查询农林牧渔业信息";
        if (StringUtils.isEmpty(type) || StringUtils.isEmpty(areaId)) {
            throw new GlobalException(ILLEGAL_PARAMETER_ERROR.getIndex(), ILLEGAL_PARAMETER_ERROR.getValue() + "，查询类型或区域编码不能为空", null, menuName);
        }
        AgriData entity = agriDataMapper.select(type, areaId);
        if (entity == null) {
            return null;
        }
        AgriFisheryRep agriFisheryRep = new AgriFisheryRep(entity);
        List<AgriDataDetail> list = agriDataDetailMapper.select(entity.getId());
        List<AgriFisheryDetail> repList = new ArrayList<>();
        for (AgriDataDetail item : list) {
            AgriFisheryDetail rep = new AgriFisheryDetail(item);
            repList.add(rep);
        }
        agriFisheryRep.setList(repList);
        log.info("查询农业产业信息完成，请求数据：type|" + type);
        return agriFisheryRep;
    }

    @Override
    public AgriTownDataRep selectTownAgriData(String areaId) {
        AgriData entity = agriDataMapper.select("8", areaId);
        if (entity == null) {
            return null;
        }
        AgriTownDataRep agriTownDataRep = new AgriTownDataRep(entity);
        List<AgriDataDetail> list = agriDataDetailMapper.select(entity.getId());
        List<AgriTownDataReps> repList = new ArrayList<>();
        List<String> areaNameList = new ArrayList<>();
        for (AgriDataDetail item : list) {
            areaNameList.add(item.getAreaName());
        }
        StringUtils.removeDuplicateWithOrder(areaNameList);

        for (String item : areaNameList) {
            AgriTownDataReps rep = new AgriTownDataReps();
            rep.setName(item);
            List<AgriFisheryDetail> detailList = new ArrayList<>();
            for (AgriDataDetail detail : list) {
                AgriFisheryDetail agriFisheryDetail = new AgriFisheryDetail(detail);
                if (item.equals(detail.getAreaName())) {
                    detailList.add(agriFisheryDetail);
                }
            }
            rep.setDetail(detailList);
            repList.add(rep);
        }
        agriTownDataRep.setList(repList);
        log.info("查询乡镇产业特色信息完成，请求数据：areaId|" + areaId);
        return agriTownDataRep;
    }

    @Override
    public AgriFisheryRep selectGroupData(String id) {
        String menuName = "园区统计-查询种植统计信息";
        if (StringUtils.isEmpty(id)) {
            throw new GlobalException(ILLEGAL_PARAMETER_ERROR.getIndex(), ILLEGAL_PARAMETER_ERROR.getValue() + "，id不能为空", null, menuName);
        }
        AgriData entity = agriDataMapper.selectById(id);
        if (entity == null) {
            return null;
        }
        SysArea sysArea = agriDataMapper.selectBySn(String.format("%-12s", entity.getPosId()).replace(' ', '0'));
        entity.setMainName(entity.getMainName().replaceAll("###",sysArea.getPosName()));
        AgriGroup agriGroup = agriGroupMapper.selectById(id);
        entity.setMainName(entity.getMainName().replaceAll("@@@",agriGroup.getGroupName()));
        AgriFisheryRep agriFisheryRep = new AgriFisheryRep(entity);
        List<AgriDataDetail> list = agriDataDetailMapper.select(entity.getId());
        List<AgriFisheryDetail> repList = new ArrayList<>();
        for (AgriDataDetail item : list) {
            AgriFisheryDetail rep = new AgriFisheryDetail(item);
            repList.add(rep);
        }
        agriFisheryRep.setList(repList);
        log.info("查询园区种植统计信息完成，请求数据：id|" + id);
        return agriFisheryRep;
    }

}

