package com.hywa.map.service.statis.impl;

import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.hywa.map.common.exception.GlobalException;
import com.hywa.map.common.utils.StringUtils;
import com.hywa.map.dao.entity.*;
import com.hywa.map.dao.mapper.AgriGroupDetailMapper;
import com.hywa.map.dao.mapper.AgriGroupItemMapper;
import com.hywa.map.dao.mapper.AgriGroupMapper;
import com.hywa.map.dao.mapper.AgriProfileMapper;
import com.hywa.map.representation.*;
import com.hywa.map.service.statis.AgriGroupService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;

import static com.hywa.map.common.enums.CommonEnum.ILLEGAL_PARAMETER_ERROR;

/**
 * t_agri_group  对应ServiceImpl 实现类
 *
 * @author hywa
 * @create by 2019-05-05 13:53:05
 */
@Slf4j
@Service
public class AgriGroupServiceImpl implements AgriGroupService {

    @Autowired
    private AgriGroupMapper agriGroupMapper;
    @Autowired
    private AgriGroupItemMapper agriGroupItemMapper;
    @Autowired
    private AgriProfileMapper agriProfileMapper;
    @Autowired
    private AgriGroupDetailMapper agriGroupDetailMapper;


    @Override
    public AgriGroupReps selectGroupAll(int pageNo, int pageSize) {
        PageHelper.startPage(pageNo, pageSize, true);
        List<AgriGroup> list = agriGroupMapper.select();
        List<AgriGroupRep> repList = new ArrayList<>();
        for (AgriGroup item : list) {
            AgriGroupRep rep = new AgriGroupRep(item);
            repList.add(rep);
        }
        log.info("查询农业组别信息完成，请求数据|" + "");
        return new AgriGroupReps(new PageInfo<>(list).getTotal(), repList);
    }

    @Override
    public AgriGroupItemReps selectGroupItem(String groupId) {
        String menuName = "态势统计-查询农业组别条目信息";
        if (StringUtils.isEmpty(groupId)) {
            throw new GlobalException(ILLEGAL_PARAMETER_ERROR.getIndex(), ILLEGAL_PARAMETER_ERROR.getValue() + "，id不能为空", null, menuName);
        }
        List<AgriGroupItem> list = agriGroupItemMapper.select(groupId);
        List<AgriGroupItemRep> repList = new ArrayList<>();
        for (AgriGroupItem item : list) {
            AgriGroupItemRep rep = new AgriGroupItemRep(item);
            repList.add(rep);
        }
        log.info("查询农业组别条目信息完成，请求数据：groupId|" + groupId);
        return new AgriGroupItemReps(new PageInfo<>(list).getTotal(), repList);
    }

    @Override
    public AgriProfileReps selectProfile(String areaId) {
        String menuName = "态势统计-查询农业概要统计信息";
        if (StringUtils.isEmpty(areaId)) {
            throw new GlobalException(ILLEGAL_PARAMETER_ERROR.getIndex(), ILLEGAL_PARAMETER_ERROR.getValue() + "，区域编码不能为空", null, menuName);
        }
        List<AgriProfile> list = agriProfileMapper.select(areaId);
        List<AgriProfileRep> repList = new ArrayList<>();
        for (AgriProfile item : list) {
            AgriProfileRep rep = new AgriProfileRep(item);
            repList.add(rep);
        }
        log.info("查询农业概要统计信息完成，请求数据|" + "");
        return new AgriProfileReps(new PageInfo<>(list).getTotal(), repList);
    }

    /*@Override
    public AgriGroupDetailRep selectGroupDetail(String id, String areaId) {
        String menuName = "态势统计-查询农业地标统计信息";
        if (StringUtils.isEmpty(id) || StringUtils.isEmpty(areaId)) {
            throw new GlobalException(ILLEGAL_PARAMETER_ERROR.getIndex(), ILLEGAL_PARAMETER_ERROR.getValue() + "，id或地区编码不能为空", null, menuName);
        }
        AgriGroupDetailRep rep = new AgriGroupDetailRep();
        AgriGroupItem agriGroupItem = agriGroupDetailMapper.selectAlias(id);
        rep.setMainName(agriGroupItem.getItemName() + "种养植户数量统计");
        String icon = agriGroupItem.getItemIcon();
        List<AgriGroupDetails> list = new ArrayList<>();
        if (icon.indexOf("kind") == -1) {
            list = agriGroupDetailMapper.select(id, areaId);
            for (AgriGroupDetails item : list) {
                item.setIcon(icon);
            }
        } else {
            list = agriGroupDetailMapper.selectByGroupId(id, areaId);
        }
        rep.setList(list);
        log.info("查询农业地标统计信息完成，请求数据：id|" + id + "，areaId|" + areaId);
        return rep;
    }*/

    @Override
    public AgriGroupDetailRep selectGroupDetail(String id, String areaId) {
        String menuName = "态势统计-查询农业地标统计信息";
        if (StringUtils.isEmpty(id) || StringUtils.isEmpty(areaId)) {
            throw new GlobalException(ILLEGAL_PARAMETER_ERROR.getIndex(), ILLEGAL_PARAMETER_ERROR.getValue() + "，id或地区编码不能为空", null, menuName);
        }
        AgriGroupDetailRep rep = new AgriGroupDetailRep();
        List<AgriGroupDetails> list = new ArrayList<>();
        list = agriGroupDetailMapper.selectByGroupIdAndAreaId(id, areaId);
        rep.setList(list);
        log.info("查询农业地标统计信息完成，请求数据：id|" + id + "，areaId|" + areaId);
        return rep;
    }
}

