package com.hywa.map.service.topic.impl;

import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.hywa.map.common.exception.GlobalException;
import com.hywa.map.common.utils.StringUtils;
import com.hywa.map.common.utils.UUIDUtils;
import com.hywa.map.config.EnvProperties;
import com.hywa.map.dao.entity.*;
import com.hywa.map.dao.entity.TopicIcon;
import com.hywa.map.dao.mapper.*;
import com.hywa.map.representation.*;
import com.hywa.map.service.topic.TopicInfoService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import static com.hywa.map.common.enums.CommonEnum.ILLEGAL_PARAMETER_ERROR;
import static com.hywa.map.common.enums.CommonEnum.TOPIC_NAME_FOUND;

/**
 * t_topic_info  对应ServiceImpl 实现类
 *
 * @author hywa
 * @create by 2019-05-10 13:50:25
 */
@Slf4j
@Service
public class TopicInfoServiceImpl implements TopicInfoService {

    @Autowired
    private TopicInfoMapper topicInfoMapper;
    @Autowired
    private TopicDbMapper topicDbMapper;
    @Autowired
    private TopicTemplateItemMapper topicTemplateItemMapper;
    @Autowired
    private TopicTemplateMapper topicTemplateMapper;
    @Autowired
    private AreaDataMapper areaDataMapper;

    @Autowired
    EnvProperties envProperties;

    @Override
    public TopicInfoReps selectTopicInfo(int pageNo, int pageSize, String topicName) {
        PageHelper.startPage(pageNo, pageSize, true);
        List<TopicDetailRep> list = topicInfoMapper.select(topicName);
        log.info("查询专题信息完成，请求数据：pageNo|" + pageNo + "，pageSize|" + pageSize + "，topicName|" + topicName);
        return new TopicInfoReps(new PageInfo<>(list).getTotal(), list);
    }

    @Override
    public ViewTemplateReps selectTopicDetail(String id, String areaName) {
        String menuName = "专题管理-查询专题详细信息";
        if (StringUtils.isEmpty(id) || StringUtils.isEmpty(areaName)) {
            throw new GlobalException(ILLEGAL_PARAMETER_ERROR.getIndex(), ILLEGAL_PARAMETER_ERROR.getValue() + "，id或地区名称不能为空", null, menuName);
        }
        List<TopicTemplateItem> templateItem = topicTemplateItemMapper.select(id);
        List<CommonView> list = topicDbMapper.select(id);
        if (list == null || list.size() == 0) {
            return null;
        }
        TopicSource topicSource = topicDbMapper.selectSource(id);
        ViewTemplateReps rep = new ViewTemplateReps();
        List<ViewTemplate> repList = doTemplate(topicSource, templateItem, list, areaName);
        rep.setList(repList);
        log.info("查询专题详细信息完成，请求数据：id|" + id + "，areaName|" + areaName);
        return rep;
    }

    @Override
    public TopicTemplateReps selectTopicTemplate() {
        List<TopicTemplate> list = topicTemplateMapper.select();
        List<TopicTemplateRep> repList = new ArrayList<>();
        for (TopicTemplate item : list) {
            TopicTemplateRep repObj = new TopicTemplateRep(item);
            repList.add(repObj);
        }
        log.info("查询专题模板信息完成，请求数据|" + "");
        return new TopicTemplateReps(new PageInfo<>(list).getTotal(), repList);
    }

    @Override
    public ViewTemplateReps selectTopicModel(String id) {
        String menuName = "专题管理-查询专题模版样式信息";
        if (StringUtils.isEmpty(id)) {
            throw new GlobalException(ILLEGAL_PARAMETER_ERROR.getIndex(), ILLEGAL_PARAMETER_ERROR.getValue() + "，id不能为空", null, menuName);
        }
        List<TopicTemplateItem> templateItem = topicTemplateItemMapper.selectByTemplateId(id);
        List<ViewTemplate> vTList = new ArrayList<>();
        for (TopicTemplateItem item : templateItem) {
            ViewTemplate kp = new ViewTemplate();
            List<CommonViewRep> viewList = new ArrayList<>();
            String viewType = item.getViewType();
            String[] context = item.getContent().split("\\|");
            String viewVal = item.getDemoData();
            if ("2".equals(viewType)) {
                for (int i = 0; i < context.length; i++) {
                    CommonViewRep cv = new CommonViewRep();
                    cv.setViewName(context[i]);
                    cv.setViewVal(Integer.parseInt(viewVal.split("\\|")[i]));
                    viewList.add(cv);
                    kp.setViewList(viewList);
                }
            } else if ("1".equals(viewType)) {
                String[] val = viewVal.split("\\,");
                for (int j = 0; j < val.length; j++) {
                    CommonViewRep cv = new CommonViewRep();
                    cv.setViewName(val[j].split("\\|")[0]);
                    cv.setViewVal(Integer.parseInt(val[j].split("\\|")[1]));
                    viewList.add(cv);
                }
                kp.setViewList(viewList);
            }
            kp.setViewTitle(item.getItemName());
            vTList.add(kp);
        }
        ViewTemplateReps rep = new ViewTemplateReps();
        rep.setList(vTList);
        log.info("查询专题模版样式信息完成，请求数据：id|" + id);
        return rep;
    }

    @Override
    public TopicDbReps selectDb(int pageNo, int pageSize) {
        PageHelper.startPage(pageNo, pageSize, true);
        List<TopicDb> list = topicDbMapper.selectDb();
        List<TopicDbRep> repList = new ArrayList<>();
        for (TopicDb item : list) {
            TopicDbRep repObj = new TopicDbRep(item);
            repList.add(repObj);
        }
        log.info("查询资源数据库信息完成，请求数据：pageNo|" + pageNo + "，pageSize|" + pageSize);
        return new TopicDbReps(new PageInfo<>(list).getTotal(), repList);
    }

    @Override
    public TopicDbItemReps selectDbItem(int pageNo, int pageSize, String dbId) {
        PageHelper.startPage(pageNo, pageSize, true);
        List<TopicDbItem> list = topicDbMapper.selectDbItem(dbId);
        List<TopicDbItemRep> repList = new ArrayList<>();
        for (TopicDbItem item : list) {
            TopicDbItemRep repObj = new TopicDbItemRep(item);
            repList.add(repObj);
        }
        log.info("查询资源数据库集信息完成，请求数据：pageNo|" + pageNo + "，pageSize|" + pageSize + "，dbId|" + dbId);
        return new TopicDbItemReps(new PageInfo<>(list).getTotal(), repList);
    }

    @Override
    public TopicDbFieldReps selectDbField(String itemId) {
        CommonView commonView = topicDbMapper.selectField(itemId);
        if (commonView == null) {
            return null;
        }
        String field = commonView.getViewName();
        String[] fields = field.split("\\|");
        List<TopicDbFieldRep> repList = new ArrayList<>();
        for (int i = 0; i < fields.length; i++) {
            TopicDbFieldRep repObj = new TopicDbFieldRep(fields[i]);
            repObj.setId(i+"");
            repList.add(repObj);
        }
        log.info("查询资源数据库集字段信息完成，请求数据：itemId|" + itemId);
        return new TopicDbFieldReps(new PageInfo<>(repList).getTotal(), repList);
    }

    @Override
    public TopicTemplateFieldReps selectTemplateField(String templateId) {
        CommonView commonView = topicTemplateItemMapper.selectTemplateField(templateId);
        if (commonView == null) {
            return null;
        }
        String field = commonView.getViewName();
        String[] fields = field.split("\\|");
        List<TopicDbFieldRep> repList = new ArrayList<>();
        for (int i = 0; i < fields.length; i++) {
            TopicDbFieldRep repObj = new TopicDbFieldRep(fields[i]);
            repList.add(repObj);
        }
        TopicTemplateFieldReps topicTemplateFieldReps = new TopicTemplateFieldReps(new PageInfo<>(repList).getTotal(), repList);
        topicTemplateFieldReps.setTemplateName(commonView.getViewVal());
        log.info("查询专题模板字段信息完成，请求数据：templateId|" + templateId);
        return topicTemplateFieldReps;
    }

    @Override
    public TopicSaveRep saveTopicInfo(TopicInfoReq reqInfo, String userId) {
        String menuName = "专题管理-新增专题基本信息";
        //1.新增专题基本信息
        if (StringUtils.isEmpty(reqInfo.getTopicName())) {
            throw new GlobalException(ILLEGAL_PARAMETER_ERROR.getIndex(), ILLEGAL_PARAMETER_ERROR.getValue() + "，专题名称不能为空", null, menuName);
        }
        if (topicInfoMapper.countByTopicName(reqInfo.getTopicName(), null) > 0) {
            throw new GlobalException(TOPIC_NAME_FOUND.getIndex(), TOPIC_NAME_FOUND.getValue(), null, menuName);
        }
        String uuid = UUIDUtils.randomUUID();
        TopicInfo topicInfo = new TopicInfo();
        topicInfo.setId(uuid);
        topicInfo.setTopicName(reqInfo.getTopicName());
        topicInfo.setTopicDesc(reqInfo.getTopicDesc());
        topicInfo.setSourceId(reqInfo.getSourceId());
        topicInfo.setShowType(reqInfo.getShowType());
        topicInfo.setPointName(reqInfo.getPointName());
        topicInfo.setAddress(reqInfo.getAddress());
        topicInfo.setShowUrl(reqInfo.getShowUrl());
        topicInfo.setCreateUser(userId);
        topicInfo.setCreateDate(new Date());
        int count = topicInfoMapper.add(topicInfo);
        TopicSaveRep topicSaveRep = new TopicSaveRep();
        topicSaveRep.setId(uuid);
        topicSaveRep.setName(reqInfo.getTopicName());
        log.info("新增专题基本信息完成，请求数据|" + reqInfo + "，操作结果|" + count);
        return topicSaveRep;
    }

    @Override
    public void updateTopicInfo(TopicInfoReq reqInfo, String userId) {
        String menuName = "专题管理-编辑专题基本信息";
        //1.新增专题基本信息
        String id = reqInfo.getId();
        if (StringUtils.isEmpty(reqInfo.getTopicName())) {
            throw new GlobalException(ILLEGAL_PARAMETER_ERROR.getIndex(), ILLEGAL_PARAMETER_ERROR.getValue() + "，专题名称不能为空", null, menuName);
        }
        if (StringUtils.isEmpty(id)) {
            throw new GlobalException(ILLEGAL_PARAMETER_ERROR.getIndex(), ILLEGAL_PARAMETER_ERROR.getValue() + "，id不能为空", null, menuName);
        }
        if (topicInfoMapper.countByTopicName(reqInfo.getTopicName(), id) > 0) {
            throw new GlobalException(TOPIC_NAME_FOUND.getIndex(), TOPIC_NAME_FOUND.getValue(), null, menuName);
        }
        TopicInfo topicInfo = new TopicInfo();
        topicInfo.setId(id);
        topicInfo.setTopicName(reqInfo.getTopicName());
        topicInfo.setTopicDesc(reqInfo.getTopicDesc());
        topicInfo.setSourceId(reqInfo.getSourceId());
        topicInfo.setShowType(reqInfo.getShowType());
        topicInfo.setPointName(reqInfo.getPointName());
        topicInfo.setAddress(reqInfo.getAddress());
        topicInfo.setShowUrl(reqInfo.getShowUrl());
        topicInfo.setUpdateUser(userId);
        topicInfo.setUpdateDate(new Date());
        int count = topicInfoMapper.update(topicInfo);
        log.info("编辑专题基本信息完成，请求数据|" + reqInfo + "，操作结果|" + count);
    }

    @Override
    public void deleteTopicInfo(TopicInfoReq reqInfo, String userId) {
        String menuName = "专题管理-删除专题基本信息";
        if (StringUtils.isEmpty(reqInfo.getId())) {
            throw new GlobalException(ILLEGAL_PARAMETER_ERROR.getIndex(), ILLEGAL_PARAMETER_ERROR.getValue() + "，id不能为空", null, menuName);
        }
        int count = topicInfoMapper.put(reqInfo.getId(), userId);
        log.info("删除专题基本信息完成，请求数据|" + reqInfo + "，操作结果|" + count);
    }

    @Override
    public TopicDetailRep selectTopicInfoById(String id) {
        String menuName = "专题管理-根据id查询专题信息";
        if (StringUtils.isEmpty(id)) {
            throw new GlobalException(ILLEGAL_PARAMETER_ERROR.getIndex(), ILLEGAL_PARAMETER_ERROR.getValue() + "，id不能为空", null, menuName);
        }
        log.info("根据id查询专题信息完成，请求数据：id|" + id);
        return topicInfoMapper.selectById(id);
    }

    public List<ViewTemplate> doTemplate(TopicSource topicSource, List<TopicTemplateItem> templateItem, List<CommonView> list, String areaName) {
        List<ViewTemplate> vTList = new ArrayList<>();
        String[] field = list.get(0).getViewVal().split("\\|");
        String[] config = templateItem.get(0).getTemplateConfig().split("\\,");
        String address = topicSource.getAddress();
        int addressIndex = StringUtils.getArrayIndex(field, address);
        for (TopicTemplateItem item : templateItem) {
            ViewTemplate kp = new ViewTemplate();
            List<CommonViewRep> viewList = new ArrayList<>();
            String viewType = item.getViewType();
            String[] context = item.getContent().split("\\|");
            String[] contextBak = item.getContent().split("\\|");
            //获取配置映射字段start
            for (int k = 0; k < context.length; k++) {
                for (int m = 0; m < config.length; m++) {
                    if (config[m].indexOf(context[k] + "|") >= 0) {
                        context[k] = config[m].split("\\|")[1];
                        break;
                    }
                }
            }
            //获取配置映射字段end
            if ("2".equals(viewType)) {
                for (int i = 0; i < context.length; i++) {
                    int index = StringUtils.getArrayIndex(field, context[i]);
                    if (index != -1) {
                        for (int j = 1; j < list.size(); j++) {
                            String[] val = list.get(j).getViewVal().split("\\|");
                            CommonViewRep cv = new CommonViewRep();
                            cv.setViewName(contextBak[i]);
                            cv.setViewVal(Integer.parseInt(val[index]));
                            if (val[addressIndex].indexOf(areaName) >= 0) {
                                viewList.add(cv);
                                kp.setViewList(viewList);
                                break;
                            }
                            //break;
                        }
                    } else {
                        CommonViewRep cv = new CommonViewRep();
                        cv.setViewName(contextBak[i]);
                        //cv.setViewVal(null);
                        viewList.add(cv);
                        kp.setViewList(viewList);
                    }
                }
            } else if ("1".equals(viewType)) {
                int indexName = StringUtils.getArrayIndex(field, context[0]);
                int indexVal = StringUtils.getArrayIndex(field, context[1]);
                if (indexName != -1 && indexVal != -1) {
                    for (int j = 1; j < list.size(); j++) {
                        String[] val = list.get(j).getViewVal().split("\\|");
                        CommonViewRep cv = new CommonViewRep();
                        cv.setViewName(val[indexName]);
                        cv.setViewVal(Integer.parseInt(val[indexVal]));
                        if (val[addressIndex].indexOf(areaName) >= 0) {
                            viewList.add(cv);
                        }
                    }
                    kp.setViewList(viewList);
                }
            }
            kp.setViewTitle(item.getItemName().replaceAll("贵州省", areaName));
            vTList.add(kp);
        }
        return vTList;
    }
}

