package com.hywa.map.service.tracing.impl;

import com.hywa.map.common.ConstantPool;
import com.hywa.map.config.EnvProperties;
import com.hywa.map.dao.entity.SysDict;
import com.hywa.map.dao.mapper.SysDictMapper;
import com.hywa.map.representation.*;
import com.hywa.map.service.tracing.TracingService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;

/**
 * 可追溯系统  对应ServiceImpl 实现类
 *
 * @author hywa
 * @create by 2018-12-13 13:41:59
 */
@Slf4j
@Service
public class TracingServiceImpl implements TracingService {

    @Autowired
    private SysDictMapper sysdictMapper;

    @Autowired
    EnvProperties envProperties;

    @Override
    public SysDictReps getFruitType() {
        List<SysDictRep> sysDictRepRepsList = getSysDictList(ConstantPool.CORP_LIBCODE, ConstantPool.DICTCODE_LEVEL_91, ConstantPool.DICT_FIRST_LENGTH, null);
        log.info("可追溯系统获取果树分类，返回结果：" + sysDictRepRepsList);
        return new SysDictReps((long) sysDictRepRepsList.size(), sysDictRepRepsList);
    }

    @Override
    public SysDictReps getFruitTree(String id) {
        List<SysDictRep> sysDictRepRepsList = getSysDictList(ConstantPool.CORP_LIBCODE, ConstantPool.DICTCODE_LEVEL_91, ConstantPool.DICT_SECOND_LENGTH, id);
        log.info("可追溯系统获取果树列表，请求数据：id|" + id + "，返回结果：" + sysDictRepRepsList);
        return new SysDictReps((long) sysDictRepRepsList.size(), sysDictRepRepsList);
    }

    @Override
    public SysDictReps getFarmType() {
        List<SysDictRep> sysDictRepRepsList = getSysDictList(ConstantPool.PLAN_LIBCODE, ConstantPool.DICTCODE_LEVEL_1, ConstantPool.DICT_FIRST_LENGTH, null);
        log.info("可追溯系统获取农事类型，返回结果：" + sysDictRepRepsList);
        return new SysDictReps((long) sysDictRepRepsList.size(), sysDictRepRepsList);
    }

    @Override
    public SysDictReps getPlantingType() {
        List<SysDictRep> sysDictRepRepsList = getSysDictList(ConstantPool.OTHER_LIBCODE, ConstantPool.DICTCODE_LEVEL_1, ConstantPool.DICT_FIRST_LENGTH, null);
        log.info("可追溯系统获取种植方式，返回结果：" + sysDictRepRepsList);
        return new SysDictReps((long) sysDictRepRepsList.size(), sysDictRepRepsList);
    }

    @Override
    public SysDictReps getPackageType() {
        List<SysDictRep> sysDictRepRepsList = getSysDictList(ConstantPool.OTHER_LIBCODE, ConstantPool.DICTCODE_LEVEL_3, ConstantPool.DICT_FIRST_LENGTH, null);
        log.info("可追溯系统获取包装规格，返回结果：" + sysDictRepRepsList);
        return new SysDictReps((long) sysDictRepRepsList.size(), sysDictRepRepsList);
    }

    @Override
    public SysDictReps getDosageUnit() {
        List<SysDictRep> sysDictRepRepsList = getSysDictList(ConstantPool.OTHER_LIBCODE, ConstantPool.DICTCODE_LEVEL_4, ConstantPool.DICT_FIRST_LENGTH, null);
        log.info("可追溯系统获取计量单位，返回结果：" + sysDictRepRepsList);
        return new SysDictReps((long) sysDictRepRepsList.size(), sysDictRepRepsList);
    }

    @Override
    public SysDictReps getDisasterType() {
        List<SysDictRep> sysDictRepRepsList = getSysDictList(ConstantPool.OTHER_LIBCODE, ConstantPool.DICTCODE_LEVEL_5);
        log.info("可追溯系统获取灾害类型，返回结果：" + sysDictRepRepsList);
        return new SysDictReps((long) sysDictRepRepsList.size(), sysDictRepRepsList);
    }

    @Override
    public List<SysDictRep> getSysDictList(String libCode, String dictCode, int codeLength, String parentId) {
        List<SysDict> sysDicts = null;
        sysDicts = sysdictMapper.selectByDictCodeAndLen(libCode, dictCode, codeLength, parentId);
        List<SysDictRep> sysDictRepRepsList = new ArrayList<>();
        for (SysDict sysDict : sysDicts) {
            SysDictRep sysDictRep = new SysDictRep(sysDict);
            sysDictRepRepsList.add(sysDictRep);
        }
        return sysDictRepRepsList;
    }
    public List<SysDictRep> getSysDictList(String libCode, String dictCode) {
        List<SysDict> sysDicts = null;
        sysDicts = sysdictMapper.selectByDictCodeNoSelf(libCode, dictCode);
        List<SysDictRep> sysDictRepRepsList = new ArrayList<>();
        for (SysDict sysDict : sysDicts) {
            SysDictRep sysDictRep = new SysDictRep(sysDict);
            sysDictRepRepsList.add(sysDictRep);
        }
        return sysDictRepRepsList;
    }
}

