package com.hywa.map.timer;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.hywa.map.common.utils.HttpUtils;
import com.hywa.map.config.redis.RedisUtil;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Configuration;
import org.springframework.scheduling.annotation.EnableScheduling;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.util.HashMap;
import java.util.Map;

@Component
@Configuration
@EnableScheduling
public class OrchardTimer {
    @Resource
    private RedisUtil redisUtil;
    @Value("${orchard.client.id}")
    private String orchardId;
    @Value("${orchard.client.secret}")
    private String orchardSecret;
    @Value("${orchard.client.url}")
    private String orchardUrl;
    private static Log log = LogFactory.getLog(OrchardTimer.class);

    /**
     * 获取农事管理系统token，保存到redis
     *
     * @return void
     */
    //@Scheduled(cron = "4 0 0/1 * * ? ")   //每小时4秒执行一次
    //@Scheduled(cron = "0/30 * * * * ? ")
    public void queryOrchardToken() {
        Map<String, String> queryParas = new HashMap<String, String>();
        queryParas.put("grant_type", "client_credentials");
        queryParas.put("client_id", orchardId);
        queryParas.put("client_secret", orchardSecret);
        String params = JSON.toJSONString(queryParas);
        try {
            String result = HttpUtils.post(orchardUrl + "oauth/token", params);
            JSONObject jsonObject = JSONObject.parseObject(result);
            String accessToken = jsonObject.getString("access_token");
            redisUtil.set("smartMapOrchardToken", accessToken);
            log.info("获取农事管理系统token成功，长度：" + accessToken.length());
        } catch (Exception e) {
            log.error("获取农事管理系统token失败|异常信息：" + e);
        }
    }
}
