package com.hywa.map.timer;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.hywa.map.common.param.PageBean;
import com.hywa.map.common.utils.HttpUtils;
import com.hywa.map.common.utils.UUIDUtils;
import com.hywa.map.dao.entity.Weather;
import com.hywa.map.dao.mapper.WeatherCityMapper;
import com.hywa.map.dao.mapper.WeatherMapper;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Configuration;
import org.springframework.scheduling.annotation.EnableScheduling;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.util.*;

@Component
@Configuration
@EnableScheduling
public class WeatherTimer {
    @Resource
    WeatherMapper weatherMapper;
    @Resource
    WeatherCityMapper weatherCityMapper;
    @Value("${lbs.amap.com.dev.key}")
    private String lbsDevKey;
    @Value("${lbs.amap.com.dev.url}")
    private String lbsDevUrl;
    private static Log log = LogFactory.getLog(WeatherTimer.class);
    private static int pageSize = 200;

    /**
     * 发送http接口获取天气信息，并保存到数据库
     *
     * @param city  高德区域编码
     * @param lcity 本地区域编码
     * @return void
     */
    private void handleRequest(String city, String lcity) {
        Map<String, String> queryParas = new HashMap<String, String>();
        queryParas.put("city", city);
        queryParas.put("key", lbsDevKey);
        queryParas.put("extensions", "base");
        try {
            String weatherInfo = HttpUtils.get(lbsDevUrl, queryParas);
            //String weatherInfo = HttpClientUtils.get(lbsDevUrl, queryParas);
            JSONObject weatherObj = JSON.parseObject(weatherInfo);
            JSONArray weatherArray = JSON.parseArray(weatherObj.getString("lives"));
            Weather weather = new Weather();
            for (Iterator iterator = weatherArray.iterator(); iterator.hasNext(); ) {
                JSONObject lives = (JSONObject) iterator.next();
                weather.setId(UUIDUtils.randomUUID());
                weather.setProvince(lives.getString("province"));
                weather.setCityName(lives.getString("city"));
                //dapWeather.setAdCode(lives.getString("adcode"));
                weather.setAdCode(lcity);
                weather.setWeather(lives.getString("weather"));
                weather.setTemperature(Integer.parseInt(lives.getString("temperature")));
                weather.setWindDire(lives.getString("winddirection"));
                weather.setWindPower(lives.getString("windpower"));
                weather.setHumidity(Integer.parseInt(lives.getString("humidity")));
                weather.setCreateUser("admin");
                weather.setCreateDate(new Date());
                break;
            }
            weatherMapper.add(weather);
        } catch (Exception e) {
            log.error("定时任务获取天气信息异常，查询城市：" + city + "|异常信息：" + e.getMessage());
        }
        log.info("定时任务获取天气信息成功，查询城市：" + city);
    }

    //@Scheduled(cron = "13 1 7 * * ?")   //每天7点1分13秒执行一次
    @Scheduled(cron = "50 0 0/1 * * ? ")   //每小时5秒执行一次
    //@Scheduled(cron="0 0/2 * * * ? ")   //每3分钟执行一次
    //@Scheduled(cron = "0/30 * * * * ? ")   //每30秒执行一次
    public void getWeatherInfo() {
        //为了避免内存溢出，需要分页查询数据，这里每次查询200条
        Map<String, Object> equalMap = new HashMap<String, Object>();
        equalMap.put("isDel", "0");
        Map<String, Object> likeMap = new HashMap<String, Object>();
        int totalCount = weatherCityMapper.searchTotals(equalMap, likeMap);
        if (totalCount > 0) {
            int cycleCount = totalCount % pageSize != 0 ? (totalCount / pageSize) + 1 : totalCount / pageSize;
            for (int i = 0; i < cycleCount; i++) {
                Map<String, Object> fieldMap = new HashMap<String, Object>();//查询字段
                fieldMap.put("adCode", "adCode");//高德区域编码
                fieldMap.put("ladCode", "ladCode");//本系统区域编码
                PageBean pageBean = new PageBean();
                pageBean.setPage(i * pageSize);
                pageBean.setRows(pageSize);
                try {
                    List<Map<String, Object>> list = weatherCityMapper.search(fieldMap, equalMap, null, pageBean, likeMap);
                    Iterator<Map<String, Object>> it = list.iterator();
                    while (it.hasNext()) {
                        Map<String, Object> map = it.next();
                        String adCode = (String) map.get("adCode");
                        String ladCode = (String) map.get("ladCode");
                        handleRequest(adCode, ladCode);
                    }
                } catch (Exception e) {
                    log.error("定时任务获取城市参数信息异常：" + e);
                }
            }
        }
    }
}
