package com.hywa.map.timer;

import com.alibaba.fastjson.JSONObject;
import com.hywa.map.common.utils.DateUtils;
import com.hywa.map.common.utils.HttpUtils;
import com.hywa.map.common.utils.StringUtils;
import com.hywa.map.config.EnvProperties;
import com.hywa.map.config.redis.RedisUtil;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.context.annotation.Configuration;
import org.springframework.scheduling.annotation.EnableScheduling;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;

@Component
@Configuration
@EnableScheduling
public class YsOpenPlatTimer {
    @Resource
    private RedisUtil redisUtil;
    @Resource
    private EnvProperties envProperties;
    public static String ysOpenPlatAccessToken = "ysOpenPlatAccessToken";
    private static Log log = LogFactory.getLog(YsOpenPlatTimer.class);

    /**
     * 获取萤石开放平台accessToken
     *
     * @return void
     */
    private void handleRequest() {
        String params = "appKey="+envProperties.getYsAppkey()+"&appSecret="+envProperties.getYsSecret();
        try {
            String currentDate = DateUtils.getCurrentDate();
            String result = HttpUtils.post(envProperties.getYsServerUrl() + "api/lapp/token/get?"+params, null);
            JSONObject jsonObject = JSONObject.parseObject(result);
            JSONObject data = jsonObject.getJSONObject("data");
            String newAccessToken = data.getString("accessToken");
            redisUtil.set(ysOpenPlatAccessToken, currentDate + "_" + newAccessToken);
            log.info("获取萤石开放平台accessToken成功，长度：" + newAccessToken.length());
        } catch (Exception e) {
            log.error("获取萤石开放平台accessToken失败|异常信息：" + e);
        }
    }

    /**
     * 获取萤石开放平台token，保存到redis
     *
     * @return void
     */
    @Scheduled(cron = "0 1 6 * * ? ")   //每天上午06:01触发
    //@Scheduled(cron = "0 41 13 * * ? ")
    public void queryAccessToken() {
        String accessToken = redisUtil.get(ysOpenPlatAccessToken);
        if (StringUtils.isEmpty(accessToken)) {
            log.info("获取萤石开放平台accessToken开始，直接获取token");
            handleRequest();
        } else {
            String lastDate = accessToken.split("\\_")[0];
            String exeDate = DateUtils.getDiffDay(-5);
            log.info("获取萤石开放平台accessToken开始，上次获取时间：" + lastDate+"，本次获取时间："+exeDate);
            if (lastDate.equals(exeDate)) {
                handleRequest();
            }
        }
    }
}
